/*
    Copyright 2007, 2008, 2009, 2010 Geyer Klaus

    This file is part of Cat'sEyE.

    Cat'sEyE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Cat'sEyE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Cat'sEyE.  If not, see <http://www.gnu.org/licenses/>.
*/



#include <gtk/gtk.h>
#ifndef NO_GIO
#include <gio/gio.h>
#endif
#include <string.h>
#include <stdlib.h>
#include <gdk/gdkkeysyms.h>


#include "helpers.h"
#include "Lists.h"
#include "mytrace.h"
#include "userdata.h"
#include "confighandling.h"
#include "CommandParser.h"
#include "globalDefinitions.h"

#include "allInclude.h"

extern GtkWindow 	*win;

int decryptPredefine(GString *Predefine, struct _predefineStruct *predefStruct);
int exchangePredefine(GString *gstrText, struct _predefineStruct *predefStruct);
int processBackSpaces(GString *gstrStartText, gboolean bMakeThreadSave);
int processMyStandardEscapeSequences(GString *gstrToChange);
int exchangeUserVarsInText(GString *gstrText, struct _DialogEntityList *DialogEntityList);
void UserDialog_FileDirChooserButton(GtkEntry *entry, gpointer data);
int processAsTextWithoutUserVars (GString *gstrDestination, GString *gstrSource, struct _predefineStruct *predefStruct, gboolean bMakeThreadSave);

////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////
//below are the new prototypes
int ParseXMLUserCommand_ParseDialog(struct _CallUserCommand_StartThreadStructure *ThreadStartStruct, struct _predefineStruct *predefineStruct, gboolean *bShowCommand);
int ParseXMLUserCommand_FillPredefineStruct (struct _predefineStruct *predefineStruct, struct _CallUserCommand_StartThreadStructure *ThreadStartStruct, GString *gstrWorkingPath );
int ParseXMLUserCommand_ReplaceVarsInString (GString **gstrpDestination, GString *gstrSource, struct _predefineStruct *predefineStruct, struct _DialogEntityList *DialogEntityList);
gboolean ParseXMLUserComman_ESCButtonHandlerForUserDialog(GtkWidget *widget, GdkEventKey *event, struct _CallUserCommand_StartThreadStructure *ThreadStartStruct);
int ParseXMLUserCommand_FreePredefineStruct (struct _predefineStruct *predefineStruct);
////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////
//starting with the new functions


int ParseXMLUserCommand (struct _CallUserCommand_StartThreadStructure *ThreadStartStruct, GString **gstrpParsedCommand, GString **gstrpQuitReplyText, GString **gstrpActionNoteText, GString *gstrWorkingPath){
    TRACEIT (10, "ParseXMLUserCommand       start");
    //in here we build the usercommand ready to be sent to the shell
    //we will create the dialog if necessary
    //and fill the Quitreply and actionNote strings
    //return: error = -1, 0=ok
    //1=user canceled one item
    //2=user canceled all items
    int iRet = 0;
    int iDRet = 0;
    gboolean bShowCommand=FALSE;
    GtkWidget *dialog, *entrybox;

    if (ThreadStartStruct==NULL || gstrpParsedCommand==NULL || gstrpQuitReplyText==NULL || gstrpActionNoteText==NULL || gstrWorkingPath==NULL){
        TRACEIT (2, "ParseXMLUserCommand        parameter NULL");
        return -1;
    }
    struct _predefineStruct predefineStruct;
    struct _UserCommandList *UserCommandList = (struct _UserCommandList*)(ThreadStartStruct->OurUserDataListItem->ListElement);

    iRet = ParseXMLUserCommand_FillPredefineStruct (&predefineStruct, ThreadStartStruct, gstrWorkingPath);

    if ( iRet == 0 && (UserCommandList->gstrDialogLabel!=NULL ||  UserCommandList->gstrDialogCaption!=NULL || UserCommandList->DialogEntityList != NULL)){
        iRet = ParseXMLUserCommand_ParseDialog(ThreadStartStruct, &predefineStruct, &bShowCommand);
    }

    if (iRet == 0 ) {
        //let us build the command
        iRet = ParseXMLUserCommand_ReplaceVarsInString (gstrpParsedCommand, UserCommandList->gstrCommand, &predefineStruct, UserCommandList->DialogEntityList);
    }

    if (iRet == 0 && bShowCommand==TRUE && *gstrpParsedCommand != NULL){
        myWrap_gdk_threads_enter("ParseXMLUserCommand");

        dialog = CreateMyDialog("Usercommand ready for take off",win,2,"Go",GTK_RESPONSE_ACCEPT,"Stop execution",GTK_RESPONSE_REJECT, NULL, 0,  TRUE);
        entrybox = gtk_entry_new();
        gtk_entry_set_text((GtkEntry *)entrybox,(*gstrpParsedCommand)->str);
        gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox),entrybox);
        gtk_widget_show_all(dialog);
        iDRet = gtk_dialog_run(GTK_DIALOG(dialog));
        if (iDRet == GTK_RESPONSE_ACCEPT){
            g_string_assign(*gstrpParsedCommand, gtk_entry_get_text ((GtkEntry *)entrybox));
            iRet = 0;
        }
        else{
            iRet = 1;
            //ThreadStartStruct->iCancelAll = 1;
        }

        gtk_widget_destroy(dialog);
        myWrap_gdk_threads_leave("ParseXMLUserCommand");
    }

    if (ThreadStartStruct->iCancelAll == 0 && iRet == 0){
        if (UserCommandList->option_gstrActionnote != NULL){
            ParseXMLUserCommand_ReplaceVarsInString (gstrpActionNoteText, UserCommandList->option_gstrActionnote, &predefineStruct, UserCommandList->DialogEntityList);
        }

        if (UserCommandList->option_gstrQuitreply != NULL){
            ParseXMLUserCommand_ReplaceVarsInString (gstrpQuitReplyText, UserCommandList->option_gstrQuitreply, &predefineStruct, UserCommandList->DialogEntityList);
        }
    }

    ParseXMLUserCommand_FreePredefineStruct (&predefineStruct);

    TRACEIT (10, "ParseXMLUserCommand       end");
    return iRet;
}


int ParseXMLUserCommand_ReplaceVarsInString (GString **gstrpDestination, GString *gstrSource, struct _predefineStruct *predefineStruct, struct _DialogEntityList *DialogEntityList){
    TRACEIT (10, "ParseXMLUserCommand_ReplaceVarsInString       start");
    //used e.g. to create the usercommand
    if (gstrpDestination == NULL || gstrSource==NULL || predefineStruct==NULL ){
        TRACEIT (2, "ParseXMLUserCommand_ReplaceVarsInString        parameter NULL");
        return -1;
    }
    int iRet=0;
    int i;

    if (*gstrpDestination == NULL){
        *gstrpDestination = g_string_new(gstrSource->str);
    }
    else{
        g_string_assign (*gstrpDestination, gstrSource->str);
    }

    processMyStandardEscapeSequences (*gstrpDestination);

    i=0;
    if (DialogEntityList != NULL){
        do{
            iRet = exchangeUserVarsInText(*gstrpDestination, DialogEntityList);
            i++;
        }while (iRet > 0 && i < 5);
    }
    if (iRet == 0){
        exchangePredefine(*gstrpDestination, predefineStruct);
    }
    if (iRet == 0){
        processBackSpaces(*gstrpDestination, TRUE);
    }
    TRACEIT (10, "ParseXMLUserCommand_ReplaceVarsInString       end");
    return 0;
}


void ParseXMLUserCommand_ButtonOk (GtkButton *button, struct _CallUserCommand_StartThreadStructure *ThreadStartStruct){
    TRACEIT (10, "ParseXMLUserCommand_ButtonOk       start");

    if ( ThreadStartStruct == NULL){
        TRACEIT (2, "ParseXMLUserCommand_ButtonOk       parameter NULL");
        return;
    }
    //GTK_RESPONSE_ACCEPT,"Cancel",GTK_RESPONSE_REJECT, "Cancel all", GTK_RESPONSE_CANCEL
    ThreadStartStruct->iUserDialogReturnValue = GTK_RESPONSE_ACCEPT;

    //do not destroy the dialog in here! as we do need the objects for reading their values, but
    //didnt increase the refernece pointer!
    //gtk_widget_destroy (ThreadStartStruct->wdgtUserDialog);
    g_cond_signal (ThreadStartStruct->gcontDialogCondition);

    TRACEIT (10, "ParseXMLUserCommand_ButtonOk       end");
}


void ParseXMLUserCommand_ButtonCancel (GtkButton *button, struct _CallUserCommand_StartThreadStructure *ThreadStartStruct){
    TRACEIT (10, "ParseXMLUserCommand_ButtonCancel       start");

    if (ThreadStartStruct == NULL){
        TRACEIT (2, "ParseXMLUserCommand_ButtonCancel       parameter NULL");
        return;
    }
    //GTK_RESPONSE_ACCEPT,"Cancel",GTK_RESPONSE_REJECT, "Cancel all", GTK_RESPONSE_CANCEL
    ThreadStartStruct->iUserDialogReturnValue = GTK_RESPONSE_REJECT;

    //do not destroy the dialog in here! as we do need the objects for reading their values, but
    //didnt increase the refernece pointer!
    //gtk_widget_destroy (ThreadStartStruct->wdgtUserDialog);
    g_cond_signal (ThreadStartStruct->gcontDialogCondition);

    TRACEIT (10, "ParseXMLUserCommand_ButtonCancel       end");
}


void ParseXMLUserCommand_ButtonCancelAll (GtkButton *button, struct _CallUserCommand_StartThreadStructure *ThreadStartStruct){
    TRACEIT (10, "ParseXMLUserCommand_ButtonCancel       start");

    if (ThreadStartStruct == NULL){
        TRACEIT (2, "ParseXMLUserCommand_ButtonCancelAll       parameter NULL");
        return;
    }
    //GTK_RESPONSE_ACCEPT,"Cancel",GTK_RESPONSE_REJECT, "Cancel all", GTK_RESPONSE_CANCEL
    ThreadStartStruct->iUserDialogReturnValue = GTK_RESPONSE_CANCEL;

    //do not destroy the dialog in here! as we do need the objects for reading their values, but
    //didnt increase the refernece pointer!
    //gtk_widget_destroy (ThreadStartStruct->wdgtUserDialog);
    g_cond_signal (ThreadStartStruct->gcontDialogCondition);


    TRACEIT (10, "ParseXMLUserCommand_ButtonCancelAll       end");
    return;
}


gboolean ParseXMLUserCommand_CreateWidgetPrototypeForUserDialog_DeleteNotification (GtkWidget *widget, GdkEvent  *event, void *data){
    //this is the handler for delete-event, which is called only if the user!! requests the widget to be closed.
    //but we want the user to be able to close only with our buttons
    //but if gtk isnt able to avoid the visibility of the close button (mainly placed in the upper right corner of each window)
    //we will end up here and then we call the cancel button.
    TRACEIT (10, "ParseXMLUserCommand_CreateWidgetPrototypeForUserDialog_DeleteNotification       start");
    struct _CallUserCommand_StartThreadStructure *ThreadStartStruct = (struct _CallUserCommand_StartThreadStructure *)data;
    if (ThreadStartStruct == NULL){
        TRACEIT (2, "ParseXMLUserCommand_CreateWidgetPrototypeForUserDialog_DeleteNotification       parameter NULL");
        return TRUE;
    }

    ParseXMLUserCommand_ButtonCancelAll (NULL, ThreadStartStruct);


    TRACEIT (10, "ParseXMLUserCommand_CreateWidgetPrototypeForUserDialog_DeleteNotification       end");
    return TRUE;    //True: widget will stay
}


GtkWidget * ParseXMLUserCommand_CreateWidgetPrototypeForUserDialog(char *ccpCaption, struct _CallUserCommand_StartThreadStructure *ThreadStartStruct, int iNumOfButtons){
    //iNumOfButtons can be 2 or 3
    TRACEIT (10, "ParseXMLUserCommand_CreateWidgetPrototypeForUserDialog       start");

    if (ccpCaption == NULL || ThreadStartStruct == NULL){
        TRACEIT (2, "ParseXMLUserCommand_CreateWidgetPrototypeForUserDialog       parameter NULL");
        return NULL;
    }
    myWrap_gdk_threads_enter("ParseXMLUserCommand_CreateWidgetPrototypeForUserDialog 1");
    GtkWidget 	*dialog = CreateMyWindow(ccpCaption, win, -1, -1, TRUE, TRUE, TRUE, NULL, FALSE, FALSE);
    myWrap_gdk_threads_leave("ParseXMLUserCommand_CreateWidgetPrototypeForUserDialog 1");
    if (dialog == NULL){
        return NULL;
    }
    GtkWidget *btnCancelAll=NULL;
    GtkWidget *btnCancel=NULL;
    GtkWidget *hboxButtons=NULL;
    GtkWidget *btnOk=NULL;
    GtkWidget *vbox=NULL;
    GtkWidget *hbox=NULL;

    myWrap_gdk_threads_enter("ParseXMLUserCommand_CreateWidgetPrototypeForUserDialog 2");

    hboxButtons         = gtk_hbox_new(TRUE, 0);
    hbox                = gtk_hbox_new(TRUE, 0);
    vbox                = gtk_vbox_new(FALSE, 3);
    btnOk               = gtk_button_new_with_label("Ok");
    btnCancel           = gtk_button_new_with_label("Cancel");
    if (iNumOfButtons == 3){
        btnCancelAll = gtk_button_new_with_label("Cancel all");
    }
    GtkWidget *separator    = gtk_hseparator_new();

    ThreadStartStruct->wdgtUserDialog = dialog;
    ThreadStartStruct->wdgtUSerDialogMainVBox = vbox;
    ThreadStartStruct->iUserDialogReturnValue = GTK_RESPONSE_NONE;

    gtk_box_pack_start ((GtkBox *)hboxButtons, btnOk, TRUE, TRUE, 3);
    if (iNumOfButtons == 3){
        gtk_box_pack_start ((GtkBox *)hboxButtons, btnCancelAll, TRUE, TRUE, 3);
        //gtk_container_add ((GtkContainer *)hbox, btnCancelAll);
    }
    gtk_box_pack_start ((GtkBox *)hboxButtons, btnCancel, TRUE, TRUE, 3);

    gtk_box_pack_end ((GtkBox *)vbox, hboxButtons, FALSE, FALSE, 5);
    gtk_box_pack_end ((GtkBox *)vbox, separator, FALSE, FALSE, 0);

    gtk_box_pack_start((GtkBox *)hbox, vbox, TRUE, TRUE, 6);

    gtk_container_add ((GtkContainer *)dialog, hbox);

    g_signal_connect (dialog,"key-press-event",G_CALLBACK(ParseXMLUserComman_ESCButtonHandlerForUserDialog),ThreadStartStruct);
    //g_signal_connect (dialog,"destroy",G_CALLBACK(ParseXMLUserCommand_CreateWidgetPrototypeForUserDialog_DestroyNotification),(void *)ThreadStartStruct);
    g_signal_connect (dialog,"delete-event",G_CALLBACK(ParseXMLUserCommand_CreateWidgetPrototypeForUserDialog_DeleteNotification), (void *)ThreadStartStruct);

    g_signal_connect (btnOk,"clicked",G_CALLBACK(ParseXMLUserCommand_ButtonOk), (void *)ThreadStartStruct);
    g_signal_connect (btnCancel,"clicked",G_CALLBACK(ParseXMLUserCommand_ButtonCancel), (void *)ThreadStartStruct);
    if (iNumOfButtons == 3){
        g_signal_connect (btnCancelAll,"clicked",G_CALLBACK(ParseXMLUserCommand_ButtonCancelAll), (void *)ThreadStartStruct);
    }

    myWrap_gdk_threads_leave("ParseXMLUserCommand_CreateWidgetPrototypeForUserDialog 2");

    TRACEIT (10, "ParseXMLUserCommand_CreateWidgetPrototypeForUserDialog       end");
    return dialog;
}


gboolean ParseXMLUserComman_ESCButtonHandlerForUserDialog(GtkWidget *widget, GdkEventKey *event, struct _CallUserCommand_StartThreadStructure *ThreadStartStruct){
	TRACEIT(10,"ParseXMLUserComman_ESCButtonHandlerForUserDialog		start");
	if (ThreadStartStruct == NULL){
        TRACEIT (2, "ParseXMLUserComman_ESCButtonHandlerForUserDialog       Parameter NULL");
        return FALSE;
    }
	guint modifiers;

	modifiers = gtk_accelerator_get_default_mod_mask ();

    if ( (event->keyval == GDK_Escape)){
        ParseXMLUserCommand_ButtonCancel (NULL, ThreadStartStruct);

		TRACEIT(10,"ParseXMLUserComman_ESCButtonHandlerForUserDialog		ends");
		return TRUE;
	}

	TRACEIT(10,"ParseXMLUserComman_ESCButtonHandlerForUserDialog		end");
	return FALSE;
}


void ParseXMLUserCommand_ShowDialogThread(struct _CallUserCommand_StartThreadStructure *ThreadStartStruct){
    //simply will show the dialog given as parameter and locks the mutes also given as parameter
    //this thread will exit after the widged was created and thats it.
    TRACEIT (10, "ParseXMLUserCommand_ShowDialogThread       start");
    if (ThreadStartStruct == NULL){
        TRACEIT (2, "ParseXMLUserCommand_ShowDialogThread       Parameter NULL");
        return;
    }
    if (ThreadStartStruct->wdgtUserDialog == NULL || ThreadStartStruct->gcontDialogCondition == NULL){
        TRACEIT (2, "ParseXMLUserCommand_ShowDialogThread       Parameter  2 NULL");
        return;
    }

    myWrap_gdk_threads_enter("ParseXMLUserCommand_ShowDialogThread 1");
    gtk_widget_show_all (ThreadStartStruct->wdgtUserDialog);
    myWrap_gdk_threads_leave("ParseXMLUserCommand_ShowDialogThread 1");

    TRACEIT (10, "ParseXMLUserCommand_ShowDialogThread       end");
    //g_thread_exit (0); //this is no thread
    return;
}


void ParseXMLUserCommand_EntryBoxActivate(GtkEntry *entry, struct _CallUserCommand_StartThreadStructure *ThreadStartStruct){
    TRACEIT (10, "ParseXMLUserCommand_EntryBoxActivate       start");
    if (ThreadStartStruct == NULL){
        TRACEIT (2, "ParseXMLUserCommand_EntryBoxActivate      parameter NULL");
        return;
    }

    ParseXMLUserCommand_ButtonOk (NULL, ThreadStartStruct);
    TRACEIT (10, "ParseXMLUserCommand_EntryBoxActivate       end");
}


int ParseXMLUserCommand_ParseDialog(struct _CallUserCommand_StartThreadStructure *ThreadStartStruct, struct _predefineStruct *predefineStruct, gboolean *bShowCommand){
    TRACEIT (10, "ParseXMLUserCommand_ParseDialog       start");
    //function which creates the dailog and saves the users answers
    //return: -1=error, 0=ok, 1=cancel this command, 2=cancel all commands (TODO)
    if (ThreadStartStruct == NULL || predefineStruct == NULL){
        TRACEIT (2, "ParseXMLUserCommand_ParseDialog            parameter NULL");
        return -1;
    }
    struct _UserCommandList *UserCommandList = (struct _UserCommandList*)(ThreadStartStruct->OurUserDataListItem->ListElement);
    if (UserCommandList == NULL){
        TRACEIT (2, "ParseXMLUserCommand_ParseDialog            parameter 2 NULL");
        return -1;
    }

    struct _DialogEntityList *DailogEntityList_BufList = NULL;
    struct _DialogEntityList *DailogEntityList = UserCommandList->DialogEntityList;
    //if (DailogEntityList == NULL){
    //    TRACEIT (2, "ParseXMLUserCommand_ParseDialog            parameter 3 NULL");
    //    return -1;
    //}

    int iRet = 0;
    int iNumOfButtons = 3;

    GString *gstrBuf = g_string_sized_new(10);
	GtkWidget *label;
	GtkWidget* CheckButton;
	GtkWidget *sep1;
	GString *gstrVarIdentifier = g_string_sized_new(10);
	GString *gstrVarValue = g_string_sized_new(10);
	GString *gstrLabel = g_string_sized_new(10);
	GString *gstrPredefine = g_string_sized_new(10);
	GString *gstrAdditional1 = g_string_sized_new(10);
	GString *gstrAdditional2 = g_string_sized_new(10);
	GtkWidget *MainVBox = NULL;
	GtkWidget *VBoxBuf = NULL;

    if (UserCommandList->gstrDialogCaption == NULL){
        UserCommandList->gstrDialogCaption = g_string_new("CEE");
    }

    if (UserCommandList->option_iCallType == USERCOMMAND_CALLTYPE_ALL){
        iNumOfButtons = 2;
    }
    ParseXMLUserCommand_CreateWidgetPrototypeForUserDialog(UserCommandList->gstrDialogCaption->str, ThreadStartStruct, iNumOfButtons);

    myWrap_gdk_threads_enter("ParseXMLUserCommand_ParseDialog 1");

    MainVBox = ThreadStartStruct->wdgtUSerDialogMainVBox;

	if (UserCommandList->gstrDialogLabel){
        processAsTextWithoutUserVars (gstrBuf, UserCommandList->gstrDialogLabel, predefineStruct, FALSE);
		label = gtk_label_new(gstrBuf->str);
		VBoxBuf = gtk_vbox_new (FALSE, 1);
		gtk_box_pack_start ((GtkBox *)VBoxBuf, label, FALSE, FALSE, 0);
		gtk_box_pack_start ((GtkBox *)MainVBox, VBoxBuf, FALSE, FALSE, 5);
	}

	DailogEntityList_BufList = DailogEntityList;
	while (DailogEntityList_BufList){
	    if (DailogEntityList_BufList->iValid != 1){
            DailogEntityList_BufList = DailogEntityList_BufList->nextItem;
            continue;
	    }
	    g_string_erase (gstrVarIdentifier, 0, -1);
	    g_string_erase (gstrVarValue, 0, -1);
	    g_string_erase (gstrLabel, 0, -1);
	    g_string_erase (gstrPredefine, 0, -1);
	    g_string_erase (gstrAdditional1, 0, -1);
	    g_string_erase (gstrAdditional2, 0, -1);

	    if (DailogEntityList_BufList->gstrVarIdentifier){
            processAsTextWithoutUserVars (gstrVarIdentifier, DailogEntityList_BufList->gstrVarIdentifier, predefineStruct, FALSE);
	    }
	    if (DailogEntityList_BufList->gstrVarValue){
            processAsTextWithoutUserVars (gstrVarValue, DailogEntityList_BufList->gstrVarValue, predefineStruct, FALSE);
	    }
	    if (DailogEntityList_BufList->gstrLabel){
            processAsTextWithoutUserVars (gstrLabel, DailogEntityList_BufList->gstrLabel, predefineStruct, FALSE);
	    }
	    if (DailogEntityList_BufList->gstrPredefine){
            processAsTextWithoutUserVars (gstrPredefine, DailogEntityList_BufList->gstrPredefine, predefineStruct, FALSE);
	    }
	    if (DailogEntityList_BufList->gstrAdditional1){
            processAsTextWithoutUserVars (gstrAdditional1, DailogEntityList_BufList->gstrAdditional1, predefineStruct, FALSE);
	    }
	    if (DailogEntityList_BufList->gstrAdditional2){
            processAsTextWithoutUserVars (gstrAdditional2, DailogEntityList_BufList->gstrAdditional2, predefineStruct, FALSE);
	    }

		if (DailogEntityList_BufList->iType == USERDIALOG_ENTITY_TYPE_QUESTION){
            DailogEntityList_BufList->wdgtLabel =  gtk_label_new(gstrLabel->str);
            gtk_misc_set_alignment ( (GtkMisc *)DailogEntityList_BufList->wdgtLabel, 0.0f, 0.0f);
            //gtk_misc_set_padding ( (GtkMisc *)DailogEntityList_BufList->wdgtLabel, 4, 0);
            DailogEntityList_BufList->wdgtEntryBox = gtk_entry_new();
            if (DailogEntityList_BufList->gstrPredefine){
                gtk_entry_set_text(	(GtkEntry *)DailogEntityList_BufList->wdgtEntryBox, gstrPredefine->str);
            }
            g_signal_connect ((GtkEntry *)DailogEntityList_BufList->wdgtEntryBox, "activate",G_CALLBACK(ParseXMLUserCommand_EntryBoxActivate),ThreadStartStruct);

            VBoxBuf = gtk_vbox_new (FALSE, 1);
            gtk_box_pack_start ((GtkBox *)VBoxBuf, DailogEntityList_BufList->wdgtLabel, FALSE, FALSE, 0);
            gtk_box_pack_start ((GtkBox *)VBoxBuf, DailogEntityList_BufList->wdgtEntryBox, FALSE, FALSE, 0);
            gtk_box_pack_start ((GtkBox *)MainVBox, VBoxBuf, FALSE, FALSE, 2);
		}
		else if (DailogEntityList_BufList->iType == USERDIALOG_ENTITY_TYPE_CHECKBOX){
            DailogEntityList_BufList->wdgtLabel = gtk_check_button_new_with_mnemonic(gstrLabel->str);
            //VBoxBuf = gtk_vbox_new (FALSE, 1);
            gtk_box_pack_start ((GtkBox *)MainVBox, DailogEntityList_BufList->wdgtLabel, FALSE, FALSE, 0);
            if (DailogEntityList_BufList->gstrPredefine){
                if (strcmp(DailogEntityList_BufList->gstrPredefine->str, "true")==0 || strcmp(gstrPredefine->str, "TRUE")==0 || strcmp(DailogEntityList_BufList->gstrPredefine->str, "True")==0 ){
                    gtk_toggle_button_set_active ( (GtkToggleButton *)DailogEntityList_BufList->wdgtLabel, TRUE);
                }
            }
		}
		else if (DailogEntityList_BufList->iType == USERDIALOG_ENTITY_TYPE_TEXT){	//0=question, 1=checkbox, 2=text
		    //g_string_assign (gstrBuf, DailogEntityList_BufList->gstrLabel->str);
	        DailogEntityList_BufList->wdgtLabel = gtk_label_new (gstrLabel->str);
            //DailogEntityList_BufList->wdgtLabel = gtk_label_new ("testttt");
            gtk_misc_set_alignment ( (GtkMisc *)DailogEntityList_BufList->wdgtLabel, 0.0f, 0.0f);
            //gtk_misc_set_padding ( (GtkMisc *)DailogEntityList_BufList->wdgtLabel, 4, 0);
            gtk_box_pack_start ((GtkBox *)MainVBox, DailogEntityList_BufList->wdgtLabel, FALSE, FALSE, 2);
		}
		else if (DailogEntityList_BufList->iType==USERDIALOG_ENTITY_TYPE_FILECHOOSER || DailogEntityList_BufList->iType == USERDIALOG_ENTITY_TYPE_DIRCHOOSER){ //0=question, 1=checkbox, 2=text, 3=filechooser, 4=dirchooser
            DailogEntityList_BufList->wdgtLabel =  gtk_label_new(gstrLabel->str);
            gtk_misc_set_alignment ( (GtkMisc *)DailogEntityList_BufList->wdgtLabel, 0.0f, 0.0f);
            //gtk_misc_set_padding ( (GtkMisc *)DailogEntityList_BufList->wdgtLabel, 4, 0);
            DailogEntityList_BufList->wdgtEntryBox = gtk_entry_new();
            if (DailogEntityList_BufList->gstrPredefine){
                gtk_entry_set_text(	(GtkEntry *)DailogEntityList_BufList->wdgtEntryBox, gstrPredefine->str);
            }
            g_signal_connect ((GtkEntry *)DailogEntityList_BufList->wdgtEntryBox, "activate",G_CALLBACK(ParseXMLUserCommand_EntryBoxActivate),ThreadStartStruct);

            DailogEntityList_BufList->wdgtAdditional2 = gtk_hbox_new (FALSE, 0);
            DailogEntityList_BufList->wdgtAdditional1 = gtk_button_new_with_label (gstrAdditional1->str);

            g_signal_connect ((GtkButton *)DailogEntityList_BufList->wdgtAdditional1, "clicked",G_CALLBACK(UserDialog_FileDirChooserButton),DailogEntityList_BufList);

            VBoxBuf = gtk_vbox_new (FALSE, 1);
            gtk_box_pack_start ((GtkBox *)DailogEntityList_BufList->wdgtAdditional2, DailogEntityList_BufList->wdgtEntryBox, TRUE, TRUE, 0);
            gtk_box_pack_start ((GtkBox *)DailogEntityList_BufList->wdgtAdditional2, DailogEntityList_BufList->wdgtAdditional1, FALSE, FALSE, 3);
            gtk_box_pack_start ((GtkBox *)VBoxBuf, DailogEntityList_BufList->wdgtLabel, FALSE, FALSE, 0);
            gtk_box_pack_start ((GtkBox *)VBoxBuf, DailogEntityList_BufList->wdgtAdditional2, FALSE, FALSE, 0);
            gtk_box_pack_start ((GtkBox *)MainVBox, VBoxBuf, FALSE, FALSE, 2);
  		}
  		else if (DailogEntityList_BufList->iType == USERDIALOG_ENTITY_TYPE_SEPARATOR){	//0=question, 1=checkbox, 2=text
	        DailogEntityList_BufList->wdgtLabel = gtk_separator_menu_item_new ();
            gtk_box_pack_start ((GtkBox *)MainVBox, DailogEntityList_BufList->wdgtLabel, FALSE, FALSE, 0);
		}
		DailogEntityList_BufList = DailogEntityList_BufList->nextItem;
	}

    sep1 = gtk_separator_menu_item_new ();
    gtk_box_pack_start ((GtkBox *)MainVBox, sep1, FALSE, FALSE, 0);

	CheckButton = gtk_check_button_new_with_label("Show me the complete command");
	gtk_box_pack_start ((GtkBox *)MainVBox, CheckButton, FALSE, FALSE, 0);

    myWrap_gdk_threads_leave("ParseXMLUserCommand_ParseDialog 1");

    int iDRet = 0;
    GMutex *mutex = g_mutex_new ();
    ThreadStartStruct->gcontDialogCondition = g_cond_new ();
    g_mutex_lock (mutex);

    ParseXMLUserCommand_ShowDialogThread ((void *)ThreadStartStruct);

    g_cond_wait (ThreadStartStruct->gcontDialogCondition, mutex);
    g_mutex_unlock (mutex);

    g_mutex_free (mutex);
    mutex = NULL;
    g_cond_free (ThreadStartStruct->gcontDialogCondition);
    ThreadStartStruct->gcontDialogCondition = NULL;
    ThreadStartStruct->gcontEachCondition = NULL;

    iDRet = ThreadStartStruct->iUserDialogReturnValue;

    myWrap_gdk_threads_enter("ParseXMLUserCommand_ParseDialog 2");

	if (iDRet== GTK_RESPONSE_ACCEPT){
		DailogEntityList_BufList = DailogEntityList;
		while (DailogEntityList_BufList){
		   	if (DailogEntityList_BufList->iValid == 1 && DailogEntityList_BufList->iType == USERDIALOG_ENTITY_TYPE_QUESTION){
				DailogEntityList_BufList->gstrVarValue = g_string_new( gtk_entry_get_text((GtkEntry *)DailogEntityList_BufList->wdgtEntryBox ) );
			}
			else if (DailogEntityList_BufList->iValid == 1 && DailogEntityList_BufList->iType == USERDIALOG_ENTITY_TYPE_CHECKBOX){
				if (gtk_toggle_button_get_active ( (GtkToggleButton *)DailogEntityList_BufList->wdgtLabel)){	//checkbox checked
					DailogEntityList_BufList->gstrVarValue = g_string_new((DailogEntityList_BufList->gstrAdditional1!=NULL)?DailogEntityList_BufList->gstrAdditional1->str:"");
				}
				else if (DailogEntityList_BufList->gstrAdditional2){
					DailogEntityList_BufList->gstrVarValue = g_string_new((DailogEntityList_BufList->gstrAdditional2!=NULL)?DailogEntityList_BufList->gstrAdditional2->str:"");
				}
			}
			else if (DailogEntityList_BufList->iValid == 1 && DailogEntityList_BufList->iType == USERDIALOG_ENTITY_TYPE_FILECHOOSER){
				DailogEntityList_BufList->gstrVarValue = g_string_new( gtk_entry_get_text((GtkEntry *)DailogEntityList_BufList->wdgtEntryBox ) );
			}
			else if (DailogEntityList_BufList->iValid == 1 && DailogEntityList_BufList->iType == USERDIALOG_ENTITY_TYPE_DIRCHOOSER){
				DailogEntityList_BufList->gstrVarValue = g_string_new( gtk_entry_get_text((GtkEntry *)DailogEntityList_BufList->wdgtEntryBox ) );
			}
			DailogEntityList_BufList = DailogEntityList_BufList->nextItem;
		}
		if (gtk_toggle_button_get_active ( (GtkToggleButton *)CheckButton)){
			*bShowCommand = TRUE;
		}
		else{
			*bShowCommand = FALSE;
		}

		iRet = 0;
	}
	else if (iDRet == GTK_RESPONSE_CANCEL){
        iRet = 2;
        ThreadStartStruct->iCancelAll = 2;
    }
    else{ //if (iDRet == GTK_RESPONSE_REJECT){  //fallback
		iRet = 1; //cancel this command
	}

    //we have to forget this things, because the list-free fnction would try to unref them
	DailogEntityList_BufList = DailogEntityList;
    while (DailogEntityList_BufList){
        if (DailogEntityList_BufList->iValid == 1 && DailogEntityList_BufList->iType == USERDIALOG_ENTITY_TYPE_QUESTION){
            DailogEntityList_BufList->wdgtLabel = NULL;
            DailogEntityList_BufList->wdgtEntryBox = NULL;
        }
        else if (DailogEntityList_BufList->iValid == 1 && DailogEntityList_BufList->iType == USERDIALOG_ENTITY_TYPE_CHECKBOX){
            DailogEntityList_BufList->wdgtLabel = NULL;
        }
        else if (DailogEntityList_BufList->iValid == 1 && DailogEntityList_BufList->iType == USERDIALOG_ENTITY_TYPE_TEXT){
            DailogEntityList_BufList->wdgtLabel = NULL;
        }
        else if (DailogEntityList_BufList->iValid == 1 && DailogEntityList_BufList->iType == USERDIALOG_ENTITY_TYPE_FILECHOOSER){
            DailogEntityList_BufList->wdgtLabel = NULL;
            DailogEntityList_BufList->wdgtEntryBox = NULL;
            DailogEntityList_BufList->wdgtAdditional1 = NULL;
            DailogEntityList_BufList->wdgtAdditional2 = NULL;
        }
        else if (DailogEntityList_BufList->iValid == 1 && DailogEntityList_BufList->iType == USERDIALOG_ENTITY_TYPE_DIRCHOOSER){
            DailogEntityList_BufList->wdgtLabel = NULL;
            DailogEntityList_BufList->wdgtEntryBox = NULL;
            DailogEntityList_BufList->wdgtAdditional1 = NULL;
            DailogEntityList_BufList->wdgtAdditional2 = NULL;
        }
        else if (DailogEntityList_BufList->iValid == 1 && DailogEntityList_BufList->iType == USERDIALOG_ENTITY_TYPE_SEPARATOR){
            DailogEntityList_BufList->wdgtLabel = NULL;
        }
        DailogEntityList_BufList = DailogEntityList_BufList->nextItem;
    }

    gtk_widget_destroy (ThreadStartStruct->wdgtUserDialog);
	myWrap_gdk_threads_leave("ParseXMLUserCommand_ParseDialog 2");

	g_string_free(gstrBuf, TRUE);
	g_string_free(gstrVarIdentifier, TRUE);
	g_string_free(gstrVarValue, TRUE);
	g_string_free(gstrLabel, TRUE);
	g_string_free(gstrPredefine, TRUE);
	g_string_free(gstrAdditional1, TRUE);
	g_string_free(gstrAdditional2, TRUE);

    TRACEIT (10, "ParseXMLUserCommand_ParseDialog       end");
    return iRet;
}


int ParseXMLUserCommand_FreePredefineStruct (struct _predefineStruct *predefineStruct){
    TRACEIT (10, "ParseXMLUserCommand_FreePredefineStruct       start");
    if (predefineStruct == NULL){
        TRACEIT (2, "ParseXMLUserCommand_FreePredefineStruct        parameter NULL");
        return -1;
    }

    if (predefineStruct->gstrAllMarkedItems_WithoutPath_Unquoted){
        g_string_free (predefineStruct->gstrAllMarkedItems_WithoutPath_Unquoted, TRUE);
        predefineStruct->gstrAllMarkedItems_WithoutPath_Unquoted = NULL;
    }
    if (predefineStruct->gstrAllMarkedItems_WithoutPath_Quoted){
        g_string_free (predefineStruct->gstrAllMarkedItems_WithoutPath_Quoted, TRUE);
        predefineStruct->gstrAllMarkedItems_WithoutPath_Quoted = NULL;
    }
    if (predefineStruct->gstrAllMarkedItems_WithPath_Unquoted){
        g_string_free (predefineStruct->gstrAllMarkedItems_WithPath_Unquoted, TRUE);
        predefineStruct->gstrAllMarkedItems_WithPath_Unquoted = NULL;
    }
    if (predefineStruct->gstrAllMarkedItems_WithPath_Quoted){
        g_string_free (predefineStruct->gstrAllMarkedItems_WithPath_Quoted, TRUE);
        predefineStruct->gstrAllMarkedItems_WithPath_Quoted = NULL;
    }
    if (predefineStruct->gstrOneMarkedItem_WithoutPath_Unquoted){
        g_string_free (predefineStruct->gstrOneMarkedItem_WithoutPath_Unquoted, TRUE);
        predefineStruct->gstrOneMarkedItem_WithoutPath_Unquoted = NULL;
    }
    if (predefineStruct->gstrOneMarkedItem_WithoutPath_Quoted){
        g_string_free (predefineStruct->gstrOneMarkedItem_WithoutPath_Quoted, TRUE);
        predefineStruct->gstrOneMarkedItem_WithoutPath_Quoted = NULL;
    }
    if (predefineStruct->gstrOneMarkedItem_WithPath_Unquoted){
        g_string_free (predefineStruct->gstrOneMarkedItem_WithPath_Unquoted, TRUE);
        predefineStruct->gstrOneMarkedItem_WithPath_Unquoted = NULL;
    }
    if (predefineStruct->gstrOneMarkedItem_WithPath_Quoted){
        g_string_free (predefineStruct->gstrOneMarkedItem_WithPath_Quoted, TRUE);
        predefineStruct->gstrOneMarkedItem_WithPath_Quoted = NULL;
    }

    TRACEIT (10, "ParseXMLUserCommand_FreePredefineStruct       end");
    return 0;
}


int ParseXMLUserCommand_FillPredefineStruct (struct _predefineStruct *predefineStruct, struct _CallUserCommand_StartThreadStructure *ThreadStartStruct, GString *gstrWorkingPath ){
    TRACEIT (10, "ParseXMLUserCommand_FillPredefineStruct       start");
    //fills the struct _predefineStruct with the values from ThreadStartStruct
    //keep in mind that some GStrings will not be copied but only their pointer!
    //so, we dont need to free them in here
    //return: 0=ok, -1=error

    if (predefineStruct==NULL || ThreadStartStruct==NULL || gstrWorkingPath==NULL){
        TRACEIT (2, "ParseXMLUserCommand_FillPredefineStruct        parameter NULL");
        return -1;
    }

    GList *glistBuf =  NULL;
    int iBuf;

    predefineStruct->gstrAllMarkedItems_WithoutPath_Unquoted    = NULL;
    predefineStruct->gstrAllMarkedItems_WithoutPath_Quoted      = NULL;
    predefineStruct->gstrAllMarkedItems_WithPath_Unquoted       = NULL;
    predefineStruct->gstrAllMarkedItems_WithPath_Quoted         = NULL;
    predefineStruct->gstrOneMarkedItem_WithoutPath_Unquoted     = NULL;
    predefineStruct->gstrOneMarkedItem_WithoutPath_Quoted       = NULL;
    predefineStruct->gstrOneMarkedItem_WithPath_Unquoted        = NULL;
    predefineStruct->gstrOneMarkedItem_WithPath_Quoted          = NULL;


    predefineStruct->gstrThisSidePath   = ThreadStartStruct->gstrThisSidePath;
    predefineStruct->gstrOtherSidePath  = ThreadStartStruct->gstrOtherSidePath;
    predefineStruct->gstrLeftSidePath   = ThreadStartStruct->gstrLeftSidePath;
    predefineStruct->gstrRightSidePath  = ThreadStartStruct->gstrRightSidePath;
    predefineStruct->gstrHomeDir        = ThreadStartStruct->gstrHomeDir;
    //predefineStruct->gstrWorkingPath    = gstrWorkingPath;

    //if we come from FileShelf gstrThisSidePath and gstrOtherSidePath are NULL
    if (ThreadStartStruct->OurUserDataListItem->iSourceType == SOURCE_FILESHELF){
        if (predefineStruct->gstrThisSidePath == NULL){
            predefineStruct->gstrThisSidePath = g_string_new(gstrWorkingPath->str);
            ThreadStartStruct->gstrThisSidePath = predefineStruct->gstrThisSidePath; //write this back to let others free t
        }
        if (predefineStruct->gstrOtherSidePath == NULL){
            predefineStruct->gstrOtherSidePath = g_string_new(ThreadStartStruct->gstrLeftSidePath->str);
            ThreadStartStruct->gstrOtherSidePath = predefineStruct->gstrOtherSidePath; //write this back to let others free t
        }
    }

    if (ThreadStartStruct->glistItemsWithPath != NULL)
        glistBuf = g_list_first (ThreadStartStruct->glistItemsWithPath);
    else
        glistBuf = NULL;
    predefineStruct->gstrAllMarkedItems_WithPath_Unquoted   = g_string_new("");
    predefineStruct->gstrAllMarkedItems_WithPath_Quoted     = g_string_new("");
    iBuf = 0;
    while (glistBuf){
        g_string_append_c (predefineStruct->gstrAllMarkedItems_WithPath_Quoted, '\"');
        g_string_append   (predefineStruct->gstrAllMarkedItems_WithPath_Quoted, ((GString *)(glistBuf->data))->str);
        g_string_append   (predefineStruct->gstrAllMarkedItems_WithPath_Quoted, "\" ");

        g_string_append   (predefineStruct->gstrAllMarkedItems_WithPath_Unquoted, ((GString *)(glistBuf->data))->str);
        g_string_append_c (predefineStruct->gstrAllMarkedItems_WithPath_Unquoted, ' ');

        iBuf = 1;

        glistBuf = glistBuf->next;
    }
    if (iBuf == 1){
        g_string_erase (predefineStruct->gstrAllMarkedItems_WithPath_Quoted, strlen (predefineStruct->gstrAllMarkedItems_WithPath_Quoted->str)-1, -1);
        g_string_erase (predefineStruct->gstrAllMarkedItems_WithPath_Unquoted, strlen (predefineStruct->gstrAllMarkedItems_WithPath_Unquoted->str)-1, -1);
    }
    if (ThreadStartStruct->glistItemsWithPath != NULL)
        glistBuf = g_list_first (ThreadStartStruct->glistItemsWithoutPath);
    else
        glistBuf = NULL;
    predefineStruct->gstrAllMarkedItems_WithoutPath_Unquoted   = g_string_new("");
    predefineStruct->gstrAllMarkedItems_WithoutPath_Quoted     = g_string_new("");
    iBuf = 0;
    while (glistBuf){
        g_string_append_c (predefineStruct->gstrAllMarkedItems_WithoutPath_Quoted, '\"');
        g_string_append   (predefineStruct->gstrAllMarkedItems_WithoutPath_Quoted, ((GString *)(glistBuf->data))->str);
        g_string_append   (predefineStruct->gstrAllMarkedItems_WithoutPath_Quoted, "\" ");

        g_string_append   (predefineStruct->gstrAllMarkedItems_WithoutPath_Unquoted, ((GString *)(glistBuf->data))->str);
        g_string_append_c (predefineStruct->gstrAllMarkedItems_WithoutPath_Unquoted, ' ');

        iBuf = 1;

        glistBuf = glistBuf->next;
    }
    if (iBuf == 1){
        g_string_erase (predefineStruct->gstrAllMarkedItems_WithoutPath_Quoted, strlen (predefineStruct->gstrAllMarkedItems_WithoutPath_Quoted->str)-1, -1);
        g_string_erase (predefineStruct->gstrAllMarkedItems_WithoutPath_Unquoted, strlen (predefineStruct->gstrAllMarkedItems_WithoutPath_Unquoted->str)-1, -1);
    }
    //if the calltype is "all", the predefine variables for MarkedItem (not MarkedItems) have to the same as for MarkedItems
    if (((struct _UserCommandList  *)(ThreadStartStruct->OurUserDataListItem->ListElement))->option_iCallType == USERCOMMAND_CALLTYPE_ALL && ThreadStartStruct->glistItemsWithoutPath!=NULL && ThreadStartStruct->glistItemsWithPath!=NULL){
        predefineStruct->gstrOneMarkedItem_WithoutPath_Unquoted = g_string_new (predefineStruct->gstrAllMarkedItems_WithoutPath_Unquoted->str);
        predefineStruct->gstrOneMarkedItem_WithoutPath_Quoted = g_string_new (predefineStruct->gstrAllMarkedItems_WithoutPath_Quoted->str);
        predefineStruct->gstrOneMarkedItem_WithPath_Unquoted = g_string_new (predefineStruct->gstrAllMarkedItems_WithPath_Unquoted->str);
        predefineStruct->gstrOneMarkedItem_WithPath_Quoted = g_string_new (predefineStruct->gstrAllMarkedItems_WithPath_Quoted->str);
    }
    else{
        if (ThreadStartStruct->glistItemsWithoutPath)
            glistBuf = g_list_nth(ThreadStartStruct->glistItemsWithoutPath, ThreadStartStruct->uiCurrentPositionInList);
        else
            glistBuf = NULL;
        if (glistBuf){
            predefineStruct->gstrOneMarkedItem_WithoutPath_Unquoted = g_string_new (((GString *)(glistBuf->data))->str);
            predefineStruct->gstrOneMarkedItem_WithoutPath_Quoted = g_string_new (((GString *)(glistBuf->data))->str);
            g_string_prepend (predefineStruct->gstrOneMarkedItem_WithoutPath_Quoted, "\"");
            g_string_append (predefineStruct->gstrOneMarkedItem_WithoutPath_Quoted, "\"");
        }
        else{
            predefineStruct->gstrOneMarkedItem_WithoutPath_Unquoted = g_string_new("");
            predefineStruct->gstrOneMarkedItem_WithoutPath_Quoted = g_string_new("");
        }

        if (ThreadStartStruct->glistItemsWithPath)
            glistBuf = g_list_nth(ThreadStartStruct->glistItemsWithPath, ThreadStartStruct->uiCurrentPositionInList);
        else
            glistBuf = NULL;
        if (glistBuf){
            predefineStruct->gstrOneMarkedItem_WithPath_Unquoted = g_string_new (((GString *)(glistBuf->data))->str);
            predefineStruct->gstrOneMarkedItem_WithPath_Quoted = g_string_new (((GString *)(glistBuf->data))->str);
            g_string_prepend (predefineStruct->gstrOneMarkedItem_WithPath_Quoted, "\"");
            g_string_append (predefineStruct->gstrOneMarkedItem_WithPath_Quoted, "\"");
        }
        else{
            predefineStruct->gstrOneMarkedItem_WithPath_Unquoted = g_string_new ("");
            predefineStruct->gstrOneMarkedItem_WithPath_Quoted = g_string_new("");
        }
    }

    TRACEIT (10, "ParseXMLUserCommand_FillPredefineStruct       end");
    return 0;
}


int processMyStandardEscapeSequences(GString *gstrToChange){
    TRACEIT (10, "processMyStandardEscapeSequences       start");

    if (gstrToChange == NULL){
        TRACEIT (2, "processMyStandardEscapeSequences       parameter NULL");
        return -1;
    }
    GString *gg = g_string_new("\\n");
    GString *tt = g_string_new("\n");
    exchangeNameWithValue(gstrToChange,gg,tt);

    g_string_assign(gg,"\\t");
    g_string_assign(tt,"\t");
    exchangeNameWithValue(gstrToChange,gg,tt);

    g_string_assign(gg,"\\\"");
    g_string_assign(tt,"\"");
    exchangeNameWithValue(gstrToChange,gg,tt);

    g_string_free(gg,TRUE);
    g_string_free(tt,TRUE);

    TRACEIT (10, "processMyStandardEscapeSequences       end");
    return 0;
}


int processAsTextWithoutUserVars (GString *gstrDestination, GString *gstrSource, struct _predefineStruct *predefStruct, gboolean bMakeThreadSave){
    TRACEIT (10, "processAsTextWithoutUserVars       start");
    //gstrSource is copied to gstrDestination
    //after that, the string in gstrDestination is checked for predefines, Backspaces and esacapeSequences
    //NO USERVARS

    if (gstrDestination == NULL || gstrSource == NULL){
        TRACEIT (2, "processAsTextWithoutUserVars       parameter NULL");
        return -1;
    }

    g_string_assign (gstrDestination, gstrSource->str);
    processMyStandardEscapeSequences (gstrDestination);
    exchangePredefine(gstrDestination, predefStruct);
    processBackSpaces (gstrDestination, bMakeThreadSave);

    TRACEIT (10, "processAsTextWithoutUserVars       end");
    return 0;
}


int exchangePredefine(GString *gstrText, struct _predefineStruct *predefStruct){
	TRACEIT(10,"exchangePredefine		start");
	if (gstrText == NULL || predefStruct == NULL){
        TRACEIT (2, "exchangePredefine      parameter NULL");
        return -1;
	}

	char *KnownPredefines[] = KNOWNPREDEFINES;
	int Count = NUMBEROFPREDEFINES;
	int i;

	GString *gstrBuf = g_string_sized_new(20);
	GString *gstrBuf2 = g_string_sized_new(20);

	for (i=0;i<Count;i++){
		g_string_assign(gstrBuf,KnownPredefines[i]);
		g_string_assign(gstrBuf2,KnownPredefines[i]);
		g_string_append_c(gstrBuf2,')');
		g_string_prepend (gstrBuf2, "$(");
		decryptPredefine(gstrBuf, predefStruct);
		exchangeNameWithValue(gstrText, gstrBuf2, gstrBuf);
	}
	g_string_free(gstrBuf2,TRUE);
	g_string_free(gstrBuf,TRUE);

	//if (ret == -Count){	//not one Predefine was found
	//	TRACEIT(10,"exchangePredefine		end");
	//	return -1;
	//}
	TRACEIT(10,"exchangePredefine		end");
	return 0;
}


int exchangeUserVarsInText(GString *gstrText, struct _DialogEntityList *DialogEntityList){
    TRACEIT(10,"exchangeUserVarsInText		start");
    if (gstrText == NULL ){
        TRACEIT(2,"exchangeUserVarsInText		end, text = NULL");
        return -1;
    }
    if (DialogEntityList == NULL){
        TRACEIT(10,"exchangeUserVarsInText		end");
        return -1;
    }
    int ret=0;
    //struct userDialogEntitiesList *bufList = inputList;
    struct _DialogEntityList *bufList =  DialogEntityList;
    GString *gstrBuf = g_string_sized_new(10);
    GString *gstrEmpty = g_string_new("");
    while (bufList){
        if (bufList->iValid == 1 &&
                (   bufList->iType==USERDIALOG_ENTITY_TYPE_QUESTION ||
                    bufList->iType==USERDIALOG_ENTITY_TYPE_CHECKBOX ||
                    bufList->iType==USERDIALOG_ENTITY_TYPE_FILECHOOSER ||
                    bufList->iType==USERDIALOG_ENTITY_TYPE_DIRCHOOSER )){
            g_string_assign(gstrBuf, bufList->gstrVarIdentifier->str);
            g_string_append_c(gstrBuf,')');
            //g_string_prepend_c(gstrBuf,'<');
            g_string_prepend(gstrBuf,"$(");
            if (bufList->gstrVarValue == NULL){    //we exchange the varlabel with an empty string (means we just delete the varlable in the command string)
                ret += exchangeNameWithValue(gstrText, gstrBuf, gstrEmpty);
            }
            else{
                ret += exchangeNameWithValue(gstrText, gstrBuf, bufList->gstrVarValue);
            }
        }
        //else if (bufList->iValid == 0){
        //    ret = -1;    //some error
        //}

        bufList=bufList->nextItem;
    }
    g_string_free(gstrBuf, TRUE);
    g_string_free(gstrEmpty, TRUE);
    TRACEIT(10,"exchangeUserVarsInText		end");
    return ret;
}


int processBackSpaces(GString *gstrStartText, gboolean bMakeThreadSave){
    // Replaces all \\b and delets the leading character
    // returns -1 when the text ends up in an empty string and leave the gstrText untouched
    // returns 0 if nothing was done
    // returns 1 when the string was processed
    TRACEIT(10,"processBackSpaces		start");
    if (gstrStartText==NULL){
        TRACEIT(2,"processBackSpaces		end gstrText=NULL, SHOULD NOT HAPPEN");
        return -2;
    }
    GString *gstrTextBuf = g_string_new(gstrStartText->str);
    GString *gstrbuf = g_string_sized_new(10);

    char Enemy[]="\\b";
    char *cStart;
	int iPosition=0;
	int iSize;
	int firstTime = 1;
	int diff=0;
	cStart = gstrTextBuf->str;
	do {
	    //printf( "gstrTextBuf: %s\n",gstrTextBuf->str);
		cStart = strstr(gstrTextBuf->str,Enemy);
		if (cStart == gstrTextBuf->str){
            //we will not try to delete something if there is nothing in front of it
            //cStart+=1; //+1 is possible because of our enemy
            break;;
		}
		if (cStart == NULL && firstTime==0){
			break;
		}
		else if(firstTime == 1 && cStart==NULL){
		    TRACEIT(10,"processBackSpaces		end, no backspace");
		    g_string_free(gstrTextBuf, TRUE);
            g_string_free(gstrbuf, TRUE);
			return 0;	//No backspace found !!!
		}
		firstTime = 0;
		iSize = strlen(Enemy);
		iPosition = 0;
		while( (gstrTextBuf->str)+iPosition != cStart){
			iPosition++;
		}
		diff=(iPosition==0)?0:1;
		iPosition-=diff;
		iSize+=diff;
		g_string_erase(gstrTextBuf,iPosition,iSize);
		//printf( "gstrTextBuf: %s\n",gstrTextBuf->str);
	}while (cStart != NULL);

    g_string_assign(gstrbuf, gstrTextBuf->str);
	cutSpacesFromGString(gstrbuf);
	if (strlen(gstrbuf->str) == 0){
        TRACEIT(5,"processBackSpaces		end, strlen ends up in 0");
        if (bMakeThreadSave)
            myWrap_gdk_threads_enter ("exchangeNameWithValue");
        MyMessageWidget (win,"UserCommandParser","Backspace error\nThe Backspaces lead to an empty string.\nCheck config file.");
        if (bMakeThreadSave)
            myWrap_gdk_threads_leave ("exchangeNameWithValue");
        //g_string_free(gstrTextBuf, TRUE);
        //g_string_free(gstrbuf, TRUE);
        //return -1;
	}
    g_string_assign(gstrStartText, gstrTextBuf->str);

    g_string_free(gstrTextBuf, TRUE);
    g_string_free(gstrbuf, TRUE);
	TRACEIT(10,"processBackSpaces		end");
	return 0;
}


void UserDialog_FileDirChooserButton(GtkEntry *entry, gpointer data){
    //used in getUserInput, connected to the file/dirchooser button
    TRACEIT(10,"UserDialog_FileDirChooserButton		start");
    //struct userDialogEntitiesList *ListItem = (struct userDialogEntitiesList *)data;
    struct _DialogEntityList *listItem = (struct _DialogEntityList *)data;

    if (listItem == NULL){
        TRACEIT(10,"UserDialog_FileDirChooserButton		parameter NULL");
        return;
    }

    GString *gstrCaption = g_string_new("choose a file");
    char *filename=NULL;

    if (listItem->gstrAdditional2){
        g_string_assign(gstrCaption, listItem->gstrAdditional2->str);
    }

    GtkWidget *dialog = gtk_file_chooser_dialog_new (gstrCaption->str,
                                                        win,
                                                        listItem->iType==USERDIALOG_ENTITY_TYPE_FILECHOOSER?GTK_FILE_CHOOSER_ACTION_OPEN:GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
                                                        GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                                        GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
                                                        NULL);

    if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT){
        filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
        gtk_entry_set_text(	(GtkEntry *)listItem->wdgtEntryBox, filename);
    }

    gtk_widget_destroy (dialog);
    if (filename)
        g_free (filename);
    g_string_free(gstrCaption, TRUE);

    TRACEIT(10,"UserDialog_FileDirChooserButton		end");
}


int decryptPredefine(GString *Predefine, struct _predefineStruct *predefStruct){
	TRACEIT(10,"decryptPredefine		start");
	char *KnownPredefines[] = KNOWNPREDEFINES;
	int Count = NUMBEROFPREDEFINES;				//Change this if you add or remove a String to/from KownPredefines
	int i;

	for (i=0; i<Count; i++){
		if (strcmp(KnownPredefines[i], Predefine->str) == 0){
			break;
		}
	}

	switch(i){
		case 0:	//ThisDirectory
		case 1:
				g_string_assign(Predefine,predefStruct->gstrThisSidePath->str);
				break;
		case 2:	//OtherDirectory
		case 3:
				g_string_assign(Predefine,predefStruct->gstrOtherSidePath->str);
				break;
		case 4:	//LeftSideDirectory
		case 5:
				g_string_assign(Predefine,predefStruct->gstrLeftSidePath->str);
				break;
		case 6:	//gstrRightSidePath
		case 7:
				g_string_assign(Predefine,predefStruct->gstrRightSidePath->str);
				break;
		case 8:	//Home
		case 9:
		case 26:
                //getHomeDirectory(&Predefine);
                g_string_assign(Predefine,predefStruct->gstrHomeDir->str);
				break;
        case 10: //gstrAllMarkedItems_WithoutPath_Unquoted
        case 11:
                g_string_assign(Predefine,predefStruct->gstrAllMarkedItems_WithoutPath_Unquoted->str);
                break;
        case 12: //gstrAllMarkedItems_WithoutPath_Quoted
        case 13:
                g_string_assign(Predefine,predefStruct->gstrAllMarkedItems_WithoutPath_Quoted->str);
                break;
        case 14: //gstrAllMarkedItems_WithPath_Unquoted
        case 15:
                g_string_assign(Predefine,predefStruct->gstrAllMarkedItems_WithPath_Unquoted->str);
                break;
        case 16: //gstrAllMarkedItems_WithPath_Quoted
        case 17:
                g_string_assign(Predefine,predefStruct->gstrAllMarkedItems_WithPath_Quoted->str);
                break;
        case 18: //gstrOneMarkedItem_WithoutPath_Unquoted
        case 19:
                g_string_assign(Predefine,predefStruct->gstrOneMarkedItem_WithoutPath_Unquoted->str);
                break;
        case 20: //gstrOneMarkedItem_WithoutPath_Quoted
        case 21:
                g_string_assign(Predefine,predefStruct->gstrOneMarkedItem_WithoutPath_Quoted->str);
                break;
        case 22: //gstrOneMarkedItem_WithPath_Unquoted
        case 23:
                g_string_assign(Predefine,predefStruct->gstrOneMarkedItem_WithPath_Unquoted->str);
                break;
        case 24: //gstrOneMarkedItem_WithPath_Unquoted
        case 25:
                g_string_assign(Predefine,predefStruct->gstrOneMarkedItem_WithPath_Quoted->str);
                break;
		default:
				Predefine->str[0] = '\0';
	};
	TRACEIT(10,"decryptPredefine		end");
	return 0;
}
