/*
    Copyright 2007, 2008, 2009, 2010 Geyer Klaus

    This file is part of Cat'sEyE.

    Cat'sEyE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Cat'sEyE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Cat'sEyE.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <gtk/gtk.h>
#ifndef NO_GIO
#include <gio/gio.h>
#endif
#include <string.h>

#include "helpers.h"
#include "globalDefinitions.h"
#include "Lists.h"
#include "listitems.h"
#include "userdata.h"
#include "myfm.h"
#include "fileshelf.h"
#include "threads.h"

#include "mytrace.h"

#include "allInclude.h"

void DNDSignal_dragdataget (GtkWidget *widget, GdkDragContext *drag_context, GtkSelectionData *selection_data, guint info, guint time, gpointer user_data);
void DNDSignal_dragdatereceive (GtkWidget *widget, GdkDragContext   *drag_context, gint x, gint y, GtkSelectionData *data, guint info, guint time, gpointer user_data);
gboolean DNDSignal_dragdrop (GtkWidget *widget, GdkDragContext *drag_context, gint x, gint y, guint time, gpointer user_data);

extern GtkWidget 		*Notebooks[SIDE_BOTH];
extern GtkNotebook      *g_FileShelfNotebook;
extern GtkWindow 		*win;

enum {
        TARGET_LISTVIEW, //treeview, basic DnD for sideviews and fileshelfs
        TARGET_MARKEDITEMSASTEXT
        //TARGET_FROMNOTEBOOK //for getting tabs from notebooks, we have this target
};

// text/uri-list text/plain     application/x-moz-file
static GtkTargetEntry targets[] = {
   /* { "widget(listview)",       GTK_TARGET_SAME_APP,    TARGET_LISTVIEW },*/
    { "STRING",                 GTK_TARGET_SAME_APP,   TARGET_MARKEDITEMSASTEXT },
    { "STRING",                 GTK_TARGET_OTHER_APP,   TARGET_MARKEDITEMSASTEXT },
    { "text/plain",             GTK_TARGET_OTHER_APP,   TARGET_MARKEDITEMSASTEXT },
  	{ "text/uri-list",          GTK_TARGET_OTHER_APP,   TARGET_MARKEDITEMSASTEXT }/*,
  	{ "application/x-moz-file", GTK_TARGET_OTHER_APP, TARGET_MARKEDITEMSASTEXT },
    { "GTK_NOTEBOOK_TAB",    GTK_TARGET_SAME_APP, TARGET_LISTVIEW }  */ //look at 'http://library.gnome.org/devel/gtk/unstable/GtkNotebook.html', search for 'gtk_notebook_set_tab_detachable'
};



/*
static GtkTargetEntry NoteBookTargets[] = {
    { "GTK_NOTEBOOK_TAB",    GTK_TARGET_SAME_APP, TARGET_LISTVIEW }   //look at 'http://library.gnome.org/devel/gtk/unstable/GtkNotebook.html', search for 'gtk_notebook_set_tab_detachable'
};
*/
/*
int prepareWidgetForDragnDrop (GtkWidget *widget, int enableNoteBookDest){    //enables dragndrop for the widget and connects signals
    //this is our standard dragndrop for copying items from/to view/Shelf
    //enableNoteBookDest: 0:no, 1:yes, the widget can be achieve a ntebook-tab
    TRACEIT(10,"prepareWidgetForDragnDrop		start");
    if (widget == NULL){
        TRACEIT(2,"prepareWidgetForDragnDrop		end, widget NULL");
        return -1;
    }
    if (enableNoteBookDest==1){
        gtk_drag_dest_set (widget, GTK_DEST_DEFAULT_ALL, NoteBookTargets, 1, GDK_ACTION_COPY|GDK_ACTION_MOVE);
    }
    else{
        gtk_drag_dest_set (widget, GTK_DEST_DEFAULT_ALL, targets, 1, GDK_ACTION_COPY);
    }
    gtk_drag_source_set (widget, GDK_BUTTON1_MASK, targets, 1, GDK_ACTION_COPY);    //Mind that only destinations who can handle Notebooktabs are allowd to get the target for notebooks

    //g_signal_connect(widget,"drag-drop",G_CALLBACK(DNDSignal_dragdrop),NULL); //Do you remember the problem that we end up in the drag-data-get and receive signal handler twice
    //after comment this out we did not have this Problem again
    g_signal_connect(widget,"drag-data-get",G_CALLBACK(DNDSignal_dragdataget),NULL);
    g_signal_connect(widget,"drag-data-received",G_CALLBACK(DNDSignal_dragdatereceive),NULL);

    TRACEIT(10,"prepareWidgetForDragnDrop		end");
    return 0;
}
*/
int prepareWidgetForDrag (GtkWidget *widget){    //enables the Drag in dragndrop for the widget and connects signals
    //this is our standard dragndrop for copying items from/to view/Shelf

    TRACEIT(10,"prepareWidgetForDrag		start");
    if (widget == NULL){
        TRACEIT(2,"prepareWidgetForDrag		end, widget NULL");
        return -1;
    }
    gtk_drag_source_set (widget, GDK_BUTTON1_MASK, targets, sizeof(targets) / sizeof(targets[0]), GDK_ACTION_COPY);    //Mind that only destinations who can handle Notebooktabs are allowd to get the target for notebooks

    //g_signal_connect(widget,"drag-drop",G_CALLBACK(DNDSignal_dragdrop),NULL); //Do you remember the problem that we end up in the drag-data-get and receive signal handler twice
    //after comment this out we did not have this Problem again
    g_signal_connect(widget,"drag-data-get",G_CALLBACK(DNDSignal_dragdataget),NULL);


    TRACEIT(10,"prepareWidgetForDrag		end");
    return 0;
}


int prepareWidgetForDrop (GtkWidget *widget, int enableNoteBookDest, void (*functionStartNotification)(), void *UserData){    //enables Drop for dragndrop for the widget and connects signals
    //this is our standard dragndrop for copying items from/to view/Shelf
    //enableNoteBookDest: 0:no, 1:yes, the widget can be achieve a ntebook-tab
    //functionStartNotification: the function to connect to the receiv-signal
    TRACEIT(10,"prepareWidgetForDrop		start");
    if (widget == NULL){
        TRACEIT(2,"prepareWidgetForDrop		end, widget NULL");
        return -1;
    }
    if (enableNoteBookDest==1){
        gtk_drag_dest_set (widget, GTK_DEST_DEFAULT_ALL, targets, sizeof(targets)/sizeof(targets[0]), GDK_ACTION_COPY);//  GDK_ACTION_COPY|GDK_ACTION_MOVE
    }
    else{
        gtk_drag_dest_set (widget, GTK_DEST_DEFAULT_ALL, targets, sizeof(targets)/sizeof(targets[0]), GDK_ACTION_COPY);
    }

    //g_signal_connect(widget,"drag-drop",G_CALLBACK(DNDSignal_dragdrop),NULL); //Do you remember the problem that we end up in the drag-data-get and receive signal handler twice
    //after comment this out we did not have this Problem again


    g_signal_connect(widget,"drag-data-received",G_CALLBACK(functionStartNotification), UserData);

    TRACEIT(10,"prepareWidgetForDrop		end");
    return 0;
}
////////////////// signal handler ////////////////////

/*
gboolean DNDSignal_dragdrop (GtkWidget *widget, GdkDragContext *drag_context, gint x, gint y, guint time, gpointer user_data){
	printf ("DID NOT THINK WE CAN COME UP HERE\n");

    return FALSE;
}
*/

//"drag-data-get" signal (source side)
//the source sets the data
void DNDSignal_dragdataget (GtkWidget *widget, GdkDragContext *drag_context, GtkSelectionData *selection_data, guint info, guint time, gpointer user_data){
    TRACEIT(10,"DNDSignal_dragdataget		start");
    //GtkTreeSelection *selection=NULL;
	if (info == TARGET_MARKEDITEMSASTEXT){
		GString *gstrDataToSend = g_string_sized_new (10);
		GString *gstrBuf = g_string_sized_new (10);
		struct CopyReMoveList *FirstItem = NULL;
		struct CopyReMoveList *BufItem = NULL;
		//The only widgets connected to this functions are the listviews from the sideObjects and the fileshelfs

		GtkTreeSelection	*selection;
		struct SideObjectList *me_sideObject=NULL;
	    struct FileShelfObjectList *me_fileshelfObject=NULL;
	    int side;
	    int iWeAre=0;   //1=sideObject, 2=fileshelf
	    char *cpConvertedString = NULL;
	    GError *error=NULL;
		SideObjectList_GetObjectByChildListView(&me_sideObject, (GtkTreeView *)widget, &side);
        if (me_sideObject == NULL){    //maybe we are a FileshelfObject
            //FileShelfObjectList_GetActiveObject(&me_fileshelfObject);
            FileShelfObjectList_GetObjectByChildListView(&me_fileshelfObject, (GtkTreeView *)widget);
            //if (me_fileshelfObject!=NULL && me_fileshelfObject->listView == widget){
            if (me_fileshelfObject!=NULL){
                iWeAre=2;
            }
        }
        else{
            iWeAre=1;
        }
		if (iWeAre == 1){
			iWeAre = SOURCE_SIDEOBJECT;
		}
		else if (iWeAre == 2){
			iWeAre = SOURCE_FILESHELF;
		}
		else{
			TRACEIT (4, "DNDSignal_dragdataget		unable to determine the source widget, isnt it a listview?");
			return;
		}

		selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(widget));
		g_string_assign (gstrDataToSend, "/NoPathHere");	//we will never use this, so it doesn't matter
		g_string_assign (gstrBuf, "/JustAnotherPath");
		if (0 == MakeList (iWeAre, &FirstItem, selection, (iWeAre==SOURCE_SIDEOBJECT)?me_sideObject->gstrPath:gstrDataToSend, gstrBuf)){	// all is good
			BufItem = FirstItem;
			g_string_erase (gstrDataToSend, 0, -1);
			while (BufItem){
			    cutCharFromGString (BufItem->string1, '\"', 0);
			    cpConvertedString = g_locale_to_utf8 (BufItem->string1->str, strlen (BufItem->string1->str), NULL, NULL, &error);
			    if (error != NULL){
                    TRACEIT (3, "DNDSignal_dragdataget error:");
                    TRACEIT (3, error->message);
                    g_error_free (error);
                    error=NULL;
                    //BufItem = BufItem->NextItem;  //do not cancel on that error
                    //continue;
			    }

				//g_string_append (gstrDataToSend, "file://");	//we only have this type in our views
                if (cpConvertedString != NULL){
                    g_string_assign (gstrBuf, cpConvertedString);
                    g_free (cpConvertedString);
                }
                else{   //a fall back if the encoding failed
                    g_string_assign (gstrBuf, BufItem->string1->str);
                }
                cpConvertedString = g_filename_to_uri (gstrBuf->str, NULL, &error);
                if (error != NULL){
                    TRACEIT (3, "DNDSignal_dragdataget error:");
                    TRACEIT (3, error->message);
                    g_error_free (error);
                    error = NULL;
                }
                if (cpConvertedString != NULL){
                    g_string_append (gstrDataToSend, cpConvertedString);
                    g_string_append_c (gstrDataToSend, '\r'); //Carriage return, Ctrl-m, ASCII code 13, 0xD
                    g_string_append_c (gstrDataToSend, '\n');
                    g_free (cpConvertedString);
                }
                else{
                    g_string_erase (gstrDataToSend, 0, -1);
                }

				BufItem = BufItem->NextItem;
			}

			if (0 < strlen (gstrDataToSend->str)){
				//data will be copied from gtk_selection_data_set.
				gtk_selection_data_set (selection_data, selection_data->target, 8, (guchar *)gstrDataToSend->str, strlen (gstrDataToSend->str));
			}
		}
		g_string_free (gstrDataToSend, TRUE);
		g_string_free (gstrBuf, TRUE);
		if (FirstItem != NULL){
			CopyReMoveList_DeleteList (FirstItem);
			FirstItem = NULL;
		}
	}

    if (info == TARGET_LISTVIEW){
        int *testpointer = (int *)widget;

        //selection  = gtk_tree_view_get_selection((GtkTreeView *)widget /*ListView*/);

        //gtk_selection_data_set (selection_data, selection_data->target, 8,(guchar *)&widget, sizeof(GtkWidget*));
        gtk_selection_data_set (selection_data, selection_data->target, 1,(guchar *)&testpointer, sizeof(int*));

    }
    TRACEIT(10,"DNDSignal_dragdataget		end");
}


//"drag-data-received" signal (drop side) for SideObjects and FileShelfs
void DNDSignal_dragdatereceive_forSideObjectAndFileShelf (GtkWidget *widget, GdkDragContext *drag_context, gint x, gint y, GtkSelectionData *selection_data, guint info, guint time, gpointer user_data){
    TRACEIT(10,"DNDSignal_dragdatereceive_forSideObjectAndFileShelf		start");
    //GtkTreeSelection	*selection;
    struct SideObjectList *me_sideObject=NULL;
    struct FileShelfObjectList *me_fileshelfObject=NULL;
    //struct SideObjectList *source_sideObject=NULL;
    //struct FileShelfObjectList *source_fileshelfObject=NULL;
    int side;
    int iWeAre=0;   //1=sideObject, 2=fileshelf, 3=ViewNotebook, 4=FSNotebook
    //int iSourceIs=0;//1=sideObject, 2=fileshelf, 3=ViewNotebook
    //GtkWidget *sourceWidget=NULL;
    gboolean DND_Done = FALSE;
    //GtkWidget **NotebookChild=NULL;
    GString *gstrPath=NULL;
    GString *gstrName=NULL;
    GError *error = NULL;
    gchar *cpConvertedString=NULL;
    gchar *cpConvertedString2=NULL;
    gchar *cpHostName=NULL;

    if((selection_data != NULL) && (selection_data->length >= 0)){
    	//determine own type (the widget is from sideObject or Fileshelf)
        if (widget == Notebooks[SIDE_LEFT] || widget == Notebooks[SIDE_RIGHT]){  //find viewNotebooks
            SideObjectList_GetActiveObject((GtkNotebook *)widget, &me_sideObject);
            if (me_sideObject){
                iWeAre=3;    //3=ViewNotebook
            }
            if (widget == Notebooks[SIDE_LEFT]){
                side=SIDE_LEFT;
            }
            else if (widget == Notebooks[SIDE_RIGHT]){
                side=SIDE_RIGHT;
            }
        }
        else if (widget == (GtkWidget *)g_FileShelfNotebook){
            FileShelfObjectList_GetActiveObject(&me_fileshelfObject);
            if (me_fileshelfObject!=NULL){
                iWeAre=4;
            }
        }
        else{
            SideObjectList_GetObjectByChildListView(&me_sideObject, (GtkTreeView *)widget, &side);
            if (me_sideObject == NULL){    //maybe we are a FileshelfObject
                FileShelfObjectList_GetActiveObject(&me_fileshelfObject);
                if (me_fileshelfObject!=NULL && me_fileshelfObject->listView == widget){
                    iWeAre=2;
                }
            }
            else{
                iWeAre=1;
            }
        }
        if (iWeAre==0){
            TRACEIT(2,"DNDSignal_dragdatereceive_forSideObjectAndFileShelf		end, could not determine own type.");
            DND_Done = FALSE;
            gtk_drag_finish (drag_context, DND_Done, FALSE, time);
            return;
        }
        if (me_sideObject!=NULL && me_sideObject->bReadOnly == TRUE){
        	MyMessageWidget(win, "Read only Tab", "The tab is marked as read only.\nPlease remove this flag to\nperform this action.\n(tab context menu)");
			TRACEIT(2,"DNDSignal_dragdatereceive_forSideObjectAndFileShelf		end, could not determine own type.");
			DND_Done = FALSE;
            gtk_drag_finish (drag_context, DND_Done, FALSE, time);
            return;
        }

    	if (info == TARGET_MARKEDITEMSASTEXT){
    		//we only handle "file://<PATHWITHFILE>" objects in here, this is a filebrowser.
    		char *cpBuf = (char *)selection_data->data;
    		gstrPath = g_string_sized_new (10);
    		gstrName = g_string_sized_new (10);
    		int iCopyToSideObject = 0;
    		struct CopyReMoveList *FirstItem = NULL;
    		for (cpBuf = strtok (cpBuf, "\r\n"); cpBuf!=NULL; cpBuf = strtok (NULL, "\r\n") ){

    		    cpConvertedString = g_locale_from_utf8 (cpBuf, strlen (cpBuf), NULL, NULL, &error);
    		    if (error != NULL){
                    TRACEIT (2, "DNDSignal_dragdatereceive_forSideObjectAndFileShelf error from locale_from_utf8:");
                    TRACEIT (2, error->message);
                    g_error_free (error);
                    error = NULL;
    		    }
    		    if (cpConvertedString == NULL){
                    //do not stop here, we will have a fall back for this case
                    cpConvertedString2 = g_filename_from_uri (cpBuf, &cpHostName, &error);
    		    }
    		    else{
                    cpConvertedString2 = g_filename_from_uri (cpConvertedString, &cpHostName, &error);
                    g_free (cpConvertedString);
                    cpConvertedString=NULL;
    		    }
    		    if (error != NULL){
                    TRACEIT (2, "DNDSignal_dragdatereceive_forSideObjectAndFileShelf error from g_filename_from_uri:");
                    TRACEIT (2, error->message);
                    g_error_free (error);
                    error = NULL;
    		    }
    		    if (cpConvertedString2 == NULL){
                    continue;
    		    }

    			if (cpHostName != NULL && strcmp (cpHostName, "file://")!=0){
    			    g_free(cpConvertedString2);
    			    cpConvertedString2=NULL;
    				continue;
    			}
    			else if (cpHostName != NULL){
                    g_free (cpHostName);
    			}

    			if (iWeAre==1 || iWeAre==3){	//sideobject copy
    				//we create a CopyReMoveList and let others do the work
    				g_string_assign (gstrPath, cpConvertedString2);
					GetPathFromPathWithName(gstrPath, 1);
					if (0 == strcmp (gstrPath->str, me_sideObject->gstrPath->str)){
					    g_free(cpConvertedString2);
                        cpConvertedString2=NULL;
						continue;
					}
    				CopyReMoveList_AddItem ( &FirstItem, cpConvertedString2, me_sideObject->gstrPath->str, NULL, me_sideObject, NULL);

    				iCopyToSideObject = 1;
    			}
    			else if (iWeAre==2 || iWeAre==4){	//fileshelf add
    				g_string_assign (gstrPath, cpConvertedString2);
    				g_string_assign (gstrName, cpConvertedString2);

    				getLastStringInPath(gstrName);
					GetPathFromPathWithName(gstrPath, 1);

    				AddItemToFileshelf (me_fileshelfObject, gstrName->str, gstrPath->str, TRUE, 1);
    				DND_Done = TRUE;
    			}
    			if (cpConvertedString2!=NULL){
                    g_free(cpConvertedString2);
                    cpConvertedString2=NULL;
    			}
    		}
    		if (iCopyToSideObject == 1){	//sideobject copy
    			ProcessItemsViaListWrap(FirstItem, 1);
    			DND_Done = TRUE;
    		}

    		g_string_free (gstrPath, TRUE);
    		g_string_free (gstrName, TRUE);
    		DND_Done = FALSE;
    	}
        /*else if (info == TARGET_LISTVIEW){
            sourceWidget = gtk_drag_get_source_widget (drag_context);

            //find source type
            if (sourceWidget == Notebooks[0] || sourceWidget == Notebooks[1]){  //find viewNotebooks
                NotebookChild = (void *)selection_data->data;
                SideObjectList_GetObjectByChildVBox(&source_sideObject, (GtkWidget *)*NotebookChild, &side);
                if (source_sideObject){
                    iSourceIs=3;    //3=Notebook
                }
            }
            else{
                SideObjectList_GetObjectByChildListView(&source_sideObject, (GtkTreeView *)sourceWidget, &side);
                if (source_sideObject == NULL){    //maybe we are a FileshelfObject
                    FileShelfObjectList_GetObjectByChildListView(&source_fileshelfObject, (GtkTreeView *)sourceWidget);
                    if (source_fileshelfObject){
                        iSourceIs=2;
                    }
                }
                else{
                    iSourceIs=1;
                }
            }
            if (iSourceIs==0){
                TRACEIT(2,"DNDSignal_dragdatereceive		end, could not determine source type.");
                gtk_drag_finish (drag_context, DND_Done, FALSE, time);
                return;
            }

            //printf("found iWeAre=%d\tiSourceIs=%d\n",iWeAre,iSourceIs);

            if ( (iWeAre==1||iWeAre==3) && iSourceIs == 1){ //From sideObject to sideObject
                //printf("View2View\n");
                if (source_sideObject != me_sideObject){
                    copyToAnOtherPannel( source_sideObject, me_sideObject);
                    DND_Done=TRUE;
                }
            }
            else if ( (iWeAre==2||iWeAre==4) && iSourceIs==1){ //From sideObject to FileShelf
                //printf("View2FS\n");
                MenuAddItemToFileshelf(NULL, (void *) source_sideObject);
                DND_Done=TRUE;
            }
            else if ( (iWeAre==1||iWeAre==3) && iSourceIs == 2){ //From fileshelf to sideObject
                //printf("FS2View\n");
                if (side == SIDE_RIGHT){
                    copyFileShelfToRightPannel(NULL, source_fileshelfObject);
                    DND_Done=TRUE;
                }
                else if (side == SIDE_LEFT){
                    copyFileShelfToLeftPannel(NULL, source_fileshelfObject);
                    DND_Done=TRUE;
                }
            }
            else if ( (iWeAre==2||iWeAre==4) && iSourceIs == 2){    //From Fileshelf to FileShelf
                //printf("FS2FS\n");
                if (source_fileshelfObject != me_fileshelfObject){
                    selection  = gtk_tree_view_get_selection(GTK_TREE_VIEW(source_fileshelfObject->listView));
                    gtk_tree_selection_selected_foreach(selection, addItemsFromFileshelfToFileshelf_Helper, source_fileshelfObject);
                    DND_Done=TRUE;
                }
            }
            else if ( (iWeAre==2||iWeAre==4) && iSourceIs == 3){    //notbook tab to Fileshelf (we put the directory viewed from the notebook to the shelf as item)
                //printf("ViewTab2FS\n");
                gstrPath = g_string_new (source_sideObject->gstrPath->str);
                gstrName = g_string_new (source_sideObject->gstrPath->str);

                getLastStringInPath(gstrName);
                GetPathFromPathWithName(gstrPath, 1);

                AddItemToFileshelf (me_fileshelfObject, gstrName->str, gstrPath->str, 0, 1);

                g_string_free(gstrPath, TRUE);
                g_string_free(gstrName, TRUE);
                DND_Done=TRUE;
            }
        }*/
    }

    gtk_drag_finish (drag_context, DND_Done, FALSE, time);
    //gtk_drag_finish (drag_context, TRUE, TRUE, time);

    TRACEIT(10,"DNDSignal_dragdatereceive_forSideObjectAndFileShelf		end");
}



void DNDSignal_dragdatereceive_forCopyReMoveWidget (GtkWidget *widget, GdkDragContext *drag_context, gint x, gint y, GtkSelectionData *selection_data, guint info, guint time, void *user_data){
    TRACEIT(10,"DNDSignal_dragdatereceive_forCopyReMoveWidget		start");
    // user_data is of type     struct ListWidgetCommunicationStruct
    GString *gstrPath = g_string_sized_new (10);
    GString *gstrName = g_string_sized_new (10);
    //int iCopyToSideObject = 0;
    char *cpBuf = NULL;
    struct CopyReMoveList *FirstItem = NULL;
    struct CopyReMoveList *AllreadyExistingList = NULL;
    struct ListWidgetCommunicationStruct *CommStru = NULL;
    GError *error = NULL;
    gchar *cpConvertedString=NULL;
    gchar *cpConvertedString2=NULL;
    gboolean DND_Done = FALSE;
    gchar *cpHostName=NULL;
    GtkTreeIter iter;
    int iListCreated=0;
    int iItemColumn=0;

    CommStru = (struct ListWidgetCommunicationStruct *)user_data;
    AllreadyExistingList = CommStru->firstItem;
    if (NULL == CommStru){
        gtk_drag_finish (drag_context, DND_Done, FALSE, time);
        TRACEIT(2, "DNDSignal_dragdatereceive_forCopyReMoveWidget		PARAMETER NULL (1)");
        return;
    }

    AllreadyExistingList = CommStru->firstItem;
    if (NULL == AllreadyExistingList){
        gtk_drag_finish (drag_context, DND_Done, FALSE, time);
        TRACEIT(2, "DNDSignal_dragdatereceive_forCopyReMoveWidget		PARAMETER NULL");
        return;
    }

    if((selection_data != NULL) && (selection_data->length >= 0)){
    	//determine own type (the widget is from sideObject or Fileshelf)

    	if (info == TARGET_MARKEDITEMSASTEXT){
    		//we only handle "file://<PATHWITHFILE>" objects in here, this is a filebrowser.
    		cpBuf = (char *)selection_data->data;

    		for (cpBuf = strtok (cpBuf, "\r\n"); cpBuf!=NULL; cpBuf = strtok (NULL, "\r\n") ){

    		    cpConvertedString = g_locale_from_utf8 (cpBuf, strlen (cpBuf), NULL, NULL, &error);
    		    if (error != NULL){
                    TRACEIT (2, "DNDSignal_dragdatereceive_forSideObjectAndFileShelf error from locale_from_utf8:");
                    TRACEIT (2, error->message);
                    g_error_free (error);
                    error = NULL;
    		    }
    		    if (cpConvertedString == NULL){
                    //do not stop here, we will have a fall back for this case
                    cpConvertedString2 = g_filename_from_uri (cpBuf, &cpHostName, &error);
    		    }
    		    else{
                    cpConvertedString2 = g_filename_from_uri (cpConvertedString, &cpHostName, &error);
                    g_free (cpConvertedString);
                    cpConvertedString=NULL;
    		    }
    		    if (error != NULL){
                    TRACEIT (2, "DNDSignal_dragdatereceive_forSideObjectAndFileShelf error from g_filename_from_uri:");
                    TRACEIT (2, error->message);
                    g_error_free (error);
                    error = NULL;
    		    }
    		    if (cpConvertedString2 == NULL){
                    continue;
    		    }

    			if (cpHostName != NULL && strcmp (cpHostName, "file://")!=0){
    			    g_free(cpConvertedString2);
    			    cpConvertedString2=NULL;
    				continue;
    			}
    			else if (cpHostName != NULL){
                    g_free (cpHostName);
    			}

                //we create a CopyReMoveList and let others do the work
                g_string_assign (gstrPath, cpConvertedString2);
                GetPathFromPathWithName(gstrPath, 1);
                if (NULL!=AllreadyExistingList->DestObject && 0 == strcmp (gstrPath->str, AllreadyExistingList->DestObject->gstrPath->str)){
                    g_free(cpConvertedString2);
                    cpConvertedString2=NULL;
                    continue;
                }

                //copy unknown parameter from the 'old' list
                CopyReMoveList_AddItem (    &FirstItem, cpConvertedString2,
                                            (AllreadyExistingList->DestObject==NULL)?NULL:AllreadyExistingList->DestObject->gstrPath->str,
                                            AllreadyExistingList->SrcObject,
                                            AllreadyExistingList->DestObject,
                                            AllreadyExistingList->FileShelfObject);
                iListCreated = 1;

    			if (cpConvertedString2!=NULL){
                    g_free(cpConvertedString2);
                    cpConvertedString2=NULL;
    			}
    		}

            //now we want to connect the old list in AllreadyExistingList with the new one in FirstItem
            struct CopyReMoveList *listbuf = NULL;
            //int iListCounter = 0;

            /*
            listbuf = AllreadyExistingList;
            iListCounter = 0;
            do{
                iListCounter++;
                listbuf = listbuf->NextItem;
            }while(listbuf!= NULL);

            listbuf = FirstItem;
            do{
                iListCounter++;
                listbuf = listbuf->NextItem;
            }while(listbuf!= NULL);
            */

            if (CommStru->ThreadStartStruct != NULL){
                iItemColumn = 1;
            }
            else{
                iItemColumn = 2;
            }

            if (iListCreated == 1){
                listbuf = FirstItem;
                do{
                    gtk_list_store_append( CommStru->listStore, &iter);
                    gtk_list_store_set ( CommStru->listStore, &iter, iItemColumn, listbuf->string1->str,-1);

                    CopyReMoveList_initializeMemberTo (listbuf,  CommStru->listStore, iter, &(CommStru->ui64TotalBytesProcessed), &(CommStru->i64StartTotalBytesCopied), &(CommStru->timeStartTimeForBytesCopied), &(CommStru->timeStartTimeForBytesCopiedSingleItem), &(CommStru->ui64TotalBytes), &(CommStru->iTotalBytesValid), (GtkProgressBar *)CommStru->progressBar, AllreadyExistingList->CopyMoveOrRemove, &(CommStru->dCopySpeed) );



                    if (CommStru->ThreadStartStruct != NULL){
                        //We are in the usercommand via list widget
                        //we have to add the new items to some more lists for the usercommand:
                        g_string_assign (gstrPath, listbuf->string1->str);
                        g_string_assign (gstrName, listbuf->string1->str);

                        getLastStringInPath(gstrName);
                        //GetPathFromPathWithName(gstrPath, 1);

                        CommStru->ThreadStartStruct->glistItemsWithPath = g_list_append (CommStru->ThreadStartStruct->glistItemsWithPath, g_string_new (gstrPath->str));
                        CommStru->ThreadStartStruct->glistItemsWithoutPath = g_list_append (CommStru->ThreadStartStruct->glistItemsWithoutPath, g_string_new(gstrName->str));
                        CommStru->ThreadStartStruct->uiSizeOfLists = g_list_length (CommStru->ThreadStartStruct->glistItemsWithPath);
                    }


                    listbuf = listbuf->NextItem;
                }while(listbuf!= NULL);

                //find the last item in the list
                for (listbuf=AllreadyExistingList; listbuf->NextItem; listbuf=listbuf->NextItem);

                //connect the two lists
                listbuf->NextItem = FirstItem;
                FirstItem->PrevItem = listbuf;


                copyReMoveWidget_updateItemCounterColumn(FirstItem);

                if (CommStru->ThreadStartStruct == NULL){
                    //means, that we are in the CopyReMoveWidget
                    if (CommStru->iTotalBytesValid==1){
                        if (FirstItem->CopyMoveOrRemove != 3){    //do not calculate this when deleting
                            g_thread_create((GThreadFunc)CopyReMoveItemsViaList_SizeCalc, (void *)CommStru , FALSE, NULL);
                        }
                    }
                }
            }

    		DND_Done=TRUE;
    	}
    }

    g_string_free (gstrPath, TRUE);
    g_string_free (gstrName, TRUE);

    gtk_drag_finish (drag_context, DND_Done, FALSE, time);

    //fired automaticly via signal drag-leave
    //gtk_button_clicked (CommStru->continueButton);

    TRACEIT(10,"DNDSignal_dragdatereceive_forCopyReMoveWidget		end");
}




////////////////////////////////////////////////////////////////////////////////////////////
/*
void                gtk_drag_dest_set                   (GtkWidget *widget,
                                                         GtkDestDefaults flags,
                                                         const GtkTargetEntry *targets,
                                                         gint n_targets,
                                                         GdkDragAction actions);

typedef struct {
  gchar *target;
  guint  flags;     //GTK_TARGET_SAME_APP
  guint  info;
} GtkTargetEntry;

typedef enum
{
  GDK_ACTION_DEFAULT = 1 << 0,
  GDK_ACTION_COPY    = 1 << 1,
  GDK_ACTION_MOVE    = 1 << 2,
  GDK_ACTION_LINK    = 1 << 3,
  GDK_ACTION_PRIVATE = 1 << 4,
  GDK_ACTION_ASK     = 1 << 5
} GdkDragAction;


//some signals
  "drag-begin"                                     : Run Last
  "drag-data-delete"                               : Run Last
  "drag-data-get"                                  : Run Last
  "drag-data-received"                             : Run Last
  "drag-drop"                                      : Run Last
  "drag-end"                                       : Run Last
  "drag-failed"                                    : Run Last
  "drag-leave"                                     : Run Last
  "drag-motion"
*/

