/*
    Copyright 2007, 2008, 2009, 2010 Geyer Klaus

    This file is part of Cat'sEyE.

    Cat'sEyE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Cat'sEyE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Cat'sEyE.  If not, see <http://www.gnu.org/licenses/>.
*/



#include <stdlib.h>
#include <gtk/gtk.h>
#ifndef NO_GIO
#include <gio/gio.h>
#endif
#include <string.h>

#include "Lists.h"
#include "mytrace.h"
#include "helpers.h"
#include "globalDefinitions.h"
#include "confighandling.h"
#include "userdata.h"
#include "myfm.h"
#include "fileshelf.h"
#include "automaticUpdate.h"
#include "threads.h"

#include "allInclude.h"

extern int g_int_AlwaysShowHidden;
extern GtkWidget 		*Notebooks[SIDE_BOTH];

/////////////////////////////////////////////////////////////////////////////////////////////////
///////////////// CopyReMove LISTS //////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////


int CopyReMoveList_AddItem(struct CopyReMoveList **FirstItem, char *string1, char *string2, struct SideObjectList *SrcObject, struct SideObjectList *DestObject, struct FileShelfObjectList *FileShelfObject){
	TRACEIT(10,"CopyReMoveList_AddItem		start");
	struct CopyReMoveList *list;
	if (FirstItem == NULL || (string1==NULL && string2==NULL)){
		TRACEIT(10,"CopyReMoveList_AddItem		ends -2");
		return -2;
	}


    struct CopyReMoveList *newItem = malloc(sizeof(struct CopyReMoveList));
    if (newItem == NULL){
        TRACEIT(10,"CopyReMoveList_AddItem		ends, Error allocating memory");
        return -1;
    }
    if (string1==NULL){
        newItem->string1 = NULL;
    }
    else{
        newItem->string1=g_string_new(string1);
    }
    if (string2 ==NULL){
        newItem->string2=NULL;
    }
    else{
        newItem->string2=g_string_new(string2);
    }
    newItem->SrcObject=SrcObject;
    newItem->DestObject=DestObject;
    newItem->NextItem=NULL;
    newItem->PrevItem=NULL;
    newItem->CopyMoveOrRemove=0;
    newItem->FileShelfObject=FileShelfObject;
    newItem->UserCommand = NULL;
    newItem->gstrCaption = NULL;
    newItem->gstrLabel = NULL;
    newItem->Errors = NULL;
    newItem->ErrorMessage = NULL;
    newItem->bOverWriteExisting=FALSE;
    newItem->threadCalcSize=NULL;
    newItem->listStore=NULL;
    newItem->iBytesToCopy=0;
    newItem->iBytesCopied=0;
    newItem->iBytesToCopyIsValid=0;
    newItem->ui64pTotalBytesCopied=NULL;
    newItem->i64pStartTotalBytesCopied=NULL;
    newItem->timepStartTimeForBytesCopied=NULL;
    newItem->timepStartTimeForBytesCopiedSingleItem=NULL;
    newItem->iTotalBytesValid=NULL;

    if (*FirstItem==NULL){
        *FirstItem = newItem;
    }
    else{
        for (list=*FirstItem; list->NextItem; list=list->NextItem);
        list->NextItem = newItem;
        newItem->PrevItem = list;
    }

	TRACEIT(10,"CopyReMoveList_AddItem		ends _ 2");
	return 0;
}


int CopyReMoveList_CopyList (struct CopyReMoveList **NewList, struct CopyReMoveList *OldList){
    TRACEIT(10,"CopyReMoveList_CopyList		start");
    if (NewList == NULL || OldList == NULL){
        TRACEIT(10,"CopyReMoveList_CopyList		end -1");
        return -1;
    }
    while (OldList){
        CopyReMoveList_AddItem(NewList, OldList->string1->str, OldList->string2->str, OldList->SrcObject, OldList->DestObject, OldList->FileShelfObject);
        OldList = OldList->NextItem;
    }

    TRACEIT(10,"CopyReMoveList_CopyList		end");
    return 0;
}


int CopyReMoveList_initializeMemberTo (struct CopyReMoveList *Item, GtkListStore *listStore, GtkTreeIter iter, guint64 *ui64pTotalBytesCopied, gint64 *i64pStartTotalBytesCopied, GTimeVal *timepStartTimeForBytesCopied, GTimeVal *timepStartTimeForBytesCopiedSingleItem, guint64 *ui64TotalBytes, int *iTotalBytesValid, GtkProgressBar *progressBar, int CopyMoveOrRemove, double *dpCopySpeed){
    TRACEIT(10,"CopyReMoveList_initializeMemberTo		end");

    Item->listStore=listStore;
    Item->iter = iter;
    Item->status = -1;
    Item->ibreak=0;
    Item->lastfinishedTotalBytes=0;
    Item->threadCalcSize=NULL;
    Item->ui64pTotalBytesCopied = ui64pTotalBytesCopied;
    Item->ui64TotalBytes = ui64TotalBytes;
    Item->iTotalBytesValid = iTotalBytesValid;
    Item->progressBar = progressBar;
    Item->CopyMoveOrRemove = CopyMoveOrRemove;
    Item->i64pStartTotalBytesCopied = i64pStartTotalBytesCopied;
    Item->timepStartTimeForBytesCopied = timepStartTimeForBytesCopied;
    Item->timepStartTimeForBytesCopiedSingleItem = timepStartTimeForBytesCopiedSingleItem;
    Item->dpCopySpeed = dpCopySpeed;

    TRACEIT(10,"CopyReMoveList_initializeMemberTo		end");
    return 0;
}


struct CopyReMoveList *CopyReMoveList_removeItemFromList (struct CopyReMoveList *theItem){
    //returns the new beginning of the list
    TRACEIT(10,"CopyReMoveList_removeItemFromList		start");
	if (theItem == NULL){
		TRACEIT(2,"CopyReMoveList_removeItemFromList		PARAMETER NULL");
		return NULL;
	}
    struct CopyReMoveList *Following;
    struct CopyReMoveList *Before;

    Following = theItem->NextItem;
    Before = theItem->PrevItem;

    if (Before != NULL){
        Before->NextItem = Following;
    }
    if (Following != NULL){
        Following->PrevItem = Before;
    }
    CopyReMoveLis_deleteItemMember (theItem);
    free (theItem);
    Before = (Before==NULL)?Following:Before;
    if (Before == NULL){
        return NULL;    //the whole list is gone
    }

    while (Before->PrevItem != NULL){
        Before = Before->PrevItem;
    }


    TRACEIT(10,"CopyReMoveList_removeItemFromList		end");
    return Before;
}


int CopyReMoveLis_deleteItemMember (struct CopyReMoveList *FirstItem){
    //will not delete the item itself but frees all the memory resevered before
    TRACEIT(10,"CopyReMoveLis_deleteItemMember		start");
	if (FirstItem == NULL){
		TRACEIT(2,"CopyReMoveLis_deleteItemMember		PARAMETER NULL");
		return -1;
	}

	if (FirstItem->string1 != NULL){
        g_string_free(FirstItem->string1,TRUE);
        FirstItem->string1=NULL;
    }
    if (FirstItem->string2 != NULL){
        g_string_free(FirstItem->string2,TRUE);
        FirstItem->string2=NULL;
    }
    if (FirstItem->UserCommand != NULL){
        g_string_free(FirstItem->UserCommand, TRUE);
        FirstItem->UserCommand=NULL;
    }
    if (FirstItem->gstrCaption != NULL){
        g_string_free(FirstItem->gstrCaption, TRUE);
        FirstItem->gstrCaption=NULL;
    }
    if (FirstItem->gstrLabel != NULL){
        g_string_free(FirstItem->gstrLabel, TRUE);
        FirstItem->gstrLabel=NULL;
    }
    if (FirstItem->Errors != NULL){
        g_string_free(FirstItem->Errors, TRUE);
        FirstItem->Errors=NULL;
    }
    if (FirstItem->ErrorMessage != NULL){
        g_string_free(FirstItem->ErrorMessage, TRUE);
        FirstItem->ErrorMessage=NULL;
    }



	TRACEIT(10,"CopyReMoveLis_deleteItemMember		end");
	return 0;
}

int CopyReMoveList_DeleteList(struct CopyReMoveList *FirstItem){
	TRACEIT(10,"CopyReMoveList_DeleteList		start");
	if (FirstItem == NULL){
		TRACEIT(2, "CopyReMoveList_DeleteList		PARAMETER NULL");
		return -2;
	}
	struct CopyReMoveList *bufItem;
	for ( bufItem=FirstItem; bufItem->PrevItem; bufItem=bufItem->PrevItem);
	do{
        CopyReMoveLis_deleteItemMember (FirstItem);

		bufItem = FirstItem;
		FirstItem = FirstItem->NextItem;
		free(bufItem);
	}while (FirstItem != NULL);
	TRACEIT(10,"CopyReMoveList_DeleteList		ends");
	return 0;
}


/////////////////////////////////////////////////////////////////////////////////////////////////
///////////////// FILESHELF OBJECT LIST /////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////
struct FileShelfObjectList *FirstFileShelfObject;

struct FileShelfObjectList *FileShelftObjectList_getFirstObject(){
	return FirstFileShelfObject;
}

struct FileShelfObjectList *FileShelfObjectList_getNextObejct(struct  FileShelfObjectList *Object){
	return Object->NextItem;
}

int FileShelfObjectList_StartWithObjectList(){
	//called in InitFileShelf (do not call it from another function)
	TRACEIT(10,"FileShelfObjectList_StartWithObjectList		start");
	FirstFileShelfObject = malloc(sizeof(struct FileShelfObjectList));
	if (FirstFileShelfObject == NULL){
		TRACEIT(8,"FileShelfObjectList_StartWithObjectList		ends");
		return -1;
	}

	FirstFileShelfObject->notebook=NULL;
	FirstFileShelfObject->listView=NULL;
	FirstFileShelfObject->listStore=NULL;
	FirstFileShelfObject->ScrollWindow=NULL;
	FirstFileShelfObject->actualSortCol=NULL;
	FirstFileShelfObject->PreviewCol=NULL;
	FirstFileShelfObject->label=NULL;
	FirstFileShelfObject->iRecursiveSizeCalculation=-1;
	FirstFileShelfObject->ID=getRandomIntNumber(-1, -1);	//-1: do just return the rand() output, do not apply any border calculations
    FirstFileShelfObject->gthreadF6Updage = NULL;
    FirstFileShelfObject->iPreview=0;

	FirstFileShelfObject->PrevItem = NULL;
	FirstFileShelfObject->NextItem = NULL;

	TRACEIT(10,"FileShelfObjectList_StartWithObjectList		ends");
	return 0;
}


int FileShelfObjectList_NewObject(struct FileShelfObjectList **RetObject){
	TRACEIT(10,"FileShelfObjectList_NewObject		start");
	if (RetObject == NULL){
		TRACEIT(2,"FileShelfObjectList_NewObject		ends, PARAMETER NULL");
		return -1;
	}

	struct  FileShelfObjectList *Object = FirstFileShelfObject;

	while (Object->NextItem != NULL){
		Object = Object->NextItem;
	}
	Object->NextItem = malloc(sizeof(struct FileShelfObjectList));
	if (FirstFileShelfObject == NULL){
		TRACEIT(1,"FileShelfObjectList_NewObject		ends, error allocating memory");
		*RetObject = NULL;
		return -1;
	}
	Object->NextItem->PrevItem = Object;
	Object = Object->NextItem;
	Object->NextItem=NULL;

	Object->notebook=NULL;
	Object->listView=NULL;
	Object->listStore=NULL;
	Object->ScrollWindow=NULL;
	Object->actualSortCol=NULL;
	Object->PreviewCol=NULL;
	Object->label=NULL;
	Object->iRecursiveSizeCalculation=-1;
	Object->ID=getRandomIntNumber(-1, -1);	//-1: do just return the rand() output, do not apply any border calculations
    Object->gthreadF6Updage = NULL;
    Object->iPreview=0;

	*RetObject = Object;
	TRACEIT(10,"FileShelfObjectList_NewObject		ends");
	return 0;
}


void FileShelfObjectList_GetActiveObject(struct FileShelfObjectList **Object){
	TRACEIT(10,"FileShelfObjectList_GetActiveObject		start");
	if (Object == NULL){
		TRACEIT(2,"FileShelfObjectList_GetActiveObject		ends, PARAMETER NULL");
		return;
	}
	struct FileShelfObjectList *BufObject=FirstFileShelfObject;

	if (BufObject == NULL){
		TRACEIT(10,"FileShelfObjectList_GetActiveObject		ends 2, fileshelf not initialized");
		*Object=NULL;
		return;
	}

	int i_activePage = gtk_notebook_get_current_page (FirstFileShelfObject->notebook);
	GtkWidget *ActiveWidget = gtk_notebook_get_nth_page(FirstFileShelfObject->notebook, i_activePage);
	while (BufObject){
		if (BufObject->ScrollWindow == ActiveWidget){
			*Object = BufObject;
			TRACEIT(10,"FileShelfObjectList_GetActiveObject		ends 1");
			return;
		}
		BufObject = BufObject->NextItem;
	};
	*Object = NULL;
	TRACEIT(10,"FileShelfObjectList_GetActiveObject		ends");
	return;
}


int FileShelfObjectList_GetPageNumberByObject(struct FileShelfObjectList *Object){
	//return: -1=error,  pagenumber on success
	TRACEIT(11,"FileShelfObjectList_GetPageNumberByObject		start");
	if (Object == NULL){
		TRACEIT(2,"FileShelfObjectList_GetPageNumberByObject		end, PARAMETER NULL");
		return -1;
	}
	int i;
	struct FileShelfObjectList *BufObject=FirstFileShelfObject;
	int iPages = gtk_notebook_get_n_pages (BufObject->notebook);
	int found = 0;
	for (i=0; i<=iPages; i++){
		FileShelfObjectList_GetObjectByPageNumber(&BufObject, i);
		if (BufObject == Object){
			found = 1;
			break;
		}
	}

	if (found == 1){
		return i;
	}
	TRACEIT(11,"FileShelfObjectList_GetPageNumberByObject		end");
	return -1;
}


int FileShelfObjectList_GetIDByPageNumber(int iPageNumber){
	//return: -1=error, ID on success
	TRACEIT(11,"FileShelfObjectList_GetIDByPageNumber		start");
	int iPages = gtk_notebook_get_n_pages (FirstFileShelfObject->notebook);
	if (iPageNumber > iPages){
		TRACEIT(4,"FileShelfObjectList_GetIDByPageNumber		end, iPageNumner>iPages");
		return -1;
	}
	struct FileShelfObjectList *BufObject;
	if (-1 == FileShelfObjectList_GetObjectByPageNumber(&BufObject, iPageNumber)){
		TRACEIT(4,"FileShelfObjectList_GetIDByPageNumber		end, could not find object to pagenumber");
		return -1;
	}
	TRACEIT(11,"FileShelfObjectList_GetIDByPageNumber		end");
	return BufObject->ID;
}


int FileShelfObjectList_GetPageNumberByID(struct FileShelfObjectList *Object, int ID){
	//return: -1=error,  pagenumber on success
	TRACEIT(11,"FileShelfObjectList_GetPageNumberByID		start");

	if (Object == NULL){
		TRACEIT(2,"FileShelfObjectList_GetPageNumberByID		end, PARAMETER NULL");
		return -1;
	}

	if (ID < 0){
		TRACEIT(2,"FileShelfObjectList_GetPageNumberByID		end, ID<0, How can this be?");
		return -1;
	}

	struct FileShelfObjectList *BufObject=FirstFileShelfObject;
	int PageNumber = -1;
	do{
		if (BufObject->ID == ID){
			PageNumber = FileShelfObjectList_GetPageNumberByObject(BufObject);
			break;
		}
		BufObject = BufObject->NextItem;
	}while(BufObject);

	TRACEIT(11,"FileShelfObjectList_GetPageNumberByID		end");
	return PageNumber;
}


struct FileShelfObjectList  *FileShelfObjectList_GetObjectByID(const int ID){
	//return NULL on error

	struct FileShelfObjectList *BufObject=FirstFileShelfObject;

	while (BufObject){
		if (ID == BufObject->ID){
			return BufObject;
		}
		BufObject = BufObject->NextItem;
	}

	return NULL;
}


int FileShelfObjectList_GetObjectByPageNumber(struct FileShelfObjectList **Object, guint Number){
	//return: -1=error, 0=success
	TRACEIT(11,"FileShelfObjectList_GetObjectByPageNumber		start");
	if (Object == NULL){
		TRACEIT(2,"FileShelfObjectList_GetObjectByPageNumber		ends, PARAMETER NULL");
		return -1;
	}

	int currentPage = Number;

	*Object = NULL;

	struct FileShelfObjectList *BufObject=FirstFileShelfObject;

	do{
		if (currentPage == gtk_notebook_page_num(FirstFileShelfObject->notebook,GTK_WIDGET(BufObject->ScrollWindow))){
			*Object = BufObject;
			TRACEIT(11,"FileShelfObjectList_GetObjectByPageNumber		ends");
			return 0;
		}
		BufObject = BufObject->NextItem;

	}while(BufObject);
	TRACEIT(8,"FileShelfObjectList_GetObjectByPageNumber		ends		NotFound ERROR");
	return -1;
}


int FileShelfObjectList_GetObjectByChildListView(struct FileShelfObjectList **Object, GtkTreeView *listView){
    // writes NULL to *Object to determine the result to be valid (in some kind)
    // searches all the Objects till the Listview given here is the same as in one if the Objects abnd
    // then fills the Obect and the side where side is the notebook (left or right)
    TRACEIT(10,"FileShelfObjectList_GetObjectByChildListView		start");
    if (Object == NULL || listView == NULL){
        TRACEIT(2,"FileShelfObjectList_GetObjectByChildListView		ends, parameter NULL");
        return -1;
    }
    struct FileShelfObjectList *Objects = FirstFileShelfObject;
    if (Objects == NULL){
        TRACEIT(2,"FileShelfObjectList_GetObjectByChildListView		ends, No Objects defined");
        return -1;
    }
    *Object=NULL;
    while (Objects){
        if (Objects->listView == (GtkWidget *)listView){
            *Object = Objects;
            break;
        }
        Objects = Objects->NextItem;
    }

    TRACEIT(10,"FileShelfObjectList_GetObjectByChildListView		end");
    return 0;
}


void FileShelfObjectList_DeleteObject(struct FileShelfObjectList *Object){
	TRACEIT(10,"FileShelfObjectList_DeleteObject	start");
	if (Object == NULL){
		TRACEIT(2,"FileShelfObjectList_DeleteObject		ends, PARAMETER NULL");
		return;
	}

	if (Object->PrevItem != NULL){
		Object->PrevItem->NextItem = Object->NextItem;
	}
	else{
		FirstFileShelfObject = Object->NextItem;
	}

	if (Object->NextItem != NULL){
		Object->NextItem->PrevItem = Object->PrevItem;
	}

    calcDirectorySizeInView_closeProcess (&(Object->iRecursiveSizeCalculation), &(Object->gthreadF6Updage), NULL, NULL, TRUE);

    SideObjectList_UnrefPreviewPixbufs (Object->listStore, COL_FLSHLF_PREVIEW);

	if (Object->listStore != NULL){
		g_object_unref(Object->listStore);
	}
	free(Object);
	TRACEIT(10,"FileShelfObjectList_DeleteObject	ends");
	return;
}


void FileShelfObjectList_ClearAllForShutdown(){
	TRACEIT(10,"SideObjectList_ClearAllForShutdown		start");
	struct FileShelfObjectList *ThisObject;

	while (FirstFileShelfObject){

		ThisObject = FirstFileShelfObject;
		FirstFileShelfObject->PrevItem=NULL;

		calcDirectorySizeInView_closeProcess (&(ThisObject->iRecursiveSizeCalculation), &(ThisObject->gthreadF6Updage), NULL, NULL, TRUE);

        SideObjectList_UnrefPreviewPixbufs (ThisObject->listStore, COL_FLSHLF_PREVIEW);

		if (FirstFileShelfObject->listStore != NULL){	//it can be NULL when a directory was loaded (e.g. on startup)
			g_object_unref(FirstFileShelfObject->listStore);	//wich does not exist then, there is no listStore defined
		}

		FirstFileShelfObject = FirstFileShelfObject->NextItem;
		free(ThisObject);
	}
	TRACEIT(10,"SideObjectList_ClearAllForShutdown		ends");
	return;
}


int FileShelfObjectList_ValidatObject(struct FileShelfObjectList *Object){
	TRACEIT(10,"FileShelfObjectList_ValidateObject		start");
	if (Object == NULL)	//this is possible when processItemsViaList from within a SideObject (for copy Remove things)
		return -1;
	struct FileShelfObjectList *buf = FirstFileShelfObject;

	while (buf != NULL){
		if (buf == Object){
			TRACEIT(10,"FileShelfObjectList_ValidateObject		ends 1");
			return 1;
		}
		buf = buf->NextItem;
	};
	TRACEIT(10,"FileShelfObjectList_ValidateObject		ends -1");
	return -1;
}

/////////////////////////////////////////////////////////////////////////////////////////////////
///////////////// SIDE OBJECT LIST //////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////

struct  SideObjectList *FirstObject;

struct SideObjectList *SideObjectList_getFirstObject(){
	return FirstObject;
}

struct SideObjectList *SideObjectList_getNextObejct(struct  SideObjectList *Object){
	return Object->NextItem;
}


int SideObjectList_StartWithObjectList(GtkNotebook *notebook, char *Path){
	TRACEIT(10,"SideObjectList_StartWithObjectList		start");
	FirstObject = malloc(sizeof(struct SideObjectList));
	if (FirstObject == NULL){
		TRACEIT(8,"SideObjectList_StartWithObjectList		ends");
		return -1;
	}
	FirstObject->PrevItem = NULL;
	FirstObject->NextItem = NULL;

	SideObjectList_ProcessNewItem(notebook, FirstObject, Path, NULL);
	TRACEIT(10,"SideObjectList_StartWithObjectList		ends");
	return 0;
}


void SideObjectList_ProcessNewItem(GtkNotebook *notebook, struct SideObjectList *Object, char *Path, struct SideObjectList *OldObject){
	TRACEIT(10,"SideObjectList_ProcessNewItem		start");
	Object->notebook = notebook;
	//predefine_NotebookTaps_Objects(Object);
	//connectSignalsToNotebookTapObject(Object);

	Object->listStore=NULL;
	Object->gstrPath = g_string_new(Path);  //dont change this, we need this for monitoring, look in updateViewWithNewFolder

	if (OldObject != NULL){
		Object->showHiddenFiles=OldObject->showHiddenFiles;
		Object->iPreview=OldObject->iPreview;
	}
	else{
		Object->showHiddenFiles=FALSE;
		Object->iPreview = 0;
	}

	if( g_int_AlwaysShowHidden==1){
		Object->alwaysShowHiddenFiles=TRUE;
	}
	else{
		Object->alwaysShowHiddenFiles=FALSE;
	}

	Object->iPreventFromUpdating = 0;
	Object->bUpdateFlag_UpdateOnTabSwitch = FALSE;
	Object->bUpdateFlag_CurrentlyUpdating = FALSE;
	Object->bUpdateFlag_UserWantsToUpdate = FALSE;
	Object->bUpdateFlag_F6 = FALSE;
	Object->gthreadF6Updage = NULL;
	Object->actualSortCol=NULL;
	Object->PreviewCol=NULL;
	Object->bPointsAddedToView=FALSE;
	Object->bReadOnly = FALSE;

    predefine_NotebookTaps_Objects(Object);
	connectSignalsToNotebookTapObject(Object);


	gtk_container_add(GTK_CONTAINER(Object->ScrollWindow),Object->listView);
	gtk_box_pack_start(GTK_BOX(Object->VBox),GTK_WIDGET(Object->ScrollWindow),TRUE,TRUE,1);
	gtk_box_pack_start(GTK_BOX(Object->VBox),GTK_WIDGET(Object->StatusBar),FALSE,FALSE,1);

	gtk_widget_show_all (GTK_WIDGET (Object->VBox));

#ifndef NO_GIO
	//add a monitor:
	autoUpdateList_AddMonitor(Path);
#endif

	gtk_notebook_append_page(notebook,GTK_WIDGET(Object->VBox),Object->label);
	gtk_notebook_set_tab_reorderable(notebook,GTK_WIDGET(Object->VBox),TRUE);
	predefine_NotebookTabs_reorderColumns ((GtkTreeView *)Object->listView);
    //gtk_notebook_set_tab_detachable (notebook, GTK_WIDGET(Object->VBox), TRUE);

    ////////// Test section
    //GtkWidget *label = gtk_notebook_get_tab_label (notebook,GTK_WIDGET(Object->VBox));
    //prepareWidgetForDrag (label);
    /////////////

	//updateViewWithNewFolder(Object,Object->gstrPath);	//is done in Signal "page-added"
	TRACEIT(10,"SideObjectList_ProcessNewItem		ends");
}


int SideObjectList_CreateNewViewWithPath(GtkNotebook *notebook, char *Path, struct SideObjectList *OldObject){	//OldObject holds object from which the new page is copied when calling movepage, copy page, dublicate page. it can be NULL
//this function is also called from within the fileshelf. And creates a new SideObject with the path saved in the Fileshelf.
//Therefor it is necessary that notebook can be NULL. In this case, the information is got from FirstObject.
	TRACEIT(10,"SideObjectList_CreateNewViewWithPath		start");
	if (FirstObject == NULL){
		TRACEIT(8,"SideObjectList_CreateNewViewWithPath		ends		ERROR");
		return -1;
	}
	struct SideObjectList *newObject = FirstObject;
	struct SideObjectList *bufObject;

	while(newObject->NextItem != NULL){
		newObject = newObject->NextItem;
	}

	newObject->NextItem = malloc(sizeof(struct SideObjectList));

	if (newObject->NextItem == NULL){
		TRACEIT(8,"SideObjectList_CreateNewViewWithPath		ends		ERROR2");
		return -1;
	}

	bufObject = newObject;
	newObject = bufObject->NextItem;
	newObject->PrevItem = bufObject;
	newObject->NextItem = NULL;

	if (notebook == NULL){
		SideObjectList_ProcessNewItem(FirstObject->notebook, newObject, Path, OldObject);
	}
	else{
		SideObjectList_ProcessNewItem(notebook, newObject, Path, OldObject);
	}
	TRACEIT(10,"SideObjectList_CreateNewViewWithPath		ends");
	return 0;
}


int SideObjectList_GetActiveObject(GtkNotebook *notebook, struct SideObjectList **Object){
	TRACEIT(11,"SideObjectList_GetActiveObject		start");
	int currentPage = gtk_notebook_get_current_page(notebook);

	if (currentPage == -1){
		currentPage = -4;
	}
	*Object = NULL;

	struct SideObjectList *bufObject = FirstObject;
	do{
		if (currentPage == gtk_notebook_page_num(notebook,GTK_WIDGET(bufObject->VBox))){
			*Object = bufObject;
			//(*Object)->PageNumber=currentPage;
			TRACEIT(11,"SideObjectList_GetActiveObject		ends");
			return 0;
		}
		bufObject = bufObject->NextItem;


	}while(bufObject);
	TRACEIT(10,"SideObjectList_GetActiveObject		ends		NotFound ERROR");
	return -1;
}


int SideObjectList_GetObjectByPageNumber(GtkNotebook *notebook, struct SideObjectList **Object, guint Number){
	TRACEIT(11,"SideObjectList_GetObjectByPageNumber		start");
	if (Object == NULL || notebook == NULL){
		TRACEIT(11,"SideObjectList_GetObjectByPageNumber		end, ERROR Object or Notebook NULL");
		return -2;
	}
	int currentPage = Number;

	*Object = NULL;

	struct SideObjectList *bufObject = FirstObject;
	if (FirstObject == NULL){
		TRACEIT(2,"SideObjectList_GetObjectByPageNumber		FirstObject==NULL");
	}

	do{
		TRACEIT(12, "in do-while");
		if (currentPage == gtk_notebook_page_num(notebook,GTK_WIDGET(bufObject->VBox))){
			*Object = bufObject;
			TRACEIT(11,"SideObjectList_GetObjectByPageNumber		ends");
			return 0;
		}
		bufObject = bufObject->NextItem;

	}while(bufObject);
	TRACEIT(10,"SideObjectList_GetObjectByPageNumber		ends		NotFound ERROR");
	return -1;
}


int SideObjectList_GetObjectByChildListView(struct SideObjectList **Object, GtkTreeView *listView, int *side){
    // writes NULL to *Object to determine the result to be valid (in some kind)
    // searches all the Objects till the Listview given here is the same as in one if the Objects abnd
    // then fills the Obect and the side where side is the notebook (left or right)
    TRACEIT(10,"SideObjectList_GetObjectByPageNumber		start");
    if (Object == NULL || listView == NULL || side == NULL){
        TRACEIT(2,"SideObjectList_GetObjectByPageNumber		ends, parameter NULL");
        return -1;
    }
    struct SideObjectList *Objects = SideObjectList_getFirstObject();
    if (Objects == NULL){
        TRACEIT(2,"SideObjectList_GetObjectByPageNumber		ends, No Objects defined");
        return -1;
    }
    *Object=NULL;
    while (Objects){
        if (Objects->listView == (GtkWidget *)listView){
            *Object = Objects;
            if (Objects->notebook == (GtkNotebook *)Notebooks[SIDE_LEFT]){
                *side = SIDE_LEFT;
            }
            else{
                *side = SIDE_RIGHT;
            }
            break;
        }
        Objects = Objects->NextItem;
    }
    TRACEIT(10,"SideObjectList_GetObjectByPageNumber		end");
    return 0;
}


int SideObjectList_GetObjectByChildVBox(struct SideObjectList **Object, GtkWidget *vbox, int *side){
    // writes NULL to *Object to determine the result to be valid (in some kind)
    // searches all the Objects till the vbox given here is the same as in one if the Objects abnd
    // then fills the Obect and the side where side is the notebook (left or right)
    //the vbox is the child which is stored in the notebook tabs
    TRACEIT(10,"SideObjectList_GetObjectByChildVBox		start");
    if (Object == NULL || vbox == NULL || side == NULL){
        TRACEIT(2,"SideObjectList_GetObjectByChildVBox		ends, parameter NULL");
        return -1;
    }
    struct SideObjectList *Objects = SideObjectList_getFirstObject();
    if (Objects == NULL){
        TRACEIT(2,"SideObjectList_GetObjectByChildVBox		ends, No Objects defined");
        return -1;
    }
    *Object=NULL;
    while (Objects){
        if ((GtkWidget *)Objects->VBox == vbox){
            *Object = Objects;
            if (Objects->notebook == (GtkNotebook *)Notebooks[SIDE_LEFT]){
                *side = SIDE_LEFT;
            }
            else{
                *side = SIDE_RIGHT;
            }
            break;
        }
        Objects = Objects->NextItem;
    }

    TRACEIT(10,"SideObjectList_GetObjectByChildVBox		end");
    return 0;
}


void SideObjectList_UpdateObject(struct SideObjectList *Object){
	TRACEIT(10,"SideObjectList_UpdateObject		start");
	GString *labelText = g_string_new(Object->gstrPath->str);
	checkPathForLastCharOtherwise(labelText);
	getLastStringInPath(labelText);

	cutStringToNChars(labelText,MAXCHARSINNOTEBOOKLABEL);

	if ( Object->showHiddenFiles || Object->alwaysShowHiddenFiles){
		g_string_append(labelText, " (h)");
	}
	if (Object->iPreventFromUpdating!=0){
        g_string_append(labelText, " (nu)");
	}
	if (Object->bReadOnly){
		g_string_append (labelText, " (ro)");
	}

	gtk_label_set_text(GTK_LABEL(Object->label),labelText->str);

	g_string_free(labelText,TRUE);
	TRACEIT(10,"SideObjectList_UpdateObject		ends");
}


void SideObjectList_DeleteActiveObject(GtkNotebook *notebook){
	TRACEIT(10,"SideObjectList_DeleteActiveObject		start");
	struct SideObjectList *Object=NULL;
	SideObjectList_GetActiveObject(notebook, &Object);

	if (Object == NULL){
		TRACEIT(10,"SideObjectList_DeleteActiveObject		ends");
		return;
	}

	if (1>=gtk_notebook_get_n_pages(notebook)){
		TRACEIT(10,"SideObjectList_DeleteActiveObject		ends");
		return;
	}
	gtk_notebook_remove_page(notebook,gtk_notebook_get_current_page (notebook));

	if (Object->NextItem!=NULL ){
		Object->NextItem->PrevItem=Object->PrevItem;
	}
	if (Object->PrevItem!=NULL){
		Object->PrevItem->NextItem=Object->NextItem;
	}
	else{
		FirstObject = Object->NextItem;
	}
	SideObjectList_ClearObject(Object);
	TRACEIT(10,"SideObjectList_DeleteActiveObject		ends");
}


void SideObjectList_MoveActiveObject(GtkNotebook *srcnotebook, GtkNotebook *destnotebook){
	TRACEIT(10,"SideObjectList_MoveActiveObject		start");

	struct SideObjectList *Object=NULL;
	SideObjectList_GetActiveObject(srcnotebook, &Object);

	if (Object == NULL){
		TRACEIT(10,"SideObjectList_MoveActiveObject		ends");
		return;
	}

	if (1>=gtk_notebook_get_n_pages(srcnotebook)){
		TRACEIT(10,"SideObjectList_MoveActiveObject		ends");
		return;
	}

	SideObjectList_CreateNewViewWithPath(destnotebook, Object->gstrPath->str, Object);
	SideObjectList_DeleteActiveObject( srcnotebook);

	TRACEIT(10,"SideObjectList_MoveActiveObject		ends");
	return;
}


int SideObjectList_ValidateObject(struct SideObjectList *Object){
	TRACEIT(10,"SideObjectList_ValidateObject		start");
	if (Object == NULL)	//this is possible when processItemsViaList is called from within the Fileshelf	(for copy Remove things)
		return -1;
	struct SideObjectList *buf = FirstObject;

	while (buf != NULL){
		if (buf == Object){
			TRACEIT(10,"SideObjectList_ValidateObject		ends 1");
			return 1;
		}
		buf = buf->NextItem;
	};
	TRACEIT(10,"SideObjectList_ValidateObject		ends -1");
	return -1;
}


void SideObjectList_ClearObject(struct SideObjectList *Object){
	TRACEIT(10,"SideObjectList_ClearObject		start");

#ifndef NO_GIO
	//remove monitor
    autoUpdateList_RemoveMonitor(Object->gstrPath->str);
#endif
    calcDirectorySizeInView_closeProcess (&(Object->iRecursiveSizeCalculation), &(Object->gthreadF6Updage), &(Object->bUpdateFlag_F6), &(Object->iPreventFromUpdating), TRUE);

    SideObjectList_UnrefPreviewPixbufs (Object->listStore, COL_PREVIEW);

    if (Object->listStore != NULL){
		g_object_unref(Object->listStore);
	}

	g_string_free(Object->gstrPath,TRUE);
	free(Object);

	TRACEIT(10,"SideObjectList_ClearObject		ends");
}


void SideObjectList_ClearAllForShutdown(){
	TRACEIT(10,"SideObjectList_ClearAllForShutdown		start");
	struct SideObjectList *ThisObject;

	while (FirstObject){
		ThisObject = FirstObject;
		FirstObject->PrevItem=NULL;

        FirstObject = FirstObject->NextItem;

		SideObjectList_ClearObject (ThisObject);
	}
	TRACEIT(10,"SideObjectList_ClearAllForShutdown		ends");
	return;
}


gboolean SideObjectList_UnrefPreviewPixbufs_foreach ( GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, void *data ){
    TRACEIT(10,"SideObjectList_UnrefPreviewPixbufs_foreach		start");

    GdkPixbuf *gdkpixBuf = NULL;
    struct AllInformationAllUseStruct *stru = (struct AllInformationAllUseStruct *)data;
    if (stru == NULL){
        TRACEIT (3, "SideObjectList_UnrefPreviewPixbufs_foreach     error, parameter NULL");
        return FALSE;
    }
    GList **glistPixBufs = NULL;
    GtkListStore *listStore=NULL;
    int iColumn = -1;

    listStore = (GtkListStore *)stru->Info_1;
    glistPixBufs = (GList **)stru->Info_2;
    iColumn = *(int *)stru->Info_3;
    if (NULL == listStore || NULL == glistPixBufs || -1==iColumn){
        TRACEIT (3, "SideObjectList_UnrefPreviewPixbufs_foreach     error, parameter NULL 2");
        return FALSE;
    }

    gtk_tree_model_get(model, iter, iColumn, &gdkpixBuf, -1);
    if (NULL != gdkpixBuf){
        *glistPixBufs = g_list_prepend (*glistPixBufs, gdkpixBuf);
        gtk_list_store_set (listStore, iter,iColumn, NULL, -1);
    }

    TRACEIT(10,"SideObjectList_UnrefPreviewPixbufs_foreach		end");
    return FALSE;   //false = continue
}



void SideObjectList_UnrefPreviewPixbufs(GtkListStore *listStore, int iColumn){
    //unrefers the pixbufs of the listview if they are set
    TRACEIT(10,"SideObjectList_ClearAllForShutdown		start");
    GList *glistPixBufs = NULL;
    GList *glistBuf = NULL;

    if (listStore == NULL){
        return;
    }

    struct AllInformationAllUseStruct stru;

    stru.Info_1 = (void *)listStore;
    stru.Info_2 = (void *)&glistPixBufs;
    stru.Info_3 = (void *)&iColumn;

    gtk_tree_model_foreach ((GtkTreeModel *)listStore, (GtkTreeModelForeachFunc)SideObjectList_UnrefPreviewPixbufs_foreach, &stru);

    if (NULL != glistPixBufs){
        glistBuf = glistPixBufs;
        while (glistBuf){
            g_object_unref (glistBuf->data);    //we need
            g_object_unref (glistBuf->data);    //two unrefs for the pixbufs, because it seems that the liststore increases the reference too, but does not free it
            glistBuf = glistBuf->next;
        }
        g_list_free (glistPixBufs);
    }

    TRACEIT(10,"SideObjectList_ClearAllForShutdown		ends");
}

/////////////////////////////////////////////////////////////////////////////////////////////////
///////////////// UPDATE LIST /////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////
// Be carefull changing this, because this list is also used in other terms (e.g. when marking all items which are of the same type as the already marked one ( markItemsWithSameType )
void updateList_addItem(struct updateList **itemlist, char *Name, char *Path, GtkTreeRowReference  *rowref, int found){
	TRACEIT(10,"updateList_addItem		start");
	if(itemlist == NULL){
		TRACEIT(2,"updateList_addItem		ends, itemlist parameter NULL");
		return;
	}
    struct updateList *newItem=NULL;
    struct updateList *bufItem=*itemlist;

    newItem = malloc(sizeof(struct updateList) );
    if (newItem == NULL){
        TRACEIT(2,"updateList_addItem		unable to allocate memory");
		return;
    }

    newItem->gstrName = NULL;
    newItem->gstrPath = NULL;
    newItem->rowref = NULL;

    if (Name)
        newItem->gstrName = g_string_new(Name);
    if (Path)
        newItem->gstrPath = g_string_new(Path);
    if (rowref)
        newItem->rowref = rowref;
    newItem->found = found;
    newItem->NextItem = NULL;
    newItem->PrevItem = NULL;

    if (*itemlist == NULL){
        *itemlist = newItem;

		TRACEIT(10,"updateList_addItem		end");
        return;
    }
    while (bufItem->NextItem){
        bufItem = bufItem->NextItem;
    }
    bufItem->NextItem = newItem;
    newItem->PrevItem = bufItem;
	TRACEIT(10,"updateList_addItem		end");
    return;
}


void updateList_getFirstItem(struct updateList **itemlist){
	TRACEIT(10,"updateList_getFirstItem		start");
    if (itemlist == NULL || *itemlist == NULL){
		TRACEIT(2,"updateList_getFirstItem		end, itemlist invalid");
        return;
	}
    while((*itemlist)->PrevItem){
        *itemlist = (*itemlist)->PrevItem;
    }
	TRACEIT(10,"updateList_getFirstItem		end");
}


void updateList_deleteList(struct updateList **itemlist){
	TRACEIT(10,"updateList_deleteList		start");
    if (itemlist == NULL || *itemlist == NULL){
		TRACEIT(2,"updateList_deleteList		end, itemlist invalid");
        return;
	}
    updateList_getFirstItem(itemlist);

    struct updateList *bufItem=*itemlist;
    struct updateList *bufItem2 = NULL;

    while (bufItem){
        if (bufItem->gstrName)
            g_string_free(bufItem->gstrName,TRUE);
        if (bufItem->gstrPath)
            g_string_free(bufItem->gstrPath, TRUE);
        if (bufItem->rowref)
            gtk_tree_row_reference_free(bufItem->rowref);

        bufItem2 = bufItem;
        bufItem = bufItem->NextItem;
        free (bufItem2);
    }
    *itemlist=NULL;
	TRACEIT(10,"updateList_deleteList		end");
	return;
}


GtkTreeRowReference *updateList_checkForItem(struct updateList **itemlist, char *Name){
	TRACEIT(10,"updateList_checkForItem		start");
    if (itemlist == NULL || *itemlist == NULL){
		TRACEIT(2,"updateList_checkForItem		end, itemlist invalid");
        return NULL;
	}
    updateList_getFirstItem(itemlist);
    struct updateList *bufItem=*itemlist;

    while (bufItem){
        if (0 ==strcmp(bufItem->gstrName->str, Name)){
                bufItem->found = 1;
                return bufItem->rowref;
        }
        bufItem = bufItem->NextItem;
    }
	TRACEIT(10,"updateList_checkForItem		end");
    return NULL;
}


int updateList_checkForItem_NoRef(struct updateList **itemlist, char *Name){
	TRACEIT(10,"updateList_checkForItem_NoRef		start");
    if (itemlist == NULL || *itemlist == NULL){
		TRACEIT(2,"updateList_checkForItem_NoRef		end, itemlist invalid");
        return -1;
	}
    updateList_getFirstItem(itemlist);
    struct updateList *bufItem=*itemlist;

    while (bufItem){
        if (0 ==strcmp(bufItem->gstrName->str, Name)){
                bufItem->found = 1;
				TRACEIT(10,"updateList_checkForItem_NoRef		end, found");
                return 1;
        }
        bufItem = bufItem->NextItem;
    }
	TRACEIT(10,"updateList_checkForItem_NoRef		end, not found");
    return 0;
}
