/*
    Copyright 2007, 2008, 2009, 2010 Geyer Klaus

    This file is part of Cat'sEyE.

    Cat'sEyE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Cat'sEyE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Cat'sEyE.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef LISTS_H_
#define LISTS_H_


/////////////////////////////////////////////////////////////////////////////////////////////////
///////////////// UPDATE LIST /////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////
// Be carefull changing this, because this list is also used in other terms (e.g. when marking all items which are of the same type as the already marked one ( markItemsWithSameType )
struct updateList{
    GString *gstrName;
    GString *gstrPath;
    GtkTreeRowReference  *rowref;
    int found;

    struct updateList *PrevItem;
    struct updateList *NextItem;
};

void updateList_addItem(struct updateList **itemlist, char *Name, char *Path, GtkTreeRowReference *rowref, int found);
void updateList_deleteList(struct updateList **itemlist);
void updateList_getFirstItem(struct updateList **itemlist);
GtkTreeRowReference *updateList_checkForItem(struct updateList **itemlist, char *Name);
int updateList_checkForItem_NoRef(struct updateList **itemlist, char *Name);


/////////////////////////////////////////////////////////////////////////////////////////////////
///////////////// SIDE OBJECT LIST //////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////

struct SideObjectList{

	GtkNotebook 		*notebook;
	GtkWidget 			*listView;
	GtkListStore		*listStore;
	GtkWidget			*ScrollWindow;
	GString 			*gstrPath;
	gboolean			showHiddenFiles;
	gboolean 			alwaysShowHiddenFiles;	//set from a global variable, tunabel by the user via the internal menu. overwrites (but not echange) the value of showHiddenFiles in case it is true.
	GtkTreeViewColumn	*actualSortCol;
	GtkTreeViewColumn   *PreviewCol;
	GtkVBox 			*VBox; 			//Holds scrollview, and StatusBar
	GtkStatusbar		*StatusBar;
	guint64				guint64sizeOfSelectedItems;
	GtkWidget			*label;
	int                 iRecursiveSizeCalculation;  //-1 if there is not such a calculation, 0 if the size of the directories viewed are currently calculated
                                                    //with 'F6' threads are created which calculate the size of each directory shown on the view. The sizes
                                                    //are also shown within the view as sizes of the items. When this Object gets a normal update (e.g. by changing the directory)
                                                    //this variable becomes -1 again and all threads stop calculating immediately.

    int                 iPreview;                   //Pictures can be previewed if the user wants it. 0 -> Preview disabled, !=0 -> Preview enabled

    //Update Flags:
    int                 iPreventFromUpdating;        //1=do not update this Object (view), 0:Update this Object when nessecary
                                                    //added because of our new automatic update, in case we rename items we have to
                                                    //prevent updating. Who ever sets this flag, has to delete it!
    gboolean            bUpdateFlag_UpdateOnTabSwitch;      // *1
    gboolean            bUpdateFlag_F6;                     // *2
    gboolean            bUpdateFlag_CurrentlyUpdating;      // *3
    gboolean            bUpdateFlag_UserWantsToUpdate;      // *4
    GThread             *gthreadF6Updage;

    gboolean            bPointsAddedToView;          //a flag inticating that the points (. ..) were added to the current view (makeLists)
	gboolean			bReadOnly;	//the user can mark a tab as read only, copying to, moving from or to or deleting in this tab is prohibited


	gpointer			data;			//UserData some Functions need, e.g. the copyToOtherPannel

	struct SideObjectList 		*NextItem;
	struct SideObjectList 		*PrevItem;
};

//  *1:     if there is a monitor callback, and the tab viewing the folder monitored is not active, this flag becomes TRUE
//          When the Tab becomes active this flag is checked whether we need to update the tabObject (TRUE) or not (FALSE).
//          gets false after update.
//  *2:     becomes TRUE if the user pressed F6 in the view and prevents object from beeing updated
//          changing tab or updating with F5 deletes this flag. This flag is used to interrupt the size calculation thread
//  *3:     The name says it all, while we Update the view wth updateViewWithNewFolder, this flag is TRUE, when it is finished it is FALSE
//  *4:     If the user explicitly updates with F5 or changes the Folder this update has highest priority, autoatic updates are ignored

int SideObjectList_StartWithObjectList(GtkNotebook *notebook, char *Path);
void SideObjectList_ProcessNewItem(GtkNotebook *notebook, struct SideObjectList *Object, char *Path, struct SideObjectList *OldObject);
int SideObjectList_CreateNewViewWithPath(GtkNotebook *notebook, char *Path, struct SideObjectList *OldObject);
int SideObjectList_GetActiveObject(GtkNotebook *notebook, struct SideObjectList **Object);
int SideObjectList_GetObjectByPageNumber(GtkNotebook *notebook, struct SideObjectList **Object, guint Number);
int SideObjectList_GetObjectByChildListView(struct SideObjectList **Object, GtkTreeView *listView, int *side);
int SideObjectList_GetObjectByChildVBox(struct SideObjectList **Object, GtkWidget *vbox, int *side);
void SideObjectList_UpdateObject(struct SideObjectList *Object);
void SideObjectList_DeleteActiveObject(GtkNotebook *notebook);
void SideObjectList_ClearObject(struct SideObjectList *Object);
void SideObjectList_MoveActiveObject(GtkNotebook *srcnotebook, GtkNotebook *destnotebook);
int SideObjectList_ValidateObject(struct SideObjectList *Object);
struct SideObjectList *SideObjectList_getFirstObject();
struct SideObjectList *SideObjectList_getNextObejct(struct  SideObjectList *Object);
void SideObjectList_ClearAllForShutdown();
void SideObjectList_UnrefPreviewPixbufs(GtkListStore *listStore, int iColumn);


/////////////////////////////////////////////////////////////////////////////////////////////////
///////////////// FILESHELF OBJECT LIST /////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////


struct FileShelfObjectList{
	GtkNotebook		    *notebook;		//This never changes for the fileshelf Object, I dont know if I gonna use this
	GtkWidget		    *listView;
	GtkListStore		*listStore;
	GtkWidget		    *ScrollWindow;
	GtkTreeViewColumn 	*actualSortCol;
	GtkTreeViewColumn   *PreviewCol;
	GtkWidget		    *label;
    int                 iRecursiveSizeCalculation;  //-1 if there is not such calculation, 0 if the size of the directories viewed are currently calculated
                                                    //with 'F6' threads are created which calculate the size of each directory shown on the view. The sizes
                                                    //are also shown within the view as sizes of the items. When this Object gets a normal update (e.g. by changing the directory)
                                                    //this variable becomes -1 again and all threads stop calculating emetiadly.

    int                 iPreview;                   //Pictures can be previewed if the user wants it. 0 -> Preview disabled, !=0 -> Preview enabled

	gpointer		    data;			//UserData some Functions need (e.g. copyFileShelfToPannel)
	int				    iCopyReMoveSide;	//Holds information to which side the items of the Fileshelf should be CopyReMoved
	int					ID;				//unique ID of this FileShelf, Randomnumber

    GThread             *gthreadF6Updage;

	struct FileShelfObjectList *NextItem;
	struct FileShelfObjectList *PrevItem;
};

struct FileShelfObjectList *FileShelftObjectList_getFirstObject();
struct FileShelfObjectList *FileShelfObjectList_getNextObejct(struct  FileShelfObjectList *Object);
int 						FileShelfObjectList_StartWithObjectList();
void 						FileShelfObjectList_ClearAllForShutdown();
void 						FileShelfObjectList_GetActiveObject(struct FileShelfObjectList **Object);
int 						FileShelfObjectList_ValidatObject(struct FileShelfObjectList *Object);
int 						FileShelfObjectList_NewObject(struct FileShelfObjectList **RetObject);
void 						FileShelfObjectList_DeleteObject(struct FileShelfObjectList *Object);

int                         FileShelfObjectList_GetObjectByChildListView(struct FileShelfObjectList **Object, GtkTreeView *listView);
int                         FileShelfObjectList_GetObjectByPageNumber(struct FileShelfObjectList **Object, guint Number);
struct FileShelfObjectList *FileShelfObjectList_GetObjectByID(const int ID);
int                         FileShelfObjectList_GetPageNumberByID(struct FileShelfObjectList *Object, int ID);
int                         FileShelfObjectList_GetPageNumberByObject(struct FileShelfObjectList *Object);
int                         FileShelfObjectList_GetIDByPageNumber(int iPageNumber);



/////////////////////////////////////////////////////////////////////////////////////////////////
///////////////// CopyReMove LISTS //////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////

struct CopyReMoveList{
	GString                 *string1;
	GString                 *string2;
	struct SideObjectList   *SrcObject;
	struct SideObjectList   *DestObject;
	struct FileShelfObjectList *FileShelfObject;


	int             status;	//-10 means cancel, 1: Item Processed, 0:Item in process, -1:Item not processed yet,
	GtkTreeIter     iter; //used for storing iter for ListView
	int             CopyMoveOrRemove; //1=copy, 2=move, 3=remove

	GString         *UserCommand; //to store the usercommand that will be used with the list
	GString         *gstrCaption;		//Used with the usercommand to view the right strings in the list widget (call via list)
	GString         *gstrLabel;			//Used with the usercommand to view the right strings in the list widget (call via list)

	GString         *Errors;    //Errors and outpuT from the system-command


    ///// New copy methode:
    //GError *error;
    guint64         iBytesToCopy;
    gint64          iBytesCopied;
    int             iBytesToCopyIsValid;    //because the bytes to copy are checked recursive, we have to check wheter it is ready or not.
                                //0=invalid, 1=valid

    guint64         *ui64pTotalBytesCopied; //a pointer to ListWidgetCommunicationStructs ui64TotalBytesProcessed
    gint64          *i64pStartTotalBytesCopied;
    GTimeVal        *timepStartTimeForBytesCopied;
    GTimeVal        *timepStartTimeForBytesCopiedSingleItem;
    guint64         *ui64TotalBytes;         // a pointer to ListWidgetCommunicationStruct's ui64TotalBytes
    int             *iTotalBytesValid;       // a pointer to ListWidgetCommunicationStructs iTotalBytesValid
    double          *dpCopySpeed;
    GtkProgressBar  *progressBar;    //pointer to ListWidgetCommunicationStruct's progressBar
    GString         *ErrorMessage; //Error while recursive procedure;
    int             ibreak; // 1 == break, 0=continue. used mainly for calculating size
    gboolean        bOverWriteExisting; //True: user said yes
    //GThread *threadCalcSizeAfterProcessing;     //holds the thread wich is start after copyReMoveRecursive ends. and calcs the size of
                                                //of the destination and compairs it with the source size, allready calculated
    GtkListStore    *listStore;        //a pointer to the list store, used in the thread CalcSizeAfterCopyMove, which calcs the size of
                                    //the destination to check it.
    goffset         lastfinishedTotalBytes; //used in copyItemRecursive_Callback (used to calc the size of processed items)
    GThread         *threadCalcSize;        //the thread which calcs the size for one item

	struct CopyReMoveList *NextItem;
	struct CopyReMoveList *PrevItem;
};
int CopyReMoveList_AddItem(struct CopyReMoveList **FirstItem, char *string1, char *string2, struct SideObjectList *SrcObject, struct SideObjectList *DestObject, struct FileShelfObjectList *FileShelfObject);
int CopyReMoveList_DeleteList(struct CopyReMoveList *FirstItem);
int CopyReMoveList_CopyList (struct CopyReMoveList **NewList, struct CopyReMoveList *OldList);
int CopyReMoveList_initializeMemberTo (struct CopyReMoveList *Item, GtkListStore *listStore, GtkTreeIter iter, guint64 *ui64pTotalBytesCopied, gint64 *i64pStartTotalBytesCopied, GTimeVal *timepStartTimeForBytesCopied, GTimeVal *timepStartTimeForBytesCopiedSingleItem, guint64 *ui64TotalBytes, int *iTotalBytesValid, GtkProgressBar *progressBar, int CopyMoveOrRemove, double *dpCopySpeed );
int CopyReMoveList_ConnectTwoLists (struct CopyReMoveList *FirstList, struct CopyReMoveList *SecondList );
struct CopyReMoveList *CopyReMoveList_removeItemFromList (struct CopyReMoveList *theItem);
int CopyReMoveLis_deleteItemMember (struct CopyReMoveList *FirstItem);

/////////////////////////////////////////////////////////////////////////////////////////////////
///////////////// COMMUNICATION STRUCT //////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////
//for communication between copyReMove threads

struct ListWidgetCommunicationStruct {  //used for communication between the signals in the procedure handling processing items via list
    GtkWidget       *mainListWidget;
    GtkWidget       *listView;
    GtkWidget       *TheLabel;
    GtkListStore    *listStore; //needs an unrefere call
    GtkButton       *cancelButton;
    GtkButton       *closeButton;
    GtkButton       *pauseButton;
    GtkButton       *continueButton;
    #ifndef NO_GIO
    GCancellable    *cancellable;
    #else
    void            *cancellable;
    #endif
    GString         *WindowName;
    GError *        gerror;

    int         SecondThreadRunning;    //set to 0 in 1st thread, 1 as long as 2nd thread runs
    GThread     *gthrdCopyReMoveThread_1; //for joining the copyRemove Thread, processing the currentItem (member of this struct)
    GThread     *gthrdCopyReMoveThread_2;
    GMutex      *mtxContinue;   //mutex for the pause-continue procedure over several threads
    int         iNextThreadStarted; //helps with the mtxContinue
   // int iNotify; //used to decide if we come from the closebutton or from the X in the right upper corner
    int         iSomeError; //becomes != 0 if there was an error while processing the list;
    int         iCancelled; //because canceling is processed over threads it may not be that fast as we need it. Therefore this becomes 1 if the list was cancelled
    int         iListPaused; //becomes 1 if the list is paused, otherwise its 0
    int         iListPausedAutomatic; //becomes one if the the list is paused because of the drag-motion signal
    int         iCancelDialogRuns;  //this becomes 1 if the cancel Dialog runs, other wise its 0
    int         iDestroyFlag;       //1 if the all is done and the next function stumble over this can free all
    gboolean (*FreeMemoryAndDeleteWidgetFunction)();  //the function to frees the memory, and destroys the widget
    void (*CancelListFunction)();   //the function used to cancel the list (listbuf->status=0,g_cancellable_cancel)
    guint64     ui64TotalBytes;         //total bytes of all items in the list going to copied/moved
    guint64     ui64TotalBytesProcessed;//bytes copied (from total);
    gint64      i64StartTotalBytesCopied;
    GTimeVal    timeStartTimeForBytesCopied;
    GTimeVal    timeStartTimeForBytesCopiedSingleItem;
    double      dCopySpeed;
    int         iTotalBytesValid;           //0=not valid, still in process. 1=valid
    GtkProgressBar *progressBar;    //the progressbar for total amount of copied bytes
    GtkTreeIter *iterLastProcessedItem; //when pausing the list, this becomes the iter of the item which was processed at this time. so we can restore the markings

    int         bOverWriteAllExisting;      //-1:not asked, 1=user said yes to all, 0=user said no to all

    struct _CallUserCommand_StartThreadStructure *ThreadStartStruct;    //for our UserCommandViaList

    struct CopyReMoveList *firstItem;
    struct CopyReMoveList *currentItem;


};


#endif //LISTS_H_


