/*
    Copyright 2007, 2008, 2009, 2010 Geyer Klaus

    This file is part of Cat'sEyE.

    Cat'sEyE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Cat'sEyE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Cat'sEyE.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <gtk/gtk.h>
#include <dirent.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#include "listitems.h"
#include "Lists.h"
#include "fileshelf.h"
#include "helpers.h"
#include "SearchFor.h"

#include "mytrace.h"

#include "allInclude.h"

extern GtkWindow 	*win;   //the main widget
extern struct SearchParams *gpSearchParamStru;


int SearchDirectoryFor(char *cpDirectory, struct SearchParams *pSearchStru, int *iFoundItems, GString *gstrErrors);
void searchThread (struct SearchParams *pSearchStru);
void search_ActionWidget(struct SearchParams *pSearchStru);
int prepare_output(char *cpShelfName, struct FileShelfObjectList **FileShelfObject);
int find_dialog (GString *gstrDirectory, struct SearchParams *pSearchParams);





struct SearchParams	*searchParams_new (){
	//returns a newly allocated SearchParams Struct, FREE IT, or NULL on error
	TRACEIT(10,"SearchParams		start");
	struct SearchParams *stru = malloc (sizeof (struct SearchParams));
	if (stru != NULL){
		stru->gstrString = g_string_sized_new (20);
		stru->cUserWasAsked = 0;
		stru->iFileShelfID = -1;
		stru->pFileShelfObject = NULL;
		stru->iBreak = 0;
		stru->iDeleteNotification = 0;
		stru->actionWidget = NULL;
		stru->actionWidgetLabel = NULL;
		stru->glistDirectories = NULL;
		//stru->iDeleteByThread = iDeleteByThread;
	}

	TRACEIT(10,"SearchParams		end");

	return stru;
}


void searchParams_delete (struct SearchParams *pStru){
	//deletes a struct SearchParams and its members
	TRACEIT(10,"searchParams_delete		start");

	if (pStru != NULL){
		g_string_free (pStru->gstrString, TRUE);

		if (pStru->glistDirectories != NULL){
			g_list_foreach(pStru->glistDirectories, (GFunc) GListFreeGStrings, NULL);
			g_list_free(pStru->glistDirectories);
		}

		free (pStru);
	}
	else{
		TRACEIT (2, "searchParams_delete PARAMETER NULL; THIS IS NOT ABOUT TO HAPPEN, There might be a memory leak");
	}

	TRACEIT(10,"searchParams_delete		start");
	return;
}


void searchParams_copyMemberForSearch (struct SearchParams *pStruDest, struct SearchParams *pStruSrc){
	//This function will not copy they whole structure but only the member wich are used for the search
	TRACEIT(10,"searchParams_copyMember		start");
	if (pStruDest == NULL || pStruSrc == NULL){
		TRACEIT (2, "searchParams_copyMember		Parameter NULL");
		return;
	}

	if (pStruDest->gstrString != NULL)
		g_string_assign (pStruDest->gstrString, pStruSrc->gstrString->str);
	else
		pStruDest->gstrString = g_string_new (pStruSrc->gstrString->str);

	TRACEIT(10,"searchParams_copyMember		end");
}




int find_simple(GList *glistDirectories){
    //called from the menuitem and the main access point to this functions
    //if cpDirectory is NULL, the home directory of the user is used.
	//ciFSID can be -1, then a new FileShelf will be created before searching. or an existing Fileshelf-ID is given here.
	//if  ciFSID != -1 we will not ask the user for input if searchStru is NOT Null at the same time.


    //return: -1 on error,
    //			0 on success

    TRACEIT(10,"find_simple		start");

	if (NULL == glistDirectories){
		TRACEIT (2, "find_simple		Parameter NULL");
		return -1;
	}

    int iCont = 1;
    GList *glistBuf = NULL;
    GString *gstrBuf = g_string_new ("new search");
    struct SearchParams *pSearchStru = NULL;

	if (g_list_length(glistDirectories) == 0){
		TRACEIT (4, "find_simple		Directory list is empty");
		return -1;
	}

	pSearchStru = searchParams_new ();

	if (NULL == pSearchStru) {
		TRACEIT (1, "find_simple	unable to create new struct, out of memory?");
		iCont = 0;
	}

	if (gpSearchParamStru!=NULL && 0 < strlen (gpSearchParamStru->gstrString->str)){
		searchParams_copyMemberForSearch (pSearchStru, gpSearchParamStru);
	}

	pSearchStru->glistDirectories = glistDirectories;

	if (0 != iCont && 0 == pSearchStru->cUserWasAsked){
		if ( 0 != find_dialog((GString *)(glistDirectories->data), pSearchStru)){	//find dialog makes checks for the correctness of the search params
			iCont = 0;
		}
		else{
			g_string_assign (gpSearchParamStru->gstrString, pSearchStru->gstrString->str);
		}
	}
	pSearchStru->glistDirectories = NULL;	//looks a little bit weird doesnt it. But find_dialog needs this list and we will need  a NULL at this position in this function bellow

	if (0 != iCont){
		if (pSearchStru->iFileShelfID == -1){

			//we have to create some name for the fileshelf, but mind, that in future versions, we might have the possibility to search for dates instead of a filename:
			g_string_printf (gstrBuf, "search for: %s", pSearchStru->gstrString->str);
			pSearchStru->iFileShelfID = prepare_output(gstrBuf->str, &(pSearchStru->pFileShelfObject));
		}
		else {
			pSearchStru->pFileShelfObject = FileShelfObjectList_GetObjectByID (pSearchStru->iFileShelfID);
		}
		if (pSearchStru->pFileShelfObject == NULL){
			iCont = 0;
		}
	}

	if (0 != iCont){
		//We must copy the whole Glist for us
		glistBuf = g_list_first(glistDirectories);
		while (glistBuf){
			pSearchStru->glistDirectories = g_list_append (pSearchStru->glistDirectories, g_string_new(((GString *)(glistBuf->data))->str));
			glistBuf = glistBuf->next;
		}

		if (pSearchStru->actionWidget == NULL){
			search_ActionWidget (pSearchStru);
		}
		g_thread_create((GThreadFunc)searchThread, pSearchStru , FALSE, NULL);
	}

	g_string_free (gstrBuf, TRUE);
	TRACEIT(10,"find_simple		end");
    return iCont;
}


/** find_dialog has to validate the search params!		*/
int find_dialog (GString *gstrDirectory, struct SearchParams *pSearchParams){
    //creates the dialog asking the user for the string to search for
    TRACEIT(10,"find_dialog		start");

	if (gstrDirectory == NULL || pSearchParams == NULL){
		TRACEIT (4, "find_dialog	Parameter NULL");
		return -1;
	}

	//GString *gstrBuf = g_string_new (((GString *)(pSearchParams->glistDirectories->data))->str);
	GString *gstrBuf = g_string_sized_new (20);
    char cpText1[] = "This searches the items in the directory,\nmentioned bellow, recursivly for the occurence\nof the string you enter bellow.\nThe result is viewed in a new FileShelf.\nOnly the names are checked.\n\nSearch for occurence of:";
    int iRet=0;

    iRet = MyEasyInputWidget_Two (win, "RSearchD", cpText1, pSearchParams->gstrString, "\nDirectory:",gstrDirectory , "Ok", "Cancel", NULL, NULL );

    if (iRet == 0){
		//lets check some things before returning:
		g_string_assign(gstrBuf, pSearchParams->gstrString->str);
        cutSpacesFromGString ( gstrBuf);
        if (strlen (gstrBuf->str) == 0){
            iRet = -1;
        }
    }

	g_string_free (gstrBuf, TRUE);
    TRACEIT(10,"find_dialog		end");
    return iRet;
}


int prepare_output(char *cpShelfName, struct FileShelfObjectList **FileShelfObject){
    //creates the Fileshelf where the results show up
    //returns: 	-1 on error
	//			FileShelf ID on success
    TRACEIT(10,"prepare_output		start");
    if (cpShelfName == NULL){
        TRACEIT(2,"prepare_output		end, parameter NULL");
        return -1;
    }
    if (0 == NewFileShelf (cpShelfName)){
        FileShelfObjectList_GetActiveObject(FileShelfObject);
        TRACEIT(10,"prepare_output		end");
        return (*FileShelfObject)->ID;
    }

    TRACEIT(5,"prepare_output		end, creating Fileshelf failed");
    return -1;
}


//void search_ActionWidget(struct AllInformationAllUseStruct *info){
void search_ActionWidget(struct SearchParams *pSearchStru){
    //creates a widget which holds a simple label, showing the current directory searched
    //Destroy it when searchThread finishes
    TRACEIT(10,"search_ActionWidget		start");
    if (pSearchStru == NULL){
        TRACEIT(2,"search_ActionWidget		end, Parameter NULL");
        return;
    }
    GtkWidget *ActionWidget = NULL;
    GtkWidget *label = NULL;
    GtkButton *CancelButton = NULL;
    GtkWidget *box = NULL;

    int *iDeleteNotification = &(pSearchStru->iDeleteNotification);
    ActionWidget = CreateMyWindow("search", win, 200, 100, TRUE, TRUE, FALSE, iDeleteNotification, FALSE, FALSE);
    label = gtk_label_new("");
    gtk_label_set_line_wrap((GtkLabel *)label, TRUE);
    gtk_misc_set_alignment ( (GtkMisc *)label, 0.0f, 0.0f);
    gtk_misc_set_padding ( (GtkMisc *)label, 4, 0);


    box = gtk_vbox_new(FALSE,0);
    //first put in the button or it can happen that it will change its possition when the directory (which is currently
    //searched for) changed.
    CancelButton = (GtkButton *)gtk_button_new_with_label("Cancel");
	g_signal_connect (CancelButton,"clicked",G_CALLBACK(CancelButtonFunctionForIBreak),&(pSearchStru->iBreak));

    gtk_box_pack_start ( (GtkBox *)box, (GtkWidget *)CancelButton, FALSE, FALSE, 0);
    gtk_box_pack_start ( (GtkBox *)box, label, TRUE, TRUE, 0);

    gtk_container_add(GTK_CONTAINER(ActionWidget),box);

    pSearchStru->actionWidget = (void *)ActionWidget;
    pSearchStru->actionWidgetLabel = (void *)label;

    TRACEIT(10,"search_ActionWidget		end");
}



void searchThread (struct SearchParams *pSearchStru){
    TRACEIT(10,"searchThread		start");

    GString *gstrDirectory = g_string_sized_new (20);
    GString *gstrErrorMessage = g_string_new ("");
    int iItemsFound=0;
	GList *glistBuf = NULL;

    myWrap_gdk_threads_enter("searchThread 1");
    gtk_widget_show_all(pSearchStru->actionWidget);
    myWrap_gdk_threads_leave("searchThread 1");


	glistBuf = g_list_first (pSearchStru->glistDirectories);
	while (glistBuf){
		g_string_assign (gstrDirectory, ((GString *)(glistBuf->data))->str );

		SearchDirectoryFor(gstrDirectory->str, pSearchStru, &iItemsFound, gstrErrorMessage);

		glistBuf = glistBuf->next;
	}

    if (5 < strlen (gstrErrorMessage->str)){
        g_string_prepend (gstrErrorMessage, "There was at least one error while traveling the directories:\n\n");
        myWrap_gdk_threads_enter("searchThread 3");
        MyMessageWidget (win, "error", gstrErrorMessage->str);
        myWrap_gdk_threads_leave("searchThread 3");
    }


    if (pSearchStru->iDeleteNotification == 1){
        myWrap_gdk_threads_enter("searchThread 2");
        gtk_widget_destroy(pSearchStru->actionWidget);
        myWrap_gdk_threads_leave("searchThread 2");
    }

	searchParams_delete (pSearchStru);
	g_string_free (gstrDirectory, TRUE);
	g_string_free (gstrErrorMessage, TRUE);


    TRACEIT(10,"searchThread		end");
    g_thread_exit(0);
}


int SearchDirectoryFor(char *cpDirectory, struct SearchParams *pSearchStru, int *ipFoundItems, GString *gstrErrors){
    //searches the directory recursively for the occurence of cpSearchstring
    //every item inherriting the search-string becomes an item on Shelf
    //if label is not NULL, it will be filled with the name of the directory currently searched in
    //gstrError may be NULL, if not, all errors are written to this gstring
    TRACEIT(10,"SearchDirectoryFor		start");
    if (cpDirectory == NULL || pSearchStru==NULL){
        TRACEIT(2,"SearchDirectoryFor		end, parameter NULL (1)");
        return -1;
    }
    if (ipFoundItems == NULL ){
        TRACEIT(2,"SearchDirectoryFor		end, parameter NULL (2)");
        return -1;
    }


    GString *gstrNames = g_string_sized_new(10);
    GString *gstrPath = g_string_sized_new(10);
	struct myLocationStruct locationStru;

	//Because we want to search within a fileshelf, we can have items in here that are no drectories but files, we have to do some check :
	if (FALSE == checkStringIsPath (cpDirectory)){
		g_string_assign (gstrNames, cpDirectory);
		g_string_assign (gstrPath, cpDirectory);
		GetPathFromPathWithName(gstrPath, 1);
		getLastStringInPath(gstrNames);
		if (TRUE == search_PerformSearchParamsOnItem (gstrNames->str, pSearchStru)){
			(*ipFoundItems)++;
			myWrap_gdk_threads_enter("SearchDirectoryFor 2");
			if (1 == FileShelfObjectList_ValidatObject(pSearchStru->pFileShelfObject)){
				AddItemToFileshelf (pSearchStru->pFileShelfObject, gstrNames->str, gstrPath->str, FALSE, 0);
			}
			myWrap_gdk_threads_leave("SearchDirectoryFor 2");
		}
		g_string_free (gstrNames, TRUE);
		g_string_free (gstrPath, TRUE);
		return 0;
	}


    g_string_assign(gstrPath, cpDirectory);
    if (0 != openLocation(gstrPath->str, &locationStru, FALSE, TRUE)){
        if (gstrErrors){
            g_string_append_printf(gstrErrors, "Error opening directory: %s\n", gstrPath->str);
        }
        g_string_free(gstrNames, TRUE);
        g_string_free(gstrPath, TRUE);
        TRACEIT(10,"SearchDirectoryFor		end");
        return -1;
    }

    if (pSearchStru->actionWidgetLabel){
        g_string_printf(gstrPath, "\nsearching for '%s'\n\nin: %s\n\nfound items: %i", pSearchStru->gstrString->str, cpDirectory, *ipFoundItems);
        myWrap_gdk_threads_enter("SearchDirectoryFor 1");
        gtk_label_set_text ((GtkLabel *)pSearchStru->actionWidgetLabel, gstrPath->str);
        myWrap_gdk_threads_leave("SearchDirectoryFor 1");
    }

    while ( pSearchStru->iBreak == 0 && pSearchStru->iDeleteNotification == 1 && 1==getNextItemInLocation(&locationStru, gstrNames, TRUE, TRUE)){
		if (strcmp(gstrNames->str,".")!=0 && strcmp(gstrNames->str,"..")!=0){
			if (TRUE == search_PerformSearchParamsOnItem (gstrNames->str, pSearchStru)){
                (*ipFoundItems)++;
                myWrap_gdk_threads_enter("SearchDirectoryFor 2");
                if (1 == FileShelfObjectList_ValidatObject(pSearchStru->pFileShelfObject)){
					AddItemToFileshelf (pSearchStru->pFileShelfObject, gstrNames->str, cpDirectory, FALSE, 0);
				}
                myWrap_gdk_threads_leave("SearchDirectoryFor 2");
		    }
			g_string_assign(gstrPath,cpDirectory);
			checkPathForLastChar(gstrPath);
			g_string_append(gstrPath,gstrNames->str);
			if (TRUE == checkStringIsPath(gstrPath->str)){  //recursive
			    if (1 != checkItemIsLink (gstrPath->str)){  //we will follow links
                    SearchDirectoryFor (gstrPath->str, pSearchStru, ipFoundItems, gstrErrors);
			    }
			}
		}
	}

    closeLocation(&locationStru);
    g_string_free(gstrNames, TRUE);
    g_string_free(gstrPath, TRUE);

    TRACEIT(10,"SearchDirectoryFor		end");
    return 0;
}


gboolean search_PerformSearchParamsOnItem (const char *cpItemName, const struct SearchParams *pSearchStru){
	TRACEIT(10,"search_PerformSearchParamsOnItem		end");
	//returns 0 if the item does not match the search params
	//return 1 if the item does match the search params

	if (cpItemName == NULL || pSearchStru == NULL){
		TRACEIT (3, "search_PerformSearchParamsOnItem		Parameter NULL");
		return FALSE;
	}


	if (strcasestr(cpItemName, pSearchStru->gstrString->str) != NULL){	//strcasestr returns charpointer
		TRACEIT(10,"search_PerformSearchParamsOnItem		end");
		return TRUE;
	}

	TRACEIT(10,"search_PerformSearchParamsOnItem		end");
	return FALSE;
}



