/*
    Copyright 2007, 2008, 2009, 2010 Geyer Klaus

    This file is part of Cat'sEyE.

    Cat'sEyE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Cat'sEyE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Cat'sEyE.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef NO_GIO

#include <gtk/gtk.h>
#ifndef NO_GIO
#include <gio/gio.h>
#endif
#include <stdlib.h>
#include <string.h>
#include <unistd.h> 	//e.g. chdir, usleep

#include "helpers.h"
#include "Lists.h"
#include "userdata.h"
#include "myfm.h"
#include "automaticUpdate.h"
#include "mytrace.h"

#include "allInclude.h"


#define AUTOUPDATE_MINTIME 500000 //us
#define MONITORRATELIMIT 5 //ms


struct autoUpdateList *g_listAutoUpdateList;


void autoUpdateList_UpdateThread (struct AllInformationAllUseStruct *info){
    //we will not update emediatly but waiting for AUTOUPDATE_MINTIME useconds before udating.
    //the monitor signal can check for the flag in buflist
    //infostruct:
    //Info_1 = struct autoUpdateList
    //Info_2 = monitor
    TRACEIT(11,"autoUpdateList_UpdateThread	    start");

    if (info == NULL){
        TRACEIT (2, "autoUpdateList_UpdateThread        parameter NULL");
        g_thread_exit(0);
    }

    struct autoUpdateList *buflist = (struct autoUpdateList *)(info->Info_1);
    GFileMonitor *monitor = (GFileMonitor *)info->Info_2;
    free (info);
    info = NULL;

    if (buflist == NULL){
        TRACEIT (2, "autoUpdateList_UpdateThread        parameter 2 NULL");
        g_thread_exit(0);
    }

    usleep (AUTOUPDATE_MINTIME);

    if ( 1 != autoUpdateList_ValidateItem (buflist, monitor)){
        TRACEIT(11,"autoUpdateList_UpdateThread	    end, invalid");
        g_thread_exit(0);
    }

    myWrap_gdk_threads_enter ("autoUpdateList_UpdateThread");
    updateObjectsViewingFolder(buflist->gstrPath->str, TRUE);
    myWrap_gdk_threads_leave ("autoUpdateList_UpdateThread");

    buflist->iUpdateThreadStarted=0;

    TRACEIT(11,"autoUpdateList_UpdateThread	    end");
    g_thread_exit(0);
}


void autoUpdateList_monitoredDirChanged (GFileMonitor *monitor, GFile *file, GFile *other_file, GFileMonitorEvent event_type, struct autoUpdateList *buflist){
    TRACEIT(11,"autoUpdateList_monitoredDirChanged	    start");
    GError *Error=NULL;

    if ( 1 != autoUpdateList_ValidateItem (buflist, monitor)){
        TRACEIT(2,"autoUpdateList_monitoredDirChanged	    invalid list item");
        return;
    }

    if (buflist->iUpdateThreadStarted == 0){
        if ( TRUE == g_mutex_trylock (buflist->mtxJoinUpdateThread)){
            if (buflist->gthreadUpdateThread != NULL){
                g_thread_join (buflist->gthreadUpdateThread);
                buflist->gthreadUpdateThread = NULL;
            }
            g_mutex_unlock (buflist->mtxJoinUpdateThread);
        }
        else {
            TRACEIT(11,"autoUpdateList_monitoredDirChanged	    end 1");
            return;
        }

        struct AllInformationAllUseStruct *info = malloc (sizeof (struct AllInformationAllUseStruct));
        if (info == NULL){
            TRACEIT(2,"autoUpdateList_monitoredDirChanged	    unable to allocate memory");
            return;
        }
        info->Info_1 = (void *)buflist;
        info->Info_2 = (void *)monitor;

        buflist->iUpdateThreadStarted = 1; //this should be close to the thread creating call
        buflist->gthreadUpdateThread = g_thread_create((GThreadFunc)autoUpdateList_UpdateThread, (void *)info , TRUE, &Error);

        if (Error != NULL){
            buflist->iUpdateThreadStarted = 0;
            free (info);
            TRACEIT(2,"autoUpdateList_monitoredDirChanged	    error starting update thread:");
            TRACEIT(2,Error->message);
            g_error_free (Error);
            Error = NULL;
        }
    }

    //printf("monitor: update signal end %s\n",buflist->gstrPath->str);

    TRACEIT(11,"autoUpdateList_monitoredDirChanged	    end");
}


void autoUpdateList_DeleteObject(struct autoUpdateList *Object){
	TRACEIT(10,"autoUpdateList_DeleteObject	    start");
	if (Object == NULL){
        TRACEIT (2, "autoUpdateList_DeleteObject    parameter NULL");
        return;
	}
    GError *Error=NULL;

	if (Object->PrevItem != NULL){
		Object->PrevItem->NextItem = Object->NextItem;
	}
	else{
		g_listAutoUpdateList = Object->NextItem;
		if (g_listAutoUpdateList != NULL){
            g_listAutoUpdateList->PrevItem = NULL;
		}
	}

	if (Object->NextItem != NULL){
		Object->NextItem->PrevItem = Object->PrevItem;
	}
    //printf("autoUpdateList_DeleteObject: going to start delete thread\n");
    //g_thread_create((GThreadFunc)autoUpdateList_deleteItemMembersAndItem, (void *)Object , FALSE, &Error);
    g_thread_create((GThreadFunc)autoUpdateList_deleteItemMembersAndItem_threadWrapper, (void *)Object , FALSE, &Error);

    if (Error != NULL){
        TRACEIT(2,"autoUpdateList_DeleteObject	    error starting delete thread:");
        TRACEIT(2,Error->message);
        g_error_free (Error);
        Error = NULL;
    }
    //why we create this thread:
    //remember the bug where we unpacked some tar file which created a directory which we entered emediatly after it apeared and
    //CatsEyE froze?
    //The error was: while such a directory creation fires 4 monitor-callbacks waiting 800ms (default now its 5ms) between each
    //callback call. The user (we) entered the directory that leads to the deletion of the monitor in some of our UpdateViewWith ...
    //functions. However, this functions are thread save because they are signal driven (remember the mainloop). But the monitor-
    //callback is called in between and the update thread has allready started, the update thread then, tries to gdk_thread_enter
    //but can not because the UpdateView... fucntion is running. But the UpdateView function waits for the monitor list to delete
    //this monitor, and the function to delete this monitor waits for the updateThread to finish. And there it is, the deadlock.

    //TZo solve this, I decided to create anohter thread when deleting a monitor, the monitor will be extracted from the list
    //and the thread will delete the item after the update thread has quit. In this case, the deletefunction will not block
    //if the updatethread is running and the UpdateView... function can continue.

    //But, in this case, the update thread has to validate the monitor item before using it, this is done too.


	TRACEIT(10,"autoUpdateList_DeleteObject	    ends");
	return;
}


void autoUpdateList_deleteItemMembersAndItem_threadWrapper (struct autoUpdateList *item){
    autoUpdateList_deleteItemMembersAndItem (item);
    g_thread_exit(0);
}


void autoUpdateList_deleteItemMembersAndItem (struct autoUpdateList *item){
    //this function will delete (free) all the members of item and after that will delete item itself
    //this function WILL NOT care about the list and linked items, this is the matter of the functions calling this
    //please mind, that this function could be a thread
    TRACEIT(10,"autoUpdateList_deleteItemMembersAndItem		start");
    if (item == NULL){
        TRACEIT(2,"autoUpdateList_deleteItemMembersAndItem		parameter NULL");
        return;
    }

    if (FALSE == g_file_monitor_cancel ( item->monitor )){
        TRACEIT(2,"autoUpdateList_DeleteObject	    monitor could not be cancelled");
    }

    if (item->mtxJoinUpdateThread != NULL){
        g_mutex_lock (item->mtxJoinUpdateThread);
        if (item->gthreadUpdateThread != NULL){
            g_thread_join (item->gthreadUpdateThread);
            item->gthreadUpdateThread = NULL;
        }
        g_mutex_unlock (item->mtxJoinUpdateThread);
    }
    //printf("deleting monitor for: %s\n",item->gstrPath->str);

    g_string_free(item->gstrPath, TRUE);
    g_object_unref(item->monitor);
    g_object_unref(item->gfilePath);

    g_thread_yield (); //this gives CPU time to other threads
    g_mutex_free (item->mtxJoinUpdateThread);
    item->mtxJoinUpdateThread = NULL;
    //printf ("autoUpdateList_deleteItemMembersAndItem: deleting item: %p\n",item);

	free(item);

    char cpBuf[41];
    getcwd (cpBuf, 40);
    //printf("current working directory: %s\n", cpBuf);

    TRACEIT(10,"autoUpdateList_deleteItemMembersAndItem		end");
    return;
}


void autoUpdateList_deleteList (){
    TRACEIT(10,"autoUpdateList_deleteList		start");
    //printf("monitor: deleting LIST\n");

    struct autoUpdateList *ThisObject;
    struct autoUpdateList *ObjectBuf;

    //autoUpdateList_debugOut ();

    ThisObject = autoUpdateList_getFirstItem (g_listAutoUpdateList);
    if (ThisObject == NULL && g_listAutoUpdateList!=NULL){
        TRACEIT (2,"autoUpdateList_deleteList       NoFirstItem?");
        ThisObject = g_listAutoUpdateList;
    }

	while (ThisObject){
	    ObjectBuf = ThisObject->NextItem;

        //delete ThisObject from global list as our updatethread will try to validate its object, therefor
        //we have to make sure that the global variable does not point to an allready freed memory.
        //TODO, think about this again, its very late now.
	    if (ThisObject->PrevItem != NULL){
            ThisObject->PrevItem->NextItem = ThisObject->NextItem;
        }
        else{
            g_listAutoUpdateList = ThisObject->NextItem;
            if (g_listAutoUpdateList != NULL){
                g_listAutoUpdateList->PrevItem = NULL;
            }
        }
        if (ThisObject->NextItem != NULL){
            ThisObject->NextItem->PrevItem = ThisObject->PrevItem;
        }

		autoUpdateList_deleteItemMembersAndItem (ThisObject);
        ThisObject = ObjectBuf;
	}
	g_listAutoUpdateList = NULL;
    //printf("monitor: LIST deleted\n");
    TRACEIT(10,"autoUpdateList_deleteList		end");
}


int autoUpdateList_RemoveMonitor (char *Path){
    TRACEIT(10,"autoUpdateList_RemoveMonitor		start");

    if (g_listAutoUpdateList == NULL){
        TRACEIT(9,"autoUpdateList_RemoveMonitor		end, No Monitor at all");
        return 0;
    }

    if (Path == NULL){
        TRACEIT(2,"autoUpdateList_RemoveMonitor		end, Parameter NULL");
        return -1;
    }
    //printf("monitor: deleting %s\n",Path);
    //search list for path,
    struct autoUpdateList *buflist = autoUpdateList_getFirstItem (g_listAutoUpdateList);
    while (buflist){
        if (strcmp(Path, buflist->gstrPath->str) == 0){
            buflist->iCountRef--;
            //printf("monitor: countRef decreased\n");
            break;
        }
        buflist = buflist->NextItem;
    }
    if (buflist == NULL){
        TRACEIT(4,"autoUpdateList_RemoveMonitor		end, path not found, not monitored");
        return -1;
    }

    //printf("autoUpdateList_RemoveMonitor: buflist->iCountRef= %i\n",buflist->iCountRef);

    if (buflist->iCountRef == 0){
        //printf("monitor: deleting monitor\n");
        autoUpdateList_DeleteObject(buflist);
    }

    TRACEIT(10,"autoUpdateList_RemoveMonitor		end");
    return 0;
}


int autoUpdateList_AddMonitor (char *Path){
    TRACEIT(10,"autoUpdateList_AddMonitor		start");
    if (Path == NULL){
        TRACEIT(2,"autoUpdateList_AddMonitor		end, Parameter NULL");
        return -1;
    }
    //printf("monitor: adding %s\n",Path);

    struct autoUpdateList *buf2list=NULL;
    //search list for path, if we allready monitor it we increase the reference count
    struct autoUpdateList *buflist = g_listAutoUpdateList;

    while (buflist){
        if (strcmp(Path, buflist->gstrPath->str) == 0){
            buflist->iCountRef++;
            //printf("monitor: CountRef increased\n");
            TRACEIT(10,"autoUpdateList_AddMonitor		end");
            return 0;
        }
        buflist = buflist->NextItem;
    }

    //if we dont find the path, start monitoring it
    if (buflist == NULL){
       // printf("monitor: new monitor\n");
        buflist = malloc( sizeof(struct autoUpdateList) );

        //printf ("autoUpdateList_AddMonitor: added item: %p, path: %s\n",buflist, Path);

        if (buflist){
            buflist->iUpdateThreadStarted = 0;
            buflist->gthreadUpdateThread = NULL;
            buflist->mtxJoinUpdateThread = g_mutex_new();

            buflist->gfilePath = g_file_new_for_path (Path);
            buflist->monitor = g_file_monitor_directory (buflist->gfilePath,G_FILE_MONITOR_NONE,NULL,NULL);
            if (buflist->monitor == NULL){
                g_object_unref(buflist->gfilePath);
                free(buflist);
                TRACEIT(2,"autoUpdateList_AddMonitor		end, cant get monitor");
                return -1;
            }
            g_file_monitor_set_rate_limit (buflist->monitor, MONITORRATELIMIT); //default value is 800, but this does not mean, that
            //the monitor skipps any messages but simply waits for X ms before calling the callback function.
            //In a matter of fact, there are several messages for each change fired. This leads to a porblem in
            //our case, and the only solution so far is to set the rate limit to very low value, where the define for
            // our AUTOUPDATE_MINTIME is much bigger than this.

            buflist->iCountRef=1;
            buflist->gstrPath = g_string_new(Path);
            g_signal_connect(buflist->monitor,"changed",G_CALLBACK(autoUpdateList_monitoredDirChanged),buflist);

            //add new item to the list
            buflist->NextItem = NULL;
            buflist->PrevItem = NULL;
            if (NULL == g_listAutoUpdateList){
                g_listAutoUpdateList = buflist;
            }
            else{
                //find the last item
                for (buf2list = g_listAutoUpdateList; buf2list->NextItem; buf2list=buf2list->NextItem );
                //printf("buf2list=%p\tbuf2list->NextItem=%p",buf2list,buf2list->NextItem);

                buf2list->NextItem = buflist;
                buflist->PrevItem = buf2list;
            }
        }
        else{
            TRACEIT(2,"autoUpdateList_AddMonitor		end, malloc failed");
            return -1;
        }
    }

    //printf("monitor: added\n");
    TRACEIT(10,"autoUpdateList_AddMonitor		end");
    return 0;
}


int autoUpdateList_ValidateItem (struct autoUpdateList *item, GFileMonitor *monitor){
    //call this function to validate a given item.
    //monitor is used as an additional indicator as the pointer adress may exist (after it was deleted and newly allocated)
    //but then, the monitor pointer is hopefully different.
    //returns -1=error, 0=invalid, 1=valid
    //monitor can be NULL
    TRACEIT(11,"autoUpdateList_ValidateItem		start");
    if (item == NULL){
        TRACEIT (2, "autoUpdateList_ValidateItem        parameter NULL");
        return -1;
    }
    int iRet = 0;

    struct autoUpdateList *buflist = autoUpdateList_getFirstItem (g_listAutoUpdateList);

    while (buflist){
        if (buflist == item){
            if (monitor != NULL){
                if (monitor == buflist->monitor){
                    iRet = 1;
                }
            }
            else{
                iRet = 1;
            }
        }
        buflist = buflist->NextItem;
    }

    TRACEIT(11,"autoUpdateList_ValidateItem		end");
    return iRet;
}


struct autoUpdateList *autoUpdateList_getFirstItem (struct autoUpdateList *theList){
    TRACEIT(11,"autoUpdateList_getFirstItem		start");
    if (theList == NULL){
        TRACEIT (2, "autoUpdateList_getFirstItem        parameter NULL");
        return NULL;
    }
    while (theList->PrevItem != NULL){
        theList = theList->PrevItem;
    }

    TRACEIT(11,"autoUpdateList_getFirstItem		end");
    return theList;
}


void autoUpdateList_debugOut (){
    struct autoUpdateList *theList = autoUpdateList_getFirstItem (g_listAutoUpdateList);
    int i=0;

    printf("Debug Out Start\n");
    while (theList){
        printf("autoUpdateList [%d]: %p\n",i++, theList);

        theList = theList->NextItem;
    }
    printf("Debug Out End\n");

}


#endif













