/*
    Copyright 2007, 2008, 2009, 2010 Geyer Klaus

    This file is part of Cat'sEyE.

    Cat'sEyE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Cat'sEyE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Cat'sEyE.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef CONFIGHANDLING_H_
#define CONFIGHANDLING_H_



struct myFiles_struct{
    FILE *File;
    //   int iWrite;                 //0=read, 1=write,create, 2=append,create
    int allreadystarted;
    GString *gstrFileName;      //including the path
    GString *gstrIdentifier;
    GString *gstrValue;
};


enum{
    USERCOMMAND_CALLTYPE_ALL=0,
    USERCOMMAND_CALLTYPE_EACH,
    USERCOMMAND_CALLTYPE_LIST
};

enum{
	CONFIGFILE_LOAD_NOTHING=0,
    CONFIGFILE_LOAD_OPTIONS=1,
    CONFIGFILE_LOAD_USERCOMMANDS=2,
    CONFIGFILE_LOAD_BOOKMARKS=4
};

enum{	//-1 = unknown
	USERDIALOG_ENTITY_TYPE_QUESTION=0,
	USERDIALOG_ENTITY_TYPE_CHECKBOX,
	USERDIALOG_ENTITY_TYPE_TEXT,
	USERDIALOG_ENTITY_TYPE_FILECHOOSER,
	USERDIALOG_ENTITY_TYPE_DIRCHOOSER,
	USERDIALOG_ENTITY_TYPE_SEPARATOR
};

//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
//!!!!!!!!!!!!!!!!!!!!!!!!! IMPORTANT!!!!!!!!!!!!!!!!!!
//! after the widgets in this struct were used, make the pointers here NULL. Or they will be unrefed when the list gets deleted
struct _DialogEntityList {
	int iType;				//see enum (e.g. USEDIALOG_ENTITY_TYPE_QUESTION)
	int iValid;				//becomes 1 if the information is complete, so we can create this entity

	GString 		*gstrVarIdentifier;	//Name of the variable (defined by the user)
    GString         *gstrVarValue;      //the value assigned with the varIdentifier (we have to leave all the other values untouched)
                                        //and this is the only value which is written
	//label
	GString			*gstrLabel;		//the text shown in the label (e.g. the questiontext for qestion entity)
	GtkWidget		*wdgtLabel;		//the pointer to the widget which holds the gstrLabel text.

	//entrybox
	GString 		*gstrPredefine; //question's predefine or true or false for checkboxes, means the checkbox is checked or not
	GtkWidget		*wdgtEntryBox;	//text entity has no entrybox.

	//other properties
	GString			*gstrAdditional1;	//for checkbox: TRUE-Value,   file/dirchooser: Text on the button
	GString			*gstrAdditional2;	//for checkbox: FALSE-Value,  file/dirchooser: caption of the chooser dialog
	GtkWidget		*wdgtAdditional1;	//for file/dirchooser: the button widget
	GtkWidget		*wdgtAdditional2;	//for file/dirchooser: the hbox container

	struct _DialogEntityList *nextItem;
	struct _DialogEntityList *prevItem;
};


struct _UserCommandList {
    int iValid;				//becomes 1 if the information is complete
    int iIsDefaultCommand; 	//for double click or enter button, 0=nothing special here, 1=its the default command
    GString *gstrName;      //the name as viewed in contextmenu
    GList *glistTargets;    //all the Targets (endings), this command belongs to
    GString *gstrCommand;   //the command as in configfile
    struct _DialogEntityList *DialogEntityList; //all infos about the entities in the dialog (question, checkbox, ....)
	GString *gstrDialogCaption;
	GString *gstrDialogLabel;
	GString *gstrWorkingDir;    //used for StandardTerminalkey

    //the options are not stored in a seperate structure as they are not dynamic we can store the information right here
    int option_iCallType;    //the call type (all/each/list) see enum (USERCOMMAND_CALLTYPE_ALL)
    GString *option_gstrQuitreply;
    GString *option_gstrActionnote;
    GString *option_gstrCaption;    //the caption for the list widget
    GString *option_gstrLabel;      //the label fot the list widget

    struct _UserCommandList *nextItem;
    struct _UserCommandList *prevItem;
};


struct _OptionsList {   //list for storing options such things like standardTerminal, startpathes, colors and all that
    GString *gstrID;        //the identifier (for options: e.g. strdld)
    GString *gstrName;
    GString *gstrValue;   //the only items which can have more than one value are the standardstartpathesLeft/Right properties.
                            //but this porperties simply are added more than ones to this list, having the same name but different
                            //values.

    struct _OptionsList *nextItem;
    struct _OptionsList *prevItem;
};


struct _Items_XML_Relation {  //stores in whitch mode we are when parsing the xml file
										//Bit/value		explanation

	gboolean bUnKnownBlock;				//0 /1
	gboolean bCatsEyEBlock;             //1 /2		we are in <CatsEyE> - Block

	gboolean bCatsEyEBookMarksBlock;	//2	/4

	gboolean bCatsEyEOptionsBlock;      //3	/8		we are in <CatsEyEOptions> - Block
	gboolean bCatsEyEUserCommandsBlock; //4 /16
    gboolean bUserCommandBlock;			//5	/32		we are in <Usercommand>-Block (within <CatsEyEUsercommands>)
    gboolean bObjectBlock;				//6 /64
    gboolean bOptionBlock;				//7	/128		we are in <Option> - Block (within <CatsEyEOptions>-Block)

	gboolean bIDBlock;					//8 /256
    gboolean bNameBlock;				//9 /512
    gboolean bValueBlock;				//10 /1024

    int	iRelation;						//all the bools are bitwise accumulated and stored as value here, call accumulateItemXMLRelation
};


struct _Parser_Mode_Struct {    //to communicate between different xml parser functions.
    void 		*storageList;      //becomes _OptionsList or _UserCommandList pointer
    void 		*currentListItem;  //becomes _OptionsList or _UserCommandList pointer, holds the item currently in use which is then stored in the storageList
    struct _OptionsList *currentOptionBlock; //needed for the usercommand section to cache the <Option> Block, therefor holds a pointer to _OptionsList
    struct _Items_XML_Relation *relation;
    int			iCurrentListType;	//stores from wich type the storageList is (see enum, CONFIGFILE_LOAD_OPTIONS, CONFIGFILE_LOAD_USERCOMMANDS, CONFIGFILE_LOAD_BOOKMARKS)
    gboolean 	bMakeItThreadSave;
    const char **cppAttributeValues;	//used in UserCommand part, where the <Object> Block has a type attribute to identify the object e.g. as question
    const char **cppAttributeNames;		//used in UserCommand part, where the <Object> Block has a type attribute to identify the object e.g. as question

    void (*functionStartNotification)();	//stores a pointer to a function which is called from the XMLParser_foundStart
    void (*functionStopNotification)();	    //stores a pointer to a function which is called from the XMLParser_foundEnd
											//the functions must be declared as follows:
											//void *startFunction (struct _Parser_Mode_Struct *info);
											//void *endFuntion (struct _Parser_Mode_Struct *info, const char *element_name);
	void (*functionTextNotification)();	    //stores a pointer to a function which is called from XMLParser_foundText;
											//the functions must be declared as follows:
											//void *startFunction (struct _Parser_Mode_Struct *info, const char *text);

};



/////////////////////////////////
///// declerations /////////////

//Main access
void                    	configFileHandling_Init();
int                     	configFileHandling_loadConfigFile(gboolean bIgnoreTimeStamp, gboolean bMakeItThreadSave);
int                         configFileHandling_saveGlobalStructsAsXMLConfigFile (const char *ccpFileName, gboolean bMakeThreadSave);
void                    	configFileHandling_clearForShutdown();
int 						configFileHandling_defineConfigFileName(const char *ccpFileName);
const char              *   configFileHandling_getConfigFileName ();
void                        configFileHandling_setPreventReloadFlag (int iValue);
void                        configFileHandling_validateUsercommandList(gboolean bMakeItThreadSave);
void 					debugOut_PrintLists();


// dialog List
int 						DialogEntityList_addItem(struct _DialogEntityList **theList, int iType);	//if iType == -1, its unknown
int 						DialogEntityList_freeList(struct _DialogEntityList **theList);
int 						DialogEntityList_editItem(struct _DialogEntityList *theItem, int iType, char *cpVarID, char *cpLabel, GtkWidget *wdgtLabel, char *cpPredefine, GtkWidget *wdgtEntryBox, char *cpAdd1, char *cpAdd2, GtkWidget *wdgtAdd1 );		//changes the content of the given item
int 						DialogEntityList_editLastItem(struct _DialogEntityList *theList, int iType, char *cpVarID, char *cpLabel, GtkWidget *wdgtLabel, char *cpPredefine, GtkWidget *wdgtEntryBox, char *cpAdd1, char *cpAdd2, GtkWidget *wdgtAdd1 );		//changes the content of the given item);	//searches the last item in the list and changes its content
struct _DialogEntityList *	DialogEntityList_getLastItem(struct _DialogEntityList *theList);
struct _DialogEntityList *	DialogEntityList_getFirstItem(struct _DialogEntityList *theList);
void 				DialogEntityList_DebugOut_printList(struct _DialogEntityList *theList);
//int 						DialogEntityList_validateList(struct _DialogEntityList *theList);	//checks the content of all items and sets iValid member


//UserCommandList
int							UserCommandList_addItem(struct _UserCommandList **theList, const char *ccpSource);
struct _UserCommandList   *	UserCommandList_deleteItem(struct _UserCommandList *theItem);
int 						UserCommandList_clearTargets(struct _UserCommandList *theItem);
int 						UserCommandList_freeList(struct _UserCommandList **theList); //has to free _DialogEntityList and glistTargets too.
int 						UserCommandList_editItem(struct _UserCommandList *theItem, int iIsDefaultCommand, const char *cpName, const char *cpTarget, const char *cpCommand, int iCallType, const char *cpOptionQuitreply, const char *cpOptionActionnote, const char *cpOptionCaption, const char *cpOptionLabel, const char *cpDialogCaption, const char *cpDialogLabel, const char *cpWorkingDir); //the target amy be added to the GList
int 						UserCommandList_deleteItemElement(struct _UserCommandList *theItem, int iName, int iTarget, int iCommand, int iOptionQuitreply, int iOptionActionnote, int iOptionCaption, int iOptionLabel, int iDialogCaption, int iDialogLabel, int iWorkingDir); //0=do nothing, 1=delete this element
int 						UserCommandList_addItemAsCopy(struct _UserCommandList **theList, struct _UserCommandList *theNewItem);	//simply adds theNewItem to theList, this will preserve us to copy the whole content of hte list (which holds 2 lists by its own!)
struct _UserCommandList *	UserCommandList_getLastItem (struct _UserCommandList *theList);
struct _UserCommandList *	UserCommandList_getFirstItem (struct _UserCommandList *theList);
GList *						UserCommandList_getCommandsForTarget (struct _UserCommandList *theWholeList, char *cpTarget);
gboolean 					UserCommandList_ItemHasTarget (struct _UserCommandList *theItem, char *cpTarget);
int 						UserCommandList_copyItemToList (struct _UserCommandList **theList, struct _UserCommandList *NewItem); //really copies!
void  				UserCommandList_DebugOut_printList(struct _UserCommandList *theList);


//_OptionsList related functions
int                     	OptionsList_addItem(struct _OptionsList **OptionsList, const char*cpID, const char *cpName, const char *cpValue);
int                     	OptionsList_editItem(struct _OptionsList *OptionsList, const char*cpID, const char *cpName, const char *cpValue);
struct _OptionsList     *	OptionsList_getFirstItem(struct _OptionsList *thelist);
struct _OptionsList     *	OptionsList_getLastItem(struct _OptionsList *thelist);
struct _OptionsList     *	OptionsList_deleteItem(struct _OptionsList *theItem);
void                    	OptionsList_freeList(struct _OptionsList **thelist);
struct _OptionsList 	*	OptionsList_getItemByName(struct _OptionsList *OptionsList, const char *cpName);
struct _OptionsList 	*	OptionsList_getItemByID(struct _OptionsList *OptionsList, const char *cpName);
const char              *   OptionsList_getValueByID (struct _OptionsList *OptionsList, const char *cpID);
int							OptionsList_GetIntValueByID (struct _OptionsList *OptionsList, const char *cpID, int iDefault);
const char              *   OptionsList_getNameByID (struct _OptionsList *OptionsList, const char *cpID);
const char              *   OptionsList_getValueAndNextItemByID (struct _OptionsList *StartItem, struct _OptionsList **NextItem, const char *cpID);

//gboolean                	OptionsList_checkForID(struct _OptionsList *OptionsList, const char *cpID);
void                   OptionsList_DebugOut_printList(struct _OptionsList *theList);


//for all
void						freeRelicts(struct _Parser_Mode_Struct *info);	//deletes currentItems in this struct if necessary

//for Options and Bookmarks
int                     	Options_loadOptionsFromFile(struct myFiles_struct *myFilestru, gboolean bMakeItThreadSave);
int                     	Options_CreateDefaultList ();
void                    	Options_FoundStartCallback(struct _Parser_Mode_Struct *info);
void                    	Options_FoundEndCallback(struct _Parser_Mode_Struct *info, const char *element_name);
void                    	Options_FoundTextCallback(struct _Parser_Mode_Struct *info, const char *text);



//_Itms_XML_Relation functions
int 						XMLRelation_updateByNewElement(struct _Items_XML_Relation *relation, const char *cpElement);
int 						XMLRelation_accumulateRelations(struct _Items_XML_Relation *relation);
int 						XMLRelation_eraseAll(struct _Items_XML_Relation *relation);
struct _OptionsList		*	XMLParser_getValidKeyWordsForState(struct _Items_XML_Relation *relation);


//XML Parser functions
int 						XMLParser_ParseFileFile (struct myFiles_struct *myFilestru, struct _Parser_Mode_Struct *modeStruct, gboolean bMakeItThreadSave);
void 						XMLParser_foundStart (GMarkupParseContext *context, const gchar *element_name, const gchar **attribute_names, const gchar **attribute_values, gpointer data, GError **error);
void 						XMLParser_foundEnd (GMarkupParseContext *context, const gchar *element_name, gpointer data, GError **error);
void 						XMLParser_foundText (GMarkupParseContext *context, const gchar *text, gsize text_len, gpointer user_data, GError **error);



////////////////////////////// XML PROCEDURE ENDS HERE ///////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////


//file only
struct myFiles_struct *myFiles_openFile(char *FileName, char *cpMode);
int myFiles_closeFile(struct myFiles_struct *fileInfo);
int myFiles_SaveValueToFile(struct myFiles_struct *fileInfo, char *cpLine);
int myFiles_findStringInFile(struct myFiles_struct *fileInfo, char *cpString);
//common
int myFiles_getValueFromIdentifier(struct myFiles_struct *fileInfo, char *cpIdentifier, int again);
int myFiles_SeparateMultiValue(struct myFiles_struct *fileInfo, GString *name, GString *command, int ModusSingleOrPair);
//configfile
//struct myFiles_struct *myFiles_openConfigTypesFile(char *cpMode);
//int myFiles_ConfigFileSimpleGetValue(char *Identifier, GString *Value);
//savefile
struct myFiles_struct *myFiles_openSaveFile( char *cpMode);
int myFiles_SaveFileSimpleSave(char *cpLine, char *Mode);
int myFiles_SaveFileSimpleGetValue(char *Identifier, GString *Value);


#endif //CONFIGHANDLING_H_













