/*
    Copyright 2007, 2008, 2009, 2010 Geyer Klaus

    This file is part of Cat'sEyE.

    Cat'sEyE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Cat'sEyE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Cat'sEyE.  If not, see <http://www.gnu.org/licenses/>.
*/



#include <gtk/gtk.h>
#include <glib.h>
#include <glib/gstdio.h>

#include <sys/types.h>
#include <sys/stat.h>

#include "confighandling.h"
#include "confighandlingDefinitions.h"
#include "globalDefinitions.h"
#include "helpers.h"
#include "internals.h"
#include "mytrace.h"

#include "allInclude.h"

#define ABOUTTEXT "\n\n    Copyright 2007, 2008, 2009, 2010 Geyer Klaus\n\
\n\
    Cat'sEyE is free software: you can redistribute it and/or modify\n\
    it under the terms of the GNU General Public License as published by    \n\
    the Free Software Foundation, either version 3 of the License, or\n\
    (at your option) any later version.\n\
\n\
    Cat'sEyE is distributed in the hope that it will be useful,\n\
    but WITHOUT ANY WARRANTY; without even the implied warranty of\n\
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n\
    GNU General Public License for more details.\n\
\n\
    You should have received a copy of the GNU General Public License\n\
    along with Cat'sEyE.  If not, see <http://www.gnu.org/licenses/>.\n\n"


extern GtkWindow 	*win;
extern GtkTextBuffer *g_TextBuffer_CommandHistory;

void internalMenuAbout(){
	TRACEIT(10,"internalMenuAbout		start");
	GtkWidget *dialog, *label1, *label2, *sep;
	GString *lbl1=g_string_sized_new(20);
	GString *gstrBuf=g_string_sized_new(20);
	sep = gtk_hseparator_new ();
	dialog = CreateMyDialog("About", NULL, 1, "close", GTK_RESPONSE_ACCEPT, NULL, 0, NULL, 0, FALSE);
	AddMyIconsToWindow((GtkWindow *)dialog);


    g_string_printf(lbl1,"\nThis is %s, a filebrowser using the GTK.\n\
        \nVersion: %s , %d\tbuild: %s,%s\n\n",MYNAME,VERSION, INTERNALBUILDNUMBER, __DATE__,__TIME__);

    g_string_printf(gstrBuf,"Tracelevel is set to %d, to change this\
        \nrebuild with '-D ITTRACE=<Num>' \nwhere <Num> is the tracelevel you want.\n",
    #ifdef ITTRACEinternalMenuCommandHistory_ShowTextThread
        ITTRACE);
    #else
        0);
    #endif
    g_string_append(lbl1, gstrBuf->str);

    #ifdef NO_GIO
    g_string_printf(gstrBuf, "NO_GIO option was enabled at compile time, no function\n\
        from the gio framework is used. Please mind, that the user has to update the \n\
        views manually and the copy- move- and remove-functions\n\
        are those used with the shell.\n");
    g_string_append(lbl1, gstrBuf->str);
    #endif

    g_string_printf(gstrBuf,"\nGTK version linked to: %d.%d.%d\
        \nGTK version of headers used to compile: %d.%d.%d\n\
        \nglib version linked to: %d.%d.%d\
        \nglib version of headers used to compile: %d.%d.%d\n",
        gtk_major_version,gtk_minor_version,gtk_micro_version,
        GTK_MAJOR_VERSION,GTK_MINOR_VERSION,GTK_MICRO_VERSION,
        glib_major_version,glib_minor_version,glib_micro_version,
        GLIB_MAJOR_VERSION,GLIB_MINOR_VERSION,GLIB_MICRO_VERSION);
    g_string_append(lbl1, gstrBuf->str);


	label1 = gtk_label_new(lbl1->str);
	label2 = gtk_label_new(ABOUTTEXT);

	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox),label1);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox),sep);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox),label2);
	gtk_widget_show_all(dialog);
	gtk_dialog_run(GTK_DIALOG(dialog));

	gtk_widget_destroy(dialog);
	g_string_free(lbl1,TRUE);
	g_string_free(gstrBuf, TRUE);
	TRACEIT(10,"internalMenuAbout		end");
}


void internalMenuCommandHistory_ShowTextThread (void *data){
    TRACEIT(10,"internalMenuCommandHistory_ShowTextThread		start");
    //myWrap_gdk_threads_enter("internalMenuCommandHistory_ShowTextThread");
    GtkWidget *label;
    GtkVBox *VBox;
    GtkWidget *TextView = gtk_text_view_new_with_buffer (g_TextBuffer_CommandHistory);
    VBox = g_object_new(GTK_TYPE_VBOX, "spacing", 5, "homogeneous", FALSE, NULL);
    GtkWidget *scrollwidget;

    label = gtk_label_new("This is the history of all commands Cat'sEyE sent to the shell.\nThis includes copy, move, remove, rename and user commands.");
    gtk_misc_set_alignment ( (GtkMisc *)label, 0.0f, 0.0f);
    gtk_misc_set_padding ( (GtkMisc *)label, 4, 0);
    gtk_label_set_line_wrap ((GtkLabel *)label, TRUE);

    scrollwidget = gtk_scrolled_window_new(NULL,NULL);

    GtkWidget *dialog = CreateMyWindow("Command history", win, 400, 300, TRUE, TRUE, FALSE, NULL, FALSE, TRUE );
    AddMyIconsToWindow((GtkWindow *)dialog);

    gtk_scrolled_window_add_with_viewport ( (GtkScrolledWindow *)scrollwidget, TextView);
    gtk_container_add((GtkContainer *)VBox,(GtkWidget *)label);
    gtk_box_set_child_packing ((GtkBox *)VBox, (GtkWidget *)label, FALSE, TRUE, 4, GTK_PACK_START);
    gtk_container_add(GTK_CONTAINER(VBox),scrollwidget);
    gtk_container_add((GtkContainer *)dialog,(GtkWidget *)VBox);

	gtk_widget_show_all(dialog);

	//myWrap_gdk_threads_leave("internalMenuCommandHistory_ShowTextThread");

	TRACEIT(10,"internalMenuCommandHistory_ShowTextThread		ends");
	//g_thread_exit(0);
}


void internalMenuCommandHistory (){
    TRACEIT(10,"internalMenuCommandHistory		start");
    if (g_TextBuffer_CommandHistory == NULL)
        return;

    //if( !g_thread_supported() ){
	//	g_thread_init(NULL);
     		//gdk_threads_init();                   // Called to initialize internal mutex "gdk_threads_mutex".
	//}
    //g_thread_create((GThreadFunc)internalMenuCommandHistory_ShowTextThread, NULL , FALSE, NULL);
    internalMenuCommandHistory_ShowTextThread (NULL);
    TRACEIT(10,"internalMenuCommandHistory		ends");
    return;
}
