/*
    Copyright 2007, 2008, 2009, 2010 Geyer Klaus

    This file is part of Cat'sEyE.

    Cat'sEyE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Cat'sEyE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Cat'sEyE.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdio.h>
#include <stdlib.h>	//for system();
#include <gtk/gtk.h>
#ifndef NO_GIO
#include <gio/gio.h>
#endif
#include <sys/types.h>  //Diese hier auch fr die Verzeichnisfunktionen und Attribute / and fork()
#include <sys/stat.h>	//Diese hier auch fr die Verzeichnisfunktionen und Attribute
#include <dirent.h>	//Verzeichnis Zeiger und DIR - Struktur
#include <string.h>
#include <time.h>
#include <unistd.h> 	//e.g. chdir, usleep
#include <errno.h>
#include <glib.h>

#include "Lists.h"
#include "listitems.h"
#include "mytrace.h"
#include "helpers.h"
#include "globalDefinitions.h"
#include "userdata.h"
#include "myfm.h"

#include "allInclude.h"

extern GtkWindow 	*win;

int makeDirectory(char *Directory, int iShowErrorMessage, GString *gstrErrorStorage){
    //if iShowErrorMessage == 1 then a message widget may pop up, showing some error, otherwise no message widget pops up
    //if gstrErrorStorage is != NULL the error message is written to it.
    //return: -1 internal error
    //return: 0 OK
    //return: 1 Error Message
    TRACEIT(10,"makeDirectory		start");

    int ret=-1;
    if (Directory == NULL){
		TRACEIT(2,"makeDirectory		ends, Parameter NULL");
		return -1;
	}
    GError *Error = NULL;
    GFile *Sourcefile = g_file_new_for_path(Directory);
    GString *gstrText=g_string_sized_new(20);
    if (FALSE == g_file_make_directory (Sourcefile, NULL, &Error) ){
        if (gstrErrorStorage){
            g_string_append(gstrErrorStorage, Error->message);
        }
        MyMessageWidget(win, "Error while creating Directory", Error->message);
        g_string_printf(gstrText, "error created Directory: \"%s\":%s",Directory, Error->message);
        textbuffer_commandHistory_addText(gstrText->str,"error");
        ret = 1;
    }
    else{
        g_string_printf(gstrText, "created Directory: %s",Directory);
        textbuffer_commandHistory_addText(gstrText->str,NULL);
        ret = 0;
    }

    if (Error)
        g_error_free(Error);
    if (Sourcefile)
        g_object_unref(Sourcefile);
    if (gstrText)
        g_string_free(gstrText, TRUE);
    Sourcefile = NULL;

    TRACEIT(10,"makeDirectory		ends");
	return ret;
}

#define UPDATETIMESINGLEITEM 100000
#define UPDATETIMETOTALBYTES 500000

void  copyItemRecursive_Callback (goffset current_num_bytes, goffset total_num_bytes, gpointer data){
    TRACEIT(11,"copyItemRecursive_Callback       start");
    if (data == NULL){
        TRACEIT(2,"copyItemRecursive_Callback       end, userdate NULL");
        return;
    }
    //we have to use list, NOT commstru->currentItem, as the currentItem is set in another thread and may
    //not be the same as the listitem aimed by this function!!
    struct CopyReMoveList *list = (struct CopyReMoveList *)data;
    GString *gstrBuf=g_string_sized_new(10);
    GString *gstrSpeed=g_string_sized_new(10);
    GString *gstrBuf2=g_string_sized_new(10);
    double percentage=0.0;
    double totalPercentage=0.0;
    double myTime;
    int iUpdateView = 0;
    int iUpdateViewSingleItem = 0;

    GTimeVal *startTime;
    GTimeVal nextTime;


    //when finished we come up here twice, this leads to an percentage of 200%
    //when you dont understand how this works, please mind, that copying a directory
    //may consist of many files, for them we have to the leading if structure.
    //every time one item finished, we add the total amount to our list variable
//list->ui64pTotalBytesCopied

    //Calc Percentage of the Item and write it to the list
    startTime = list->timepStartTimeForBytesCopiedSingleItem;

    if (list->iBytesCopied == -1){
        list->iBytesCopied = 0;
        g_get_current_time (startTime);
        g_time_val_add(startTime, UPDATETIMESINGLEITEM);
        iUpdateViewSingleItem = 1;
    }

    if (current_num_bytes == total_num_bytes){
        if (list->lastfinishedTotalBytes != total_num_bytes){
            list->iBytesCopied += total_num_bytes;
            list->lastfinishedTotalBytes = total_num_bytes;

            *(list->ui64pTotalBytesCopied) += total_num_bytes;
        }
        else{
            list->lastfinishedTotalBytes = -1;
        }
        current_num_bytes = 0;
        iUpdateViewSingleItem = 1;
    }
    if (iUpdateViewSingleItem == 0){
        g_get_current_time (&nextTime);
        myTime = GetTimeMine(*startTime) - GetTimeMine(nextTime);
        if ( myTime <= 0){
            g_get_current_time (startTime);
            g_time_val_add(startTime, UPDATETIMESINGLEITEM);
            iUpdateViewSingleItem = 1;
        }
    }

    if (iUpdateViewSingleItem == 1){
        if (list->iBytesToCopyIsValid == 1){
            percentage = 100*(double)(list->iBytesCopied+current_num_bytes)/(double)list->iBytesToCopy;
            g_string_printf(gstrBuf, "%0.2f", percentage);
        }
        else{
            g_string_assign(gstrBuf, "na");
        }

        myWrap_gdk_threads_enter("copyItemRecursive_Callback 1");
        gtk_list_store_set (list->listStore, &(list->iter), 1, gstrBuf->str,-1);
        myWrap_gdk_threads_leave("copyItemRecursive_Callback 1");
    }

    //TotalPercentag and bytes/Sec
    startTime = list->timepStartTimeForBytesCopied;

    //totalPercentage = (double)(*(list->ui64pTotalBytesCopied)+current_num_bytes) / *(list->ui64TotalBytes);
    //totalPercentage = (totalPercentage>1.0)?1.0:totalPercentage;
    if (*(list->i64pStartTotalBytesCopied)<0){    //start messuring
        *(list->i64pStartTotalBytesCopied) = *(list->ui64pTotalBytesCopied)+current_num_bytes;
        g_get_current_time (startTime);
        g_time_val_add(startTime, UPDATETIMETOTALBYTES);
    }
    g_get_current_time (&nextTime);
    myTime = GetTimeMine(*startTime) - GetTimeMine(nextTime);
    if ( myTime <= 0){
        *(list->i64pStartTotalBytesCopied) = *(list->ui64pTotalBytesCopied)+current_num_bytes - *(list->i64pStartTotalBytesCopied);
        myTime = 0.5 - myTime;
        *(list->dpCopySpeed) = *(list->i64pStartTotalBytesCopied) / myTime;

        *(list->i64pStartTotalBytesCopied)=-1.0;
        iUpdateView = 1;
    }
    if (*(list->iTotalBytesValid)==1){
        BytesToHumanityString ((guint64)*(list->dpCopySpeed), gstrSpeed, 1);
        g_string_append(gstrSpeed, "/s");

        if (totalPercentage>1.0)
            totalPercentage = 1.0;
        else if (totalPercentage<0.0)
            totalPercentage = 0.0;
    }
    else{
        g_string_assign(gstrBuf, "in progress ...");
    }


    if (iUpdateView == 1){
        totalPercentage = (double)(*(list->ui64pTotalBytesCopied)+current_num_bytes) / *(list->ui64TotalBytes);
        totalPercentage = (totalPercentage>1.0)?1.0:totalPercentage;


        BytesToHumanityString ((guint64)*(list->ui64TotalBytes), gstrBuf2, 1);

        g_string_printf(gstrBuf, "total: %0.0f%%, at %s (%s)", totalPercentage*100, gstrSpeed->str, gstrBuf2->str);

        myWrap_gdk_threads_enter("copyItemRecursive_Callback 2");
        gtk_progress_bar_set_fraction (list->progressBar, totalPercentage);
        gtk_progress_bar_set_text(list->progressBar, gstrBuf->str);
        myWrap_gdk_threads_leave("copyItemRecursive_Callback 2");
    }

    g_string_free(gstrBuf, TRUE);
    g_string_free(gstrSpeed, TRUE);
    g_string_free(gstrBuf2, TRUE);
    TRACEIT(11,"copyItemRecursive_Callback       end");
    return;
}


int copyReMoveItemRecursive(const char *cpSource, const char *cpDest, struct ListWidgetCommunicationStruct *commstru){
    //return: -1:error, 1=success, 0=copy returned false or any other error, copying failed
    //cpDest is supposed to be a path
    //while cpSource may be a path or filename
    //cpDest may be NULL (for deleting)
    TRACEIT(10,"copyReMoveItemRecursive		start");
    if ( commstru==NULL || cpSource==NULL || (cpDest==NULL && commstru->currentItem->CopyMoveOrRemove != 3) ){
        TRACEIT(2,"copyReMoveItemRecursive		end, Parameter NULL");
        return -1;
    }

/*
#ifdef NO_GIO
    int iRetErrorCode=0;    //0=error, 1=0k, -1 internal error, thsi is the value returned by this function
    GString *gstrCommandString = g_string_sized_new(20);
    GString *gstrSource = g_string_sized_new(20);
    GString *gstrDest = g_string_sized_new(20);
    gchar *error=NULL;
    int iExitStatus;
    if (commstru->currentItem->CopyMoveOrRemove == 1){  //copy
        g_string_printf(gstrCommandString, "%s \"%s\" \"%s\"", g_gstrCommandCopy->str, cpSource, cpDest);
    }
    else if (commstru->currentItem->CopyMoveOrRemove == 2){ //move
        g_string_printf(gstrCommandString, "%s \"%s\" \"%s\"", g_gstrCommandMove->str, cpSource, cpDest);
    }
    else if (commstru->currentItem->CopyMoveOrRemove == 3){ //remove
        g_string_printf(gstrCommandString, "%s \"%s\"", g_gstrCommandRemove->str, cpSource);
    }

    myWrapOf__g_spawn_command_line_sync (gstrCommandString->str, NULL, &error, &iExitStatus, NULL, TRUE);

    if (iExitStatus != 0){
        if (error != NULL){
            g_string_assign(commstru->currentItem->ErrorMessage, error);
            g_free(error);
            error=NULL;
        }
        iRetErrorCode=0;    //error
    }
    else{
        iRetErrorCode=1;    //OK
    }

    myWrap_gdk_threads_enter("copyReMoveItemRecursive 1");
    if (commstru->currentItem->CopyMoveOrRemove == 1){  //copy
        g_string_assign(gstrDest, cpDest);
        updateObjectsViewingFolder(gstrDest->str, TRUE);
    }
    else if (commstru->currentItem->CopyMoveOrRemove == 2){ //move
        g_string_assign(gstrSource, cpSource);
        GetPathFromPathWithName(gstrSource, 1);
        updateObjectsViewingFolder(gstrSource->str, TRUE);
        g_string_assign(gstrDest, cpDest);
        updateObjectsViewingFolder(gstrDest->str, TRUE);
    }
    else if (commstru->currentItem->CopyMoveOrRemove == 3){ //remove
        g_string_assign(gstrSource, cpSource);
        GetPathFromPathWithName(gstrSource, 1);
        updateObjectsViewingFolder(gstrSource->str, TRUE);
    }
    myWrap_gdk_threads_leave("copyReMoveItemRecursive 1");
    g_string_free(gstrCommandString, TRUE);
    g_string_free(gstrSource, TRUE);
    g_string_free(gstrDest, TRUE);
    TRACEIT(10,"copyReMoveItemRecursive		end");
    return iRetErrorCode;
#endif //#ifdef NO_GIO
*/
#ifndef NO_GIO

    gboolean iDoWeDelete = (commstru->currentItem->CopyMoveOrRemove == 3)?TRUE:FALSE;
    struct CopyReMoveList *currentItem = commstru->currentItem;

    GString *gstrSource=g_string_sized_new(20);
    if (!iDoWeDelete)
        g_string_assign(gstrSource,cpDest);

    //GString *gstrDest=g_string_new(cpSource);
    GString *gstrDest=g_string_sized_new(20);
    g_string_assign(gstrDest, cpSource);
    GString *gstrNames=g_string_sized_new(10);
    gboolean bRet=FALSE;

    GFile *Sourcefile = NULL;
    GFile *Destfile = NULL;

    struct myLocationStruct locationStru;

    int iRetRecursivItem=0;
    int iRetFolder=0;
    int iRetErrorCode=0;    //0=error, 1=0k, -1 internal error, thsi is the value returned by this function
    int iRecItemCount=0;
    int iOverwriteRet=0;
    gboolean bCanceled = FALSE;

    bCanceled =  g_cancellable_is_cancelled (commstru->cancellable);

    //build the complete Name of the destination:
    //get the name of the file:
    if (bCanceled == FALSE && currentItem->status==0){
        if (!iDoWeDelete){
            getLastStringInPath(gstrDest);
            checkPathForLastChar(gstrSource);   //gstrSource is filled with cpDest
            g_string_prepend(gstrDest,gstrSource->str); //now we have the complete destination
        }
        g_string_assign(gstrSource, cpSource);  //now the source is valid too.
        //printf("comming in: Source=%s\n",gstrSource->str);

        //check existance of sourceitem, and ask for permission for overwrite
        if (!iDoWeDelete && TRUE == g_file_test( gstrDest->str, G_FILE_TEST_EXISTS)){  //Ask for permission to overwirte existing file
            if (commstru->bOverWriteAllExisting==-1 && currentItem->bOverWriteExisting==FALSE){
                myWrap_gdk_threads_enter("copyReMoveItemRecursive 2");
                iOverwriteRet = MyEasyInputWidget_Two ((GtkWindow *)commstru->mainListWidget, "overwrite", "There is allready an item with the\nsame name in the destination directory.\nDo you want to overwrite it?\n", NULL, NULL, NULL, "Overwrite", "Overwrite all", "No to all", "No");
                myWrap_gdk_threads_leave("copyReMoveItemRecursive 2");
				if (iOverwriteRet == 0){  //Overwrite this item
				    //printf("overwrite item\n");
					currentItem->bOverWriteExisting = TRUE;
				}
				else if (iOverwriteRet == 1){ //Overwirte all items
				    //printf("overwrite ALL item\n");
                    currentItem->bOverWriteExisting = TRUE;
                    commstru->bOverWriteAllExisting = 1;
				}
				else if (iOverwriteRet == 2){ //No to all, this means we may cancel the whole list.
				    //printf("NO to ALL item\n");
				    commstru->bOverWriteAllExisting = 0;
				    currentItem->status=-10;
                    //commstru->CancelListFunction(commstru);
				}
				else if (iOverwriteRet == 3){ //No to this item, we cancel this item
				    //printf("No to this item\n");
                    currentItem->status=-10;    //mark it as cancelled
				}
            }
            else if (commstru->bOverWriteAllExisting==0){//-1:not asked, 1=user said yes to all, 0=user said no to all
                 currentItem->status=-10;    //mark it as cancelled
            }
        }

        if (currentItem->status == 0){     //item in process, this means this function
            //do not follow links, event if they are pointing to a directory
            if (TRUE == checkStringIsPath( gstrSource->str ) && FALSE == g_file_test( gstrSource->str, G_FILE_TEST_IS_SYMLINK)){ //open directory and give one item after each other recursive to this function
                //printf("Identified as path\n");
                //printf("found dir:%s\n",gstrSource->str);
                if (!iDoWeDelete){
                    if (TRUE == g_file_test( gstrDest->str, G_FILE_TEST_EXISTS)){
                        if (TRUE == checkStringIsPath( gstrDest->str )){ //check if the directory allready exists
                            iRetFolder = 0;    //folder exists, we do not need to build it.
                        }
                        else{
                            iRetFolder = -1; //it seems there is all ready a file with this name, prefents us creating the directory
                        }
                    }
                    else{   //we buld the directory
                        //printf("create Dir: %s\n",gstrDest->str);
                        //gdk_threads_enter();
                        //iRetFolder = makeDirectory(gstrDest->str,0,gstrNames); //create the other directory, stores error message in gstrNames
                        //return 0=ok, 1=error , -1=internal error
                        //myWrap_gdk_threads_leave();
                        Sourcefile = g_file_new_for_path(gstrDest->str);
                        if (FALSE == g_file_make_directory (Sourcefile, commstru->cancellable, &(commstru->gerror)) ){
                            g_string_append(currentItem->ErrorMessage, commstru->gerror->message);
                            iRetFolder = -1;
                        }
                        else{
                            iRetFolder = 0;
                        }
                        if (Sourcefile)
                            g_object_unref(Sourcefile);
                        Sourcefile = NULL;
                    }
                }
                else{
                    iRetFolder = 0;
                }
                if (0 == iRetFolder){  //directory exists lets get in there
                    iRetErrorCode=1;    //so far so good
                    //printf("created Drectory:%s\n",gstrDest->str);
                    //printf("going to open Dir:%s\n", gstrSource->str);
                    if (0 == openLocation(gstrSource->str, &locationStru, FALSE, TRUE)){  //read directories entries and process them
                        iRetErrorCode=1;    //so far so good
                        iRecItemCount = 0;
                        iRetRecursivItem = 0;
                        while ( 1==getNextItemInLocation(&locationStru, gstrNames, FALSE, TRUE)){
                            if (strcmp(gstrNames->str,".")!=0 && strcmp(gstrNames->str,"..")!=0){
                                g_string_assign(gstrSource, cpSource);
                                checkPathForLastChar(gstrSource);
                                g_string_append(gstrSource, gstrNames->str);

                                //printf("new source is:%s\nnew dest is:%s\n",gstrSource->str, gstrDest->str);
                                if (currentItem->CopyMoveOrRemove == 1){    //copy
                                    iRetRecursivItem += copyReMoveItemRecursive(gstrSource->str, gstrDest->str, commstru);
                                }
                                else if (currentItem->CopyMoveOrRemove == 2){    //move
                                    //printf("moving %s\n",gstrSource->str);
                                    iRetRecursivItem += copyReMoveItemRecursive(gstrSource->str, gstrDest->str, commstru);
                                }
                                else if (currentItem->CopyMoveOrRemove == 3){    //remove
                                    //printf("deleting %s\n",gstrSource->str);
                                    iRetRecursivItem += copyReMoveItemRecursive(gstrSource->str, NULL, commstru);
                                }
                                iRecItemCount++;
                            }
                        }
                        //closeDirectory(&Directory);
                        closeLocation(&locationStru);
                        if (iRetRecursivItem != iRecItemCount){ //error on one item in the current Directory
                            iRetErrorCode=0;
                        }
                        if (currentItem->CopyMoveOrRemove == 3 || currentItem->CopyMoveOrRemove == 2){  //delete directory when ReMoved
                            if (iRetRecursivItem == iRecItemCount){  //Dont know this makes sence
                                Sourcefile = g_file_new_for_path(cpSource);
                                //printf("try to delete directory %s\n",cpSource);
                                if (commstru->gerror){
                                    g_error_free(commstru->gerror);
                                    commstru->gerror = NULL;
                                }
                                if (FALSE == g_file_delete (Sourcefile, commstru->cancellable, &(commstru->gerror))){
                                    //error deleting source directory
                                    g_string_assign ( currentItem->ErrorMessage, "Error: Failed to delete source directory code 1" );
                                    if (commstru->gerror){
                                        g_string_append(currentItem->ErrorMessage, " Message: ");
                                        g_string_append(currentItem->ErrorMessage, commstru->gerror->message);
                                    }
                                    //printf("error deleting directory 1\n");
                                    //printf("Message: %s\n",commstru->gerror->message);
                                    iRetErrorCode=0;
                                }
                                else{
                                    iRetErrorCode=1;
                                }
                                //printf("going to delete directory:%s\n",cpSource);
                                if (Sourcefile){
                                    g_object_unref(Sourcefile);
                                    Sourcefile=NULL;
                                }
                            }
                            else{   //error deleting source directory, not empty
                                g_string_assign ( currentItem->ErrorMessage, "Error: Failed to delete source directory code 2" );
                                //printf("error deleting directory 2\n");
                                iRetErrorCode=0;
                            }
                        }
                    }
                    else{
                        g_string_assign ( currentItem->ErrorMessage, "Error: Failed to open source directory" );
                        //printf("error opening source directory\n");
                        iRetErrorCode=0;
                        //failed to open source directory
                    }
                }
                else if (iRetFolder == 1){
                    //g_string_assign (currentItem->ErrorMessage, gstrNames->str);
                    iRetErrorCode=0;
                }
                else{
                    g_string_assign (currentItem->ErrorMessage, "Error: Failed to create destination directory");
                    //printf("error create destination directory\n");
                    //failed to create destination directory
                }
            }
            else{   //copy a file
                //printf("Identified as file\n");
                //prepare listitems member
                Sourcefile = getCorrectGFileTypeFromLocation(gstrSource->str, NULL);
                //Sourcefile = g_file_new_for_path(gstrSource->str);
                if (Sourcefile != NULL){
                    if (!iDoWeDelete)
                        Destfile = g_file_new_for_path(gstrDest->str);
                    if (commstru->gerror != NULL){
                        g_error_free (commstru->gerror);
                        commstru->gerror = NULL;
                        TRACEIT(5,"copyReMoveItemRecursive		ErrorStruct was not NULL, checkpoint 1");
                    }
                    if (currentItem->CopyMoveOrRemove == 1){    //copy
                        bRet = g_file_copy (  Sourcefile, Destfile, G_FILE_COPY_OVERWRITE | G_FILE_COPY_NOFOLLOW_SYMLINKS, commstru->cancellable, copyItemRecursive_Callback, (void *)currentItem, &(commstru->gerror));
                    }
                    else if (currentItem->CopyMoveOrRemove == 2){    //move
                        bRet = g_file_move (  Sourcefile, Destfile, G_FILE_COPY_OVERWRITE | G_FILE_COPY_NOFOLLOW_SYMLINKS, commstru->cancellable, copyItemRecursive_Callback, (void *)currentItem, &(commstru->gerror));
                    }
                    else if (currentItem->CopyMoveOrRemove == 3){    //remove
                        bRet = g_file_delete (Sourcefile, commstru->cancellable, &(commstru->gerror));
                    }

                    //sprintf("copied: _%s_ TO _%s_\n",gstrSource->str, gstrDest->str);
                    if (bRet)
                        iRetErrorCode = 1;
                    else
                        iRetErrorCode = 0;
                }
                else{
                    iRetErrorCode = 0;
                    g_string_assign (currentItem->ErrorMessage, "Error acquiring pointer for source object");
                }
            }
        }
        else{   //list->status!=-0
            iRetErrorCode = 0;
            g_string_assign (currentItem->ErrorMessage, "Cancelled for overwriting");
        }
    }
    //else{
    //    printf("copyReMoveItemRecursive got some bad news for you\n");
    //}

    if (Sourcefile)
        g_object_unref(Sourcefile);
    if (Destfile)
        g_object_unref(Destfile);

    g_string_free(gstrSource, TRUE);
    g_string_free(gstrDest, TRUE);
    g_string_free(gstrNames, TRUE);

    TRACEIT(10,"copyReMoveItemRecursive		end");
    return iRetErrorCode;

    #endif //#ifndef NO_GIO
}


void CalcSizeForCopyDialog(struct CopyReMoveList *list){
    TRACEIT(10,"CalcSizeForCopyDialog		start");
    if (list == NULL ){
        TRACEIT(2,"CalcSizeForCopyDialog		end, Parameter NULL");
        g_thread_exit(0);
    }
    if (list->string1 == NULL){
        TRACEIT(2,"CalcSizeForCopyDialog		end, Parameter2 NULL");
        g_thread_exit(0);
    }
    list->iBytesToCopyIsValid=0;
    list->iBytesToCopy=0;

    if (0==getDirectorySize(&(list->iBytesToCopy), list->string1, &(list->ibreak), FALSE )){
        list->iBytesToCopyIsValid=1;
    }
    else{
        list->iBytesToCopyIsValid=2;    //in CopyReMoveItemsViaList, in the case we want to move, we have to wait for
        //the the size thread to finish. n CopyReMoveItemsViaList we check this variable for 1 or 2 to continue
    }

    TRACEIT(10,"CalcSizeForCopyDialog		end");
    g_thread_exit(0);
}


void CopyReMoveItemsViaList_SizeCalc (void *data){
    TRACEIT(10,"CopyReMoveItemsViaList_SizeCalc		start");
	if (data == NULL){
		TRACEIT(2,"CopyReMoveItemsViaList_SizeCalc		ends, parameter NULL" );
		g_thread_exit(0);
	}
	struct ListWidgetCommunicationStruct *commStru=(struct ListWidgetCommunicationStruct *)data;
	struct CopyReMoveList *list = commStru->firstItem;

    commStru->ui64TotalBytes=0;
    commStru->iTotalBytesValid=0;
    while (list){
        list->threadCalcSize = g_thread_create((GThreadFunc)CalcSizeForCopyDialog, list , TRUE, NULL);
        if (list->threadCalcSize != NULL){
        	//printf("Joining thread      at CopyReMoveItemsViaList_SizeCalc\n");
	        g_thread_join (list->threadCalcSize);
	        list->threadCalcSize = NULL;	//do not join again!
	        //printf("Leaving thread join at CopyReMoveItemsViaList_SizeCalc\n");
    	    commStru->ui64TotalBytes += list->iBytesToCopy;
    	}
        list=list->NextItem;
    }
    commStru->iTotalBytesValid=1;
	TRACEIT(10,"CopyReMoveItemsViaList_SizeCalc		end");
	g_thread_exit(0);
}


void CopyReMoveItemsViaList(void *data){
    TRACEIT(10,"CopyReMoveItemsViaList		start");
	if (data == NULL){
		TRACEIT(2,"CopyReMoveItemsViaList		ends, parameter NULL" );
		g_thread_exit(0);
	}
	struct ListWidgetCommunicationStruct *commStru=(struct ListWidgetCommunicationStruct *)data;
	struct CopyReMoveList *list = NULL;
	GString *gstrError = g_string_sized_new(20);
	GString *gstrForTextBuffer = g_string_sized_new(20);
    GtkTreePath *pTreePath;
    GtkTreeSelection *selection;
    int iRet=0;
    int iBreak=0;
    myWrap_gdk_threads_enter("CopyReMoveItemsViaList 1");
    selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(commStru->listView));
    myWrap_gdk_threads_leave("CopyReMoveItemsViaList 1");
    //int iSomeError=0;
    int iStillValid=1;
    GThread *threadCalcSource=NULL;

    /*
        this function will process only one item: commStru->currentItem and nor more!
        after this function anther thread (actully the same function) is called to continue the list in commStru->currentItem.

        The threads have to be created as joinable, therefore we have to join the last thread before  continuing!
    */

    commStru->SecondThreadRunning = 1;

    if (NULL != commStru->gthrdCopyReMoveThread_2){
        //this means that this is NOT the first CopyReMove Thread, as this thread will create the second thread to continue the list
        if (NULL != commStru->gthrdCopyReMoveThread_1){
            g_thread_join (commStru->gthrdCopyReMoveThread_1);  //make sure, that only one CopyReMove Thread is running at a time and the last threads memory gets freed
        }
        commStru->gthrdCopyReMoveThread_1 = commStru->gthrdCopyReMoveThread_2;  //forget the old thread and store this thread at the 1st position;
        commStru->gthrdCopyReMoveThread_2 = NULL;
    }


    list = commStru->currentItem;   //for faster writing. and shorter sentences
    if (NULL == list){
        //because this function will set the next currentItem for the following thread, the currentItem can be NULL, as the list ends.
        //So we reached the end of the list and we only have to clear everything as usual
        iBreak = 2;
    }


    if (iBreak==0 && (list->string1==NULL || ( (list->CopyMoveOrRemove == 1 || list->CopyMoveOrRemove == 2) && list->string2==NULL) )){
        list->ibreak=1;
        iBreak = 1;
    }
    if (iBreak==0 && (list->status == -10 || commStru->iCancelled==1)){
        list->ibreak=1;
        iBreak = 1;
    }




    if (iBreak==0 && NULL!=commStru->currentItem && commStru->iCancelled==0){

        //list->ibreak=0;   //done in ProcessItemsViaList, the previouse function
        myWrap_gdk_threads_enter("CopyReMoveItemsViaList 2");
        gtk_tree_selection_select_iter(selection, &(list->iter));

        //make the scrolling follow the marked Item
        pTreePath = gtk_tree_model_get_path (gtk_tree_view_get_model ((GtkTreeView *)commStru->listView),&(list->iter));
        gtk_tree_view_scroll_to_cell ((GtkTreeView *)commStru->listView,pTreePath,NULL,TRUE,0.0,0.0);
        gtk_tree_path_free(pTreePath);
        myWrap_gdk_threads_leave("CopyReMoveItemsViaList 2");


        if (list->ErrorMessage)
            g_string_free(list->ErrorMessage,TRUE);
        list->ErrorMessage = g_string_sized_new(10);    //make clear this pointer is valid  //TODO FREE IT AND FILL IT WITH 0 on list creation

        if (commStru->gerror){
            g_error_free(commStru->gerror);
            commStru->gerror=NULL;
            TRACEIT(10,"copyItemRecursive		error struct was not NULL, checkpoint 2");
        }

        if (list->status == -1){
            list->status = 0;

            //calc size //the size is calculated in a nother thread now
            //list->iBytesToCopy = 0;
            //list->iBytesToCopyIsValid = 0;
            list->iBytesCopied = -1; //initialize with -1, so we now when to start messuring the time
            if (list->CopyMoveOrRemove != 3){    //do not calculate this when deleting
                //threadCalcSource = g_thread_create((GThreadFunc)CalcSizeForCopyDialog, list , TRUE, NULL);
                threadCalcSource = list->threadCalcSize;
            }
            //tell the textBuffer what we are giong to do:
            if (list->CopyMoveOrRemove == 1){
                g_string_printf(gstrForTextBuffer, "copying \"%s\" TO  \"%s\"",list->string1->str,list->string2->str);
            }
            else if (list->CopyMoveOrRemove == 2){
                while (list->iBytesToCopyIsValid == 0){
                    usleep(10000);
                //if (list->iBytesToCopyIsValid==0 && threadCalcSource ){
                //     g_thread_join (threadCalcSource); //we should wait for the calcthread before moving.
                }
                g_string_printf(gstrForTextBuffer, "moving \"%s\" TO  \"%s\"",list->string1->str,list->string2->str);
            }
            else if (list->CopyMoveOrRemove == 3){
                g_string_printf(gstrForTextBuffer, "deleting \"%s\"",list->string1->str);
                // Set the row for percentage to some text, its better to view something than nothing, because nothing means not processed yet
                myWrap_gdk_threads_enter("CopyReMoveItemsViaList 3");
                gtk_list_store_set (commStru->listStore, &(list->iter), 1, "deleting",-1);
                myWrap_gdk_threads_leave("CopyReMoveItemsViaList 3");
            }
            myWrap_gdk_threads_enter("CopyReMoveItemsViaList 4");
            textbuffer_commandHistory_addText(gstrForTextBuffer->str, NULL);
            myWrap_gdk_threads_leave("CopyReMoveItemsViaList 4");

            //start copying, this call is synchron
            iRet = copyReMoveItemRecursive(list->string1->str, (list->string2)?list->string2->str:NULL, commStru);

            if (iRet==1){   //iRet == 1 : Ok
                list->ibreak=0;
                myWrap_gdk_threads_enter("CopyReMoveItemsViaList 5");
                gtk_list_store_set (list->listStore, &(list->iter), 1, "100.00",-1); //when no error occured we assume that we copied 100%
                myWrap_gdk_threads_leave("CopyReMoveItemsViaList 5");
                //if (list->CopyMoveOrRemove != 3){
                    //list->threadCalcSizeAfterProcessing = g_thread_create((GThreadFunc)CalcSizeAfterCopyMove, list , TRUE, NULL);
                //}
            }
            else{
                list->ibreak=1;
            }
        }

        if (iRet == 0 ){    //Error copying look GError for information
            g_string_erase(gstrError,0,-1);
            if (commStru->gerror){
                if (G_IO_ERROR_CANCELLED != commStru->gerror->code){
                    g_string_printf(gstrError, "Error: code: %d, message:%s",commStru->gerror->code, commStru->gerror->message);
                    textbuffer_commandHistory_addText(gstrError->str, "error");
                    g_error_free(commStru->gerror);
                    commStru->gerror=NULL;
                    commStru->iSomeError = 1;
                }
                else
                    commStru->iSomeError = 2; //do not show the hint that there was an error
            }
            else if (strlen (list->ErrorMessage->str) != 0){
                g_string_assign (gstrError, list->ErrorMessage->str);
                textbuffer_commandHistory_addText(gstrError->str, "error");
                commStru->iSomeError = 1;
            }
            else{
                g_string_assign(gstrError, "Unknown error");
                textbuffer_commandHistory_addText(gstrError->str, "error");
                commStru->iSomeError = 1;
            }
            if (strlen(gstrError->str)>0){
                myWrap_gdk_threads_enter("CopyReMoveItemsViaList 6");
                gtk_list_store_set (commStru->listStore, &(list->iter), 3, gstrError->str,-1);
                myWrap_gdk_threads_leave("CopyReMoveItemsViaList 6");
                commStru->iSomeError=1;
            }
        }
        else if(iRet == -1){    //internal Error
            g_string_assign(gstrError, "BAD INTERNAL ERROR, INFORM THE PROGRAMMER");
            textbuffer_commandHistory_addText(gstrError->str, "error");
            myWrap_gdk_threads_enter("CopyReMoveItemsViaList 7");
            gtk_list_store_set (commStru->listStore, &(list->iter), 3, gstrError->str,-1);
            myWrap_gdk_threads_leave("CopyReMoveItemsViaList 7");
            commStru->iSomeError=1;
        }
        //if (commStru->iCancelled==1){
        //    list->ibreak=1;
        //    break;
        //}
        //commStru->currentItem = commStru->currentItem->NextItem;
    }

    //list->ibreak=1; //for security reason, this cancels the size calculation thread
    //we can come up here when there was
    //some error, in this case we may have a cancel
    //just a cancel
    //we finished, no error, no cancel
    //only in the last case we going to close the widget from here

    g_mutex_lock (commStru->mtxContinue);
    iStillValid = 1;
    //printf("after thread 2 loop\n");
    if (iBreak == 2){   //iBreak=2 was set if the currentItem was NULL, in this case, this thread is the last one and we have finished the list
        //printf("last item in list\n");
        if (commStru->iCancelled==1 || commStru->iSomeError!=0){ //error or cancel, its not our turn to close the widget, but to update some things
            //printf("Error or cancel\n");
            myWrap_gdk_threads_enter("CopyReMoveItemsViaList 8");
            if (commStru->iSomeError == 1){
                MyMessageWidget((GtkWindow *)commStru->mainListWidget, "ERROR", "There was at least one Error \nwhile processing the list.\nLook at the column errors and scroll up\ntill you find the error message.");
                g_string_append(commStru->WindowName," ERROR");
            }
            else if (commStru->iCancelled == 1){
                g_string_append(commStru->WindowName," Cancelled");
            }
            gtk_window_set_title((GtkWindow *)commStru->mainListWidget,commStru->WindowName->str);

            gtk_widget_show(GTK_WIDGET(commStru->closeButton));
            gtk_widget_hide(GTK_WIDGET(commStru->cancelButton));

            gtk_widget_queue_draw ((GtkWidget *)commStru->mainListWidget);
            myWrap_gdk_threads_leave("CopyReMoveItemsViaList 8");

        }
        else if (commStru->iCancelDialogRuns==0){
            //printf("check 2\n");
            commStru->SecondThreadRunning = 0;
            g_mutex_unlock (commStru->mtxContinue);
            iStillValid = 0;
            myWrap_gdk_threads_enter("CopyReMoveItemsViaList 9");
            (commStru->FreeMemoryAndDeleteWidgetFunction)( commStru);  //here Notify MUST BE 2 other wise this call will lead to a deadlock becuase of SecondThreadRunning
            myWrap_gdk_threads_leave("CopyReMoveItemsViaList 9");
        }
        else{ //the cancel dialog is open
            //printf("check 3\n");
            commStru->iDestroyFlag = 1;
        }

    }
    else if (0 == commStru->iListPaused){
        //we will start the next thread, in the case that the next item is NULL, the thread is created without the option to join it:
        commStru->iNextThreadStarted = 1;

        commStru->currentItem = commStru->currentItem->NextItem;

        if (NULL != commStru->currentItem){
            commStru->gthrdCopyReMoveThread_2 = g_thread_create((GThreadFunc)CopyReMoveItemsViaList, (void *)commStru , TRUE, NULL);
        }
        else {
            g_thread_create((GThreadFunc)CopyReMoveItemsViaList, (void *)commStru , FALSE, NULL);
        }
    }
    else{
        commStru->iNextThreadStarted = 0;
    }


    if (1 == iStillValid){
        commStru->SecondThreadRunning = 0;
        g_mutex_unlock (commStru->mtxContinue);
    }



    //free things
    g_string_free(gstrError, TRUE);
    g_string_free(gstrForTextBuffer, TRUE);
    //commStru->SecondThreadRunning = 0; NOT HERE, because of the last else if

	TRACEIT(10,"CopyReMoveItemsViaList		ends");
	g_thread_exit(0);
}


int fileAttributesGet(struct FileAttributs *attribs,char *path, char *name){
    //DO NOT FORGETT: FREE MEMORY (GSTRING IF IT IS A LINK)
    //example:
    //      if (attribs->gstrLinkTarget)
    //          g_string_free(attribs->gstrLinkTarget, TRUE);
    //
    //name can be NULL, in this case path is expected to be the whole name with path
	TRACEIT(11,"fileAttributesGet		start");
	int i;
	if (attribs == NULL || path == NULL){
		TRACEIT(10,"fileAttributesGet		ends");
		return -1;
	}

	GString *gstrFullName = g_string_new(path);
	GString *gstrBuf = g_string_sized_new(10);
	GString *gstrName = g_string_sized_new(10);
	struct stat fileattributes;

    attribs->size = 0;
    attribs->owner = 0;
    attribs->group = 0;
    sprintf(attribs->permission,"-");
    strcat(attribs->permission,"-");
    strcat(attribs->permission,"-");
    strcat(attribs->permission,"-");
    strcat(attribs->permission,"-");
    strcat(attribs->permission,"-");
    strcat(attribs->permission,"-");
    strcat(attribs->permission,"-");
    strcat(attribs->permission,"-");
    sprintf(attribs->accessed,"na");	//keep in mind, that the accessed array is 30 bytes in size (defined in listitems.h)
    sprintf(attribs->changed,"na");
    attribs->gstrLinkTarget=NULL;

    if ( name!=NULL ){
        checkPathForLastChar(gstrFullName);
        g_string_append(gstrFullName, name);
    }

	if (FALSE == g_file_test(gstrFullName->str,G_FILE_TEST_EXISTS)){
		g_string_free(gstrFullName,TRUE);
		g_string_free(gstrBuf,TRUE);
		g_string_free(gstrName, TRUE);
		TRACEIT(11,"fileAttributesGet		end, File not found");
		return -1;
	}

	if (stat(gstrFullName->str, &fileattributes) == -1){
        sprintf(attribs->accessed,"fstab: errno=%d", errno);	//keep in mind, that the accessed array is 30 bytes in size (defined in listitems.h)
	}
	else{
		//size
		attribs->size = (guint64)fileattributes.st_size;

		//owner
		attribs->owner = fileattributes.st_uid;

		//group
		attribs->group = fileattributes.st_gid;

		//permission
		sprintf(attribs->permission,(S_IRUSR & fileattributes.st_mode)?"r":"-");
		strcat(attribs->permission,(S_IWUSR & fileattributes.st_mode)?"w":"-");
		strcat(attribs->permission,(S_IXUSR & fileattributes.st_mode)?"x":"-");
		strcat(attribs->permission,(S_IRGRP & fileattributes.st_mode)?"r":"-");
		strcat(attribs->permission,(S_IWGRP & fileattributes.st_mode)?"w":"-");
		strcat(attribs->permission,(S_IXGRP & fileattributes.st_mode)?"x":"-");
		strcat(attribs->permission,(S_IROTH & fileattributes.st_mode)?"r":"-");
		strcat(attribs->permission,(S_IWOTH & fileattributes.st_mode)?"w":"-");
		strcat(attribs->permission,(S_IXOTH & fileattributes.st_mode)?"x":"-");

        attribs->type_d = (S_IFDIR & fileattributes.st_mode)?4:0;
		//accessed
		attribs->utaccessed=fileattributes.st_atime;
		sprintf(attribs->accessed,"%s",	ctime(&fileattributes.st_atime));
		attribs->accessed[strlen(attribs->accessed)-1]='\0';

		//changed
		attribs->utchanged=fileattributes.st_mtime;
		sprintf(attribs->changed,"%s",	ctime(&fileattributes.st_mtime));
		attribs->changed[strlen(attribs->changed)-1]='\0';

		//hiddenfile
		if (name == NULL){
		    g_string_assign(gstrBuf, gstrFullName->str);
            getLastStringInPath(gstrBuf);
            name = gstrBuf->str;
		}
		if (name[0] == '.' && strcmp(name,".") != 0 && strcmp(name,"..") != 0)
			attribs->hiddenfile = TRUE;
		else
			attribs->hiddenfile = FALSE;
	}
	//check if we have a Link and set the appropriate variables
#ifndef NO_GIO
	attribs->gstrLinkTarget=NULL;
	GFileInfo *SymlinkInfo=NULL;
	GFile *Symfile=NULL;
	const char *cpTarget=NULL;
	int iItsASymlink=0; //1 if it is a symlink, but its no garantee the links exists!!
    if (TRUE == g_file_test( gstrFullName->str, G_FILE_TEST_IS_SYMLINK)){
        //printf("found symlink: %s\n", gstrFullName->str);
        iItsASymlink=1;
        attribs->type_d = 10;
        Symfile = g_file_new_for_path(gstrFullName->str);
        SymlinkInfo = g_file_query_info (Symfile, G_FILE_ATTRIBUTE_STANDARD_SYMLINK_TARGET,
                                                         G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
                                                         NULL,
                                                         NULL);
        cpTarget = g_file_info_get_attribute_byte_string (SymlinkInfo,G_FILE_ATTRIBUTE_STANDARD_SYMLINK_TARGET);
        if (cpTarget){
            //printf("target: %s\n", cpTarget);
            attribs->gstrLinkTarget = g_string_new(cpTarget);
            if (g_file_test( attribs->gstrLinkTarget->str, G_FILE_TEST_EXISTS)){
                if (checkStringIsPath(attribs->gstrLinkTarget->str)){
                    attribs->type_d=4; //handle as path
                }
                else{
                    attribs->type_d=0; //handle as ordinary file
                }
            }
        }
        else{
           // printf("No target found\n");
        }

        if (Symfile)
            g_object_unref(Symfile);
        if (SymlinkInfo)
            g_object_unref(SymlinkInfo);
    }
#endif

	//type

#ifndef NO_GIO
	if (iItsASymlink==1 && attribs->gstrLinkTarget !=NULL){
        g_string_assign (gstrName, attribs->gstrLinkTarget->str);
        getLastStringInPath(gstrName);
	}
	else{
        g_string_assign(gstrName, name);
	}
#endif //#ifndef NO_GIO
#ifdef NO_GIO
    g_string_assign(gstrName, name);
#endif //#ifdef NO_GIO
	int len = strlen(gstrName->str);
	int foundit=0;
	for (i=1; i<=MAXTYPELENGTH; i++){
		//was len-i<=1 , but then e.g. 1.jpg was not recognized
		//if (len - i < 1 || dirzeiger->d_type == 4 || dirzeiger->d_type == 10){	//removed because we need this function for the fileshelf too, and do not have an directory opened
		if (len - i < 1 || attribs->type_d == 4 || attribs->type_d == 10){
		    if (attribs->type_d == 10)  //without this, short files will be marked with "-", and than could not be distinguished from directories. important, when marking all items of the same type
                foundit = 10;
			else if (attribs->type_d == 4)
                foundit = 4;
			break;
		}
		if (gstrName->str[len-i] == '.'){
			foundit = 1;
			strncpy (attribs->type,&(gstrName->str[len-i+1]),FIELDSIZE);
			break;
		}
	}
	if (foundit==0)
		strncpy(attribs->type,"-",FIELDSIZE);   //this things are used to distinguish between types, used when marking all items of the same type
    else if(foundit==10)
        strncpy(attribs->type,"---",FIELDSIZE);
    else if(foundit==4)
        strncpy(attribs->type,"--",FIELDSIZE);

    g_string_free(gstrName,TRUE);
	g_string_free(gstrFullName,TRUE);
	g_string_free(gstrBuf,TRUE);

	TRACEIT(11,"fileAttributesGet		ends");
	return 0;
}


int openLocation (char *cpLocation, struct myLocationStruct *struLocation, gboolean bUseMessageWidgets, gboolean bMakeThreadSave){
    //this is NOT THREAD SAVE if bUseMessageWidgets == TRUE
    //bUseMassageWidget=True: show error messages
    //bMakeThreadSave=True: use gdk_threads_enter/leave
    //return: 0: success opening directory, -1: error
    //the struLocation must be a pointer to a valid structure, it is not created inside here, nor it is freed!!
    //however the things which are going to be filled into this structure are freed in closeLocation
    //errors on opneing a location are shown as messageboxes from within this function,so the calling function does not have to interprete errors
    //EVEN ON AN ERROR YOU HAVE TO CALL closeLocation to free the structure struLocation
    TRACEIT(10,"openLocation		start");
    GString *gstrBuf = g_string_sized_new(10);
    if (cpLocation==NULL || struLocation==NULL){
        g_string_free(gstrBuf, TRUE);
        TRACEIT(2,"openLocation		end, Parameter NULL");
        return -1;
    }

    //first step is to initialize the structure (not the structure itself but its member)
    struLocation->gfileLocation=NULL;
    struLocation->error=NULL;
    struLocation->gstrLocation=NULL;
    struLocation->enumerator=NULL;
    struLocation->gfileInfo=NULL;

    int iItsPath = -1;  //-1 unknown, 0=uri, 1=path
    //first we have to decide if we have a path, or a uri:
    //when the first character is a '/' its supposed to be a path.
    if (strlen(cpLocation) >= 2){
        if (cpLocation[0]=='/')
            iItsPath = 1;
        else
            iItsPath = 0;
    }
    else{//its a path
        iItsPath = 1;
    }

    if (iItsPath == 1 && checkStringIsPath(cpLocation) == FALSE){
        g_string_free(gstrBuf, TRUE);
        TRACEIT(7,"openLocation		end, directory not found:");
        TRACEIT(7,cpLocation);
        return -1;
    }

    struLocation->gstrLocation = g_string_new(cpLocation);

    //Now lets create the GFile Object
    if (iItsPath == 0){ //uri
        struLocation->gfileLocation = g_file_new_for_uri (cpLocation);
    }
    else if (iItsPath == 1){ //path
        struLocation->gfileLocation = g_file_new_for_path (cpLocation);
    }
    else{
        g_string_free(gstrBuf, TRUE);
        if (bUseMessageWidgets){
            if (bMakeThreadSave)
                myWrap_gdk_threads_enter("openLocation 1");
            MyMessageWidget(win, "Error", "The type of the location could not be recognized.\nError opening new location");
            if (bMakeThreadSave)
                myWrap_gdk_threads_leave("openLocation 1");
        }
        TRACEIT(5,"openLocation		end, type not recognized:");
        TRACEIT(5,cpLocation);
        return -1;
    }

    struLocation->enumerator = g_file_enumerate_children (struLocation->gfileLocation, "standard::name", G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS, NULL, &(struLocation->error));
    if (struLocation->error != NULL){
        if (bUseMessageWidgets){
            g_string_printf(gstrBuf, "Unable to open location:\n%s\nError:%d:%s",cpLocation, struLocation->error->code, struLocation->error->message);
            if (bMakeThreadSave)
                myWrap_gdk_threads_enter("openLocation 2");
            MyMessageWidget(NULL,"Error",gstrBuf->str);
            if (bMakeThreadSave)
                myWrap_gdk_threads_leave("openLocation 2");
        }
        g_error_free(struLocation->error);
        struLocation->error=NULL;
        g_string_free(gstrBuf, TRUE);
        TRACEIT(5,"openLocation		end, location not supported:");
        TRACEIT(5,cpLocation);
        return -1;
    }

    g_string_free(gstrBuf, TRUE);
    TRACEIT(10,"openLocation		end");
    return 0;   //success, directory opened
}


int getNextItemInLocation(struct myLocationStruct *struLocation, GString *gstrName, gboolean bUseMessageWidgets, gboolean bMakeThreadSave){
    //return: -1=error, 1=success, 0=no item left
    TRACEIT(10,"getNextItemInLocation		start");
    const char *cpName=NULL;
    if (struLocation==NULL || gstrName==NULL){
        TRACEIT(2,"getNextItemInLocation		end, parameter NULL");
        return -1;
    }

    if (struLocation->gfileLocation==NULL || struLocation->enumerator==NULL){
        g_string_erase(gstrName, 0, -1);
        TRACEIT(4,"getNextItemInLocation		end, struLoction not initialized properly");
        return -1;
    }

    if (struLocation->gfileInfo!=NULL){
        g_object_unref(struLocation->gfileInfo);
        struLocation->gfileInfo=NULL;
    }

    struLocation->gfileInfo = g_file_enumerator_next_file (struLocation->enumerator, NULL, &(struLocation->error));

    if (struLocation->error != NULL){
        if (bUseMessageWidgets){
            if (bMakeThreadSave)
                myWrap_gdk_threads_enter("getNextItemInLocation 1");
            MyMessageWidget(NULL,"Error",struLocation->error->message);
            if (bMakeThreadSave)
                myWrap_gdk_threads_enter("getNextItemInLocation 1");
        }
        g_error_free(struLocation->error);
        struLocation->error=NULL;
    }

    if (struLocation->gfileInfo == NULL){   //no item left at location
        g_string_erase(gstrName, 0, -1);
        TRACEIT(10,"getNextItemInLocation		end, location finished");
        return 0;   //no more items to come
    }

    //G_FILE_ATTRIBUTE_STANDARD_COPY_NAME , G_FILE_ATTRIBUTE_STANDARD_NAME, G_FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME
    //cpName = g_file_info_get_attribute_as_string (struLocation->gfileInfo, G_FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME);
    cpName = g_file_info_get_name(struLocation->gfileInfo);
    if (cpName==NULL){
        //MyMessageWidget(NULL,"Error","Error retreiving name of item at location, aborting.");
        TRACEIT(4,"getNextItemInLocation		end, Error retreiving name of item at location");
    }
    else{
        g_string_assign(gstrName, cpName);
    }

    if (struLocation->gfileInfo!=NULL){
        g_object_unref(struLocation->gfileInfo);
        struLocation->gfileInfo=NULL;
    }

    TRACEIT(10,"getNextItemInLocation		end");
    return 1;   //more items to come
}


int closeLocation (struct myLocationStruct *struLocation){
    //frees all the things created and stored in struLocation.
    //make sure you call this function even if one of the leading functions (openLocation ...) fails!
    //return: 0=ok, -1=internal error;
    TRACEIT(10,"closeLocation		start");
    if (struLocation == NULL){
        TRACEIT(2,"closeLocation		end, Parameter NULL");
    }

    if (struLocation->gstrLocation!=NULL){
        g_string_free(struLocation->gstrLocation, TRUE);
        struLocation->gstrLocation=NULL;
    }

    if (struLocation->error!=NULL){
        g_error_free(struLocation->error);
        struLocation->error=NULL;
    }
    if (struLocation->gfileInfo!=NULL){
        g_object_unref(struLocation->gfileInfo);
        struLocation->gfileInfo=NULL;
    }

    if (struLocation->enumerator!=NULL){
        g_file_enumerator_close (struLocation->enumerator,NULL,NULL);
        g_object_unref(struLocation->enumerator);
        struLocation->enumerator=NULL;
    }

    if (struLocation->gfileLocation!=NULL){
        g_object_unref(struLocation->gfileLocation);
        struLocation->gfileLocation=NULL;
    }

    TRACEIT(10,"closeLocation		end");
    return 0;
}


static void getSizeOfSelectedItemsRecursive(struct RecursiveSizeStructure *stru){
	TRACEIT(10,"getSizeOfSelectedItemsRecursive		start");

	GString *gstrItem=g_string_sized_new(20);
	int i;

	for (i=0; i<stru->ItemCounter; i++){
		if (*(stru->ibreak) != 0){
			break;
		}
		if (stru->intPTypes[i] == 4 /*|| stru->intPTypes[i] == 10*/){
			if (stru->SideObjectOrFileshelfItem==SOURCE_SIDEOBJECT){
	    		g_string_assign(gstrItem, stru->gstrPath->str);
				checkPathForLastChar(gstrItem);
    			g_string_append(gstrItem, (stru->gstrDPItems)[i]->str);
			}
			else{
				g_string_assign(gstrItem, (stru->gstrDPItems)[i]->str);
			}
			if (0 != getDirectorySize(&(stru->guint64_size), gstrItem, stru->ibreak, FALSE)){  //Error opening directory
                stru->ErrorOccured=1;
			}
        }
    	else{
			stru->guint64_size+=stru->intPSizes[i];
    	}
    	if (*(stru->ibreak) != 0){
    			break;
    	}
    }

	//clean up
	g_string_free(gstrItem,TRUE);

	//We are ready for clean up the structure, tell this the other thread:
	stru->iReadyForCleanUp = 1;

	TRACEIT(10,"getSizeOfSelectedItemsRecursive		ends");
	g_thread_exit(0);
}


void getSizeOfSelectedItemsRecursive_dialog(void *data){
	TRACEIT(10,"getSizeOfSelectedItemsRecursive_dialog		start");
	GtkWidget *dialog, *label;
	int i;
	int len;
	struct RecursiveSizeStructure *stru = (struct RecursiveSizeStructure *) data;
	GString *gstrBuf = g_string_sized_new(10);
	GString *gstrBuf2 = g_string_sized_new(10);
	char buf[320];  //just needed to hold the size (only the number), g_printf does not know how to handle 64bit numbers
	GtkButton *CancelButton;
	GtkVBox *VBox;
	GError *error = NULL;
	//GThread *thread = NULL;

	myWrap_gdk_threads_enter("getSizeOfSelectedItemsRecursive_dialog 1");

	stru->ibreak = malloc (sizeof(int));
	*(stru->ibreak) = 0;
	stru->theLabel = gtk_label_new("sdf");
	stru->iReadyForCleanUp=1;
	stru->guint64_size=0;
	stru->ErrorOccured=0;

	int iNotify=0;

	//simple dialog
	dialog = CreateMyWindow("Size of selected items",stru->mainWin,400,250,FALSE, TRUE, TRUE, &iNotify, FALSE, FALSE);
	//dialog = CreateMyWindow("Size of selected items",NULL,400,250,FALSE, FALSE);
	AddMyIconsToWindow((GtkWindow *)dialog);

	CancelButton = (GtkButton *)gtk_button_new_with_label("Ok");
	g_signal_connect (CancelButton,"clicked",G_CALLBACK(CancelButtonFunctionForIBreak),stru->ibreak);

	VBox = g_object_new(GTK_TYPE_VBOX, "spacing", 5, "homogeneous", FALSE, NULL);

	g_string_printf(gstrBuf,"\tCalculates the size of the selected items.\t\n\tDirectories are searched recursively.\t\n\n\tItems selected: %d\n",stru->ItemCounter);

	label = gtk_label_new(gstrBuf->str);

	gtk_box_pack_start(GTK_BOX(VBox),label,TRUE,TRUE,1);
	gtk_box_pack_start(GTK_BOX(VBox),stru->theLabel,TRUE,TRUE,1);
	gtk_box_pack_start(GTK_BOX(VBox),GTK_WIDGET(CancelButton),FALSE,FALSE,1);
	gtk_container_add((GtkContainer *)dialog,(GtkWidget *)VBox);

	gtk_widget_show_all(dialog);
	myWrap_gdk_threads_leave("getSizeOfSelectedItemsRecursive_dialog 1");

	if (stru->ItemStringCounter != stru->ItemCounter){	//Error allocation memory for storing ItemInformation
		gtk_label_set_text ((GtkLabel *)stru->theLabel, "Sorry, an error occured while memory allocation.\nUnable to calculate size.");
		*(stru->ibreak) = 1;
		stru->iReadyForCleanUp = 1;
	}
	else{
		stru->iReadyForCleanUp=0;
		//thread = g_thread_create((GThreadFunc)getSizeOfSelectedItemsRecursive, (void *)stru , TRUE, &error);
		g_thread_create((GThreadFunc)getSizeOfSelectedItemsRecursive, (void *)stru , FALSE, &error);
		if (error != NULL){
            TRACEIT (3 ,"getSizeOfSelectedItemsRecursive_dialog     error creating thread:");
            TRACEIT (3 ,error->message);
            g_error_free (error);
            error = NULL;
		}
		//do not make this thread joinable and join in here, because in this case the little widget showing the size
		//will not be updated anymore.
		//g_thread_join (thread);
	}

	do{
        BytesToHumanityString (stru->guint64_size, gstrBuf2, 0);
		snprintf(buf,99,"%lld",stru->guint64_size);
        g_string_assign(gstrBuf,buf);
		len = strlen(gstrBuf->str)-3;
		while (len>0){
			g_string_insert_c(gstrBuf,len,'.');
			len -=3;
		}
		strcpy(buf,gstrBuf->str);
		if (stru->ErrorOccured == 1){
            g_string_sprintf(gstrBuf,"\tSelected need %s (%s bytes)\t\nBut there was at least one directory\nthat could not be opened.",gstrBuf2->str, buf);
		}
		else{
            g_string_sprintf(gstrBuf,"\tSelected need %s (%s bytes)\t",gstrBuf2->str, buf);
		}
		myWrap_gdk_threads_enter("getSizeOfSelectedItemsRecursive_dialog 2" );
		if (iNotify==1){    //as long this is 1 the dialog is valid and was not destroyed
            gtk_label_set_text ((GtkLabel *)stru->theLabel, gstrBuf->str);
		}
		else{
            *(stru->ibreak)=1;
		}
		myWrap_gdk_threads_leave("getSizeOfSelectedItemsRecursive_dialog 2");
		usleep(100000);
	}while(*(stru->ibreak) == 0);

	while (stru->iReadyForCleanUp != 1){
		//The other thread has not finished yet, we should wait for him.
		usleep(100000);
	}

	myWrap_gdk_threads_enter("getSizeOfSelectedItemsRecursive_dialog 3");
	if (iNotify==1){    //then, the window was closed by our ok-button and not by upper-right-corner-X, so we have to destroy the window by ourself
        gtk_widget_destroy(dialog);
	}
	myWrap_gdk_threads_leave("getSizeOfSelectedItemsRecursive_dialog3 ");

	//free everything in the struct and the struct itself
	free(stru->ibreak);
	if (stru->gstrPath != NULL){
		g_string_free(stru->gstrPath,TRUE);
	}
	for (i=0; i<stru->ItemStringCounter; i++){
		g_string_free(stru->gstrDPItems[i], TRUE);
	}
    free(stru->gstrDPItems);
	free(stru->intPSizes);
	free(stru->intPTypes);
	free(stru);
	stru = NULL;
	g_string_free(gstrBuf,TRUE);
	g_string_free(gstrBuf2, TRUE);

	TRACEIT(10,"getSizeOfSelectedItemsRecursive_dialog		ends");
	g_thread_exit(0);
}


int getDirectorySize(guint64 *size, GString *gstrDirectory, int *ibreak, gboolean bFollowLinks){
    //calls openLocation which is thread save in here, because this function is called
    //only from other threads
    //stops when ibreak == 1
	TRACEIT(10,"getDirectorySize		start");
	if (size == NULL || gstrDirectory == NULL || ibreak == NULL){
        TRACEIT(2,"getDirectorySize		end, Parameter NULL");
        return -1;
	}
	if (FALSE == g_file_test( gstrDirectory->str, G_FILE_TEST_EXISTS)){
        TRACEIT(2,"getDirectorySize		end, error opening item:");
        return -1;
	}
	GString *gstrNames = g_string_new(gstrDirectory->str);
	GString *Path= g_string_new(gstrDirectory->str);
	struct FileAttributs attribs;
    struct myLocationStruct locationStru;
    int iFoundLink = 0;

    //check if we have an Directory
    if (FALSE == checkStringIsPath( gstrDirectory->str )){ //then we have no directory
        checkPathForLastCharOtherwise(gstrNames);
        getLastStringInPath(gstrNames);
        GetPathFromPathWithName(Path, 1);
        if (strcmp(gstrNames->str,".")!=0 && strcmp(gstrNames->str,"..")!=0){
            fileAttributesGet( &attribs, Path->str, gstrNames->str);
            if (attribs.gstrLinkTarget){
                g_string_free(attribs.gstrLinkTarget, TRUE);
            }
            *size+=attribs.size;
        }
        g_string_free(gstrNames,TRUE);
        g_string_free(Path,TRUE);
        TRACEIT(10,"getDirectorySize		end");
        return 0;
    }

	if (0 != openLocation(gstrDirectory->str, &locationStru, FALSE, TRUE)){
	    closeLocation(&locationStru);
	    g_string_free(gstrNames,TRUE);
        g_string_free(Path,TRUE);
        TRACEIT(2,"getDirectorySize		end, could not open directory");
        return -1;
    }

	while ( 1 == getNextItemInLocation(&locationStru, gstrNames, FALSE, TRUE) && *ibreak == 0){

		//fileAttributesGet(dirzeiger, DirectoryOpened, &attribs, gstrDirectory->str, gstrNames->str);
		fileAttributesGet(&attribs, gstrDirectory->str, gstrNames->str);
		if (attribs.gstrLinkTarget){
            iFoundLink = 1;
            g_string_free(attribs.gstrLinkTarget, TRUE);
		}
		if (strcmp(gstrNames->str,".")!=0 && strcmp(gstrNames->str,"..")!=0 &&  ( attribs.type_d==4 && (iFoundLink!=1 || bFollowLinks==TRUE ) ) ){
			g_string_assign(Path,gstrDirectory->str);
			checkPathForLastChar(Path);
			g_string_append(Path,gstrNames->str);
			getDirectorySize(size, Path, ibreak, bFollowLinks);
		}
		else if (strcmp(gstrNames->str,".")!=0 && strcmp(gstrNames->str,"..")!=0){
			*size+=attribs.size;
		}
	}

	//Subern:
	closeLocation(&locationStru);
	g_string_free(gstrNames,TRUE);
	g_string_free(Path,TRUE);

    TRACEIT(10,"getDirectorySize		end");
    if (*ibreak == 0){
        return 0;
    }
    else{
        return -1;
    }
}


static void markedNamesToClipboard_Helper(GtkTreeModel *model,GtkTreePath *path,GtkTreeIter *iter,gpointer data){
	TRACEIT(11,"markedNamesToClipboard_Helper		start");
	if (data == NULL){
		TRACEIT(2,"markedNamesToClipboard_Helper		end, PARAMETER NULL");
		return;
	}
	struct AllInformationAllUseStruct *stru = (struct AllInformationAllUseStruct *)data;
	gchar *Name=NULL;
	gchar *Path=NULL;

	GString *gstrNames = (GString *)stru->Info_1;
	int iColumnName = *(int *)stru->Info_2;
	gboolean bWithPath = *(gboolean *)stru->Info_3;
	int iColumnPath = *(int *)stru->Info_4;
	char *cpPath = (char *)stru->Info_5;

	gtk_tree_model_get(model, iter, iColumnName, &Name, -1);

	if (gstrNames->len != 0){
		g_string_append(gstrNames, " ");
	}

	if (bWithPath == TRUE){
		if (cpPath != NULL){
			g_string_append(gstrNames, cpPath);
		}
		else if (iColumnPath > -1){
			gtk_tree_model_get(model, iter, iColumnPath, &Path, -1);
			g_string_append(gstrNames, Path);
		}
		else{
			TRACEIT(2,"markedNamesToClipboard_Helper		parameter validation error");
		}
		checkPathForLastChar(gstrNames);
	}

	g_string_append(gstrNames, Name);

	if (Name)
		g_free(Name);
	if (Path)
		g_free(Path);

	TRACEIT(11,"markedNamesToClipboard_Helper		end");
}


void markedNamesToClipboard(GtkTreeView *TreeView, gboolean bWithPath, int iColumnName, int iColumnPath, char *cpPath){
	//copies the names marked in the TreeView to the real clipboard, if more than one item is marked, all names are copied to one single line
	//if bWithPath is TRUE, the pathes are added to the names.
	//if bWithPath is FALSE, only the names are copied
	//the names are searched in the column iColumnName
	//when bWithPath is TRUE, iColumnPath or cpPath must be valid. cpPath is checked for NULL, iColumnPath is checked for < 0
	TRACEIT(10,"markedNamesToClipboard		start");
	GtkTreeSelection	*selection;
	struct AllInformationAllUseStruct stru;
	GString *gstrNames=NULL;
	GtkClipboard *ClipBoard=NULL;

	if (TreeView == NULL){
		TRACEIT(2,"markedNamesToClipboard		end, PARAMETER NULL");
		return;
	}

	//validate:
	if (bWithPath == TRUE && cpPath==NULL && iColumnPath<0){
		TRACEIT(2,"markedNamesToClipboard		end, parameter validation error");
		return;
	}

	gstrNames = g_string_sized_new(10);
	stru.Info_1 = (void *)gstrNames;
	stru.Info_2 = (void *)&iColumnName;
	stru.Info_3 = (void *)&bWithPath;
	stru.Info_4 = (void *)&iColumnPath;
	stru.Info_5 = (void *)cpPath;

	selection  = gtk_tree_view_get_selection(TreeView);
	gtk_tree_selection_selected_foreach(selection, markedNamesToClipboard_Helper, (void *)&stru);

	//printf("gstrNames = %s\nlen=%d\n",gstrNames->str, gstrNames->len);

	ClipBoard = gtk_clipboard_get (GDK_SELECTION_CLIPBOARD);
	if (ClipBoard != NULL){
		gtk_clipboard_set_text (ClipBoard, gstrNames->str, gstrNames->len);
	}
	else{
		TRACEIT(2,"markedNamesToClipboard		end, could not achive clipboard object");
	}

	if (gstrNames)
		g_string_free(gstrNames, TRUE);

	TRACEIT(10,"markedNamesToClipboard		end");
}


static void mySelectAllFunction_Helper(GtkTreeModel *model,GtkTreePath *path,GtkTreeIter *iter,gpointer data){
    //if (gtk_tree_model_get_iter(model, iter, path)){
    TRACEIT(10,"mySelectAllFunction_Helper		start");

    // 1'st POSSIBILITY
    struct AllInformationAllUseStruct *stru = (struct AllInformationAllUseStruct *)data;
    GtkTreeSelection **selection = (GtkTreeSelection **)stru->Info_2;
	int *found = (int *)stru->Info_1;
	if (*found >= 2){
	    TRACEIT(10,"mySelectAllFunction_Helper		end, . and .. already found");
        return;
	}
    gchar *Name=NULL;
	gtk_tree_model_get(model, iter, COL_NAME, &Name, -1);

	if ((strcmp(Name, "..")==0 || strcmp(Name,".")==0)){
        (*found)+=1;
        gtk_tree_selection_unselect_iter(*selection, iter);
	}

    if (Name)
        g_free(Name);
        // END 1'st POSSIBILITY

/*

    // 2'st POSSIBILITY
    gchar *Name=NULL;
    GtkTreeSelection **selection = (GtkTreeSelection **)data;
    gtk_tree_model_get(model, iter, COL_NAME, &Name, -1);
    //gtk_tree_model_get(model, iter, COL_NAME, &Name, -1);
    if (Name != NULL && !(strcmp(Name, "..")==0 || strcmp(Name,".")==0)){
        gtk_tree_selection_select_iter(*selection, iter);
	}
    if (Name)
        g_free(Name);
    // END 2'st POSSIBILITY
*/
    TRACEIT(10,"mySelectAllFunction_Helper		end");
}


gboolean mySelectAllFunction( GtkTreeView *treeview, gpointer user_data){
    //catches the signel select-all from within the listviews (side views only)
    TRACEIT(10,"mySelectAllFunction		start");

    // 1'st POSSIBILITY
    GtkTreeSelection	*selection;
    struct AllInformationAllUseStruct stru;
    int found=0;
    stru.Info_1 = (void *)&found;
    stru.Info_2 = (void *)&selection;
    stru.Info_3 = NULL;
    stru.Info_4 = NULL;
    stru.Info_5 = NULL;

    selection  = gtk_tree_view_get_selection(treeview);
    gtk_tree_selection_select_all ( selection );
    gtk_tree_selection_selected_foreach(selection, mySelectAllFunction_Helper, (void *)&stru);
    // END 1'st POSSIBILITY


/*
    // 2'st POSSIBILITY
    GtkTreeSelection *selection;    // Tested within the library directory with more than 200 entries: this solution is realy slow!!
    selection = gtk_tree_view_get_selection(treeview);
    struct SideObjectList *Object = (struct SideObjectList *)user_data;
    gtk_tree_model_foreach(GTK_TREE_MODEL(Object->listStore),(GtkTreeModelForeachFunc) mySelectAllFunction_Helper,(void *)&selection);
    // END 2'st POSSIBILITY
*/
    TRACEIT(10,"mySelectAllFunction		end");
    return TRUE;
}


static void markItemsWithSameType_Helper_1(GtkTreeModel *model,GtkTreePath *path,GtkTreeIter *iter,gpointer data){
	TRACEIT(11,"markItemsWithSameType_Helper_1		start");

    struct AllInformationAllUseStruct *stru = (struct AllInformationAllUseStruct *)data;
	struct updateList **typeList=(struct updateList **)stru->Info_1;
	int *ColType = stru->Info_2;

	gchar *type=NULL;
	gchar *typebuf=NULL;

    gtk_tree_model_get(model, iter, *ColType,&type, -1);

    typebuf = g_utf8_strdown(type,-1);
    if (*typeList != NULL){
        if (0==updateList_checkForItem_NoRef(typeList, typebuf)){
            updateList_addItem(typeList, typebuf, NULL, NULL, 0);
        }
    }
    else{
        updateList_addItem(typeList, typebuf, NULL, NULL, 0);
    }

    if (type)
        g_free(type);
    if (typebuf)
        g_free(typebuf);
	TRACEIT(11,"markItemsWithSameType_Helper_1		end");
}


static void markItemsWithSameType_Helper_2(GtkTreeModel *model,GtkTreePath *path,GtkTreeIter *iter,gpointer data){
	TRACEIT(11,"markItemsWithSameType_Helper_2		start");

    struct AllInformationAllUseStruct *stru = (struct AllInformationAllUseStruct *)data;
	struct updateList **typeList=(struct updateList **)stru->Info_1;
	int *ColType = stru->Info_2;
	GtkTreeSelection *selection = (GtkTreeSelection *)stru->Info_3;

	gchar *type=NULL;
	gchar *typebuf=NULL;

    gtk_tree_model_get(model, iter, *ColType,&type, -1);

    typebuf = g_utf8_strdown(type,-1);
    struct updateList *bufList = *typeList;
    updateList_getFirstItem(&bufList);

    if (0==updateList_checkForItem_NoRef(&bufList, typebuf)){
        gtk_tree_selection_unselect_iter(selection, iter);
    }

    if (type)
        g_free(type);
    if (typebuf)
        g_free(typebuf);
	TRACEIT(11,"markItemsWithSameType_Helper_2		end");
}


void markItemsWithSameType(GtkTreeView *treeview, int ColType){
    //selects all items in treeview wich are from the same type as the already marked one.
    //ColType holds the ID of the coloumn with the type information


    //get the types of all marked items and put them to in a list
    struct AllInformationAllUseStruct stru;
    struct updateList *typeList=NULL;

    stru.Info_1 = (void *)&typeList;
    stru.Info_2 = (void *)&ColType;

    GtkTreeSelection *selection;
	selection = gtk_tree_view_get_selection(treeview);
	gtk_tree_selection_selected_foreach(selection, markItemsWithSameType_Helper_1, (void *)&stru);

    //The list is ready, now mark all items and walk throught the view and unmark the right ones.
    stru.Info_3 = (void *)selection;
    gtk_tree_selection_select_all(selection);

    gtk_tree_selection_selected_foreach(selection, markItemsWithSameType_Helper_2, (void *)&stru);


    //now the . and .. may be marked: check this
    int found=0;
    stru.Info_1 = (void *)&found;
    stru.Info_2 = (void *)&selection;
    gtk_tree_selection_selected_foreach(selection, mySelectAllFunction_Helper, (void *)&stru);

    updateList_deleteList(&typeList);

}

