/*
    Copyright 2007, 2008, 2009, 2010 Geyer Klaus

    This file is part of Cat'sEyE.

    Cat'sEyE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Cat'sEyE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Cat'sEyE.  If not, see <http://www.gnu.org/licenses/>.
*/


//#include <gdk/gdk.h>
#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <gdk/gdkkeysyms.h>
#include <unistd.h>

#include "confighandling.h"
#include "confighandlingDefinitions.h"
#include "mytrace.h"
#include "globalDefinitions.h"
#include "helpers.h"
#include "Lists.h"
#include "userdata.h"
#include "myfm.h"
#include "CreateUsercommandDialog.h"

#include "allInclude.h"


#define INFOTEXT "\
Find all information at:\n\
catseye-fm-temp.sourceforge.net\n\
\n\
Options need to be saved before they take effect.\n\
Same counts for items moved up or down in the lists.\n\
\n\
Here are the predefined variables you can use within the\n\
command section (use them like this: $(TDIR)):\n\
\n\
\tThisDirectory\t\t=\tTDIR\n\
\tOtherDirectory\t\t=\tODIR\n\
\tLeftSideDirectory\t=\tLDIR\n\
\tRightSideDirectory\t=\tRDIR\n\
\tHome = HOME = home\n\
\tAllItemsWithoutPathUnQuoted\t\t= MIS_NP_UQ\n\
\tAllItemsWithoutPathQuoted\t\t= MIS_NP_Q\n\
\tAllItemsWithPathUnQuoted\t\t= MIS_WP_UQ\n\
\tAllItemsWithPathQuoted\t\t\t= MIS_WP_Q\n\
\tItemWithoutPathUnQuoted\t\t= MI_NP_UQ\n\
\tItemWithoutPathQuoted\t\t\t= MI_NP_Q\n\
\ttemWithPathUnQuoted\t\t\t= MI_WP_UQ\n\
\tItemWithPathQuoted\t\t\t= MI_WP_Q\n\
\n\
\tMIS_* means allways all marked files in one row\n\
\tMI_* is filled different in order which call type is active:\n\
\t\tCalltype = all: same as MIS_*\n\
\t\tCalltype = each | list: only one item at a time\n\
\n\
\tExample: If you have marked 2 files: m1.txt and m2.txt, here is what some\n\
\t\tvariables would look like:\n\
\t\t$(MIS_NP_Q) will be replaced by : 'm1.txt m2.txt'\n\
\t\t$(MIS_NP_UQ) will be replaced by : '\"m1.txt\" \"m2.txt\"'\n\
\n\
\n\
Hints:\n\
\tPressing the middle mouse button over a specific type of file\n\
\twill automaticaly mark all files of the same type in the current view.\n\
\n\
\tMiddle mouse button on notebook tab will close the current tab.\n\
\n\
\tDouble click on a notebook will add a new page or FileShelf\n\
\n\
\tDouble click an item will open the item with the default command,\n\
\tdefined in the usercommand section of the property page.\n\
\n\
\tChanging the position of a bookmark or a usercommand in the property page\n\
\tonly takes effect after saving the currently used config file.\n\
\n\
\tUse Ctrl+d to easily add all marked items to a new FileShelf.\n\
\n\
\tCat'sEyE distinguishes filetypes by their ending.\n\
\n\
\tA usercommand must be connected to some file type.\n\
\tIf you want a usercommand to be connected to all types of files\n\
\tuse the \"allfls\" keyword as a target.\n\
\n\
\tCtrl+p enables or disables the preview of pictures. You can define\n\
\twhich pictures to preview within the options section of the property page,\n\
\twhere you can define the size of the preview too\n\
\n\
\n\
Shortcuts:\n\
\tctrl + c|x|v\tcopy,cut,paste. Only works in Cat'sEyE.\n\
\t\tif you want to copy the file name to the clipboard use:\n\
\tctrl + y\t\tcopies the marked name(s) to the clipboard\n\
\talt + y\t\tcopyies the marked name(s) including their directories to the clipboard\n\
\tctrl + n\t\tnew page\n\
\tctrl + i\t\tsize of selected items\n\
\tctrl + h\t\tshow/hide hidden files\n\
\tctrl + a\t\tselect all items in a view\n\
\tctrl + s\t\tadd selected items to active fileshelf\n\
\tctrl + d\t\tadd selected items to new fileshelf\n\
\tctrl + p\t\tenable/disable picture preview\n\
\tctrl + r\t\t(un)mark current tab as read only (Cat'sEyE only)\n\
\talt + f\t\tsearch string in highlighted items\n\
\talt + c|x\t\tcopy,move selected items to the other side\n\
\n\
\tF2\t\t\trename items\n\
\tF3\t\t\tnew directory\n\
\tF4\t\t\topen terminal\n\
\tF5\t\t\tupdate view manually\n\
\tF6\t\t\tupdate view and get the size of each directorie in the current view.\n\
\t\t\tAttention: This may block Cat'sEyE for a long time in order how many\n\
\t\t\tsubdirectories there are!\n\
\n\
\n\
"

#define PROPPAGE_CAPTION "CEE Property Page"

extern const char *Names[];

enum {
    PROPPAGE_COL_USCMND_NAME,
    PROPPAGE_COL_USCMND_TARGET,
    PROPPAGE_COL_USCMND_COMMAND,
    PROPPAGE_COL_USCMND_DEFAULT,
    PROPPAGE_COL_USCMND_ITEMPOINTER,
    PROPPAGE_COL_USCMND_VISIBLE,
    PROPPAGE_COL_USCMND_NUM
};

#define PROPPAGE_USRCMND_COLTYPES  G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_POINTER, G_TYPE_BOOLEAN
#define PROPPAGE_USRCMND_COLNAMES {"Name", "Target", "Command", "Default", "ItemPointer", "Visible"}

enum {
    PROPPAGE_COL_BKMRK_NAME,
    PROPPAGE_COL_BKMRK_PATH,
    PROPPAGE_COL_BKMRK_ITEMPOINTER,
    //PROPPAGE_COL_BKMRK_VISIBLE,
    PROPPAGE_COL_BKMRK_NUM
};

#define PROPPAGE_BKMRK_COLTYPES  G_TYPE_STRING, G_TYPE_STRING, G_TYPE_POINTER//, G_TYPE_BOOLEAN
#define PROPPAGE_BKMRK_COLNAMES { "Name", "Path", "ItemPointer"/*, "Visible"*/}


enum {
	PROPPAGE_COL_COLORDER_NAME,
	PROPPAGE_COL_COLORDER_NUM
};

#define PROPPAGE_COLORDER_COLTYPES G_TYPE_STRING
#define PROPPAGE_COLORDER_COLNAMES { "Name" }



struct _propPage_BkmrkAddItem_BufStru{
    GtkWidget *Parent;
    GtkWidget *EditBox;
};

struct _propPage_ReoderLists_Stru {
    int iColumn;
    GList *glistListPointer;
};

struct _PropPage_SetVisibility_Stru {
    int iColumn;
    gboolean bValue;
};

struct _PropPage_UnHideIfKeywordFound_Stru {
    int iColumnSearch;
    int iColumnVisibility;
    char *cpKeyword;
};

struct _propPage_MainEntities {
    GtkWidget *Main_Widget;
    GtkWidget *Main_MainVBox;
    GtkWidget *Main_MainHBox;
    GtkWidget *Main_Notebook;
    GtkWidget *Main_Buttons_HBox;
    GtkWidget *Main_Button_Save;
    GtkWidget *Main_Button_Close;
    GtkWidget *Main_Button_SaveTo;
    GtkWidget *Main_Button_LoadFrom;
    GtkWidget *Main_Button_Reload;
    GtkWidget *Main_LabelOpendFile;
};

struct _propPage_InfoEntities {
    GtkWidget *Info_MainVBox;
    GtkWidget *Info_Label;
    GtkWidget *Info_ScrollWindow;
};

struct _propPage_BookmarkEntities {
	GtkWidget *Bkmrk_MainHBox;
	GtkListStore *Bkmrk_ListStore;
    GtkWidget *Bkmrk_ListView;
    GtkWidget *Bkmrk_ScrollWindow;
	GtkWidget *Bkmrk_Buttons_VBox;
	GtkWidget *Bkmrk_Button_New;
	GtkWidget *Bkmrk_Button_Edit;
	GtkWidget *Bkmrk_Button_Delete;
};

struct _propPage_UserCommandsEntities {
    GtkWidget *UsrCmnd_MainVBox;
    GtkWidget *UsrCmnd_MainHBox;
	GtkListStore *UsrCmnd_ListStore;
    GtkWidget *UsrCmnd_ListView;
    GtkWidget *UsrCmnd_ScrollWindow;
	GtkWidget *UsrCmnd_Buttons_VBox;
	GtkWidget *UsrCmnd_Button_New;
	GtkWidget *UsrCmnd_Button_Edit;
	GtkWidget *UsrCmnd_Button_Delete;
	GtkWidget *UsrCmnd_Filter_MainHBox;
	GtkWidget *UsrCmnd_Filter_1stVBox;
	GtkWidget *UsrCmnd_Filter_2ndVBox;
	GtkWidget *UsrCmnd_Filter_NameVBox;
	GtkWidget *UsrCmnd_Filter_NameLabel;
	GtkWidget *UsrCmnd_Filter_NameEntry;
	GtkWidget *UsrCmnd_Filter_TargetVBox;
	GtkWidget *UsrCmnd_Filter_TargetLabel;
	GtkWidget *UsrCmnd_Filter_TargetEntry;
	GtkWidget *UsrCmnd_Filter_CommandVBox;
	GtkWidget *UsrCmnd_Filter_CommandLabel;
	GtkWidget *UsrCmnd_Filter_CommandEntry;
	GtkWidget *UsrCmnd_Filter_DefaultVBox;
	GtkWidget *UsrCmnd_Filter_DefaultLabel;
	GtkWidget *UsrCmnd_Filter_DefaultEntry;
	GtkWidget *UsrCmnd_Filter_ResetButton;
	GtkWidget *UsrCmnd_Filter_ResetButtonVBox;
};

struct _propPage_OptionsEntities {
    GtkWidget *Option_MainVbox;
    GdkColormap *Option_Colormap;
    GtkWidget *Option_Stded_HBox_Value;
    GtkWidget *Option_Stded_Label_Value;
    GtkWidget *Option_Stded_Edit_Value;
    GtkWidget *Option_Stded_HBox_Name;
    GtkWidget *Option_Stded_Label_Name;
    GtkWidget *Option_Stded_Edit_Name;
    GtkWidget *Option_Trmnl_HBox_Value;
    GtkWidget *Option_Trmnl_Label_Value;
    GtkWidget *Option_Trmnl_Edit_Value;
    GtkWidget *Option_Trmnl_HBox_Name;
    GtkWidget *Option_Trmnl_Label_Name;
    GtkWidget *Option_Trmnl_Edit_Name;
    GtkWidget *Option_Stdd_HBox;
    GtkWidget *Option_Stdd_Label;
    GtkWidget *Option_Stdd_Edit;
    GtkWidget *Option_Stdd_Button;
    GtkWidget *Option_Svpths_HBox;
    GtkWidget *Option_Svpths_Check;
    GtkWidget *Option_Strtdl_HBox;
    GtkWidget *Option_Strtdl_Label;
    GtkWidget *Option_Strtdl_Edit;
    GtkWidget *Option_Strtdl_Button;
    GtkWidget *Option_Strtdr_HBox;
    GtkWidget *Option_Strtdr_Label;
    GtkWidget *Option_Strtdr_Edit;
    GtkWidget *Option_Strtdr_Button;
    GtkWidget *Option_ClrItm_HBox;
    GtkWidget *Option_ClrItm_Label;
    GtkWidget *Option_ClrItm_Button;
    GdkColor  Option_ClrItm_gdkColor;
    GtkWidget *Option_ClrDir_HBox;
    GtkWidget *Option_ClrDir_Label;
    GtkWidget *Option_ClrDir_Button;
    GdkColor  Option_ClrDir_gdkColor;
    GtkWidget *Option_ClrLnk_HBox;
    GtkWidget *Option_ClrLnk_Label;
    GtkWidget *Option_ClrLnk_Button;
    GdkColor  Option_ClrLnk_gdkColor;
    GtkWidget *Option_ClrMss_HBox;
    GtkWidget *Option_ClrMss_Label;
    GtkWidget *Option_ClrMss_Button;
    GdkColor  Option_ClrMss_gdkColor;
    GtkWidget *Option_PreviewSize_Label;
    GtkWidget *Option_PreviewSize_Edit;
    GtkWidget *Option_PreviewSize_HBox;
    GtkWidget *Option_PreviewEndings_Label;
    GtkWidget *Option_PreviewEndings_Edit;
    GtkWidget *Option_PreviewEndings_HBox;
    GtkWidget *Option_SaveSessionEveryMinutes_Label;
    GtkWidget *Option_SaveSessionEveryMinutes_Edit;
    GtkWidget *Option_SaveSessionEveryMinutes_HBox;
};


struct _propPage_ColumnOrder {
	GtkWidget *ClmOrder_MainVBox;
    GtkListStore *ClmOrder_ListStore;
    GtkWidget *ClmOrder_ListView;
    GtkWidget *ClmOrder_ScrollWindow;
	GtkWidget *ClmOrder_Buttons_VBox;
	GtkWidget *ClmOrder_Button_Reset;
};

//this is our main communication struct in here.
struct _propPage_MainStruct {

	struct _propPage_MainEntities 			mainEntities;
	struct _propPage_BookmarkEntities		bkmrkEntities;
	struct _propPage_OptionsEntities		OptionsEntities;
	struct _propPage_UserCommandsEntities	UserCommandsEntities;
	struct _propPage_InfoEntities           InfoEntities;
	struct _propPage_ColumnOrder			ColumnOrder;


	int iSaved;	//our saved flag, 0=not saved after last change, 1=saved after last change
};
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
//////////////////////////	globals 	////////////////////////////////////////

extern GtkWindow 		*win;
extern struct _UserCommandList *g_UserCommandList;
extern struct _OptionsList *g_OptionsList;
extern struct _OptionsList *g_BookMarkList;


////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
//////////////////////////	prototypes 	////////////////////////////////////////

int propPage_createOptionsMainWidgets (struct _propPage_MainStruct *mainStruct);
int propPage_arangeListViewAndCo(GtkListStore *listStore, GtkWidget *listView, GtkWidget *scrollwidget, char **cppNames, int iNumOfColums, int iInvisibleColumns, int iVisibilityColumn);
int propPage_createUserCommandsMainWidgets (struct _propPage_MainStruct *mainStruct);
int propPage_createBkmrkMainWidgets (struct _propPage_MainStruct *mainStruct);
int propPage_createPropPageWidgets (struct _propPage_MainStruct *mainStruct);
int propPage_openConfigFile (const char *ccpFileName, struct _propPage_MainStruct *mainStruct);
void propPage_freeMainStruct (struct _propPage_MainStruct *mainStruct);
void propPage_destroyEvent ( GtkObject *object , struct _propPage_MainStruct *mainStruct);
int propPage_fillPropPageWidgets (struct _propPage_MainStruct *mainStruct);
int propPage_fillOptionsMainWidgets (struct _propPage_MainStruct *mainStruct);
int propPage_fillBkmrkMainWidgets (struct _propPage_MainStruct *mainStruct);
int propPage_fillUserCommandsMainWidgets (struct _propPage_MainStruct *mainStruct);
int propPage_updateFileNameLabel (struct _propPage_MainStruct *mainStruct);
int propPage_assignOptionsFunctionality (struct _propPage_MainStruct *mainStruct);
int propPage_assignBkmrkFunctionality (struct _propPage_MainStruct *mainStruct);
int propPage_assignUserCommandsFunctionality (struct _propPage_MainStruct *mainStruct);
void propPage_BkmrkdeleteItem (GtkButton *button, struct _propPage_MainStruct *mainStruct);
int propPage_BkmrkAddItem (struct _propPage_MainStruct *mainStruct, int iEditOrAdd );
int propPage_assignFunctionality (struct _propPage_MainStruct *mainStruct);
void propPage_BkmrkButtonEdit (GtkButton *Button, struct _propPage_MainStruct *mainStruct);
void propPage_BkmrkButtonNew (GtkButton *Button, struct _propPage_MainStruct *mainStruct);
void propPage_UserCommandDeleteItem (GtkButton *button, struct _propPage_MainStruct *mainStruct);
int propPage_UserCommandAddItem (struct _UserCommandList *UserCommandListItem, GtkWidget *Parent);
void propPage_UserCommandButtonNew (GtkButton *button, struct _propPage_MainStruct *mainStructs);
void propPage_UserCommandButtonEdit (GtkButton *button, struct _propPage_MainStruct *mainStruct);
void propPage_OptionsChooseDirStrtdl (GtkButton *Button, struct _propPage_MainStruct *mainStruct);
void propPage_OptionsChooseDirStrtdr (GtkButton *Button, struct _propPage_MainStruct *mainStruct);
void propPage_OptionsChooseDirStdd (GtkButton *Button, struct _propPage_MainStruct *mainStruct);
int propPage_OptionsChooseDir (GtkWidget *Parent, GtkWidget *entrybox);
//gboolean propPage_OptionsoneKeyPressedInMainWidget(GtkWidget *widget, GdkEventKey *event, struct _propPage_MainStruct *mainStruct);
int propPage_assignMainFunctionality (struct _propPage_MainStruct *mainStruct);
void propPage_MainButtonClose (GtkButton *Button, struct _propPage_MainStruct *mainStruct);
void propPage_MainButtonReload (GtkButton *Button, struct _propPage_MainStruct *mainStruct);
void propPage_MainButtonLoadFrom (GtkButton *Button, struct _propPage_MainStruct *mainStruct);
void propPage_BkmrkAddItemPathButton (GtkButton *Button, struct _propPage_BkmrkAddItem_BufStru *BufStru);
gboolean propPage_BkmrkOneMouseButtonPressedInView(GtkWidget *treeview, GdkEventButton *event, struct _propPage_MainStruct *mainStruct);
gboolean propPage_BkmrkContextMenuOnListView(GtkWidget *treeview, GdkEventButton *event,  struct _propPage_MainStruct *mainStruct);
gboolean propPage_BkmrkOneKeyPressedInView(GtkWidget *widget, GdkEventKey *event, struct _propPage_MainStruct *mainStruct);
void propPage_BkmrkRowActivated (GtkTreeView *tree_view, GtkTreePath *path, GtkTreeViewColumn *column, struct _propPage_MainStruct *mainStruct);
gboolean propPage_UsrCmndContextMenuOnListView(GtkWidget *treeview, GdkEventButton *event,  struct _propPage_MainStruct *mainStruct);
gboolean propPage_UsrCmndOneMouseButtonPressedInView(GtkWidget *treeview, GdkEventButton *event, struct _propPage_MainStruct *mainStruct);
gboolean propPage_UsrCmndOneKeyPressedInView(GtkWidget *widget, GdkEventKey *event, struct _propPage_MainStruct *mainStruct);
void propPage_UsrCmndRowActivated (GtkTreeView *tree_view, GtkTreePath *path, GtkTreeViewColumn *column, struct _propPage_MainStruct *mainStruct);
int propPage_saveAllToXMLFile (struct _propPage_MainStruct *mainStruct, const char *FileName);
int propPage_writeOptionsToGlobalStruct (struct _propPage_MainStruct *mainStruct);
void propPage_MainButtonSave (GtkButton *Button, struct _propPage_MainStruct *mainStruct);
void propPage_MainButtonSaveTo (GtkButton *Button, struct _propPage_MainStruct *mainStruct);
int propPage_reorderUsercommandListByListView (struct _propPage_MainStruct *mainStruct);
int propPage_reorderBookmarkListByListView (struct _propPage_MainStruct *mainStruct);
gboolean getPointerFromListView (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, struct _propPage_ReoderLists_Stru *Struct );
void propPage_UserCommandFilterEntryBoxActivate (GtkEntry *entry, struct _propPage_UserCommandsEntities *UserCommandsEntities);
int propPage_UserCommandFilterApply (struct _propPage_UserCommandsEntities *UserCommandsEntities);
gboolean propPage_setVisibilityInListView (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, struct _PropPage_SetVisibility_Stru *Struct );
void propPage_UserCommandClearFilter (GtkButton *button, struct _propPage_UserCommandsEntities *UserCommandsEntities);
gboolean propPage_UserCommandFilterApply_foreach ( GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, struct _propPage_UserCommandsEntities *UserCommandsEntities );
int propPage_UserCommand_WouldThisFilterHideIt( GtkEntry *Entry, int iColumnSearch, GtkTreeModel *model, GtkTreeIter *iter );
int propPage_createInfoMainWidgets (struct _propPage_MainStruct *mainStruct);
int propPage_createColumnOrderWidgets (struct _propPage_MainStruct *mainStruct);
int propPage_fillColumnOrderWidgets (struct _propPage_MainStruct *mainStruct, int iUseDefault);
int propPage_assignColumnOrderFunctionality (struct _propPage_MainStruct *mainStruct);
void propPage_ClmnOrderButtonReset (GtkButton *Button, struct _propPage_MainStruct *mainStruct);
int propPage_writeColumnOrderToGlobalStruct (struct _propPage_MainStruct *mainStruct);

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
//////////////////////////	My globals 	////////////////////////////////////////

static struct _propPage_MainStruct *g_mainStruct=NULL;  //we need this in propPage_external_fillPropPageWidgets and in CreateUsercommand.c
static int g_iWidgetIsRunning=0;
static GList *g_glistUserCommandWidgets=NULL;   //stores pointer to widgets from CreateUserCommandDialog.c if the usercommandstructitem isnt NULL

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
//////////////////////////	functions 	////////////////////////////////////////

void propPage_startPropPage (){
	TRACEIT (10, "propPage_startPropPage			start");
	//printf("g_iWidgetIsRunning = %d\n",g_iWidgetIsRunning);
	if (g_iWidgetIsRunning != 0){
        TRACEIT (10, "propPage_startPropPage            end, allready running");
        return;
	}
	g_iWidgetIsRunning = 1;
    //This is our main entry point for the property pages.
    g_mainStruct = NULL;
	struct _propPage_MainStruct *mainStruct = malloc (sizeof (struct _propPage_MainStruct));
	if (mainStruct == NULL){
		TRACEIT (2, "propPage_startPropPage			unable to allocate memory");
		return;
	}
    g_mainStruct = mainStruct;
	// initialize mainStruct members
	mainStruct->iSaved = 1;	//1=saved after last change, because we just have loaded the information

	propPage_openConfigFile (NULL, mainStruct);	//Null loads file which is currently in use, confighandling.c knows more

    propPage_createPropPageWidgets (mainStruct);

    propPage_fillPropPageWidgets (mainStruct);

    propPage_assignFunctionality (mainStruct);

    gtk_widget_show_all(mainStruct->mainEntities.Main_Widget);

	TRACEIT (10, "propPage_startPropPage			end");
}


void propPage_destroyEvent ( GtkObject *object , struct _propPage_MainStruct *mainStruct){
	//connected to the destroy event of the main widget, frees all the memory reseverd
	TRACEIT (10, "propPage_destroyEvent			start");
	if (mainStruct == NULL){
		TRACEIT (2, "propPage_destroyEvent			parameter NULL");
		return;
	}

    configFileHandling_setPreventReloadFlag (0);
	propPage_freeMainStruct (mainStruct);
    g_iWidgetIsRunning = 0;
	TRACEIT (10, "propPage_destroyEvent			end");
}


void propPage_freeMainStruct (struct _propPage_MainStruct *mainStruct){
	TRACEIT (10, "propPage_freeMainStruct			start");

	gdk_colormap_free_colors (mainStruct->OptionsEntities.Option_Colormap, &(mainStruct->OptionsEntities.Option_ClrItm_gdkColor), 1);
    gdk_colormap_free_colors (mainStruct->OptionsEntities.Option_Colormap, &(mainStruct->OptionsEntities.Option_ClrLnk_gdkColor), 1);
    gdk_colormap_free_colors (mainStruct->OptionsEntities.Option_Colormap, &(mainStruct->OptionsEntities.Option_ClrDir_gdkColor), 1);
    gdk_colormap_free_colors (mainStruct->OptionsEntities.Option_Colormap, &(mainStruct->OptionsEntities.Option_ClrMss_gdkColor), 1);

    g_object_ref(mainStruct->OptionsEntities.Option_Colormap);

	free (mainStruct);

	g_list_free (g_glistUserCommandWidgets);

	TRACEIT (10, "propPage_freeMainStruct			end");
}


int propPage_openConfigFile (const char *ccpFileName, struct _propPage_MainStruct *mainStruct){
	TRACEIT (10, "propPage_openConfigFile			start");
	//opens the file in ccpFileName and reads in its properties.
	//this functions uses the functions from confighandling.c
	//this means, loading a file with this will effect whole Cat'sEyE
	//if ccpFileName == NULL, the currently file in use simply will be reloaded.
    if (mainStruct == NULL){
        TRACEIT (10, "propPage_openConfigFile			parameter NULL");
        return -1;
    }

    if (g_glistUserCommandWidgets != NULL){ //then the lists holds all the CreateUsercommandDialogs which hold a reverence to a
        //userCommandListItem which most propably becomes invalid after we have loaded a file
        GtkWidget *dialog = CreateMyDialog("Close",(GtkWindow *)mainStruct->mainEntities.Main_Widget,2,"Ok",GTK_RESPONSE_ACCEPT,"Cancel",GTK_RESPONSE_REJECT, NULL, 0,  TRUE);
        GtkWidget *label = gtk_label_new("There is at least one UserCommandDialog open which holds\na reverence on an internal struct.\nBefore loading a new file, you have to close this dialogs.\n\nPress 'Cancel' to close the dialogs by yourself or\n'Ok' if you want me to close the dialogs for you and load the specified file.\n");
        gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox),label);
        gtk_widget_show_all(dialog);
        int iDRet = gtk_dialog_run(GTK_DIALOG(dialog));
        if (iDRet != GTK_RESPONSE_ACCEPT){
            gtk_widget_destroy(dialog);
            TRACEIT(10,"propPage_openConfigFile		end");
            return 0;
        }
        gtk_widget_destroy(dialog);

        GtkWidget *wBuf = NULL;
        while (g_glistUserCommandWidgets){
            while (g_glistUserCommandWidgets != NULL && wBuf == (GtkWidget *)g_glistUserCommandWidgets->data){
                usleep(10);
            }
            wBuf = (GtkWidget *)g_glistUserCommandWidgets->data;
            gtk_widget_destroy ((GtkWidget *)g_glistUserCommandWidgets->data);
        }
    }

	if (ccpFileName != NULL){
		configFileHandling_defineConfigFileName (ccpFileName);
	}
	configFileHandling_setPreventReloadFlag (0);
	configFileHandling_loadConfigFile(TRUE, FALSE);
    configFileHandling_setPreventReloadFlag (1);

	TRACEIT (10, "propPage_openConfigFile			end");
	return 0;
}


int propPage_updateFileNameLabel (struct _propPage_MainStruct *mainStruct){
    TRACEIT (10, "propPage_updateFileNameLabel			start");
	if (mainStruct == NULL){
		TRACEIT (2, "propPage_updateFileNameLabel			parameter NULL");
		return -1;
	}
    const char *ccpConfigFileName = configFileHandling_getConfigFileName ();
    if (ccpConfigFileName == NULL){
        gtk_label_set_text ((GtkLabel *)mainStruct->mainEntities.Main_LabelOpendFile , "n.a.");
    }
    else{
        gtk_label_set_text ((GtkLabel *)mainStruct->mainEntities.Main_LabelOpendFile , ccpConfigFileName);
    }

    TRACEIT (10, "propPage_updateFileNameLabel			end");
    return 0;
}


int propPage_createPropPageWidgets (struct _propPage_MainStruct *mainStruct){
	TRACEIT (10, "propPage_createPropPageWidgets			start");
	if (mainStruct == NULL){
		TRACEIT (2, "propPage_createPropPageWidgets			parameter NULL");
		return -1;
	}

	struct _propPage_MainEntities *mainEntities = &(mainStruct->mainEntities);

    mainEntities->Main_Widget                   = CreateMyWindow(PROPPAGE_CAPTION, win, -1, -1, TRUE, TRUE, FALSE, NULL, FALSE, TRUE);
	mainEntities->Main_MainVBox                 = gtk_vbox_new (FALSE, 5);
	mainEntities->Main_MainHBox                 = gtk_hbox_new (FALSE, 0);
    mainEntities->Main_Notebook                 = gtk_notebook_new ();
    mainEntities->Main_Buttons_HBox             = gtk_hbox_new (TRUE, 0);
    mainEntities->Main_Button_Save              = gtk_button_new_with_label ("Save");
    mainEntities->Main_Button_Close             = gtk_button_new_with_label ("Close");
    mainEntities->Main_Button_SaveTo            = gtk_button_new_with_label ("Save as");
    mainEntities->Main_Button_LoadFrom          = gtk_button_new_with_label ("Load from");
    mainEntities->Main_Button_Reload            = gtk_button_new_with_label ("Reload");
    mainEntities->Main_LabelOpendFile           = gtk_label_new ("n.a.");

    gtk_box_pack_start ( (GtkBox *)mainEntities->Main_Buttons_HBox, mainEntities->Main_Button_Save, FALSE, FALSE, 5);
    gtk_box_pack_start ( (GtkBox *)mainEntities->Main_Buttons_HBox, mainEntities->Main_Button_SaveTo, FALSE, FALSE, 5);
    gtk_box_pack_start ( (GtkBox *)mainEntities->Main_Buttons_HBox, mainEntities->Main_Button_LoadFrom, FALSE, FALSE, 5);
    gtk_box_pack_start ( (GtkBox *)mainEntities->Main_Buttons_HBox, mainEntities->Main_Button_Reload, FALSE, FALSE, 5);
    gtk_box_pack_start ( (GtkBox *)mainEntities->Main_Buttons_HBox, mainEntities->Main_Button_Close, FALSE, FALSE, 5);

    gtk_box_pack_start ( (GtkBox *)mainEntities->Main_MainVBox, mainEntities->Main_Notebook, TRUE, TRUE, 5);
    gtk_box_pack_start ( (GtkBox *)mainEntities->Main_MainVBox, mainEntities->Main_LabelOpendFile, FALSE, TRUE, 0);
    gtk_box_pack_start ( (GtkBox *)mainEntities->Main_MainVBox, gtk_hseparator_new(), FALSE, FALSE, 2);
    gtk_box_pack_start ( (GtkBox *)mainEntities->Main_MainVBox, mainEntities->Main_Buttons_HBox, FALSE, TRUE, 3);

	propPage_createOptionsMainWidgets (mainStruct);
	propPage_createBkmrkMainWidgets (mainStruct);
	propPage_createUserCommandsMainWidgets (mainStruct);
	propPage_createColumnOrderWidgets (mainStruct);
	propPage_createInfoMainWidgets (mainStruct);

    gtk_notebook_append_page ((GtkNotebook *)mainEntities->Main_Notebook, mainStruct->OptionsEntities.Option_MainVbox, gtk_label_new("Options"));
    gtk_notebook_append_page ((GtkNotebook *)mainEntities->Main_Notebook, mainStruct->bkmrkEntities.Bkmrk_MainHBox, gtk_label_new("Bookmarks"));
    gtk_notebook_append_page ((GtkNotebook *)mainEntities->Main_Notebook, mainStruct->UserCommandsEntities.UsrCmnd_MainVBox, gtk_label_new("User commands"));
    gtk_notebook_append_page ((GtkNotebook *)mainEntities->Main_Notebook, mainStruct->ColumnOrder.ClmOrder_MainVBox, gtk_label_new("Column order"));
    gtk_notebook_append_page ((GtkNotebook *)mainEntities->Main_Notebook, mainStruct->InfoEntities.Info_MainVBox, gtk_label_new("Info"));

    gtk_box_pack_start ( (GtkBox *)mainEntities->Main_MainHBox, mainEntities->Main_MainVBox, TRUE, TRUE, 5);
    gtk_container_add(GTK_CONTAINER(mainEntities->Main_Widget),mainEntities->Main_MainHBox);

    g_signal_connect (mainEntities->Main_Widget, "destroy", G_CALLBACK(propPage_destroyEvent),mainStruct);

	TRACEIT (10, "propPage_createPropPageWidgets			end");
	return 0;
}


int propPage_createInfoMainWidgets (struct _propPage_MainStruct *mainStruct){
    TRACEIT (10, "propPage_createInfoMainWidgets			start");
	if (mainStruct == NULL){
		TRACEIT (2, "propPage_createInfoMainWidgets			parameter NULL");
		return -1;
	}
    mainStruct->InfoEntities.Info_MainVBox      = gtk_vbox_new(FALSE, 0);
    mainStruct->InfoEntities.Info_Label         = gtk_label_new( INFOTEXT );
    mainStruct->InfoEntities.Info_ScrollWindow  = gtk_scrolled_window_new (NULL,NULL);

    gtk_label_set_selectable ( (GtkLabel *)mainStruct->InfoEntities.Info_Label, TRUE);

    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(mainStruct->InfoEntities.Info_ScrollWindow),GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);
    gtk_scrolled_window_add_with_viewport((GtkScrolledWindow *)(mainStruct->InfoEntities.Info_ScrollWindow),mainStruct->InfoEntities.Info_Label );

    gtk_container_add(GTK_CONTAINER(mainStruct->InfoEntities.Info_MainVBox), mainStruct->InfoEntities.Info_ScrollWindow);

    TRACEIT (10, "propPage_createInfoMainWidgets			end");
	return -1;
}


int propPage_createBkmrkMainWidgets (struct _propPage_MainStruct *mainStruct){
    TRACEIT (10, "propPage_createBkmrkMainWidgets			start");
	if (mainStruct == NULL){
		TRACEIT (2, "propPage_createBkmrkMainWidgets			parameter NULL");
		return -1;
	}
    struct _propPage_BookmarkEntities *bkmrkEntities = &(mainStruct->bkmrkEntities);


    bkmrkEntities->Bkmrk_MainHBox     = gtk_hbox_new(FALSE, 5);
	bkmrkEntities->Bkmrk_ListStore    = gtk_list_store_new(PROPPAGE_COL_BKMRK_NUM, PROPPAGE_BKMRK_COLTYPES);
    bkmrkEntities->Bkmrk_ListView     = gtk_tree_view_new ();
    bkmrkEntities->Bkmrk_ScrollWindow = gtk_scrolled_window_new (NULL,NULL);
	bkmrkEntities->Bkmrk_Buttons_VBox = gtk_vbox_new(FALSE, 5);
	bkmrkEntities->Bkmrk_Button_New   = gtk_button_new_with_label("New");
	bkmrkEntities->Bkmrk_Button_Edit  = gtk_button_new_with_label("Edit");
	bkmrkEntities->Bkmrk_Button_Delete= gtk_button_new_with_label("Delete");

    char *ccpNames[] = PROPPAGE_BKMRK_COLNAMES;

    propPage_arangeListViewAndCo(bkmrkEntities->Bkmrk_ListStore, bkmrkEntities->Bkmrk_ListView, bkmrkEntities->Bkmrk_ScrollWindow, ccpNames, PROPPAGE_COL_BKMRK_NUM, PROPPAGE_COL_BKMRK_ITEMPOINTER, -1);

    gtk_box_pack_start ((GtkBox *)bkmrkEntities->Bkmrk_Buttons_VBox, bkmrkEntities->Bkmrk_Button_New, FALSE, FALSE, 0);
    gtk_box_pack_start ((GtkBox *)bkmrkEntities->Bkmrk_Buttons_VBox, bkmrkEntities->Bkmrk_Button_Edit, FALSE, FALSE, 0);
    gtk_box_pack_start ((GtkBox *)bkmrkEntities->Bkmrk_Buttons_VBox, bkmrkEntities->Bkmrk_Button_Delete, FALSE, FALSE, 0);

    gtk_box_pack_start ((GtkBox *)bkmrkEntities->Bkmrk_MainHBox, bkmrkEntities->Bkmrk_ScrollWindow, TRUE, TRUE, 0);
    gtk_box_pack_start ((GtkBox *)bkmrkEntities->Bkmrk_MainHBox, bkmrkEntities->Bkmrk_Buttons_VBox, FALSE, FALSE, 0);


    gtk_tree_view_set_reorderable ((GtkTreeView *)bkmrkEntities->Bkmrk_ListView, TRUE );

    TRACEIT (10, "propPage_createBkmrkMainWidgets			end");
	return 0;
}


int propPage_createColumnOrderWidgets (struct _propPage_MainStruct *mainStruct){
    TRACEIT (10, "propPage_createColumnOrderWidgets			start");
	if (mainStruct == NULL){
		TRACEIT (2, "propPage_createColumnOrderWidgets			parameter NULL");
		return -1;
	}

	struct _propPage_ColumnOrder *clmOrder = &(mainStruct->ColumnOrder);

	clmOrder->ClmOrder_MainVBox = gtk_vbox_new(FALSE, 5);
	clmOrder->ClmOrder_ListStore = gtk_list_store_new (PROPPAGE_COL_COLORDER_NUM, PROPPAGE_COLORDER_COLTYPES);
	clmOrder->ClmOrder_ListView = gtk_tree_view_new ();
	clmOrder->ClmOrder_ScrollWindow = gtk_scrolled_window_new (NULL, NULL);
	clmOrder->ClmOrder_Buttons_VBox = gtk_vbox_new (FALSE, 5);
	clmOrder->ClmOrder_Button_Reset = gtk_button_new_with_label ("reset to default");

	char *cppNames[] = PROPPAGE_COLORDER_COLNAMES;

	propPage_arangeListViewAndCo(clmOrder->ClmOrder_ListStore, clmOrder->ClmOrder_ListView, clmOrder->ClmOrder_ScrollWindow, cppNames, PROPPAGE_COL_COLORDER_NUM, -1, -1);

	gtk_box_pack_start ((GtkBox *)clmOrder->ClmOrder_Buttons_VBox, clmOrder->ClmOrder_Button_Reset, FALSE, FALSE, 0);

    gtk_box_pack_start ((GtkBox *)clmOrder->ClmOrder_MainVBox, clmOrder->ClmOrder_ScrollWindow, TRUE, TRUE, 0);
    gtk_box_pack_start ((GtkBox *)clmOrder->ClmOrder_MainVBox, clmOrder->ClmOrder_Buttons_VBox, FALSE, FALSE, 0);

    gtk_tree_view_set_reorderable ((GtkTreeView *)clmOrder->ClmOrder_ListView, TRUE );
	return 0;
}


int propPage_createUserCommandsMainWidgets (struct _propPage_MainStruct *mainStruct){
    TRACEIT (10, "propPage_createUserCommandsMainWidgets			start");
	if (mainStruct == NULL){
		TRACEIT (2, "propPage_createUserCommandsMainWidgets			parameter NULL");
		return -1;
	}
    struct _propPage_UserCommandsEntities *UserCommandsEntities = &(mainStruct->UserCommandsEntities);

    UserCommandsEntities->UsrCmnd_MainVBox     = gtk_vbox_new(FALSE, 0);
    UserCommandsEntities->UsrCmnd_MainHBox     = gtk_hbox_new(FALSE, 5);
	UserCommandsEntities->UsrCmnd_ListStore    = gtk_list_store_new(PROPPAGE_COL_USCMND_NUM, PROPPAGE_USRCMND_COLTYPES);
    UserCommandsEntities->UsrCmnd_ListView     = gtk_tree_view_new ();
    UserCommandsEntities->UsrCmnd_ScrollWindow = gtk_scrolled_window_new (NULL,NULL);
	UserCommandsEntities->UsrCmnd_Buttons_VBox = gtk_vbox_new(FALSE, 5);
	UserCommandsEntities->UsrCmnd_Button_New   = gtk_button_new_with_label("New");
	UserCommandsEntities->UsrCmnd_Button_Edit  = gtk_button_new_with_label("Edit");
	UserCommandsEntities->UsrCmnd_Button_Delete= gtk_button_new_with_label("Delete");
	UserCommandsEntities->UsrCmnd_Filter_MainHBox       = gtk_hbox_new(FALSE, 5);
	UserCommandsEntities->UsrCmnd_Filter_1stVBox        = gtk_vbox_new(FALSE, 0);
	UserCommandsEntities->UsrCmnd_Filter_2ndVBox        = gtk_vbox_new(FALSE, 0);
	UserCommandsEntities->UsrCmnd_Filter_NameVBox       = gtk_vbox_new(FALSE, 0);
	UserCommandsEntities->UsrCmnd_Filter_NameLabel      = gtk_label_new ("Name(s):");
	UserCommandsEntities->UsrCmnd_Filter_NameEntry      = gtk_entry_new ();
	UserCommandsEntities->UsrCmnd_Filter_TargetVBox     = gtk_vbox_new(FALSE, 0);
	UserCommandsEntities->UsrCmnd_Filter_TargetLabel    = gtk_label_new ("Target(s):");
	UserCommandsEntities->UsrCmnd_Filter_TargetEntry    = gtk_entry_new ();
	UserCommandsEntities->UsrCmnd_Filter_CommandVBox    = gtk_vbox_new(FALSE, 0);
	UserCommandsEntities->UsrCmnd_Filter_CommandLabel   = gtk_label_new ("Command(s):");
	UserCommandsEntities->UsrCmnd_Filter_CommandEntry   = gtk_entry_new ();
	UserCommandsEntities->UsrCmnd_Filter_DefaultVBox    = gtk_vbox_new(FALSE, 0);
	UserCommandsEntities->UsrCmnd_Filter_DefaultLabel   = gtk_label_new ("Default:");
	UserCommandsEntities->UsrCmnd_Filter_DefaultEntry   = gtk_entry_new ();
	UserCommandsEntities->UsrCmnd_Filter_ResetButtonVBox= gtk_vbox_new(FALSE, 0);
	UserCommandsEntities->UsrCmnd_Filter_ResetButton    = gtk_button_new_with_label ("reset filter");

    char *ccpNames[] = PROPPAGE_USRCMND_COLNAMES;

    propPage_arangeListViewAndCo(UserCommandsEntities->UsrCmnd_ListStore, UserCommandsEntities->UsrCmnd_ListView, UserCommandsEntities->UsrCmnd_ScrollWindow, ccpNames, PROPPAGE_COL_USCMND_NUM, PROPPAGE_COL_USCMND_ITEMPOINTER, PROPPAGE_COL_USCMND_VISIBLE);

    gtk_box_pack_start ((GtkBox *)UserCommandsEntities->UsrCmnd_Buttons_VBox, UserCommandsEntities->UsrCmnd_Button_New, FALSE, FALSE, 0);
    gtk_box_pack_start ((GtkBox *)UserCommandsEntities->UsrCmnd_Buttons_VBox, UserCommandsEntities->UsrCmnd_Button_Edit, FALSE, FALSE, 0);
    gtk_box_pack_start ((GtkBox *)UserCommandsEntities->UsrCmnd_Buttons_VBox, UserCommandsEntities->UsrCmnd_Button_Delete, FALSE, FALSE, 0);

    gtk_box_pack_start ((GtkBox *)UserCommandsEntities->UsrCmnd_Filter_NameVBox, UserCommandsEntities->UsrCmnd_Filter_NameLabel, FALSE, FALSE, 0);
    gtk_box_pack_start ((GtkBox *)UserCommandsEntities->UsrCmnd_Filter_NameVBox, UserCommandsEntities->UsrCmnd_Filter_NameEntry, TRUE, TRUE, 0);
    gtk_box_pack_start ((GtkBox *)UserCommandsEntities->UsrCmnd_Filter_TargetVBox, UserCommandsEntities->UsrCmnd_Filter_TargetLabel, FALSE, FALSE, 0);
    gtk_box_pack_start ((GtkBox *)UserCommandsEntities->UsrCmnd_Filter_TargetVBox, UserCommandsEntities->UsrCmnd_Filter_TargetEntry, TRUE, TRUE, 0);
    gtk_box_pack_start ((GtkBox *)UserCommandsEntities->UsrCmnd_Filter_CommandVBox, UserCommandsEntities->UsrCmnd_Filter_CommandLabel, FALSE, FALSE, 0);
    gtk_box_pack_start ((GtkBox *)UserCommandsEntities->UsrCmnd_Filter_CommandVBox, UserCommandsEntities->UsrCmnd_Filter_CommandEntry, TRUE, TRUE, 0);
    gtk_box_pack_start ((GtkBox *)UserCommandsEntities->UsrCmnd_Filter_DefaultVBox, UserCommandsEntities->UsrCmnd_Filter_DefaultLabel, FALSE, FALSE, 0);
    gtk_box_pack_start ((GtkBox *)UserCommandsEntities->UsrCmnd_Filter_DefaultVBox, UserCommandsEntities->UsrCmnd_Filter_DefaultEntry, TRUE, TRUE, 0);
    gtk_box_pack_end ((GtkBox *)UserCommandsEntities->UsrCmnd_Filter_ResetButtonVBox, UserCommandsEntities->UsrCmnd_Filter_ResetButton, FALSE, FALSE, 0);

    gtk_box_pack_start ((GtkBox *)UserCommandsEntities->UsrCmnd_Filter_1stVBox, UserCommandsEntities->UsrCmnd_Filter_NameVBox, TRUE, TRUE, 0);
    gtk_box_pack_start ((GtkBox *)UserCommandsEntities->UsrCmnd_Filter_1stVBox, UserCommandsEntities->UsrCmnd_Filter_TargetVBox, TRUE, TRUE, 0);

    gtk_box_pack_start ((GtkBox *)UserCommandsEntities->UsrCmnd_Filter_2ndVBox, UserCommandsEntities->UsrCmnd_Filter_CommandVBox, TRUE, TRUE, 0);
    gtk_box_pack_start ((GtkBox *)UserCommandsEntities->UsrCmnd_Filter_2ndVBox, UserCommandsEntities->UsrCmnd_Filter_DefaultVBox, TRUE, TRUE, 0);

    gtk_box_pack_start ((GtkBox *)UserCommandsEntities->UsrCmnd_Filter_MainHBox, UserCommandsEntities->UsrCmnd_Filter_1stVBox, TRUE, TRUE, 0);
    gtk_box_pack_start ((GtkBox *)UserCommandsEntities->UsrCmnd_Filter_MainHBox, UserCommandsEntities->UsrCmnd_Filter_2ndVBox, TRUE, TRUE, 0);
    gtk_box_pack_start ((GtkBox *)UserCommandsEntities->UsrCmnd_Filter_MainHBox, UserCommandsEntities->UsrCmnd_Filter_ResetButtonVBox, FALSE, FALSE, 0);

    gtk_box_pack_start ((GtkBox *)UserCommandsEntities->UsrCmnd_MainHBox, UserCommandsEntities->UsrCmnd_ScrollWindow, TRUE, TRUE, 0);
    gtk_box_pack_start ((GtkBox *)UserCommandsEntities->UsrCmnd_MainHBox, UserCommandsEntities->UsrCmnd_Buttons_VBox, FALSE, FALSE, 0);

    gtk_box_pack_start ((GtkBox *)UserCommandsEntities->UsrCmnd_MainVBox, UserCommandsEntities->UsrCmnd_MainHBox, TRUE, TRUE, 0);
    gtk_box_pack_start ((GtkBox *)UserCommandsEntities->UsrCmnd_MainVBox, gtk_hseparator_new(), FALSE, FALSE, 4);
    gtk_box_pack_start ((GtkBox *)UserCommandsEntities->UsrCmnd_MainVBox, UserCommandsEntities->UsrCmnd_Filter_MainHBox, FALSE, FALSE, 0);

    gtk_tree_view_set_reorderable ((GtkTreeView *)UserCommandsEntities->UsrCmnd_ListView, TRUE );

    TRACEIT (10, "propPage_createUserCommandsMainWidgets			end");
	return 0;
}


int propPage_arangeListViewAndCo(GtkListStore *listStore, GtkWidget *listView, GtkWidget *scrollwidget, char **cppNames, int iNumOfColums, int iInvisibleColumns, int iVisibilityColumn){
    TRACEIT (10, "propPage_arangeListViewAndCo			start");

    if (listStore==NULL || listView==NULL || scrollwidget==NULL || cppNames==NULL){
        TRACEIT (2, "propPage_arangeListViewAndCo			parameter NULL");
        return -1;
    }

    GtkTreeViewColumn   *col;
	GtkCellRenderer     *renderer;
    int i;

	for (i=0; i<iNumOfColums ;i++){
		col=gtk_tree_view_column_new();

		gtk_tree_view_column_set_resizable(col,TRUE);
		gtk_tree_view_column_set_reorderable(col,TRUE);
		gtk_tree_view_column_set_title(col, cppNames[i]);
		if (iInvisibleColumns != -1 && i >= iInvisibleColumns){
            gtk_tree_view_column_set_visible(col,FALSE);
        }
        else{
            renderer = gtk_cell_renderer_text_new();
            gtk_tree_view_column_pack_start(col, renderer, TRUE);
            gtk_tree_view_column_add_attribute(col,renderer,"text",i);
            gtk_tree_view_append_column(GTK_TREE_VIEW(listView),col);
            //gtk_tree_view_column_set_sort_column_id(col,i);
            if (iVisibilityColumn != -1){
                gtk_tree_view_column_add_attribute(col,renderer,"visible",iVisibilityColumn);
            }
        }
	}

    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrollwidget),GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);
    gtk_tree_view_set_model (GTK_TREE_VIEW(listView), GTK_TREE_MODEL(listStore));
	gtk_container_add(GTK_CONTAINER(scrollwidget),listView);

    g_object_unref(listStore);

    TRACEIT (10, "propPage_arangeListViewAndCo			end");
    return 0;
}


int propPage_createOptionsMainWidgets (struct _propPage_MainStruct *mainStruct){
	TRACEIT (10, "propPage_createOptionsMainWidgets			start");
	if (mainStruct == NULL){
		TRACEIT (2, "propPage_createOptionsMainWidgets			parameter NULL");
		return -1;
	}
    struct _propPage_OptionsEntities *OptionsEntities = &(mainStruct->OptionsEntities);

    OptionsEntities->Option_MainVbox    = gtk_vbox_new (FALSE, 5);
    OptionsEntities->Option_Stded_HBox_Name  = gtk_hbox_new (FALSE, 0);
    OptionsEntities->Option_Stded_Label_Name = gtk_label_new ("Standard editor name:");
    OptionsEntities->Option_Stded_Edit_Name  = gtk_entry_new ();
    OptionsEntities->Option_Stded_HBox_Value  = gtk_hbox_new (FALSE, 0);
    OptionsEntities->Option_Stded_Label_Value = gtk_label_new ("Standard editor:");
    OptionsEntities->Option_Stded_Edit_Value  = gtk_entry_new ();
    OptionsEntities->Option_Trmnl_HBox_Name  = gtk_hbox_new (FALSE, 0);
    OptionsEntities->Option_Trmnl_Label_Name = gtk_label_new ("Terminal name:");
    OptionsEntities->Option_Trmnl_Edit_Name  = gtk_entry_new ();
    OptionsEntities->Option_Trmnl_HBox_Value  = gtk_hbox_new (FALSE, 0);
    OptionsEntities->Option_Trmnl_Label_Value = gtk_label_new ("Terminal:");
    OptionsEntities->Option_Trmnl_Edit_Value  = gtk_entry_new ();
    OptionsEntities->Option_Stdd_HBox   = gtk_hbox_new (FALSE, 0);
    OptionsEntities->Option_Stdd_Label  = gtk_label_new ("Standard directory:");
    OptionsEntities->Option_Stdd_Edit   = gtk_entry_new ();
    OptionsEntities->Option_Stdd_Button = gtk_button_new_with_label ("search dir");
    OptionsEntities->Option_Svpths_HBox = gtk_hbox_new (FALSE, 0);
    OptionsEntities->Option_Svpths_Check = gtk_check_button_new_with_label ("Save sideview pathes on exit.");
    OptionsEntities->Option_Strtdl_HBox = gtk_hbox_new (FALSE, 0);
    OptionsEntities->Option_Strtdl_Label = gtk_label_new ("Start directory left:");
    OptionsEntities->Option_Strtdl_Edit = gtk_entry_new ();
    OptionsEntities->Option_Strtdl_Button = gtk_button_new_with_label ("search dir");
    OptionsEntities->Option_Strtdr_HBox = gtk_hbox_new (FALSE, 0);
    OptionsEntities->Option_Strtdr_Label = gtk_label_new ("Start directory right:");
    OptionsEntities->Option_Strtdr_Edit = gtk_entry_new ();
    OptionsEntities->Option_Strtdr_Button = gtk_button_new_with_label ("search dir");
    OptionsEntities->Option_ClrItm_HBox = gtk_hbox_new (FALSE, 0);
    OptionsEntities->Option_ClrItm_Label = gtk_label_new ("Color for items:");
    OptionsEntities->Option_ClrItm_Button = gtk_color_button_new();//gtk_button_new_with_label ("search dir");
    OptionsEntities->Option_ClrDir_HBox = gtk_hbox_new (FALSE, 0);
    OptionsEntities->Option_ClrDir_Label = gtk_label_new ("Color for directories:");
    OptionsEntities->Option_ClrDir_Button = gtk_color_button_new();//;gtk_button_new_with_label ("search dir");
    OptionsEntities->Option_ClrLnk_HBox = gtk_hbox_new (FALSE, 0);
    OptionsEntities->Option_ClrLnk_Label = gtk_label_new ("Color for links:");
    OptionsEntities->Option_ClrLnk_Button = gtk_color_button_new();//gtk_button_new_with_label ("color");
    OptionsEntities->Option_ClrMss_HBox = gtk_hbox_new (FALSE, 0);
    OptionsEntities->Option_ClrMss_Label = gtk_label_new ("Color for missing items:");
    OptionsEntities->Option_ClrMss_Button = gtk_color_button_new();//gtk_button_new_with_label ("search dir");
    OptionsEntities->Option_PreviewSize_Label = gtk_label_new ("Preview size:");
    OptionsEntities->Option_PreviewSize_Edit = gtk_entry_new ();
    OptionsEntities->Option_PreviewSize_HBox = gtk_hbox_new (FALSE, 0);
    OptionsEntities->Option_PreviewEndings_Label = gtk_label_new ("Preview of files ending:");
    OptionsEntities->Option_PreviewEndings_Edit = gtk_entry_new ();
    OptionsEntities->Option_PreviewEndings_HBox = gtk_hbox_new (FALSE, 0);
    OptionsEntities->Option_SaveSessionEveryMinutes_Label =  gtk_label_new ("Save Session every N Minutes (0=off):\n(needs a restart of Cat'sEyE)");
    OptionsEntities->Option_SaveSessionEveryMinutes_Edit = gtk_entry_new();
    OptionsEntities->Option_SaveSessionEveryMinutes_HBox = gtk_hbox_new (FALSE, 0);

    gtk_box_pack_start ((GtkBox *)OptionsEntities->Option_Stded_HBox_Name, OptionsEntities->Option_Stded_Label_Name, FALSE, FALSE, 5);
    gtk_box_pack_end ((GtkBox *)OptionsEntities->Option_Stded_HBox_Name, OptionsEntities->Option_Stded_Edit_Name, TRUE, TRUE, 5);
    gtk_box_pack_start ((GtkBox *)OptionsEntities->Option_Stded_HBox_Value, OptionsEntities->Option_Stded_Label_Value, FALSE, FALSE, 5);
    gtk_box_pack_end ((GtkBox *)OptionsEntities->Option_Stded_HBox_Value, OptionsEntities->Option_Stded_Edit_Value, TRUE, TRUE, 5);

    gtk_box_pack_start ((GtkBox *)OptionsEntities->Option_Trmnl_HBox_Name, OptionsEntities->Option_Trmnl_Label_Name, FALSE, FALSE, 5);
    gtk_box_pack_end ((GtkBox *)OptionsEntities->Option_Trmnl_HBox_Name, OptionsEntities->Option_Trmnl_Edit_Name, TRUE, TRUE, 5);
    gtk_box_pack_start ((GtkBox *)OptionsEntities->Option_Trmnl_HBox_Value, OptionsEntities->Option_Trmnl_Label_Value, FALSE, FALSE, 5);
    gtk_box_pack_end ((GtkBox *)OptionsEntities->Option_Trmnl_HBox_Value, OptionsEntities->Option_Trmnl_Edit_Value, TRUE, TRUE, 5);

    gtk_box_pack_start ((GtkBox *)OptionsEntities->Option_Stdd_HBox, OptionsEntities->Option_Stdd_Label, FALSE, FALSE, 5);
    gtk_box_pack_start ((GtkBox *)OptionsEntities->Option_Stdd_HBox, OptionsEntities->Option_Stdd_Edit, TRUE, TRUE, 5);
    gtk_box_pack_end ((GtkBox *)OptionsEntities->Option_Stdd_HBox, OptionsEntities->Option_Stdd_Button, FALSE, FALSE, 5);

    gtk_box_pack_start ((GtkBox *)OptionsEntities->Option_Svpths_HBox, OptionsEntities->Option_Svpths_Check, FALSE, FALSE, 5);

    gtk_box_pack_start ((GtkBox *)OptionsEntities->Option_Strtdl_HBox, OptionsEntities->Option_Strtdl_Label, FALSE, FALSE, 5);
    gtk_box_pack_start ((GtkBox *)OptionsEntities->Option_Strtdl_HBox, OptionsEntities->Option_Strtdl_Edit, TRUE, TRUE, 5);
    gtk_box_pack_end ((GtkBox *)OptionsEntities->Option_Strtdl_HBox, OptionsEntities->Option_Strtdl_Button, FALSE, FALSE, 5);

    gtk_box_pack_start ((GtkBox *)OptionsEntities->Option_Strtdr_HBox, OptionsEntities->Option_Strtdr_Label, FALSE, FALSE, 5);
    gtk_box_pack_start ((GtkBox *)OptionsEntities->Option_Strtdr_HBox, OptionsEntities->Option_Strtdr_Edit, TRUE, TRUE, 5);
    gtk_box_pack_end ((GtkBox *)OptionsEntities->Option_Strtdr_HBox, OptionsEntities->Option_Strtdr_Button, FALSE, FALSE, 5);

    gtk_box_pack_start ((GtkBox *)OptionsEntities->Option_ClrItm_HBox, OptionsEntities->Option_ClrItm_Label, FALSE, FALSE, 5);
    gtk_box_pack_end ((GtkBox *)OptionsEntities->Option_ClrItm_HBox, OptionsEntities->Option_ClrItm_Button, FALSE, FALSE, 5);

    gtk_box_pack_start ((GtkBox *)OptionsEntities->Option_ClrDir_HBox, OptionsEntities->Option_ClrDir_Label, FALSE, FALSE, 5);
    gtk_box_pack_end ((GtkBox *)OptionsEntities->Option_ClrDir_HBox, OptionsEntities->Option_ClrDir_Button, FALSE, FALSE, 5);

    gtk_box_pack_start ((GtkBox *)OptionsEntities->Option_ClrLnk_HBox, OptionsEntities->Option_ClrLnk_Label, FALSE, FALSE, 5);
    gtk_box_pack_end ((GtkBox *)OptionsEntities->Option_ClrLnk_HBox, OptionsEntities->Option_ClrLnk_Button, FALSE, FALSE, 5);

    gtk_box_pack_start ((GtkBox *)OptionsEntities->Option_ClrMss_HBox, OptionsEntities->Option_ClrMss_Label, FALSE, FALSE, 5);
    gtk_box_pack_end ((GtkBox *)OptionsEntities->Option_ClrMss_HBox, OptionsEntities->Option_ClrMss_Button, FALSE, FALSE, 5);

    gtk_box_pack_start ((GtkBox *)OptionsEntities->Option_PreviewSize_HBox, OptionsEntities->Option_PreviewSize_Label, FALSE, FALSE, 5);
    gtk_box_pack_end ((GtkBox *)OptionsEntities->Option_PreviewSize_HBox, OptionsEntities->Option_PreviewSize_Edit, FALSE, FALSE, 5);

    gtk_box_pack_start ((GtkBox *)OptionsEntities->Option_PreviewEndings_HBox, OptionsEntities->Option_PreviewEndings_Label, FALSE, FALSE, 5);
    gtk_box_pack_end ((GtkBox *)OptionsEntities->Option_PreviewEndings_HBox, OptionsEntities->Option_PreviewEndings_Edit, FALSE, FALSE, 5);

	gtk_box_pack_start ((GtkBox *)OptionsEntities->Option_SaveSessionEveryMinutes_HBox, OptionsEntities->Option_SaveSessionEveryMinutes_Label, FALSE, FALSE, 5);
	gtk_box_pack_end ((GtkBox *)OptionsEntities->Option_SaveSessionEveryMinutes_HBox, OptionsEntities->Option_SaveSessionEveryMinutes_Edit, FALSE, FALSE, 5);


    gtk_box_pack_start ((GtkBox *)OptionsEntities->Option_MainVbox, OptionsEntities->Option_Stded_HBox_Name, FALSE, FALSE, 0);
    gtk_box_pack_start ((GtkBox *)OptionsEntities->Option_MainVbox, OptionsEntities->Option_Stded_HBox_Value, FALSE, FALSE, 0);
    gtk_box_pack_start ((GtkBox *)OptionsEntities->Option_MainVbox, OptionsEntities->Option_Trmnl_HBox_Name, FALSE, FALSE, 0);
    gtk_box_pack_start ((GtkBox *)OptionsEntities->Option_MainVbox, OptionsEntities->Option_Trmnl_HBox_Value, FALSE, FALSE, 0);
    gtk_box_pack_start ((GtkBox *)OptionsEntities->Option_MainVbox, OptionsEntities->Option_Stdd_HBox, FALSE, FALSE, 0);
    gtk_box_pack_start ((GtkBox *)OptionsEntities->Option_MainVbox, OptionsEntities->Option_Svpths_HBox, FALSE, FALSE, 0);
    gtk_box_pack_start ((GtkBox *)OptionsEntities->Option_MainVbox, OptionsEntities->Option_Strtdl_HBox, FALSE, FALSE, 0);
    gtk_box_pack_start ((GtkBox *)OptionsEntities->Option_MainVbox, OptionsEntities->Option_Strtdr_HBox, FALSE, FALSE, 0);
    gtk_box_pack_start ((GtkBox *)OptionsEntities->Option_MainVbox, OptionsEntities->Option_ClrItm_HBox, FALSE, FALSE, 0);
    gtk_box_pack_start ((GtkBox *)OptionsEntities->Option_MainVbox, OptionsEntities->Option_ClrDir_HBox, FALSE, FALSE, 0);
    gtk_box_pack_start ((GtkBox *)OptionsEntities->Option_MainVbox, OptionsEntities->Option_ClrLnk_HBox, FALSE, FALSE, 0);
    gtk_box_pack_start ((GtkBox *)OptionsEntities->Option_MainVbox, OptionsEntities->Option_ClrMss_HBox, FALSE, FALSE, 0);
    gtk_box_pack_start ((GtkBox *)OptionsEntities->Option_MainVbox, OptionsEntities->Option_PreviewSize_HBox, FALSE, FALSE, 0);
    gtk_box_pack_start ((GtkBox *)OptionsEntities->Option_MainVbox, OptionsEntities->Option_PreviewEndings_HBox, FALSE, FALSE, 0);
    gtk_box_pack_start ((GtkBox *)OptionsEntities->Option_MainVbox, OptionsEntities->Option_SaveSessionEveryMinutes_HBox, FALSE, FALSE, 0);


	TRACEIT (10, "propPage_createOptionsMainWidgets			end");
	return 0;
}


int propPage_external_fillPropPageWidgets (){
    TRACEIT (10, "propPage_external_fillPropPageWidgets			start");
    if (g_mainStruct != NULL){
        propPage_fillPropPageWidgets (g_mainStruct);
        g_mainStruct->iSaved = 0;
    }
    TRACEIT (10, "propPage_external_fillPropPageWidgets			end");
    return 0;
}


int propPage_external_deleteItemFromglistUserCommandWidgets (GtkWidget *widget){
    TRACEIT (10, "propPage_external_deleteItemFromglistUserCommandWidgets			start");
    if (widget == NULL){
        TRACEIT (2, "propPage_external_deleteItemFromglistUserCommandWidgets			parameter NULL");
        return -1;
    }

    if (g_glistUserCommandWidgets)
        g_glistUserCommandWidgets = g_list_remove (g_glistUserCommandWidgets, widget);

    TRACEIT (10, "propPage_external_deleteItemFromglistUserCommandWidgets			end");
    return 0;
}


int propPage_fillPropPageWidgets (struct _propPage_MainStruct *mainStruct){
	TRACEIT (10, "propPage_fillPropPageWidgets			start");
	if (mainStruct == NULL){
		TRACEIT (2, "propPage_fillPropPageWidgets			parameter NULL");
		return -1;
	}

    propPage_updateFileNameLabel (mainStruct);
    propPage_fillOptionsMainWidgets (mainStruct);
    propPage_fillBkmrkMainWidgets (mainStruct);
    propPage_fillColumnOrderWidgets (mainStruct, 0);
	propPage_fillUserCommandsMainWidgets (mainStruct);

    TRACEIT (10, "propPage_fillPropPageWidgets			end");
    return 0;
}


int propPage_fillUserCommandsMainWidgets (struct _propPage_MainStruct *mainStruct){
    TRACEIT (10, "propPage_fillUserCommandsMainWidgets			start");
	if (mainStruct == NULL){
		TRACEIT (2, "propPage_fillUserCommandsMainWidgets			parameter NULL");
		return -1;
	}
    struct _propPage_UserCommandsEntities *UserCommandsEntities = &(mainStruct->UserCommandsEntities);

    GtkTreeIter iter;
    int i;
    GString *gstrBuf = g_string_sized_new (10);
    GList *glistBuf = NULL;
    struct _UserCommandList *listBuf = UserCommandList_getFirstItem (g_UserCommandList);

    gtk_list_store_clear (UserCommandsEntities->UsrCmnd_ListStore);

    i=1;
    while (listBuf){
        gtk_list_store_append (UserCommandsEntities->UsrCmnd_ListStore, &iter);
        //Position
        //gtk_list_store_set (UserCommandsEntities->UsrCmnd_ListStore, &iter, PROPPAGE_COL_USCMND_POSITION, i, -1);
        //DefaultCommand
        if (listBuf->iIsDefaultCommand == 1){
            g_string_assign (gstrBuf, "true");
        }
        else{
            g_string_assign (gstrBuf, "false");
        }
        gtk_list_store_set (UserCommandsEntities->UsrCmnd_ListStore, &iter, PROPPAGE_COL_USCMND_DEFAULT, gstrBuf->str, -1);
        //Command
        g_string_assign (gstrBuf, listBuf->gstrCommand->str);
        //g_string_truncate (gstrBuf, 20);
        gtk_list_store_set (UserCommandsEntities->UsrCmnd_ListStore, &iter, PROPPAGE_COL_USCMND_COMMAND, gstrBuf->str, -1);
        //Targets
        g_string_assign (gstrBuf, "");
        glistBuf = listBuf->glistTargets;
        while (glistBuf){
            g_string_append (gstrBuf, ((GString *)(glistBuf->data))->str);
            g_string_append (gstrBuf, " ");

            glistBuf = glistBuf->next;
        }
        gtk_list_store_set (UserCommandsEntities->UsrCmnd_ListStore, &iter, PROPPAGE_COL_USCMND_TARGET, gstrBuf->str, -1);
        //Itempointer
        gtk_list_store_set (UserCommandsEntities->UsrCmnd_ListStore, &iter, PROPPAGE_COL_USCMND_ITEMPOINTER, listBuf, -1);
        //Name
        gtk_list_store_set (UserCommandsEntities->UsrCmnd_ListStore, &iter, PROPPAGE_COL_USCMND_NAME, listBuf->gstrName->str, -1);
        //visibility
        gtk_list_store_set (UserCommandsEntities->UsrCmnd_ListStore, &iter, PROPPAGE_COL_USCMND_VISIBLE, TRUE, -1);
        i++;
        listBuf = listBuf->nextItem;
    }

    propPage_UserCommandFilterApply (UserCommandsEntities);

    g_string_free (gstrBuf, TRUE);
    TRACEIT (10, "propPage_fillUserCommandsMainWidgets			end");
    return 0;
}

int propPage_fillColumnOrderWidgets (struct _propPage_MainStruct *mainStruct, int iUseDefault){
	//iUseDefault == 0 : Do not use default if there is some user information
	//				!= 0 : Default will be used
	TRACEIT (10, "propPage_fillColumnOrderWidgets			start");
	if (mainStruct == NULL){
		TRACEIT (2, "propPage_fillColumnOrderWidgets			parameter NULL");
		return -1;
	}
	struct _propPage_ColumnOrder *clmnOrder = &(mainStruct->ColumnOrder);

	//try to load users order, then start filling the listview
	const char *ccpUsersOrder = OptionsList_getValueByID (g_OptionsList, COLUMN_ORDER);
	int i;
	GString *gstrColumnNames = g_string_sized_new (100);
	GString *gstrToken = g_string_sized_new (15);
	GtkTreeIter iter;

	if (iUseDefault == 0 && NULL != ccpUsersOrder ){
		g_string_assign (gstrColumnNames, ccpUsersOrder);
	}
	else{
		g_string_append (gstrColumnNames, "\"");
		for (i=0; i<COL_TIMESTAMPACCESSED; i++){
			g_string_append (gstrColumnNames, Names[i]);

			g_string_append (gstrColumnNames, COLUMN_ORDER_SEPARATOR);
		}
		g_string_append (gstrColumnNames, "\"");
		g_string_erase (gstrColumnNames, gstrColumnNames->len-strlen(COLUMN_ORDER_SEPARATOR), -1);
	}

	gtk_list_store_clear (clmnOrder->ClmOrder_ListStore);
	//fill the listview
	for (i=0 ; 1 == myTokenFunction(gstrColumnNames, i, COLUMN_ORDER_SEPARATOR, gstrToken) ; i++){
		cutCharFromGString(gstrToken, '\"', 0);
		if ( 0 == strlen (gstrToken->str)){
			break;
		}

		gtk_list_store_append (clmnOrder->ClmOrder_ListStore, &iter);
        gtk_list_store_set (clmnOrder->ClmOrder_ListStore, &iter, PROPPAGE_COL_COLORDER_NAME, gstrToken->str, -1);
	}

	g_string_free (gstrToken, TRUE);
	g_string_free (gstrColumnNames, TRUE);

	TRACEIT (10, "propPage_fillColumnOrderWidgets			end");
	return 0;
}

int propPage_fillBkmrkMainWidgets (struct _propPage_MainStruct *mainStruct){
	TRACEIT (10, "propPage_fillBkmrkMainWidgets			start");
	if (mainStruct == NULL){
		TRACEIT (2, "propPage_fillBkmrkMainWidgets			parameter NULL");
		return -1;
	}
    struct _propPage_BookmarkEntities *bkmrkEntities = &(mainStruct->bkmrkEntities);

    GtkTreeIter iter;
    int i;

    struct _OptionsList *listBuf = OptionsList_getFirstItem (g_BookMarkList);

    gtk_list_store_clear (bkmrkEntities->Bkmrk_ListStore);

    i=1;
    while (listBuf){
        gtk_list_store_append (bkmrkEntities->Bkmrk_ListStore, &iter);
        //Position
        //gtk_list_store_set (bkmrkEntities->Bkmrk_ListStore, &iter, PROPPAGE_COL_BKMRK_POSITION, i, -1);
        //Name
        if (listBuf->gstrName)
            gtk_list_store_set (bkmrkEntities->Bkmrk_ListStore, &iter, PROPPAGE_COL_BKMRK_NAME, listBuf->gstrName->str, -1);
        //Path
        if (listBuf->gstrValue)
            gtk_list_store_set (bkmrkEntities->Bkmrk_ListStore, &iter, PROPPAGE_COL_BKMRK_PATH, listBuf->gstrValue->str, -1);
        //Itempointer
        gtk_list_store_set (bkmrkEntities->Bkmrk_ListStore, &iter, PROPPAGE_COL_BKMRK_ITEMPOINTER, listBuf, -1);

        i++;
        listBuf = listBuf->nextItem;
    }

    TRACEIT (10, "propPage_fillBkmrkMainWidgets			end");
    return 0;
}


int propPage_fillOptionsMainWidgets (struct _propPage_MainStruct *mainStruct){
	TRACEIT (10, "propPage_fillOptionsMainWidgets			start");
	if (mainStruct == NULL){
		TRACEIT (2, "propPage_fillOptionsMainWidgets			parameter NULL");
		return -1;
	}
    struct _propPage_OptionsEntities *OptionsEntities = &(mainStruct->OptionsEntities);

    const char *cpColorDir=NULL;
	const char *cpColorItm=NULL;
	const char *cpColorLnk=NULL;
	const char *cpColorMss=NULL;
	const char cpColorFallBackItms[] = DEFAULT_COLOR_ITEMS_DEFAULT;
	const char cpColorFallBackDirs[] = DEFAULT_COLOR_DIRECTORIES;
	const char cpColorFallBackLnks[] = DEFAULT_COLOR_LINKS;
	const char cpColorFallBackMss[]  = DEFAULT_COLOR_ITEMS_MISSING;

	cpColorDir = OptionsList_getValueByID (g_OptionsList, COLOR_DIRECTORIES);
	cpColorItm = OptionsList_getValueByID (g_OptionsList, COLOR_ITEMS_DEFAULT);
	cpColorLnk = OptionsList_getValueByID (g_OptionsList, COLOR_LINKS);
	cpColorMss = OptionsList_getValueByID (g_OptionsList, COLOR_ITEMS_MISSING);

    if (cpColorDir == NULL)
        cpColorDir = cpColorFallBackDirs;
    if (cpColorItm == NULL)
        cpColorItm = cpColorFallBackItms;
    if (cpColorLnk == NULL)
        cpColorLnk = cpColorFallBackLnks;
    if (cpColorMss == NULL)
        cpColorMss = cpColorFallBackMss;

    OptionsEntities->Option_Colormap = gdk_colormap_get_system ();

    gdk_colormap_alloc_color (OptionsEntities->Option_Colormap, &(OptionsEntities->Option_ClrItm_gdkColor), TRUE, TRUE);
    gdk_colormap_alloc_color (OptionsEntities->Option_Colormap, &(OptionsEntities->Option_ClrDir_gdkColor), TRUE, TRUE);
    gdk_colormap_alloc_color (OptionsEntities->Option_Colormap, &(OptionsEntities->Option_ClrLnk_gdkColor), TRUE, TRUE);
    gdk_colormap_alloc_color (OptionsEntities->Option_Colormap, &(OptionsEntities->Option_ClrMss_gdkColor), TRUE, TRUE);

    gdk_color_parse (cpColorItm, &(OptionsEntities->Option_ClrItm_gdkColor));
    gdk_color_parse (cpColorDir, &(OptionsEntities->Option_ClrDir_gdkColor));
    gdk_color_parse (cpColorLnk, &(OptionsEntities->Option_ClrLnk_gdkColor));
    gdk_color_parse (cpColorMss, &(OptionsEntities->Option_ClrMss_gdkColor));

    gtk_color_button_set_color ((GtkColorButton *)OptionsEntities->Option_ClrItm_Button, &(OptionsEntities->Option_ClrItm_gdkColor));
    gtk_color_button_set_color ((GtkColorButton *)OptionsEntities->Option_ClrDir_Button, &(OptionsEntities->Option_ClrDir_gdkColor));
    gtk_color_button_set_color ((GtkColorButton *)OptionsEntities->Option_ClrLnk_Button, &(OptionsEntities->Option_ClrLnk_gdkColor));
    gtk_color_button_set_color ((GtkColorButton *)OptionsEntities->Option_ClrMss_Button, &(OptionsEntities->Option_ClrMss_gdkColor));

    const char *ccpBuf = NULL;

    ccpBuf = OptionsList_getValueByID (g_OptionsList, STANDARDEDITORKEY);
    gtk_entry_set_text ((GtkEntry *)OptionsEntities->Option_Stded_Edit_Value, "");
    if (ccpBuf != NULL){
        gtk_entry_set_text ((GtkEntry *)OptionsEntities->Option_Stded_Edit_Value, ccpBuf);
    }
    ccpBuf = OptionsList_getNameByID (g_OptionsList, STANDARDEDITORKEY);
    gtk_entry_set_text ((GtkEntry *)OptionsEntities->Option_Stded_Edit_Name, "");
    if (ccpBuf != NULL){
        gtk_entry_set_text ((GtkEntry *)OptionsEntities->Option_Stded_Edit_Name, ccpBuf);
    }
    ccpBuf = OptionsList_getValueByID (g_OptionsList, TERMINALKEY);
    gtk_entry_set_text ((GtkEntry *)OptionsEntities->Option_Trmnl_Edit_Value, "");
    if (ccpBuf != NULL){
        gtk_entry_set_text ((GtkEntry *)OptionsEntities->Option_Trmnl_Edit_Value, ccpBuf);
    }
    ccpBuf = OptionsList_getNameByID (g_OptionsList, TERMINALKEY);
    gtk_entry_set_text ((GtkEntry *)OptionsEntities->Option_Trmnl_Edit_Name, "");
    if (ccpBuf != NULL){
        gtk_entry_set_text ((GtkEntry *)OptionsEntities->Option_Trmnl_Edit_Name, ccpBuf);
    }
    ccpBuf = OptionsList_getValueByID (g_OptionsList, STANDARDSTARTDIRECTORY);
    gtk_entry_set_text ((GtkEntry *)OptionsEntities->Option_Stdd_Edit, "");
    if (ccpBuf != NULL){
        gtk_entry_set_text ((GtkEntry *)OptionsEntities->Option_Stdd_Edit, ccpBuf);
    }
    ccpBuf = OptionsList_getValueByID (g_OptionsList, STARTDIRECTORIESLEFTKEY);
    gtk_entry_set_text ((GtkEntry *)OptionsEntities->Option_Strtdl_Edit, "");
    if (ccpBuf != NULL){
        gtk_entry_set_text ((GtkEntry *)OptionsEntities->Option_Strtdl_Edit, ccpBuf);
    }
    ccpBuf = OptionsList_getValueByID (g_OptionsList, STARTDIRECTORIESRIGHTKEY);
    gtk_entry_set_text ((GtkEntry *)OptionsEntities->Option_Strtdr_Edit, "");
    if (ccpBuf != NULL){
        gtk_entry_set_text ((GtkEntry *)OptionsEntities->Option_Strtdr_Edit, ccpBuf);
    }
    ccpBuf = OptionsList_getValueByID (g_OptionsList, SAVEPATHESONEXIT);
    if (ccpBuf != NULL && ( 0==strcmp(ccpBuf, "TRUE") || 0==strcmp(ccpBuf, "true") || 0==strcmp(ccpBuf, "True"))){
        gtk_toggle_button_set_active ( (GtkToggleButton *)OptionsEntities->Option_Svpths_Check, TRUE);
    }
    else{
        gtk_toggle_button_set_active ( (GtkToggleButton *)OptionsEntities->Option_Svpths_Check, FALSE);
    }
    ccpBuf = OptionsList_getValueByID (g_OptionsList, PREVIEWSIZE);
    gtk_entry_set_text ((GtkEntry *)OptionsEntities->Option_PreviewSize_Edit, "");
    if (ccpBuf != NULL){
        gtk_entry_set_text ((GtkEntry *)OptionsEntities->Option_PreviewSize_Edit, ccpBuf);
    }
    ccpBuf = OptionsList_getValueByID (g_OptionsList, PREVIEWTYPES);
    gtk_entry_set_text ((GtkEntry *)OptionsEntities->Option_PreviewEndings_Edit, "");
    if (ccpBuf != NULL){
        gtk_entry_set_text ((GtkEntry *)OptionsEntities->Option_PreviewEndings_Edit, ccpBuf);
    }
	ccpBuf = OptionsList_getValueByID (g_OptionsList, SAVESESSIONMINUTESTYPES);
    gtk_entry_set_text ((GtkEntry *)OptionsEntities->Option_SaveSessionEveryMinutes_Edit, "");
    if (ccpBuf != NULL){
        gtk_entry_set_text ((GtkEntry *)OptionsEntities->Option_SaveSessionEveryMinutes_Edit, ccpBuf);
    }
    else{
		gtk_entry_set_text ((GtkEntry *)OptionsEntities->Option_SaveSessionEveryMinutes_Edit, "0");
    }

    TRACEIT (10, "propPage_fillOptionsMainWidgets			end");
    return 0;
}


int propPage_assignFunctionality (struct _propPage_MainStruct *mainStruct){
	TRACEIT (10, "propPage_assignFunctionality			start");
	if (mainStruct == NULL){
		TRACEIT (2, "propPage_assignFunctionality			parameter NULL");
		return -1;
	}

    propPage_assignMainFunctionality (mainStruct);
	propPage_assignOptionsFunctionality (mainStruct);
    propPage_assignBkmrkFunctionality (mainStruct);
    propPage_assignColumnOrderFunctionality (mainStruct);
	propPage_assignUserCommandsFunctionality (mainStruct);
	TRACEIT (10, "propPage_assignFunctionality			end");
    return 0;
}


int propPage_assignMainFunctionality (struct _propPage_MainStruct *mainStruct){
    TRACEIT (10, "propPage_assignMainFunctionality			start");
	if (mainStruct == NULL){
		TRACEIT (2, "propPage_assignMainFunctionality			parameter NULL");
		return -1;
	}

    g_signal_connect(mainStruct->mainEntities.Main_Button_Close,"clicked",G_CALLBACK(propPage_MainButtonClose), (void *)mainStruct);
    g_signal_connect(mainStruct->mainEntities.Main_Button_Reload,"clicked",G_CALLBACK(propPage_MainButtonReload), (void *)mainStruct);
    g_signal_connect(mainStruct->mainEntities.Main_Button_LoadFrom,"clicked",G_CALLBACK(propPage_MainButtonLoadFrom), (void *)mainStruct);

    g_signal_connect(mainStruct->mainEntities.Main_Button_SaveTo,"clicked",G_CALLBACK(propPage_MainButtonSaveTo), (void *)mainStruct);
    g_signal_connect(mainStruct->mainEntities.Main_Button_Save,"clicked",G_CALLBACK(propPage_MainButtonSave), (void *)mainStruct);

    TRACEIT (10, "propPage_assignMainFunctionality			end");
    return 0;
}


void propPage_MainButtonSave (GtkButton *Button, struct _propPage_MainStruct *mainStruct){
    TRACEIT(10,"propPage_MainButtonSave		start");
    if (mainStruct == NULL){
        TRACEIT(2,"propPage_MainButtonSave		parameter NULL");
        return;
    }
    GtkWidget *dialog = CreateMyDialog("Overwrite",(GtkWindow *)mainStruct->mainEntities.Main_Widget,2,"Ok",GTK_RESPONSE_ACCEPT,"Cancel",GTK_RESPONSE_REJECT, NULL, 0,  TRUE);
    GtkWidget *label = gtk_label_new("Do you want to overwrite the\ncurrently used config file?\n");
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox),label);
    gtk_widget_show_all(dialog);
    int iDRet = gtk_dialog_run(GTK_DIALOG(dialog));
    if (iDRet != GTK_RESPONSE_ACCEPT){
        gtk_widget_destroy(dialog);
        TRACEIT(10,"propPage_MainButtonSave		end");
        return;
    }

    gtk_widget_destroy(dialog);
    propPage_saveAllToXMLFile (mainStruct, NULL);

    TRACEIT(10,"propPage_MainButtonSave		end");
    return;
}


void propPage_MainButtonSaveTo (GtkButton *Button, struct _propPage_MainStruct *mainStruct){
    TRACEIT(10,"propPage_MainButtonSaveTo		start");
    if (mainStruct == NULL){
        TRACEIT(2,"propPage_MainButtonSaveTo		parameter NULL");
        return;
    }

    GtkWidget *dialog;
    GString *HomeDir = g_string_sized_new (20);
    char *filename = NULL;

    dialog = gtk_file_chooser_dialog_new ("Save configuration as",
				      (GtkWindow *)mainStruct->mainEntities.Main_Widget,
				      GTK_FILE_CHOOSER_ACTION_SAVE,
				      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
				      GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
				      NULL);
    gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (dialog), TRUE);

    getHomeDirectory(HomeDir);
    checkPathForLastChar(HomeDir);
    g_string_append(HomeDir,CATSEYEHOMEDIR);
    gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog), HomeDir->str);

    getHomeDirectory(HomeDir);
    checkPathForLastChar(HomeDir);
    g_string_assign(HomeDir, CONFIG_FILE_TYPES);
    getLastStringInPath (HomeDir);

    gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (dialog), HomeDir->str);

    if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT){
        filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
        gtk_widget_destroy (dialog);
        if (filename){
            propPage_saveAllToXMLFile (mainStruct, filename);
        }
    }
    else{
        gtk_widget_destroy (dialog);
    }

    if (HomeDir){
        g_string_free (HomeDir, TRUE);
    }
    if (filename)
        g_free (filename);

    TRACEIT(10,"propPage_MainButtonSaveTo		end");
    return;
}


int propPage_saveAllToXMLFile (struct _propPage_MainStruct *mainStruct, const char *FileName){
    TRACEIT(10,"propPage_saveAllToXMLFile		start");
    if (mainStruct == NULL){
        TRACEIT(2,"propPage_saveAllToXMLFile		parameter NULL");
        return -1;
    }
    mainStruct->iSaved = 1;
    propPage_writeOptionsToGlobalStruct (mainStruct);
    propPage_writeColumnOrderToGlobalStruct (mainStruct);
    propPage_reorderBookmarkListByListView (mainStruct);
    propPage_reorderUsercommandListByListView (mainStruct);

    configFileHandling_saveGlobalStructsAsXMLConfigFile (FileName, FALSE);

    TRACEIT(10,"propPage_saveAllToXMLFile		end");
    return 0;
}


gboolean getPointerFromListView (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, struct _propPage_ReoderLists_Stru *Struct ){
    TRACEIT(10,"getPointerFromListView		start");
    if (Struct == NULL){
        TRACEIT(10,"getPointerFromListView		parameter NULL");
        return TRUE;
    }

    void *vP=NULL;
    gtk_tree_model_get(model, iter, Struct->iColumn, &vP, -1);

    Struct->glistListPointer = g_list_append (Struct->glistListPointer, vP);

    TRACEIT(10,"getPointerFromListView		end");
    return FALSE; //to continue, TRUE to stop iterating
}


int propPage_reorderBookmarkListByListView (struct _propPage_MainStruct *mainStruct){
    TRACEIT(10,"propPage_reorderBookmarkListByListView		start");
    if (mainStruct == NULL){
        TRACEIT(2,"propPage_reorderBookmarkListByListView		parameter NULL");
        return -1;
    }
    struct _propPage_ReoderLists_Stru Struct;
    GList *glistBuf=NULL;
    struct _OptionsList *BkmrkListNewFirstItem=NULL;
    Struct.glistListPointer = NULL;
    Struct.iColumn = PROPPAGE_COL_BKMRK_ITEMPOINTER;

    gtk_tree_model_foreach ((GtkTreeModel *)mainStruct->bkmrkEntities.Bkmrk_ListStore, (GtkTreeModelForeachFunc)getPointerFromListView, &Struct);

    glistBuf = Struct.glistListPointer;
    if (glistBuf)
        BkmrkListNewFirstItem = (struct _OptionsList *)glistBuf->data;
    while (glistBuf){
        ((struct _OptionsList *)glistBuf->data)->prevItem = (glistBuf->prev)?((struct _OptionsList *)glistBuf->prev->data):NULL;
        ((struct _OptionsList *)glistBuf->data)->nextItem = (glistBuf->next)?((struct _OptionsList *)glistBuf->next->data):NULL;
        glistBuf = glistBuf->next;
    }
    g_BookMarkList = BkmrkListNewFirstItem;
    g_list_free (Struct.glistListPointer);

    TRACEIT(10,"propPage_reorderBookmarkListByListView		end");
    return 0;
}


int propPage_reorderUsercommandListByListView (struct _propPage_MainStruct *mainStruct){
    TRACEIT(10,"propPage_reorderUsercommandListByListView		start");
    if (mainStruct == NULL){
        TRACEIT(2,"propPage_reorderBookmarkListByListView		parameter NULL");
        return -1;
    }
    struct _propPage_ReoderLists_Stru Struct;
    GList *glistBuf=NULL;
    struct _UserCommandList *UserCmndListNewFirstItem=NULL;
    Struct.glistListPointer = NULL;
    Struct.iColumn = PROPPAGE_COL_USCMND_ITEMPOINTER;

    gtk_tree_model_foreach ((GtkTreeModel *)mainStruct->UserCommandsEntities.UsrCmnd_ListStore, (GtkTreeModelForeachFunc)getPointerFromListView, &Struct);

    glistBuf = Struct.glistListPointer;
    if (glistBuf)
        UserCmndListNewFirstItem = (struct _UserCommandList *)glistBuf->data;
    while (glistBuf){
        ((struct _UserCommandList *)glistBuf->data)->prevItem = (glistBuf->prev)?((struct _UserCommandList *)glistBuf->prev->data):NULL;
        ((struct _UserCommandList *)glistBuf->data)->nextItem = (glistBuf->next)?((struct _UserCommandList *)glistBuf->next->data):NULL;
        glistBuf = glistBuf->next;
    }
    g_UserCommandList = UserCmndListNewFirstItem;
    g_list_free (Struct.glistListPointer);

    TRACEIT(10,"propPage_reorderBookmarkListByListView		end");
    return 0;
}


gboolean porpPage_checkIfNewColOrderIsDefault_helper (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, GList **pglistListOrder ){
    TRACEIT(10,"porpPage_checkIfNewColOrderIsDefault_helper		start");
    if (pglistListOrder == NULL){
        TRACEIT(10,"porpPage_checkIfNewColOrderIsDefault_helper		parameter NULL");
        return TRUE;
    }

	gchar *Name=NULL;
	//do not free here!
    gtk_tree_model_get(model, iter, PROPPAGE_COL_COLORDER_NAME, &Name, -1);

	if (NULL != Name){
		*pglistListOrder = g_list_append (*pglistListOrder, Name);
	}

    TRACEIT(10,"porpPage_checkIfNewColOrderIsDefault_helper		end");
    return FALSE; //to continue, TRUE to stop iterating
}


int porpPage_checkIfNewColOrderIsDefault (struct _propPage_MainStruct *mainStruct){
	// return: 	-1 = error
	//			0 = NOT default order
	//			1 = default order
	TRACEIT(10,"porpPage_checkIfNewColOrderIsDefault		start");

	int iOrderChange = 1;
	int i;
	GList *glistListOrder = NULL;
	GList *glistBuf = NULL;

    gtk_tree_model_foreach ((GtkTreeModel *)mainStruct->ColumnOrder.ClmOrder_ListStore, (GtkTreeModelForeachFunc)porpPage_checkIfNewColOrderIsDefault_helper, &glistListOrder);

	if (NULL == glistListOrder){
		return iOrderChange;
	}

	glistBuf = glistListOrder;

	i=0;
	while (glistBuf){
		i+=1;
		if (0!=strcmp (Names[i], (char *)glistBuf->data)){
			iOrderChange = 0;
			break;
		}
		glistBuf = glistBuf->next;
	}

	g_list_foreach(glistListOrder, (GFunc) g_free, NULL);
	g_list_free(glistListOrder);


	TRACEIT(10,"porpPage_checkIfNewColOrderIsDefault		end");
	return iOrderChange;
}


int propPage_writeColumnOrderToGlobalStruct (struct _propPage_MainStruct *mainStruct){
	TRACEIT(10,"propPage_writeColumnOrderToGlobalStruct		start");
    if (mainStruct == NULL){
        TRACEIT(2,"propPage_writeColumnOrderToGlobalStruct		parameter NULL");
        return -1;
    }

	//we will delete the existing item before
	//then, check if the new order is the same as the default order.
	//default order will not be stored.

	struct _OptionsList *ListItem = NULL;
	GList *glistListOrder = NULL;
	GList *glistBuf = NULL;
	GString *gstrValue = g_string_sized_new(100);

	ListItem = OptionsList_getItemByID(g_OptionsList, COLUMN_ORDER);

	if (ListItem != NULL){
		g_OptionsList = OptionsList_deleteItem(ListItem);
	}

	gtk_tree_model_foreach ((GtkTreeModel *)mainStruct->ColumnOrder.ClmOrder_ListStore, (GtkTreeModelForeachFunc)porpPage_checkIfNewColOrderIsDefault_helper, &glistListOrder);

	if (NULL == glistListOrder){
		TRACEIT(2,"propPage_writeColumnOrderToGlobalStruct		No entries in list?");
		return 0;
	}

	glistBuf = glistListOrder;
	if (NULL != glistBuf){
		g_string_append (gstrValue, "\"");
		while (glistBuf){
			g_string_append (gstrValue, (char *)glistBuf->data);
			g_string_append (gstrValue, COLUMN_ORDER_SEPARATOR);

			glistBuf = glistBuf->next;
		}
		g_string_erase (gstrValue, gstrValue->len - strlen(COLUMN_ORDER_SEPARATOR), -1);
		g_string_append (gstrValue, "\"");

		OptionsList_addItem(&g_OptionsList, COLUMN_ORDER, NULL, gstrValue->str);

		reorderAllTabsColumns ();

		if (0 != porpPage_checkIfNewColOrderIsDefault(mainStruct)){
			ListItem = OptionsList_getItemByID(g_OptionsList, COLUMN_ORDER);
			if (NULL != ListItem){
				g_OptionsList = OptionsList_deleteItem(ListItem);
			}
		}

		g_list_foreach(glistListOrder, (GFunc) g_free, NULL);
		g_list_free(glistListOrder);
	}

	g_string_free (gstrValue, TRUE);
    TRACEIT(10,"propPage_writeColumnOrderToGlobalStruct		end");
    return 0;
}


int propPage_writeOptionsToGlobalStruct (struct _propPage_MainStruct *mainStruct){
    TRACEIT(10,"propPage_writeOptionsToGlobalStruct		start");
    if (mainStruct == NULL){
        TRACEIT(2,"propPage_writeOptionsToGlobalStruct		parameter NULL");
        return -1;
    }
    struct _propPage_OptionsEntities *OptionsEntities = &(mainStruct->OptionsEntities);
    const char *ccpBuf1 = NULL;
    const char *ccpBuf2 = NULL;
    gchar *cpColorString = NULL;
    gboolean bState;

    if (g_OptionsList){
        OptionsList_freeList(&g_OptionsList);
    }

    ccpBuf1 = gtk_entry_get_text ((GtkEntry *)OptionsEntities->Option_Stded_Edit_Value);
    ccpBuf2 = gtk_entry_get_text ((GtkEntry *)OptionsEntities->Option_Stded_Edit_Name);
    if (ccpBuf1 && ccpBuf2 && 0<strlen(ccpBuf1) && 0<strlen(ccpBuf2) ){
        OptionsList_addItem(&g_OptionsList, STANDARDEDITORKEY, ccpBuf2, ccpBuf1);
    }
    ccpBuf1 = gtk_entry_get_text ((GtkEntry *)OptionsEntities->Option_Trmnl_Edit_Value);
    ccpBuf2 = gtk_entry_get_text ((GtkEntry *)OptionsEntities->Option_Trmnl_Edit_Name);
    if (ccpBuf1 && ccpBuf2 && 0<strlen(ccpBuf1) && 0<strlen(ccpBuf2) ){
        OptionsList_addItem(&g_OptionsList, TERMINALKEY, ccpBuf2, ccpBuf1);
    }
    ccpBuf1 = gtk_entry_get_text ((GtkEntry *)OptionsEntities->Option_Stdd_Edit);
    if (ccpBuf1 && 0<strlen(ccpBuf1) ){
        OptionsList_addItem(&g_OptionsList, STANDARDSTARTDIRECTORY, NULL, ccpBuf1);
    }
    ccpBuf1 = gtk_entry_get_text ((GtkEntry *)OptionsEntities->Option_Strtdl_Edit);
    if (ccpBuf1 && 0<strlen(ccpBuf1) ){
        OptionsList_addItem(&g_OptionsList, STARTDIRECTORIESLEFTKEY, NULL, ccpBuf1);
    }
    ccpBuf1 = gtk_entry_get_text ((GtkEntry *)OptionsEntities->Option_Strtdr_Edit);
    if (ccpBuf1 && 0<strlen(ccpBuf1) ){
        OptionsList_addItem(&g_OptionsList, STARTDIRECTORIESRIGHTKEY, NULL, ccpBuf1);
    }
    ccpBuf1 = gtk_entry_get_text ((GtkEntry *)OptionsEntities->Option_Strtdr_Edit);
    if (ccpBuf1 && 0<strlen(ccpBuf1) ){
        OptionsList_addItem(&g_OptionsList, STARTDIRECTORIESRIGHTKEY, NULL, ccpBuf1);
    }

    bState = gtk_toggle_button_get_active ((GtkToggleButton *)OptionsEntities->Option_Svpths_Check);
    if (bState){
        OptionsList_addItem(&g_OptionsList, SAVEPATHESONEXIT, NULL, "true");
    }
    else{
        OptionsList_addItem(&g_OptionsList, SAVEPATHESONEXIT, NULL, "false");
    }

    gtk_color_button_get_color ( (GtkColorButton *)OptionsEntities->Option_ClrItm_Button, &OptionsEntities->Option_ClrItm_gdkColor);
    cpColorString = gdk_color_to_string (&OptionsEntities->Option_ClrItm_gdkColor);
    if (cpColorString){
        OptionsList_addItem(&g_OptionsList, COLOR_ITEMS_DEFAULT, NULL, cpColorString);
        g_free(cpColorString);
    }
    gtk_color_button_get_color ( (GtkColorButton *)OptionsEntities->Option_ClrDir_Button, &OptionsEntities->Option_ClrDir_gdkColor);
    cpColorString = gdk_color_to_string (&OptionsEntities->Option_ClrDir_gdkColor);
    if (cpColorString){
        OptionsList_addItem(&g_OptionsList, COLOR_DIRECTORIES, NULL, cpColorString);
        g_free(cpColorString);
    }
    gtk_color_button_get_color ( (GtkColorButton *)OptionsEntities->Option_ClrLnk_Button, &OptionsEntities->Option_ClrLnk_gdkColor);
    cpColorString = gdk_color_to_string (&OptionsEntities->Option_ClrLnk_gdkColor);
    if (cpColorString){
        OptionsList_addItem(&g_OptionsList, COLOR_LINKS, NULL, cpColorString);
        g_free(cpColorString);
    }
    gtk_color_button_get_color ( (GtkColorButton *)OptionsEntities->Option_ClrMss_Button, &OptionsEntities->Option_ClrMss_gdkColor);
    cpColorString = gdk_color_to_string (&OptionsEntities->Option_ClrMss_gdkColor);
    if (cpColorString){
        OptionsList_addItem(&g_OptionsList, COLOR_ITEMS_MISSING, NULL, cpColorString);
        g_free(cpColorString);
    }

    ccpBuf1 = gtk_entry_get_text ((GtkEntry *)OptionsEntities->Option_PreviewSize_Edit);
    if (ccpBuf1 && 0<strlen(ccpBuf1) ){
        OptionsList_addItem(&g_OptionsList, PREVIEWSIZE, NULL, ccpBuf1);
    }
    ccpBuf1 = gtk_entry_get_text ((GtkEntry *)OptionsEntities->Option_PreviewEndings_Edit);
    if (ccpBuf1 && 0<strlen(ccpBuf1) ){
        OptionsList_addItem(&g_OptionsList, PREVIEWTYPES, NULL, ccpBuf1);
    }

    ccpBuf1 = gtk_entry_get_text ((GtkEntry *)OptionsEntities->Option_SaveSessionEveryMinutes_Edit);
    if (ccpBuf1 && 0<strlen(ccpBuf1) ){
        OptionsList_addItem(&g_OptionsList, SAVESESSIONMINUTESTYPES, NULL, ccpBuf1);
    }


    TRACEIT(10,"propPage_writeOptionsToGlobalStruct		end");
    return 0;
}


void propPage_MainButtonLoadFrom (GtkButton *Button, struct _propPage_MainStruct *mainStruct){
    TRACEIT(10,"propPage_MainButtonLoadFrom		start");
    if (mainStruct == NULL){
        TRACEIT(2,"propPage_MainButtonLoadFrom		parameter NULL");
        return;
    }
    char *cpFile=NULL;
    GtkWidget *dialog = NULL;
    GString *HomeDir = g_string_sized_new(20);;

    dialog = gtk_file_chooser_dialog_new ("choose Cat'sEyE XML config file to load",
                                            //NULL,
                                            (GtkWindow *)mainStruct->mainEntities.Main_Widget,
                                            GTK_FILE_CHOOSER_ACTION_OPEN,
                                            GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                            GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
                                            NULL);

    getHomeDirectory(HomeDir);
    checkPathForLastChar(HomeDir);
    g_string_append(HomeDir,CATSEYEHOMEDIR);
    gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog), HomeDir->str);


    if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT){
        cpFile = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
        if (cpFile){
            propPage_openConfigFile (cpFile, mainStruct);
            propPage_fillPropPageWidgets ( mainStruct);
        }
    }

    gtk_widget_destroy (dialog);
    if (cpFile)
        g_free (cpFile);
    if (HomeDir)
        g_string_free (HomeDir, TRUE);

    TRACEIT(10,"propPage_MainButtonLoadFrom		end");
    return;
}


void propPage_MainButtonReload (GtkButton *Button, struct _propPage_MainStruct *mainStruct){
    TRACEIT(10,"propPage_MainButtonReload		start");
    if (mainStruct == NULL){
        TRACEIT(2,"propPage_MainButtonReload		parameter NULL");
        return;
    }
    propPage_openConfigFile (NULL, mainStruct);
    propPage_fillPropPageWidgets ( mainStruct);
    TRACEIT(10,"propPage_MainButtonReload		end");
    return;
}


void propPage_MainButtonClose (GtkButton *Button, struct _propPage_MainStruct *mainStruct){
    TRACEIT(10,"propPage_MainButtonClose		start");
    if (mainStruct == NULL){
        TRACEIT(2,"propPage_MainButtonClose		parameter NULL");
        return;
    }
    int iCloseIt=1;
    if (mainStruct->iSaved == 0){
        GtkWidget *dialog = CreateMyDialog("Close",(GtkWindow *)mainStruct->mainEntities.Main_Widget,2,"Back",GTK_RESPONSE_REJECT,"Close anyway",GTK_RESPONSE_ACCEPT, NULL, 0,  TRUE);
        GtkWidget *label = gtk_label_new("The changes you have made were not saved.\nDo you really want to close?\nYou will loose all your changes.");
        gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox),label);
        gtk_widget_show_all(dialog);
        if (GTK_RESPONSE_ACCEPT != gtk_dialog_run(GTK_DIALOG(dialog))){
            iCloseIt = 0;
        }
        gtk_widget_destroy(dialog);
    }

    if (iCloseIt == 1)
        gtk_widget_destroy (mainStruct->mainEntities.Main_Widget);

    TRACEIT(10,"propPage_MainButtonClose		end");
    return;
}


int propPage_assignOptionsFunctionality (struct _propPage_MainStruct *mainStruct){
    TRACEIT (10, "propPage_assignOptionsMainWidgets			start");
	if (mainStruct == NULL){
		TRACEIT (2, "propPage_assignOptionsMainWidgets			parameter NULL");
		return -1;
	}

	struct _propPage_OptionsEntities *OptionsEntities = &(mainStruct->OptionsEntities);

    g_signal_connect(OptionsEntities->Option_Stdd_Button,"clicked",G_CALLBACK(propPage_OptionsChooseDirStdd), (void *)mainStruct);
    g_signal_connect(OptionsEntities->Option_Strtdr_Button,"clicked",G_CALLBACK(propPage_OptionsChooseDirStrtdr), (void *)mainStruct);
    g_signal_connect(OptionsEntities->Option_Strtdl_Button,"clicked",G_CALLBACK(propPage_OptionsChooseDirStrtdl), (void *)mainStruct);

//  mainStruct->mainEntities.Main_Widget       OptionsEntities->Option_MainVbox
//    g_signal_connect (mainStruct->mainEntities.Main_Widget , "key-press-event", G_CALLBACK(propPage_OptionsoneKeyPressedInMainWidget),(void *)mainStruct);

    TRACEIT (10, "propPage_assignOptionsMainWidgets			end");
    return 0;
}

/*
this does not work if we connect it to OptionsEntities->Option_MainVbox
gboolean propPage_OptionsoneKeyPressedInMainWidget(GtkWidget *widget, GdkEventKey *event, struct _propPage_MainStruct *mainStruct){
	TRACEIT(10,"createUserCommand_oneKeyPressedInMainWidget		start");
	printf("Iam here\n");
	if (mainStruct == NULL){
		TRACEIT (2, "createUserCommand_oneKeyPressedInMainWidget			parameter NULL");
		return FALSE;
	}

    mainStruct->iSaved = 0;

	TRACEIT(10,"createUserCommand_oneKeyPressedInMainWidget		end");
	return FALSE;
}
*/


void propPage_OptionsChooseDirStdd (GtkButton *Button, struct _propPage_MainStruct *mainStruct){
    TRACEIT(10,"propPage_OptionsChooseDirStdd		start");
    if ( 0 == propPage_OptionsChooseDir (mainStruct->mainEntities.Main_Widget , mainStruct->OptionsEntities.Option_Stdd_Edit)){
        mainStruct->iSaved = 0;
    }
    TRACEIT(10,"propPage_OptionsChooseDirStdd		end");
}

void propPage_OptionsChooseDirStrtdr (GtkButton *Button, struct _propPage_MainStruct *mainStruct){
    TRACEIT(10,"propPage_OptionsChooseDirStrtdr		start");
    if (0 == propPage_OptionsChooseDir (mainStruct->mainEntities.Main_Widget , mainStruct->OptionsEntities.Option_Strtdr_Edit)){
        mainStruct->iSaved = 0;
    }
    TRACEIT(10,"propPage_OptionsChooseDirStrtdr		end");
}

void propPage_OptionsChooseDirStrtdl (GtkButton *Button, struct _propPage_MainStruct *mainStruct){
    TRACEIT(10,"propPage_OptionsChooseDirStrtdl		start");
    if (0==propPage_OptionsChooseDir (mainStruct->mainEntities.Main_Widget , mainStruct->OptionsEntities.Option_Strtdl_Edit)){
        mainStruct->iSaved = 0;
    }
    TRACEIT(10,"propPage_OptionsChooseDirStrtdl		end");
}


int propPage_OptionsChooseDir (GtkWidget *Parent, GtkWidget *entrybox){
    //return: 0= directory changed, 1=user canceled, -1=error
    TRACEIT(10,"propPage_OptionsChooseDir		start");
    if (entrybox == NULL){
        TRACEIT(2,"propPage_OptionsChooseDir		parameter NULL");
        return -1;
    }
    char *cpDirectory=NULL;
    GtkWidget *dialog = NULL;
    int iRet = 0;

    dialog = gtk_file_chooser_dialog_new ("choose directory",
                                            //NULL,
                                            (GtkWindow *)Parent,
                                            GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
                                            GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                            GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
                                            NULL);

    if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT){
        iRet = 1;
        cpDirectory = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
        gtk_entry_set_text(	(GtkEntry *)entrybox, cpDirectory);
    }
    gtk_widget_destroy (dialog);
    if (cpDirectory)
        g_free (cpDirectory);

    TRACEIT(10,"propPage_OptionsChooseDir		end");
    return iRet;
}


int propPage_assignColumnOrderFunctionality (struct _propPage_MainStruct *mainStruct){
	TRACEIT (10, "propPage_assignColumnOrderFunctionality			start");
	if (mainStruct == NULL){
		TRACEIT (2, "propPage_assignColumnOrderFunctionality			parameter NULL");
		return -1;
	}

	struct _propPage_ColumnOrder *ColOrder = &(mainStruct->ColumnOrder);

	g_signal_connect(ColOrder->ClmOrder_Button_Reset,"clicked",G_CALLBACK(propPage_ClmnOrderButtonReset), (void *)mainStruct);

	TRACEIT (10, "propPage_assignColumnOrderFunctionality			end");
	return 0;
}


void propPage_ClmnOrderButtonReset (GtkButton *Button, struct _propPage_MainStruct *mainStruct){
    TRACEIT(10,"propPage_ClmnOrderButtonReset		start");
    propPage_fillColumnOrderWidgets (mainStruct, 1);
    TRACEIT(10,"propPage_ClmnOrderButtonReset		end");
}


int propPage_assignBkmrkFunctionality (struct _propPage_MainStruct *mainStruct){
    TRACEIT (10, "propPage_assignBkmrkFunctionality			start");
	if (mainStruct == NULL){
		TRACEIT (2, "propPage_assignBkmrkFunctionality			parameter NULL");
		return -1;
	}

    struct _propPage_BookmarkEntities *bkmrkEntities = &(mainStruct->bkmrkEntities);

    g_signal_connect(bkmrkEntities->Bkmrk_Button_Delete,"clicked",G_CALLBACK(propPage_BkmrkdeleteItem), (void *)bkmrkEntities);
    g_signal_connect(bkmrkEntities->Bkmrk_Button_New,"clicked",G_CALLBACK(propPage_BkmrkButtonNew), (void *)mainStruct);
    g_signal_connect(bkmrkEntities->Bkmrk_Button_Edit,"clicked",G_CALLBACK(propPage_BkmrkButtonEdit), (void *)mainStruct);

    g_signal_connect (bkmrkEntities->Bkmrk_ListView, "button-press-event",G_CALLBACK(propPage_BkmrkOneMouseButtonPressedInView), (void *)mainStruct);
    g_signal_connect (bkmrkEntities->Bkmrk_ListView, "row-activated",G_CALLBACK(propPage_BkmrkRowActivated), (void *)mainStruct);
    g_signal_connect (bkmrkEntities->Bkmrk_ListView, "key-press-event",G_CALLBACK(propPage_BkmrkOneKeyPressedInView),(void *)mainStruct);

    TRACEIT (10, "propPage_assignBkmrkFunctionality			end");
    return 0;
}


void propPage_BkmrkRowActivated (GtkTreeView *tree_view, GtkTreePath *path, GtkTreeViewColumn *column, struct _propPage_MainStruct *mainStruct){
    TRACEIT(10,"propPage_BkmrkRowActivated		start");
    propPage_BkmrkAddItem (mainStruct, 1 );
    TRACEIT(10,"propPage_BkmrkRowActivated		end");
    return;
}


gboolean propPage_BkmrkOneKeyPressedInView(GtkWidget *widget, GdkEventKey *event, struct _propPage_MainStruct *mainStruct){
    //Be aware of using widget, as this function is called when a key is pressed on a notebook (tab) too
    //data = struct SideObjectList
	TRACEIT(10,"propPage_BkmrkOneKeyPressedInView		start");
	if (mainStruct == NULL){
		TRACEIT (2, "propPage_BkmrkOneKeyPressedInView			parameter NULL");
		return FALSE;
	}

	guint modifiers;

	modifiers = gtk_accelerator_get_default_mod_mask ();

    if ( (event->keyval == GDK_Delete)){
	    //Delete selected files with 'del-key'
		propPage_BkmrkdeleteItem (NULL, mainStruct);
		TRACEIT(10,"propPage_BkmrkOneKeyPressedInView		ends");
		return TRUE;
	}

	TRACEIT(10,"propPage_BkmrkOneKeyPressedInView		end");
	return FALSE;
}


gboolean propPage_BkmrkOneMouseButtonPressedInView(GtkWidget *treeview, GdkEventButton *event, struct _propPage_MainStruct *mainStruct){
    TRACEIT(10,"propPage_BkmrkOneMouseButtonPressedInView		start");
    if (event->type == GDK_BUTTON_PRESS  &&  event->button == 3){
		propPage_BkmrkContextMenuOnListView(treeview,event, mainStruct);
		TRACEIT(10,"propPage_BkmrkOneMouseButtonPressedInView		end");
		return TRUE;
	}
	TRACEIT(10,"propPage_BkmrkOneMouseButtonPressedInView		end");
	return FALSE;
}


gboolean propPage_BkmrkContextMenuOnListView(GtkWidget *treeview, GdkEventButton *event,  struct _propPage_MainStruct *mainStruct){
	TRACEIT(10,"propPage_BkmrkContextMenuOnListView		start");
    if (mainStruct == NULL){
		TRACEIT (2, "propPage_BkmrkContextMenuOnListView			parameter NULL");
		return TRUE;
	}

	GtkWidget *menu, *menuitem;
	GtkWidget *sep2;
    GtkTreeSelection *selection;
    int iSelectedRows;

	menu = gtk_menu_new();

    selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));
    iSelectedRows = gtk_tree_selection_count_selected_rows(selection);

    selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));
	if (iSelectedRows <= 1){   //Event can be NULL if the key for contextmenu is presed
		GtkTreePath *path;
        TRACEIT(11,"propPage_BkmrkContextMenuOnListView		mark the item coverd by mousepointer");
        if (gtk_tree_view_get_path_at_pos(GTK_TREE_VIEW(treeview), (gint) event->x, (gint) event->y, &path, NULL, NULL, NULL)){
			gtk_tree_selection_unselect_all(selection);
			gtk_tree_selection_select_path(selection, path);
			gtk_tree_path_free(path);
		}
	}

	sep2 = gtk_separator_menu_item_new ();

    menuitem = gtk_menu_item_new_with_label("new ...");
	g_signal_connect(menuitem,"activate",G_CALLBACK(propPage_BkmrkButtonNew),mainStruct);
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);

    menuitem = gtk_menu_item_new_with_label("edit ...");
	g_signal_connect(menuitem,"activate",G_CALLBACK(propPage_BkmrkButtonEdit),mainStruct);
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);

    gtk_menu_shell_append(GTK_MENU_SHELL(menu), sep2);

    menuitem = gtk_menu_item_new_with_label("delete");
	g_signal_connect(menuitem,"activate",G_CALLBACK(propPage_BkmrkdeleteItem), mainStruct);
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);


    gtk_widget_show_all(menu);
	// Note: event can be NULL here when called from view_onPopupMenu;
	// gdk_event_get_time() accepts a NULL argument
	gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, (event != NULL) ? event->button : 0, gdk_event_get_time((GdkEvent*)event));

	TRACEIT(10,"propPage_BkmrkContextMenuOnListView		ends");
	return TRUE;
}


void propPage_BkmrkButtonNew (GtkButton *Button, struct _propPage_MainStruct *mainStruct){
    TRACEIT(10,"propPage_BkmrkButtonNew		start");
    propPage_BkmrkAddItem (mainStruct, 0 );
    TRACEIT(10,"propPage_BkmrkButtonNew		end");
}


void propPage_BkmrkButtonEdit (GtkButton *Button, struct _propPage_MainStruct *mainStruct){
    TRACEIT(10,"propPage_BkmrkButtonEdit		start");
    propPage_BkmrkAddItem (mainStruct, 1 );
    TRACEIT(10,"propPage_BkmrkButtonEdit		end");
}


void propPage_BkmrkdeleteItem (GtkButton *button, struct _propPage_MainStruct *mainStruct){
    TRACEIT(10,"propPage_BkmrkdeleteItem		start");
    if (mainStruct == NULL){
        TRACEIT(2,"propPage_BkmrkdeleteItem		parameter NULL");
        return;
    }
    GtkTreeModel *model=NULL;
    GtkTreeIter iter;
    GtkTreeSelection	*selection;
    struct _propPage_BookmarkEntities *bkmrkEntities = &(mainStruct->bkmrkEntities);
    selection  = gtk_tree_view_get_selection((GtkTreeView *)bkmrkEntities->Bkmrk_ListView);

    struct _OptionsList *BkmrkListBuf = NULL;
    GList *glistPathList = gtk_tree_selection_get_selected_rows (selection, &model);
    GList *glistBuf;

    if (glistPathList == NULL){
        return;
    }

    glistBuf = glistPathList;
    while (glistBuf){
        gtk_tree_model_get_iter(GTK_TREE_MODEL(bkmrkEntities->Bkmrk_ListStore), &iter, (GtkTreePath *)glistBuf->data);

        gtk_tree_model_get (GTK_TREE_MODEL(bkmrkEntities->Bkmrk_ListStore), &iter, PROPPAGE_COL_BKMRK_ITEMPOINTER, &BkmrkListBuf, -1);
        if (BkmrkListBuf != NULL){
            g_BookMarkList = OptionsList_deleteItem(BkmrkListBuf);
            gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
        }
        else{
            TRACEIT(4,"propPage_BkmrkdeleteItem		error searching for bookmark pointer");
        }
        glistBuf = glistBuf->next;
    }
    mainStruct->iSaved=0;
    g_list_foreach (glistPathList,(GFunc) gtk_tree_path_free, NULL);
    g_list_free (glistPathList);
    TRACEIT(10,"propPage_BkmrkdeleteItem		end");
    return;
}



int propPage_BkmrkAddItem (struct _propPage_MainStruct *mainStruct, int iEditOrAdd ){
    //also used for editing.
    //iEditOrAdd: 0=add, 1=edit
    TRACEIT(10,"propPage_BkmrkAddItem		end");

    if ( mainStruct == NULL){
        TRACEIT (2, "propPage_BkmrkAddItem      Parameter NULL");
        return -1;
    }

    int iRet = 0;
    GtkTreeModel *model = NULL;
    GString *gstrName = g_string_sized_new(10);
    GString *gstrPath = g_string_sized_new(10);
    GtkWidget *dialog;
    GtkTreeIter iter;
    struct _propPage_BookmarkEntities *bkmrkEntities = NULL;
    GtkTreeSelection	*selection;
    GList *glistPathList = NULL;
    struct _OptionsList *BkmrkListBuf = NULL;
    struct _propPage_BkmrkAddItem_BufStru BufStru;

    bkmrkEntities = &(mainStruct->bkmrkEntities);

    dialog = CreateMyDialog("Bookmark", (GtkWindow *)mainStruct->mainEntities.Main_Widget, 2, "Ok", GTK_RESPONSE_ACCEPT, "Cancel", 1,NULL,0, TRUE);

    GtkWidget *vboxMain = gtk_vbox_new (FALSE, 0);
    GtkWidget *lblName = gtk_label_new ("Name:");
    GtkWidget *lblPath = gtk_label_new ("Path:");
    GtkWidget *entryName = gtk_entry_new ();
    GtkWidget *entryPath = gtk_entry_new ();
    GtkWidget *pathButton = gtk_button_new_with_label ("open path");
    GtkWidget *hbox = gtk_hbox_new (FALSE, 0);
    g_signal_connect (entryName, "activate",G_CALLBACK(MyEasyInputWidget_EntryBoxActivate),dialog);
    g_signal_connect (entryPath, "activate",G_CALLBACK(MyEasyInputWidget_EntryBoxActivate),dialog);

    //if we want to edit the item we have to get all the information:
    if (iEditOrAdd == 1){
        selection  = gtk_tree_view_get_selection((GtkTreeView *)bkmrkEntities->Bkmrk_ListView);
        glistPathList = gtk_tree_selection_get_selected_rows (selection, &model);

        if (glistPathList == NULL){
            iRet = -1;
        }
        else {
            gtk_tree_model_get_iter(GTK_TREE_MODEL(bkmrkEntities->Bkmrk_ListStore), &iter, (GtkTreePath *)glistPathList->data);

            gtk_tree_model_get (GTK_TREE_MODEL(bkmrkEntities->Bkmrk_ListStore), &iter, PROPPAGE_COL_BKMRK_ITEMPOINTER, &BkmrkListBuf, -1);
            if (BkmrkListBuf == NULL){
                TRACEIT(4,"propPage_BkmrkdeleteItem		error searching for bookmark pointer");
                iRet = -1;
            }
            else{
                gtk_entry_set_text ((GtkEntry *)entryName, BkmrkListBuf->gstrName->str);
                gtk_entry_set_text ((GtkEntry *)entryPath, BkmrkListBuf->gstrValue->str);
            }
        }
    }

    if (iRet == 0){
        BufStru.EditBox = entryPath;
        BufStru.Parent = mainStruct->mainEntities.Main_Widget;
        g_signal_connect(pathButton,"clicked",G_CALLBACK(propPage_BkmrkAddItemPathButton), (void *)&BufStru);

        gtk_box_pack_start ((GtkBox *)hbox, entryPath, TRUE, FALSE, 0);
        gtk_box_pack_start ((GtkBox *)hbox, pathButton, FALSE, FALSE, 0);

        gtk_box_pack_start ((GtkBox *)vboxMain, lblName, TRUE, FALSE, 0);
        gtk_box_pack_start ((GtkBox *)vboxMain, entryName, TRUE, FALSE, 0);
        gtk_box_pack_start ((GtkBox *)vboxMain, lblPath, TRUE, FALSE, 0);
        gtk_box_pack_start ((GtkBox *)vboxMain, hbox, TRUE, FALSE, 0);

        gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox),vboxMain);

        gtk_widget_show_all(dialog);
    }

    if (iRet == 0 && gtk_dialog_run(GTK_DIALOG(dialog))== GTK_RESPONSE_ACCEPT){
        mainStruct->iSaved = 0;
        g_string_assign (gstrName, gtk_entry_get_text ((GtkEntry *)entryName));
        g_string_assign (gstrPath, gtk_entry_get_text ((GtkEntry *)entryPath));

        if (iEditOrAdd == 0){    //0=add, 1=edit
            OptionsList_addItem(&g_BookMarkList, NULL, gstrName->str, gstrPath->str);
            gtk_list_store_append (bkmrkEntities->Bkmrk_ListStore, &iter);
            gtk_list_store_set (bkmrkEntities->Bkmrk_ListStore, &iter, PROPPAGE_COL_BKMRK_ITEMPOINTER, OptionsList_getLastItem(g_BookMarkList), -1);
        }
        else{
            OptionsList_editItem(BkmrkListBuf, NULL, gstrName->str, gstrPath->str);
        }

        gtk_list_store_set (bkmrkEntities->Bkmrk_ListStore, &iter, PROPPAGE_COL_BKMRK_NAME, gstrName->str, -1);
        gtk_list_store_set (bkmrkEntities->Bkmrk_ListStore, &iter, PROPPAGE_COL_BKMRK_PATH, gstrPath->str, -1);
    }

    if (iRet == 0){
        gtk_widget_destroy(dialog);
    }

    g_string_free(gstrName, TRUE);
    g_string_free(gstrPath, TRUE);

    if (glistPathList){
        g_list_foreach (glistPathList,(GFunc) gtk_tree_path_free, NULL);
        g_list_free (glistPathList);
    }

    TRACEIT(10,"propPage_BkmrkAddItem		end");
    return 0;
}


void propPage_BkmrkAddItemPathButton (GtkButton *Button, struct _propPage_BkmrkAddItem_BufStru *BufStru){
    TRACEIT(10,"propPage_OptionsChooseDirStdd		start");
    propPage_OptionsChooseDir ( BufStru->Parent, BufStru->EditBox );
    TRACEIT(10,"propPage_OptionsChooseDirStdd		end");
}



int propPage_assignUserCommandsFunctionality (struct _propPage_MainStruct *mainStruct){
    TRACEIT (10, "propPage_assignUserCommandsFunctionality			start");
	if (mainStruct == NULL){
		TRACEIT (2, "propPage_assignUserCommandsFunctionality			parameter NULL");
		return -1;
	}
    struct _propPage_UserCommandsEntities *UserCommandsEntities = &(mainStruct->UserCommandsEntities);

    g_signal_connect (UserCommandsEntities->UsrCmnd_Button_Delete,"clicked",G_CALLBACK(propPage_UserCommandDeleteItem), (void *)mainStruct);
    g_signal_connect (UserCommandsEntities->UsrCmnd_Button_New,"clicked",G_CALLBACK(propPage_UserCommandButtonNew), (void *)mainStruct);
    g_signal_connect (UserCommandsEntities->UsrCmnd_Button_Edit,"clicked",G_CALLBACK(propPage_UserCommandButtonEdit), (void *)mainStruct);

    g_signal_connect (UserCommandsEntities->UsrCmnd_Filter_ResetButton,"clicked",G_CALLBACK(propPage_UserCommandClearFilter), (void *)UserCommandsEntities);
    g_signal_connect (UserCommandsEntities->UsrCmnd_Filter_NameEntry, "activate",G_CALLBACK(propPage_UserCommandFilterEntryBoxActivate),UserCommandsEntities);
    g_signal_connect (UserCommandsEntities->UsrCmnd_Filter_TargetEntry, "activate",G_CALLBACK(propPage_UserCommandFilterEntryBoxActivate),UserCommandsEntities);
    g_signal_connect (UserCommandsEntities->UsrCmnd_Filter_CommandEntry, "activate",G_CALLBACK(propPage_UserCommandFilterEntryBoxActivate),UserCommandsEntities);
    g_signal_connect (UserCommandsEntities->UsrCmnd_Filter_DefaultEntry, "activate",G_CALLBACK(propPage_UserCommandFilterEntryBoxActivate),UserCommandsEntities);

    g_signal_connect (UserCommandsEntities->UsrCmnd_ListView, "button-press-event",G_CALLBACK(propPage_UsrCmndOneMouseButtonPressedInView), (void *)mainStruct);
    g_signal_connect (UserCommandsEntities->UsrCmnd_ListView, "row-activated",G_CALLBACK(propPage_UsrCmndRowActivated), (void *)mainStruct);
    g_signal_connect (UserCommandsEntities->UsrCmnd_ListView, "key-press-event",G_CALLBACK(propPage_UsrCmndOneKeyPressedInView),(void *)mainStruct);

    TRACEIT (10, "propPage_assignUserCommandsFunctionality			end");
    return 0;
}


void propPage_UserCommandFilterEntryBoxActivate (GtkEntry *entry, struct _propPage_UserCommandsEntities *UserCommandsEntities){
    TRACEIT(10,"propPage_UserCommandFilterEntryBoxActivate		start");
    if (UserCommandsEntities == NULL){
        TRACEIT(2,"propPage_UserCommandFilterEntryBoxActivate		parameter NULL");
        return;
    }

    propPage_UserCommandFilterApply (UserCommandsEntities);

    TRACEIT(10,"propPage_UserCommandFilterEntryBoxActivate		end");
    return;
}


int propPage_UserCommandFilterApply (struct _propPage_UserCommandsEntities *UserCommandsEntities){
    TRACEIT(10,"propPage_UserCommandFilterApply		start");
    if (UserCommandsEntities == NULL){
        TRACEIT(2,"propPage_UserCommandFilterApply		parameter NULL");
        return -1;
    }
    struct _PropPage_SetVisibility_Stru Struct;
    Struct.iColumn = PROPPAGE_COL_USCMND_VISIBLE;
    Struct.bValue = FALSE;
    gtk_tree_model_foreach ((GtkTreeModel *)UserCommandsEntities->UsrCmnd_ListStore, (GtkTreeModelForeachFunc)propPage_setVisibilityInListView, &Struct);

    gtk_tree_model_foreach ((GtkTreeModel *)UserCommandsEntities->UsrCmnd_ListStore, (GtkTreeModelForeachFunc)propPage_UserCommandFilterApply_foreach, UserCommandsEntities);

    TRACEIT(10,"propPage_UserCommandFilterApply		end");
    return 0;
}


gboolean propPage_UserCommandFilterApply_foreach ( GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, struct _propPage_UserCommandsEntities *UserCommandsEntities ){
    TRACEIT(10,"propPage_setVisibilityInListView		start");
    if (UserCommandsEntities == NULL){
        TRACEIT(2,"propPage_setVisibilityInListView		parameter NULL");
        return TRUE;
    }
    int iHideIt = 1;
    int counter = 0;
    if (0 ==propPage_UserCommand_WouldThisFilterHideIt((GtkEntry *)UserCommandsEntities->UsrCmnd_Filter_NameEntry, PROPPAGE_COL_USCMND_NAME, model, iter )){
        iHideIt = 0;
        counter++;
    }
    if (0 == propPage_UserCommand_WouldThisFilterHideIt((GtkEntry *)UserCommandsEntities->UsrCmnd_Filter_TargetEntry, PROPPAGE_COL_USCMND_TARGET, model, iter )){
        iHideIt = 0;
        counter++;
    }
    if (0 == propPage_UserCommand_WouldThisFilterHideIt((GtkEntry *)UserCommandsEntities->UsrCmnd_Filter_CommandEntry, PROPPAGE_COL_USCMND_COMMAND, model, iter )){
        iHideIt = 0;
        counter++;
    }
    if (0 == propPage_UserCommand_WouldThisFilterHideIt((GtkEntry *)UserCommandsEntities->UsrCmnd_Filter_DefaultEntry, PROPPAGE_COL_USCMND_DEFAULT, model, iter )){
        iHideIt = 0;
        counter++;
    }

    if (counter == 4)
        gtk_list_store_set ((GtkListStore *)model, iter, PROPPAGE_COL_USCMND_VISIBLE, TRUE, -1);

    TRACEIT(10,"propPage_setVisibilityInListView		start");
    return FALSE;
}


int propPage_UserCommand_WouldThisFilterHideIt( GtkEntry *Entry, int iColumnSearch, GtkTreeModel *model, GtkTreeIter *iter ){
    TRACEIT(10,"propPage_UserCommand_WouldThisFilterHideIt		start");
    if (Entry==NULL || model==NULL || iter==NULL){
        TRACEIT(10,"propPage_UserCommand_WouldThisFilterHideIt		parameter NULL");
        return -1;
    }
    char *cpBuf1 = NULL;
    char *cpBuf2 = NULL;
    const char *ccpBuf = NULL;
    ccpBuf = gtk_entry_get_text ((GtkEntry *)Entry);
    GString *gstrBuf = g_string_sized_new (10);
    int iRet = 1;   //1=I would hide it, 0=I would not hide it

    gtk_tree_model_get(model, iter, iColumnSearch, &cpBuf2, -1);
    if (cpBuf2 != NULL && 1<=strlen (ccpBuf)){
        g_string_assign (gstrBuf, ccpBuf);
        for (cpBuf1=strtok(gstrBuf->str, " ") ; cpBuf1!=NULL ; cpBuf1=strtok(NULL, " ")){
            if (strstr (cpBuf2, cpBuf1) != NULL){
                iRet = 0;
            }
        }
    }
    else{
        iRet = 0;
    }

    if (cpBuf2)
        free (cpBuf2);
    g_string_free (gstrBuf, TRUE);
    TRACEIT(10,"propPage_UserCommand_WouldThisFilterHideIt		end");
    return iRet;
}


gboolean propPage_setVisibilityInListView (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, struct _PropPage_SetVisibility_Stru *Struct ){
    TRACEIT(10,"propPage_setVisibilityInListView		start");
    if (Struct == NULL){
        TRACEIT(2,"propPage_setVisibilityInListView		parameter NULL");
        return TRUE;
    }

    gtk_list_store_set ((GtkListStore *)model, iter, Struct->iColumn, Struct->bValue, -1);

    TRACEIT(10,"propPage_setVisibilityInListView		end");
    return FALSE;
}


void propPage_UserCommandClearFilter (GtkButton *button, struct _propPage_UserCommandsEntities *UserCommandsEntities){
    TRACEIT(10,"propPage_UserCommandDeleteItem		start");
    if (UserCommandsEntities == NULL){
        TRACEIT(2,"propPage_UserCommandDeleteItem		parameter NULL");
        return;
    }

    gtk_entry_set_text ((GtkEntry *)UserCommandsEntities->UsrCmnd_Filter_NameEntry, "");
    gtk_entry_set_text ((GtkEntry *)UserCommandsEntities->UsrCmnd_Filter_TargetEntry, "");
    gtk_entry_set_text ((GtkEntry *)UserCommandsEntities->UsrCmnd_Filter_CommandEntry, "");
    gtk_entry_set_text ((GtkEntry *)UserCommandsEntities->UsrCmnd_Filter_DefaultEntry, "");

    struct _PropPage_SetVisibility_Stru Struct;
    Struct.iColumn = PROPPAGE_COL_USCMND_VISIBLE;
    Struct.bValue = TRUE;
    gtk_tree_model_foreach ((GtkTreeModel *)UserCommandsEntities->UsrCmnd_ListStore, (GtkTreeModelForeachFunc)propPage_setVisibilityInListView, &Struct);

    TRACEIT(10,"propPage_UserCommandDeleteItem		end");
    return;
}


void propPage_UsrCmndRowActivated (GtkTreeView *tree_view, GtkTreePath *path, GtkTreeViewColumn *column, struct _propPage_MainStruct *mainStruct){
    TRACEIT(10,"propPage_UsrCmndRowActivated		start");
    propPage_UserCommandButtonEdit (NULL, mainStruct);
    TRACEIT(10,"propPage_UsrCmndRowActivated		end");
    return;
}


gboolean propPage_UsrCmndOneKeyPressedInView(GtkWidget *widget, GdkEventKey *event, struct _propPage_MainStruct *mainStruct){
    //Be aware of using widget, as this function is called when a key is pressed on a notebook (tab) too
    //data = struct SideObjectList
	TRACEIT(10,"propPage_UsrCmndOneKeyPressedInView		start");
	if (mainStruct == NULL){
		TRACEIT (2, "propPage_UsrCmndOneKeyPressedInView			parameter NULL");
		return FALSE;
	}

	guint modifiers;

	modifiers = gtk_accelerator_get_default_mod_mask ();

    if ( (event->keyval == GDK_Delete)){
	    //Delete selected files with 'del-key'
		propPage_UserCommandDeleteItem (NULL, mainStruct);
		TRACEIT(10,"propPage_UsrCmndOneKeyPressedInView		ends");
		return TRUE;
	}

	TRACEIT(10,"propPage_UsrCmndOneKeyPressedInView		end");
	return FALSE;
}


gboolean propPage_UsrCmndOneMouseButtonPressedInView(GtkWidget *treeview, GdkEventButton *event, struct _propPage_MainStruct *mainStruct){
    TRACEIT(10,"propPage_UsrCmndOneMouseButtonPressedInView		start");
    if (event->type == GDK_BUTTON_PRESS  &&  event->button == 3){
		propPage_UsrCmndContextMenuOnListView(treeview,event, mainStruct);
		TRACEIT(10,"propPage_UsrCmndOneMouseButtonPressedInView		end");
		return TRUE;
	}
	TRACEIT(10,"propPage_UsrCmndOneMouseButtonPressedInView		end");
	return FALSE;
}


gboolean propPage_UsrCmndContextMenuOnListView(GtkWidget *treeview, GdkEventButton *event,  struct _propPage_MainStruct *mainStruct){
	TRACEIT(10,"propPage_BkmrkContextMenuOnListView		start");
    if (mainStruct == NULL){
		TRACEIT (2, "propPage_BkmrkContextMenuOnListView			parameter NULL");
		return TRUE;
	}

	GtkWidget *menu, *menuitem;
	GtkWidget *sep2;
    GtkTreeSelection *selection;
    int iSelectedRows;

	menu = gtk_menu_new();

    selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));
    iSelectedRows = gtk_tree_selection_count_selected_rows(selection);

    selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));
	if (iSelectedRows <= 1){   //Event can be NULL if the key for contextmenu is presed
		GtkTreePath *path;
        TRACEIT(11,"propPage_BkmrkContextMenuOnListView		mark the item coverd by mousepointer");
        if (gtk_tree_view_get_path_at_pos(GTK_TREE_VIEW(treeview), (gint) event->x, (gint) event->y, &path, NULL, NULL, NULL)){
			gtk_tree_selection_unselect_all(selection);
			gtk_tree_selection_select_path(selection, path);
			gtk_tree_path_free(path);
		}
	}

	sep2 = gtk_separator_menu_item_new ();

    menuitem = gtk_menu_item_new_with_label("new ...");
	g_signal_connect(menuitem,"activate",G_CALLBACK(propPage_UserCommandButtonNew),mainStruct);
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);

    menuitem = gtk_menu_item_new_with_label("edit ...");
	g_signal_connect(menuitem,"activate",G_CALLBACK(propPage_UserCommandButtonEdit),mainStruct);
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);

    gtk_menu_shell_append(GTK_MENU_SHELL(menu), sep2);

    menuitem = gtk_menu_item_new_with_label("delete");
	g_signal_connect(menuitem,"activate",G_CALLBACK(propPage_UserCommandDeleteItem), &(mainStruct->UserCommandsEntities));
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);


    gtk_widget_show_all(menu);
	// Note: event can be NULL here when called from view_onPopupMenu;
	// gdk_event_get_time() accepts a NULL argument
	gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, (event != NULL) ? event->button : 0, gdk_event_get_time((GdkEvent*)event));

	TRACEIT(10,"propPage_UsrCmndContextMenuOnListView		ends");
	return TRUE;
}


void propPage_UserCommandButtonEdit (GtkButton *button, struct _propPage_MainStruct *mainStruct){
    TRACEIT(10,"propPage_UserCommandButtonEdit		start");
    if (mainStruct == NULL){
        TRACEIT(2,"propPage_UserCommandButtonEdit		parameter NULL");
        return;
    }

    GtkTreeModel *model=NULL;
    GtkTreeIter iter;
    GtkTreeSelection	*selection;
    selection  = gtk_tree_view_get_selection((GtkTreeView *)mainStruct->UserCommandsEntities.UsrCmnd_ListView);

    struct _UserCommandList *UserCommandListBuf = NULL;

    GList *glistPathList = gtk_tree_selection_get_selected_rows (selection, &model);

    if (glistPathList == NULL){
        TRACEIT(10,"propPage_UserCommandButtonEdit		pathlist NULL, nothing marked, end");
        return;
    }
    gtk_tree_model_get_iter(GTK_TREE_MODEL(mainStruct->UserCommandsEntities.UsrCmnd_ListStore), &iter, (GtkTreePath *)glistPathList->data);

    gtk_tree_model_get (GTK_TREE_MODEL(mainStruct->UserCommandsEntities.UsrCmnd_ListStore), &iter, PROPPAGE_COL_USCMND_ITEMPOINTER, &UserCommandListBuf, -1);
    if (UserCommandListBuf == NULL){
        TRACEIT(4,"propPage_UserCommandDeleteItem		error searching for bookmark pointer");
    }

    g_list_foreach (glistPathList,(GFunc) gtk_tree_path_free, NULL);
    g_list_free (glistPathList);

    propPage_UserCommandAddItem (UserCommandListBuf, mainStruct->mainEntities.Main_Widget);

    TRACEIT(10,"propPage_UserCommandButtonEdit		end");
}


void propPage_UserCommandButtonNew (GtkButton *button, struct _propPage_MainStruct *mainStruct){
    TRACEIT(10,"propPage_UserCommandButtonNew		start");
    if (mainStruct == NULL){
        TRACEIT(2,"propPage_UserCommandButtonNew		parameter NULL");
        return;
    }

    propPage_UserCommandAddItem (NULL, mainStruct->mainEntities.Main_Widget);

    TRACEIT(10,"propPage_UserCommandButtonNew		end");
}


int propPage_UserCommandAddItem (struct _UserCommandList *UserCommandListItem, GtkWidget *Parent){
    TRACEIT(10,"propPage_UserCommandAddItem		start");


    GtkWidget *wBuf=NULL;

    wBuf = createUserCommand_start (UserCommandListItem, (GtkWindow *)Parent);
    if (wBuf){
        g_glistUserCommandWidgets = g_list_append (g_glistUserCommandWidgets, wBuf);
    }

    TRACEIT(10,"propPage_UserCommandAddItem		end");
    return 0;
}


void propPage_UserCommandDeleteItem (GtkButton *button, struct _propPage_MainStruct *mainStruct){
    TRACEIT(10,"propPage_UserCommandDeleteItem		start");
    if (mainStruct == NULL){
        TRACEIT(2,"propPage_UserCommandDeleteItem		parameter NULL");
        return;
    }
    GtkTreeModel *model=NULL;
    GtkTreeIter iter;
    GtkTreeSelection	*selection;
    struct _propPage_UserCommandsEntities *UserCommandsEntities = &(mainStruct->UserCommandsEntities);
    selection = gtk_tree_view_get_selection((GtkTreeView *)UserCommandsEntities->UsrCmnd_ListView);

    struct _UserCommandList *UserCommandListBuf = NULL;

    GList *glistPathList = gtk_tree_selection_get_selected_rows (selection, &model);
    GList *glistBuf;

    if (glistPathList == NULL){
        TRACEIT(10,"propPage_UserCommandDeleteItem		end");
        return;
    }

    glistBuf = glistPathList;
    while (glistBuf){
        gtk_tree_model_get_iter(GTK_TREE_MODEL(UserCommandsEntities->UsrCmnd_ListStore), &iter, (GtkTreePath *)glistBuf->data);

        gtk_tree_model_get (GTK_TREE_MODEL(UserCommandsEntities->UsrCmnd_ListStore), &iter, PROPPAGE_COL_USCMND_ITEMPOINTER, &UserCommandListBuf, -1);
        if (UserCommandListBuf != NULL){
            g_UserCommandList = UserCommandList_deleteItem(UserCommandListBuf);
            gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
        }
        else{
            TRACEIT(4,"propPage_UserCommandDeleteItem		error searching for bookmark pointer");
        }
        glistBuf = glistBuf->next;
    }
    mainStruct->iSaved=0;
    g_list_foreach (glistPathList,(GFunc) gtk_tree_path_free, NULL);
    g_list_free (glistPathList);
    TRACEIT(10,"propPage_UserCommandDeleteItem		end");
    return;
}

