/*
    Copyright 2007, 2008, 2009, 2010 Geyer Klaus

    This file is part of Cat'sEyE.

    Cat'sEyE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Cat'sEyE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Cat'sEyE.  If not, see <http://www.gnu.org/licenses/>.
*/


#include <gtk/gtk.h>
#ifndef NO_GIO
#include <gio/gio.h>
#endif
#include <string.h>
#include <stdlib.h>
#include <dirent.h>	//Verzeichnis Zeiger und DIR - Struktur
#include <unistd.h> //usleep
#include <gdk/gdkkeysyms.h>

#include "listitems.h"
#include "mytrace.h"
#include "Lists.h"
#include "helpers.h"
#include "DragnDrop.h"
#include "globalDefinitions.h"
#include "userdata.h"
#include "CommandParser.h"
#include "myfm.h"
#include "fileshelf.h"
#include "threads.h"

#include "allInclude.h"

void ProcessItemsViaList(void *data);
void calcDirectorySizeInView_thread(struct AllInformationAllUseStruct *stru);
void CopyReSizeWidget_deleteSelectedItems (struct ListWidgetCommunicationStruct *CommStru);


extern GtkWindow 	*win;

int ProcessItemsViaListWrap(struct CopyReMoveList *FirstItem, int process){
	TRACEIT(10,"ProcessItemsViaListWrap		start");
	GError *Error=NULL;
	if (FirstItem == NULL){
		TRACEIT(10,"ProcessItemsViaListWrap		ends -1");
		return -1;
	}
	if( !g_thread_supported() ){
		g_thread_init(NULL);
    }

	struct CopyReMoveList *bufitem = FirstItem;
	while (bufitem){
        bufitem->CopyMoveOrRemove=process;
        bufitem = bufitem->NextItem;
	}

	g_thread_create((GThreadFunc)ProcessItemsViaList, (void *)FirstItem , FALSE, &Error);
	if (Error){
		TRACEIT(2,"ProcessItemsViaListWrap: error in g_thread_create:");
		TRACEIT(2,Error->message);
		g_error_free(Error);
		Error=NULL;
	}
	TRACEIT(10,"ProcessItemsViaListWrap		ends");
	return 0;
}


gboolean FreeMemoryAndDeleteWidgetFunction (struct ListWidgetCommunicationStruct *commStru){
    //ATTENTION!!
    //do not access commStru after this call, it is free after this.
    //printf("FREE MEMORY\n");
    //free all memory in the list
    TRACEIT(10,"FreeMemoryAndDeleteWidgetFunction		start");
    if (commStru->cancellable)
        g_object_unref(commStru->cancellable);
    if (commStru->gerror)
        g_error_free (commStru->gerror);
    g_object_unref(commStru->listStore);

    CopyReMoveList_DeleteList (commStru->firstItem);

    g_string_free(commStru->WindowName, TRUE);

    //detroy the widget
    gtk_widget_destroy(commStru->mainListWidget);

    g_mutex_lock (commStru->mtxContinue);
    g_mutex_unlock (commStru->mtxContinue);
    g_mutex_free (commStru->mtxContinue);

    free(commStru);
    TRACEIT(10,"FreeMemoryAndDeleteWidgetFunction		ends");
    return TRUE;
}


void ListWidget_CancelList(struct ListWidgetCommunicationStruct *commStru){
    TRACEIT(10,"ListWidget_CancelList		start");
    //struct CopyReMoveList *list = (struct CopyReMoveList *)commStru->firstItem;
    struct CopyReMoveList *buflist = (struct CopyReMoveList *)commStru->firstItem;
    while (buflist != NULL){
        if (buflist->status == -1){
            buflist->status = -10;
        }
        buflist->ibreak=1;
        buflist = buflist->NextItem;
    };
    #ifndef NO_GIO
    if (commStru->cancellable)
        g_cancellable_cancel (commStru->cancellable);
    #endif
    commStru->iCancelled=1;

    gtk_widget_show(GTK_WIDGET(commStru->closeButton));
    gtk_widget_hide(GTK_WIDGET(commStru->cancelButton));
    gtk_widget_hide(GTK_WIDGET(commStru->continueButton));
    gtk_widget_hide(GTK_WIDGET(commStru->pauseButton));
    TRACEIT(10,"ListWidget_CancelList		end");
}


void ListWidget_CancelButton(GtkButton *button, gpointer data){
    //simply asks the user for canceling
    //and if the answer is ok, sets status of buflist to -10 (cancelled)
    //and cancels  the commstru->cancellable object
	TRACEIT(10,"ListWidget_CancelButton		start");
	GtkWidget *dialog, *label;
	struct ListWidgetCommunicationStruct *commStru = (struct ListWidgetCommunicationStruct *)data;

	if (commStru->iCancelled == 1){
	    TRACEIT(4,"ListWidget_CancelButton		ends, allready cancelled");
        return;
	}
    commStru->iCancelDialogRuns=1;
    dialog = CreateMyDialog("Do you want to cancel Process?", (GtkWindow *)commStru->mainListWidget, 2, "Ok",GTK_RESPONSE_ACCEPT,"Cancel",GTK_RESPONSE_REJECT, NULL, 0, TRUE);
    label = gtk_label_new("This cancels the processes in this list\nDo you want to continue?\n");
    gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox),label);
    gtk_widget_show_all(dialog);
    if (gtk_dialog_run(GTK_DIALOG(dialog))== GTK_RESPONSE_ACCEPT){
        ListWidget_CancelList(commStru);
    }
    gtk_widget_destroy(dialog);
    commStru->iCancelDialogRuns=0;

    //check if the thread 2 allready exists and we can close all
    if (commStru->iDestroyFlag == 1 && commStru->SecondThreadRunning == 0){
        FreeMemoryAndDeleteWidgetFunction(commStru);
    }

	TRACEIT(10,"ListWidget_CancelButton		ends");
}


gboolean ListWidget_DeleteEvent (GtkWidget *widget, GdkEvent  *event, struct ListWidgetCommunicationStruct *commStru){
    TRACEIT(10,"ListWidget_DeleteEvent		start");
    //this is used to catch the signal emited when the user presses the X in the upper right corner
    //printf("ListWidget_DeleteEvent DELETE EVENT\n");
    if (commStru == NULL){
        TRACEIT(2,"ListWidget_DeleteEvent		end, Parameter NULL");
        return FALSE;
    }
    //thread 2 may still run -> we have to ask for canceling
   //we may have allready cancelled -> we have to close

    if (commStru->iCancelled == 1){
        if (commStru->SecondThreadRunning == 1){
            //myWrap_gdk_threads_enter("ListWidget_DeleteEvent 1");
            MyMessageWidget((GtkWindow *)commStru->mainListWidget, "Error", "List item is still active, please close the aplication\nwhich is connect to the item first.");
            //myWrap_gdk_threads_leave("ListWidget_DeleteEvent 1");
        }
        else
            FreeMemoryAndDeleteWidgetFunction(commStru);
        return TRUE;
    }
    if (commStru->SecondThreadRunning == 1 && commStru->iCancelled == 0){
        ListWidget_CancelButton(NULL, (void *)commStru);
        if (commStru->iCancelled == 1){
            commStru->iDestroyFlag = 1;
        }
        return TRUE;
    }
    if (commStru->iDestroyFlag == 1){
        FreeMemoryAndDeleteWidgetFunction(commStru);
        return TRUE;
    }

    TRACEIT(10,"ListWidget_DeleteEvent		end");
    return TRUE;
//    return gRet;   //when this is TRUE, the widget will stay even when the user tries to close it. And because of our notification we d not close it either
                    //FALSE will destroy the widget
}


void ListWidget_CloseButton(GtkButton *button, struct ListWidgetCommunicationStruct *commStru){
    TRACEIT(10,"ListWidget_CloseButton		start");
    if (commStru == NULL){
        TRACEIT(2,"ListWidget_CloseButton		ends, Parameter NULL");
        return;
    }
    //if the close_button is visible, the thread 2 has allready finished there cant be even an cancel dialog when pressing this
    if (commStru->SecondThreadRunning == 1){
        //myWrap_gdk_threads_enter("ListWidget_CloseButton");
        MyMessageWidget((GtkWindow *)commStru->mainListWidget, "Error", "List item is still active, please close the aplication\nwhich is connect to the item first.");
        //myWrap_gdk_threads_leave("ListWidget_CloseButton");
    }
    else
        FreeMemoryAndDeleteWidgetFunction(commStru);

    TRACEIT(10,"ListWidget_CloseButton		ends");
    return;
}


void ListWidget_pauseButton(GtkButton *button, struct ListWidgetCommunicationStruct *commStru){
    TRACEIT(10,"ListWidget_pauseButton		start");
    if (commStru == NULL){
        TRACEIT(2,"ListWidget_pauseButton		ends, Parameter NULL");
        return;
    }

    commStru->iListPaused += 1;
    commStru->iterLastProcessedItem = &(commStru->currentItem->iter);

    gtk_widget_hide(GTK_WIDGET(commStru->pauseButton));
    gtk_widget_show(GTK_WIDGET(commStru->continueButton));


//printf("pauseButton change selection\n");

    GtkTreeSelection	*selection;
  	selection  = gtk_tree_view_get_selection((GtkTreeView *)commStru->listView);
	gtk_tree_selection_set_mode(selection, GTK_SELECTION_MULTIPLE);



    TRACEIT(10,"ListWidget_pauseButton		ends");
    return;
}


void ListWidget_continueButton(GtkButton *button, struct ListWidgetCommunicationStruct *commStru){
    TRACEIT(10,"ListWidget_continueButton		start");
    if (commStru == NULL){
        TRACEIT(2,"ListWidget_continueButton		ends, Parameter NULL");
        return;
    }

    commStru->iListPaused -= 1;
    if (commStru->iListPaused <= 0){
        commStru->iListPaused = 0;
    }
    else{
        TRACEIT(10,"ListWidget_continueButton		end");
        return;
    }

    GtkTreeSelection	*selection;
  	selection  = gtk_tree_view_get_selection((GtkTreeView *)commStru->listView);
	gtk_tree_selection_set_mode(selection, GTK_SELECTION_SINGLE);

	gtk_tree_selection_select_iter(selection, commStru->iterLastProcessedItem);

    gtk_widget_hide(GTK_WIDGET(commStru->continueButton));
    gtk_widget_show(GTK_WIDGET(commStru->pauseButton));

    g_mutex_lock (commStru->mtxContinue);
    //if we had to wait for the mutex, we know, that the next thread has not been started

    commStru->iListPaused = 0;

    if (0 == commStru->iNextThreadStarted){
        //Start next thread
        commStru->iNextThreadStarted = 1;

        commStru->currentItem = commStru->currentItem->NextItem;

        if (commStru->ThreadStartStruct == NULL){
            //if this is NULL, we assume that we are in CopyReMoveMode
            if (NULL != commStru->currentItem){
                commStru->gthrdCopyReMoveThread_2 = g_thread_create((GThreadFunc)CopyReMoveItemsViaList, (void *)commStru , TRUE, NULL);
            }
            else {
                g_thread_create((GThreadFunc)CopyReMoveItemsViaList, (void *)commStru , FALSE, NULL);
            }
        }
        else{
            //if it is not NULL, we are in UserCommandListMode
            if (NULL != commStru->currentItem){
                commStru->gthrdCopyReMoveThread_2 = g_thread_create((GThreadFunc)CallUserCommand_ProcessItemThread, (void *)commStru , TRUE, NULL);
            }
            else {
                g_thread_create((GThreadFunc)CallUserCommand_ProcessItemThread, (void *)commStru , FALSE, NULL);
            }
        }

    }


    g_mutex_unlock (commStru->mtxContinue);

    TRACEIT(10,"ListWidget_continueButton		ends");
    return;
}


void copyReMoveWidget_updateItemCounterColumn(struct CopyReMoveList *theList){
    TRACEIT(10,"copyReMoveWidget_updateItemCounterColumn		start");
    if (NULL == theList){
        TRACEIT(2,"copyReMoveWidget_updateItemCounterColumn		PARAMETER NULL");
        return;
    }

    struct CopyReMoveList *listbuf = NULL;
    int intItemsCount = 0;
    int i;
    GString *gstrBuf = g_string_sized_new (10);

    for (; theList->PrevItem; theList=theList->PrevItem);

    listbuf = theList;
    while(listbuf!= NULL){
		intItemsCount++;
		listbuf = listbuf->NextItem;
	}
	listbuf = theList;
	i=1;
	while(listbuf!= NULL){
        g_string_printf (gstrBuf, "%d/%d", i++, intItemsCount);
		gtk_list_store_set (listbuf->listStore, &(listbuf->iter), 0, gstrBuf->str,-1);
		listbuf = listbuf->NextItem;
	}


    g_string_free (gstrBuf, TRUE);
    TRACEIT(10,"copyReMoveWidget_updateItemCounterColumn		ends");
    return;
}


void ProcessItemsViaList(void *data){
	TRACEIT(10,"ProcessItemsViaList		start");
	if (data == NULL){
		TRACEIT(2,"ProcessItemsViaList		ends, parameter NULL");
		g_thread_exit(0);
	}
	GtkWidget *Window, *scrollWindow, *listView;
	GtkListStore *listStore;
	GtkTreeIter iter;
	GtkTreeViewColumn   	*col_Items, *col_ItemNumbers, *col_Errors, *col_percentage;
	GtkCellRenderer     	*renderer, *renderer2;
	GtkButton *CancelButton;
	GtkButton *CloseButton;
	GtkButton *continueButton;
	GtkButton *pauseButton;
	GtkWidget *TheLabel;
	GtkWidget *progressBar;
	GtkVBox *VBox;
	GtkWidget *ButtonBox;
	char charLabelText[301];
	GError *Error=NULL;
	GString *WindowName = g_string_new(MYNAMESHORT); //freed when the commStru is freed
	struct CopyReMoveList *list = (struct CopyReMoveList *)data;
	struct CopyReMoveList *listbuf = list;
	int i=0;
	int intItemsCount=0;
    struct ListWidgetCommunicationStruct *commStru = malloc(sizeof(struct ListWidgetCommunicationStruct));
	myWrap_gdk_threads_enter("ProcessItemsViaList");

	Window = CreateMyWindow(WindowName->str, NULL, 400,350,TRUE, FALSE, TRUE, NULL, FALSE, FALSE);
	if (Window == NULL){
        g_string_free(WindowName, TRUE);
        free(commStru);
        TRACEIT(1,"ProcessItemsViaList		end, window could not be created");
        return;
	}
    g_signal_connect (Window,"delete-event",G_CALLBACK(ListWidget_DeleteEvent),commStru);
    //g_signal_connect (Window, "destroy", G_CALLBACK(ListWidget_End_FreeMemory),commStru);

	AddMyIconsToWindow((GtkWindow *)Window);

  	char buf[320];

	scrollWindow = gtk_scrolled_window_new (NULL,NULL);
	listView = gtk_tree_view_new ();
	listStore = gtk_list_store_new( 4 , G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING  );
    progressBar = gtk_progress_bar_new();

    col_Items=gtk_tree_view_column_new();
	col_ItemNumbers=gtk_tree_view_column_new();
	col_Errors=gtk_tree_view_column_new();
	col_percentage=gtk_tree_view_column_new();
	gtk_tree_view_column_set_resizable(col_Items,TRUE);
	gtk_tree_view_column_set_resizable(col_ItemNumbers,TRUE);
	gtk_tree_view_column_set_resizable(col_Errors,TRUE);
	gtk_tree_view_column_set_resizable(col_percentage,TRUE);

	gtk_tree_view_column_set_title(col_ItemNumbers, "#");
	gtk_tree_view_append_column(GTK_TREE_VIEW(listView),col_ItemNumbers);
	renderer2 = gtk_cell_renderer_text_new();
	gtk_tree_view_column_pack_start(col_ItemNumbers, renderer2, TRUE);
	gtk_tree_view_column_add_attribute(col_ItemNumbers,renderer2,"text",0);

    gtk_tree_view_column_set_title(col_percentage, "%");
	gtk_tree_view_append_column(GTK_TREE_VIEW(listView),col_percentage);
	renderer = gtk_cell_renderer_text_new();
	gtk_tree_view_column_pack_start(col_percentage, renderer, TRUE);
	gtk_tree_view_column_add_attribute(col_percentage,renderer,"text",1);

	gtk_tree_view_column_set_title(col_Items, "items to process");
	gtk_tree_view_append_column(GTK_TREE_VIEW(listView),col_Items);
	renderer = gtk_cell_renderer_text_new();
	gtk_tree_view_column_pack_start(col_Items, renderer, TRUE);
	gtk_tree_view_column_add_attribute(col_Items,renderer,"text",2);

	gtk_tree_view_column_set_title(col_Errors, "errors");
	gtk_tree_view_append_column(GTK_TREE_VIEW(listView),col_Errors);
	renderer = gtk_cell_renderer_text_new();
	gtk_tree_view_column_pack_start(col_Errors, renderer, TRUE);
	gtk_tree_view_column_add_attribute(col_Errors,renderer,"text",3);

	//g_signal_connect (listView,"button-press-event",G_CALLBACK(oneMouseButtonPressedInLittleWindow),NULL);
	//g_signal_connect (listView,"button-press-event",G_CALLBACK(oneMouseButtonPressed),NULL);
	//g_signal_connect (listView,"button-release-event",G_CALLBACK(oneMouseButtonPressed),NULL);

	do{
		intItemsCount++;
		listbuf = listbuf->NextItem;
	}while(listbuf!= NULL);
	listbuf = list;
	i=1;
	do{
		snprintf(buf,319,"%d/%d",i,intItemsCount);
		gtk_list_store_append(listStore,&iter);
		gtk_list_store_set (listStore, &iter, 0, buf, 2, listbuf->string1->str,-1);


        CopyReMoveList_initializeMemberTo (listbuf, listStore, iter, &(commStru->ui64TotalBytesProcessed), &(commStru->i64StartTotalBytesCopied), &(commStru->timeStartTimeForBytesCopied), &(commStru->timeStartTimeForBytesCopiedSingleItem), &(commStru->ui64TotalBytes), &(commStru->iTotalBytesValid), (GtkProgressBar *)progressBar, list->CopyMoveOrRemove, &(commStru->dCopySpeed) );
        /*
        listbuf->listStore=listStore;
		listbuf->iter = iter;
		listbuf->status = -1;
		listbuf->ibreak=0;
		listbuf->lastfinishedTotalBytes=0;
		listbuf->threadCalcSize=NULL;
		listbuf->ui64pTotalBytesCopied = &(commStru->ui64TotalBytesProcessed);
		listbuf->ui64TotalBytes = &(commStru->ui64TotalBytes);
		listbuf->iTotalBytesValid = &(commStru->iTotalBytesValid);
		listbuf->progressBar = (GtkProgressBar *)progressBar;
		listbuf->CopyMoveOrRemove = list->CopyMoveOrRemove;
        */
		listbuf = listbuf->NextItem;
		i++;
	}while(listbuf!= NULL);

    CancelButton = (GtkButton *)gtk_button_new_with_label("cancel all");
	g_signal_connect (CancelButton,"clicked",G_CALLBACK(ListWidget_CancelButton),(void *)commStru);

	CloseButton = (GtkButton *)gtk_button_new_with_label("close");
	g_signal_connect (CloseButton,"clicked",G_CALLBACK(ListWidget_CloseButton),commStru);

    continueButton = (GtkButton *)gtk_button_new_with_label("continue");
    g_signal_connect (continueButton, "clicked", G_CALLBACK(ListWidget_continueButton), commStru);

    pauseButton = (GtkButton *)gtk_button_new_with_label("pause");
    g_signal_connect (pauseButton, "clicked", G_CALLBACK(ListWidget_pauseButton), commStru);

	gtk_tree_view_set_model (GTK_TREE_VIEW(listView), GTK_TREE_MODEL(listStore));

    //TheLabel = gtk_label_new(charLabelText);
	if (list->CopyMoveOrRemove == 1){ //COPY
		g_string_append(WindowName," copy");
		snprintf(charLabelText, 300, "Copying listed items to\n%s",list->DestObject->gstrPath->str );
	}
	else if(list->CopyMoveOrRemove == 2){ //MOVE
		g_string_append(WindowName," move");
		snprintf(charLabelText, 300, "Moving listed items to\n%s",list->DestObject->gstrPath->str );
	}
	else{	//REMOVE
		g_string_append(WindowName," remove");
		snprintf(charLabelText, 300, "Deleting listed items");
	}

	TheLabel = gtk_label_new(charLabelText);

	gtk_container_add((GtkContainer *)scrollWindow, listView);
	VBox = g_object_new(GTK_TYPE_VBOX, "spacing", 5, "homogeneous", FALSE, NULL);
	gtk_box_pack_start(GTK_BOX(VBox),TheLabel,FALSE,FALSE,1);
	gtk_box_pack_start(GTK_BOX(VBox),GTK_WIDGET(scrollWindow),TRUE,TRUE,1);
	#ifndef NO_GIO
	if (list->CopyMoveOrRemove != 3){   //do not show progressbar when deleting
        gtk_box_pack_start(GTK_BOX(VBox),GTK_WIDGET(progressBar),FALSE,FALSE,1);
	}
    #endif

    ButtonBox = gtk_hbox_new ( TRUE, 3);

    gtk_box_pack_start(GTK_BOX(ButtonBox),GTK_WIDGET(pauseButton),TRUE,TRUE,1);
    gtk_box_pack_start(GTK_BOX(ButtonBox),GTK_WIDGET(continueButton),TRUE,TRUE,1);
	gtk_box_pack_start(GTK_BOX(ButtonBox),GTK_WIDGET(CancelButton),TRUE,TRUE,1);
    gtk_box_pack_start(GTK_BOX(ButtonBox),GTK_WIDGET(CloseButton),TRUE,TRUE,1);

    gtk_box_pack_start(GTK_BOX(VBox),GTK_WIDGET(ButtonBox),FALSE,FALSE,1);



    gtk_container_add((GtkContainer *)Window,(GtkWidget *)VBox);

    commStru->mainListWidget=Window;
    commStru->listView=listView;
    commStru->cancelButton=CancelButton;
    commStru->closeButton=CloseButton;
    commStru->continueButton=continueButton;
    commStru->pauseButton=pauseButton;
    commStru->firstItem=list;
    commStru->currentItem=list;
    commStru->listStore=listStore;
    #ifndef NO_GIO
    commStru->cancellable=g_cancellable_new();
    #else
    commStru->cancellable=NULL;
    #endif
    commStru->gerror=NULL;
    commStru->SecondThreadRunning=0;
    commStru->TheLabel=TheLabel;
    commStru->WindowName=WindowName;
    commStru->iSomeError=0;
    commStru->iCancelled=0;  //becomes 1 if the process is cancelled
    commStru->iListPaused=0;
    commStru->iListPausedAutomatic=0;
    commStru->FreeMemoryAndDeleteWidgetFunction = FreeMemoryAndDeleteWidgetFunction;
    commStru->CancelListFunction = ListWidget_CancelList;
    commStru->iCancelDialogRuns=0;
    commStru->iDestroyFlag =0;
    commStru->bOverWriteAllExisting=-1;
    commStru->ui64TotalBytes=0;
    commStru->ui64TotalBytesProcessed=0;
    commStru->i64StartTotalBytesCopied=-1.0;
    commStru->dCopySpeed=0.0;
    //commStru->timeStartTimeForBytesCopied=0;
    commStru->iTotalBytesValid=0;
    commStru->progressBar=(GtkProgressBar *)progressBar;
    commStru->gthrdCopyReMoveThread_1 = NULL;
    commStru->gthrdCopyReMoveThread_2 = NULL;
    commStru->mtxContinue = g_mutex_new ();
    commStru->iNextThreadStarted = 1;
    commStru->ThreadStartStruct = NULL;

	gtk_widget_show_all(Window);
	gtk_widget_hide(GTK_WIDGET(CloseButton));
	gtk_widget_hide(GTK_WIDGET(continueButton));

	prepareWidgetForDrop ((GtkWidget *)commStru->listView, 1, DNDSignal_dragdatereceive_forCopyReMoveWidget, commStru);
	g_signal_connect ((GtkWidget *)commStru->listView,"drag-motion",G_CALLBACK(CopyReSizeWidget_dragmotion_signal),commStru);
	g_signal_connect ((GtkWidget *)commStru->listView,"drag-leave",G_CALLBACK(CopyReSizeWidget_dragleave_signal),commStru);
    g_signal_connect ((GtkWidget *)commStru->listView,"key-press-event",G_CALLBACK(CopyReSizeWidget_oneKeyPressedInView),commStru);
    g_signal_connect ((GtkWidget *)commStru->listView,"button-press-event",G_CALLBACK(ListWidget_MousePressEventCatcher),commStru);

	myWrap_gdk_threads_leave("ProcessItemsViaList");

    if (list->CopyMoveOrRemove != 3){    //do not calculate this when deleting
        g_thread_create((GThreadFunc)CopyReMoveItemsViaList_SizeCalc, (void *)commStru , FALSE, NULL);
    }
	//g_thread_create((GThreadFunc)CopyReMoveItemsViaList, (void *)commStru , FALSE, &Error); //why using here error at all when we do not check it
	commStru->gthrdCopyReMoveThread_1 = g_thread_create((GThreadFunc)CopyReMoveItemsViaList, (void *)commStru , TRUE, &Error); //why using here error at all when we do not check it

    if (Error){
        g_error_free(Error);
    }

	TRACEIT(10,"ProcessItemsViaList		ends");
	g_thread_exit(0);
}



gboolean CopyReSizeWidget_oneKeyPressedInView(GtkWidget *widget, GdkEventKey *event, struct ListWidgetCommunicationStruct *CommStru){
    //Be aware of using widget, as this function is called when a key is pressed on a notebook (tab) too
    //data = struct SideObjectList
	TRACEIT(10,"CopyReSizeWidget_oneKeyPressedInView		start");
	guint modifiers;

	modifiers = gtk_accelerator_get_default_mod_mask ();

	if ( (event->keyval == GDK_Delete || event->keyval == GDK_BackSpace)){
        //delete selected items
        if (CommStru->iListPaused != 0){
            CopyReSizeWidget_deleteSelectedItems (CommStru);
            copyReMoveWidget_updateItemCounterColumn(CommStru->firstItem);

            if (CommStru->ThreadStartStruct == NULL){
                if (CommStru->iTotalBytesValid==1){
                    if (CommStru->firstItem->CopyMoveOrRemove != 3){    //do not calculate this when deleting
                        g_thread_create((GThreadFunc)CopyReMoveItemsViaList_SizeCalc, (void *)CommStru , FALSE, NULL);
                    }
                }
            }
            return TRUE;
        }
	}



    TRACEIT(10,"CopyReSizeWidget_oneKeyPressedInView		end");
    return FALSE;
}


void CopyReSizeWidget_deleteSelectedItems (struct ListWidgetCommunicationStruct *CommStru){
    TRACEIT(10,"CopyReSizeWidget_deleteSelectedItems		start");
    //we will delete the marked items, which are not processed yet
    GtkTreeSelection *selection;
    struct CopyReMoveList *buflist = NULL;
    struct CopyReMoveList *buflist2 = NULL;

    selection  = gtk_tree_view_get_selection((GtkTreeView *)CommStru->listView);
    GList *glistBuf1 = NULL;
    GList *glistBuf2 = NULL;
    GList *glistBuf = NULL;
    int i;
    int a,b;

    buflist = CommStru->firstItem;

    i=0;
    while (buflist){
        if (TRUE == gtk_tree_selection_iter_is_selected (selection, &(buflist->iter))){
            if (buflist->status != -1){ //-1=Not processed yet
                //gtk_tree_selection_unselect_iter (selection, &(buflist->iter));
            }
            else{
                gtk_list_store_remove(CommStru->listStore, &(buflist->iter));

                if (CommStru->ThreadStartStruct != NULL){
                    glistBuf1 = g_list_prepend ( glistBuf1, g_list_nth (CommStru->ThreadStartStruct->glistItemsWithPath, i));
                    glistBuf2 = g_list_prepend ( glistBuf2, g_list_nth (CommStru->ThreadStartStruct->glistItemsWithoutPath, i));
                    //then we are in UserCommandViaList mode
                    //and we have to delete the selected items in the glists of ThreadStartStruct and the sizeOfTheList member variables
                   //CommStru->ThreadStartStruct->uiSizeOfLists = g_list_length (CommStru->ThreadStartStruct->glistItemsWithP

                }
                buflist2 = buflist;
                buflist = buflist->NextItem;
                CommStru->firstItem = CopyReMoveList_removeItemFromList (buflist2);
                continue;
            }
        }
        i++;
        buflist = buflist->NextItem;
    }

    if (CommStru->ThreadStartStruct != NULL){
        glistBuf = glistBuf1;
        while (glistBuf){
            if ( ((GList *)glistBuf->data)->data != NULL){
                g_string_free ( (GString *)(((GList *)glistBuf->data)->data), TRUE);
                ((GList *)glistBuf->data)->data = NULL;
            }
            glistBuf = glistBuf->next;
        }
        glistBuf = glistBuf2;
        while (glistBuf){
            if ( ((GList *)glistBuf->data)->data != NULL){
                g_string_free ( (GString *)(((GList *)glistBuf->data)->data), TRUE);
                ((GList *)glistBuf->data)->data = NULL;
            }
            glistBuf = glistBuf->next;
        }


        glistBuf = CommStru->ThreadStartStruct->glistItemsWithPath;
        while (glistBuf){
            if (glistBuf->data == NULL){
                CommStru->ThreadStartStruct->glistItemsWithPath =  g_list_delete_link ( CommStru->ThreadStartStruct->glistItemsWithPath, glistBuf);
                glistBuf = CommStru->ThreadStartStruct->glistItemsWithPath;
                continue;
            }
            glistBuf = glistBuf->next;
        }
        glistBuf = CommStru->ThreadStartStruct->glistItemsWithoutPath;
        while (glistBuf){
            if (glistBuf->data == NULL){
                CommStru->ThreadStartStruct->glistItemsWithoutPath =  g_list_delete_link ( CommStru->ThreadStartStruct->glistItemsWithoutPath, glistBuf);
                glistBuf = CommStru->ThreadStartStruct->glistItemsWithoutPath;
                continue;
            }
            glistBuf = glistBuf->next;
        }

        a = g_list_length (CommStru->ThreadStartStruct->glistItemsWithoutPath);
        b = g_list_length (CommStru->ThreadStartStruct->glistItemsWithPath);
        if (a != b ){
            TRACEIT(10,"CopyReSizeWidget_deleteSelectedItems		Internal Error, list sizes unequal");
        }
        a = (a<=b)?a:b;
        CommStru->ThreadStartStruct->uiSizeOfLists = a;
    }



    g_list_free (glistBuf1);
    g_list_free (glistBuf2);
    TRACEIT(10,"CopyReSizeWidget_deleteSelectedItems		end");
    return;
}


//The "drag-motion" signal
gboolean CopyReSizeWidget_dragmotion_signal (GtkWidget *widget, GdkDragContext *drag_context, gint x, gint y, guint time, struct ListWidgetCommunicationStruct *CommStru){
    TRACEIT(10,"CopyReSizeWidget_dragmotion_signal		start");

    if (CommStru->iListPausedAutomatic == 0){
        CommStru->iListPausedAutomatic=1;
        gtk_button_clicked (CommStru->pauseButton );
    }



    TRACEIT(10,"CopyReSizeWidget_dragmotion_signal		ends");
    return TRUE;
}


//The "drag-leave" signal
void CopyReSizeWidget_dragleave_signal (GtkWidget *widget, GdkDragContext *drag_context, guint time, struct ListWidgetCommunicationStruct *CommStru){
    TRACEIT(10,"CopyReSizeWidget_dragleave_signal		start");

    CommStru->iListPausedAutomatic=0;
    gtk_button_clicked (CommStru->continueButton );

    TRACEIT(10,"CopyReSizeWidget_dragleave_signal		ends");
}


/*
gboolean oneMouseButtonPressedInLittleWindow(GtkWidget *treeview, GdkEventButton *event, gpointer data){
	TRACEIT(10,"oneMouseButtonPressedInLittleWindow		start - ends");
	return TRUE;
	//return FALSE;
}
*/

void calcDirectorySizeInView_closeProcess (int *iRecursiveSizeCalculation, GThread **threadToJoin, gboolean *bpUpdateFlag_F6, int *ipPreventFromUpdating, gboolean bAbortThread){
    //be sure to call this function every time the user could have started the F6-update
    //in case of bAbortthread is FALSE, we will wait for the threat to finish, if it is TRUE, we will
    //set iRecursiveSizeCalculation to -1 and will wait for the thread.
    TRACEIT(10,"calcDirectorySizeInView_closeProcess		start");

    int iThreadIsRunnnig=0;

    if (iRecursiveSizeCalculation != NULL){
        if (*iRecursiveSizeCalculation == 0)
            iThreadIsRunnnig = 1;
        if (bAbortThread==TRUE){
            *iRecursiveSizeCalculation = -1;    //set abort flag
        }
    }

    if (threadToJoin != NULL && *threadToJoin != NULL){
        //printf("Trying to join thread: %p\n", *threadToJoin);
        //if (iThreadIsRunnnig == 1)  //we are signal driven, arent we, all signals are threadsave, but the F6 Thread needs to enter and leave this too, so we will have to leave the signal thread mutex here before joining the F6 thread
        //    myWrap_gdk_threads_leave ("calcDirectorySizeInView_closeProcess");
        //while (iRecursiveSizeCalculation == 0){
        //    usleep (100000);
        //}
        //printf("Thread joined : %p\n",*threadToJoin );
        g_thread_join ( *threadToJoin );  //wait for thread
        //if (iThreadIsRunnnig == 1)
        //    myWrap_gdk_threads_enter ("calcDirectorySizeInView_closeProcess");
        //printf ("Joining ended\n");
        *threadToJoin = NULL;
    }

    if (bpUpdateFlag_F6 != NULL){
        *bpUpdateFlag_F6 = FALSE;
    }

    if (ipPreventFromUpdating != NULL && ipPreventFromUpdating > 0){
        manipulateCounter (ipPreventFromUpdating, FALSE, "calcDirectorySizeInView_closeProcess");
    }

    TRACEIT(10,"calcDirectorySizeInView_closeProcess		end");
}


void calcDirectorySizeInView_wrap(struct AllInformationAllUseStruct *stru){
    TRACEIT(10,"calcDirectorySizeInView_wrap		start");
    GError *Error=NULL;
    if(!g_thread_supported() ){
		g_thread_init(NULL);
     		//gdk_threads_init();                   // Called to initialize internal mutex "gdk_threads_mutex".
   	}
   	//stru->source=SOURCE_FILESHELF;
    //stru->Info_1 = data;
    //stru->Info_2 = &(Object->gthreadF6Updage);
    //stru->Info_3 = &(Object->iRecursiveSizeCalculation);
    //stru->Info_4 = &(Object->bUpdateFlag_F6);

    if (stru == NULL ){
        TRACEIT(2,"calcDirectorySizeInView_wrap:       parameter NULL");
        return;
    }

    if (stru->Info_2==NULL){
        free(stru);
        TRACEIT(2,"calcDirectorySizeInView_wrap:       parameter 2 NULL");
        return;
    }
    int *iRecursiveSizeCalculation = (int *)(stru->Info_3);
    GThread *gthreadF6 = *(GThread **)(stru->Info_2);

    if (*iRecursiveSizeCalculation == 0) {   //-1 = canceled, finished, 0=running
        TRACEIT(10,"calcDirectorySizeInView_wrap		end");
        free(stru);
        return;
    }

    if (gthreadF6 != NULL){
        //printf("Thread joined : %p\n",gthreadF6 );
        g_thread_join (gthreadF6);
        *((GThread **)(stru->Info_2)) = NULL;
    }
    *iRecursiveSizeCalculation = 0;

    if (stru->Info_4 != NULL){
        *((gboolean *)stru->Info_4) = TRUE;
    }

    *((GThread **)(stru->Info_2)) = g_thread_create((GThreadFunc)calcDirectorySizeInView_thread, (void *)stru , TRUE, &Error);
    //printf("Thread created: %p\n", *((GThread **)(stru->Info_2)) );
    //g_thread_create((GThreadFunc)calcDirectorySizeInView_thread, (void *)stru , FALSE, &Error);
    if (Error){
		TRACEIT(2,"calcDirectorySizeInView_wrap: error in g_thread_create:");
		TRACEIT(2,Error->message);
		g_error_free(Error);
		Error=NULL;
		free(stru);
		*iRecursiveSizeCalculation = -1;
		if (stru->Info_4 != NULL){
            *((gboolean *)stru->Info_4) = FALSE;
        }
	}
    TRACEIT(10,"calcDirectorySizeInView_wrap		end");
}


static void calcDirectorySizeInView_thread_helper (GtkTreeModel *model,GtkTreePath *path,GtkTreeIter *iter, void *data){
    TRACEIT(10,"calcDirectorySizeInView_thread_helper		start");
    struct AllInformationAllUseStruct *stru = (struct AllInformationAllUseStruct *)data;
    int type_d;
    gchar *gcharname=NULL;
    gchar *gcharpath=NULL;
    GString *gstrPath=NULL;
    guint64 size=0;
    int ColumnSize;
    int *iRecursiveSizeCalculation = (int *)stru->Info_3;
    GtkListStore *listStore=NULL;

    if (iRecursiveSizeCalculation == NULL){
        TRACEIT(2,"calcDirectorySizeInView_thread_helper		parameter NULL");
        return;
    }

    if (*iRecursiveSizeCalculation != 0){
        return;
    }

    gstrPath = g_string_sized_new(10);

    if (stru->source==SOURCE_SIDEOBJECT){
        struct SideObjectList* Object = stru->Info_1;
        //iRecursiveSizeCalculation = &(Object->iRecursiveSizeCalculation);

        ColumnSize = COL_SIZE;
        listStore = Object->listStore;
        //myWrap_gdk_threads_enter("calcDirectorySizeInView_thread_helper 1");
        gtk_tree_model_get(model, iter,COL_NAME,&gcharname,COL_TYPE_D,&type_d, -1);
        //myWrap_gdk_threads_leave("calcDirectorySizeInView_thread_helper 1");
        g_string_assign(gstrPath, Object->gstrPath->str);
    }
    else{
        struct FileShelfObjectList* Object = stru->Info_1;
        //iRecursiveSizeCalculation = &(Object->iRecursiveSizeCalculation);
        ColumnSize = COL_FLSHLF_SIZE;
        listStore = Object->listStore;
        //myWrap_gdk_threads_enter("calcDirectorySizeInView_thread_helper 2");
        gtk_tree_model_get(model, iter, COL_FLSHLF_PATH, &gcharpath, COL_FLSHLF_NAME, &gcharname, COL_FLSHLF_TYPE_D, &type_d, -1);
        //myWrap_gdk_threads_leave("calcDirectorySizeInView_thread_helper 2");
        g_string_assign(gstrPath, gcharpath);
    }

    if (strcmp(gcharname, ".")!=0 && strcmp(gcharname, "..")!=0 && *iRecursiveSizeCalculation == 0 && type_d == 4){
        checkPathForLastChar(gstrPath);
        g_string_append(gstrPath, gcharname);
        if (0 != getDirectorySize(&size, gstrPath, iRecursiveSizeCalculation, FALSE)){ //error opening the directory
            size=0;
        }
        //myWrap_gdk_threads_enter("calcDirectorySizeInView_thread_helper 3");
        if (*iRecursiveSizeCalculation == 0){
            gtk_list_store_set (listStore, iter, ColumnSize, size, -1);
        }
        if (stru->source==SOURCE_SIDEOBJECT)
            updateStatusBar((struct SideObjectList *)stru->Info_1);
        //myWrap_gdk_threads_leave("calcDirectorySizeInView_thread_helper 3");

    }

    if (gcharname)
        g_free(gcharname);
    if (gcharpath)
        g_free(gcharpath);
    g_string_free(gstrPath, TRUE);
    TRACEIT(10,"calcDirectorySizeInView_thread_helper		end");
}


void calcDirectorySizeInView_thread(struct AllInformationAllUseStruct *stru){
    TRACEIT(10,"calcDirectorySizeInView_thread		start");
    GtkListStore		*listStore=NULL;
    int sort_column_id;
    GtkSortType order;
    //stru->source=SOURCE_FILESHELF;
    //stru->Info_1 = data;
    //stru->Info_2 = &(Object->gthreadF6Updage);
    //stru->Info_3 = &(Object->iRecursiveSizeCalculation);
    int *iRecursiveSizeCalculation = (int *)stru->Info_3;

    if (stru->source==SOURCE_SIDEOBJECT){
        struct SideObjectList* Object = stru->Info_1;
        listStore = Object->listStore;

        manipulateCounter (&(Object->iPreventFromUpdating), TRUE, "calcDirectorySizeInView_thread");
        SideObjectList_UpdateObject(Object);
        g_thread_yield ();  //give some cpu time to other threads, other wise its very unlikely that SideObjectList_UpdateObject changes take place in the views tab
                            //however, this isnt a garantie that the changes take place before this thread quits


        //When the actual sort column is the size column there is a problem.
        //Every time we update an item with its size, the list will be reordered.
        //This means, when we come to an directory ehich is very little and therefore is placed to the end
        //of the list, the next item in the function for_each... is simply nothing because the list ends
        //with the current item and we break the function and may leave many directories unchecked.
        //To avoid this we set the current sort column to the definition GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID
        //and the list is not going to reorder its items. But we have to set the right column when we are finished.
        myWrap_gdk_threads_enter("calcDirectorySizeInView_thread 1");
        gtk_tree_sortable_get_sort_column_id (GTK_TREE_SORTABLE(Object->listStore), &sort_column_id, &order);
        if (COL_SIZE == sort_column_id){
            gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE(Object->listStore),GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID, order);
        }
        //myWrap_gdk_threads_leave("calcDirectorySizeInView_thread 1");

        gtk_tree_model_foreach(GTK_TREE_MODEL(listStore),(GtkTreeModelForeachFunc) calcDirectorySizeInView_thread_helper,(void *)stru);

        //myWrap_gdk_threads_enter("calcDirectorySizeInView_thread 2");
        if (COL_SIZE == sort_column_id){
            gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE(Object->listStore),COL_SIZE, order);
        }
        myWrap_gdk_threads_leave("calcDirectorySizeInView_thread 2");
    }
    else{
        struct FileShelfObjectList* Object = stru->Info_1;
        listStore = Object->listStore;

        //When the actual sort column is the size column there is a problem.
        //Every time we update an item with its size, the list will be reordered.
        //This means, when we come to an directory ehich is very little and therefore is placed to the end
        //of the list, the next item in the function for_each... is simply nothing because the list ends
        //with the current item and we break the function and may leave many directories unchecked.
        //To avoid this we set the current sort column to the definition GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID
        //and the list is not going to reorder its items. But we have to set the right column when we are finished.
        myWrap_gdk_threads_enter("calcDirectorySizeInView_thread 3");
        gtk_tree_sortable_get_sort_column_id (GTK_TREE_SORTABLE(Object->listStore), &sort_column_id, &order);
        if (COL_FLSHLF_SIZE == sort_column_id){
            gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE(Object->listStore),GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID, order);
        }
        //myWrap_gdk_threads_leave("calcDirectorySizeInView_thread 3");
        gtk_tree_model_foreach(GTK_TREE_MODEL(listStore),(GtkTreeModelForeachFunc) calcDirectorySizeInView_thread_helper,(void *)stru);
        //myWrap_gdk_threads_enter("calcDirectorySizeInView_thread 4");
        if (COL_FLSHLF_SIZE == sort_column_id){
            gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE(Object->listStore),COL_FLSHLF_SIZE, order);
        }
        myWrap_gdk_threads_leave("calcDirectorySizeInView_thread 4");
    }
    *iRecursiveSizeCalculation = -1;

    free(stru);

    TRACEIT(10,"calcDirectorySizeInView_thread		end");
    g_thread_exit(0);
}

