/*
    Copyright 2007, 2008, 2009, 2010 Geyer Klaus

    This file is part of Cat'sEyE.

    Cat'sEyE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Cat'sEyE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Cat'sEyE.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdlib.h>
#include <gtk/gtk.h>
#include <string.h>
#include <unistd.h>


#include "Lists.h"
#include "confighandling.h"
#include "confighandlingDefinitions.h"
#include "userdata.h"
#include "CommandParser.h"
#include "helpers.h"
#include "DragnDrop.h"
#include "threads.h"
#include "mytrace.h"
#include "myfm.h"
#include "globalDefinitions.h"

#include "allInclude.h"

void CallUserCommand_MainEntryThread (void *data);
int CallUserCommand_StartThreadStructure_freeStruct (struct _CallUserCommand_StartThreadStructure **ThreadStartStruct);
int CallUserCommand_CallToSystem (void *data);
int CallUserCommand_ProcessViaList(struct _CallUserCommand_StartThreadStructure *ThreadStartStruct);
int CallUserCommand_ParseCommand(struct _CallUserCommand_StartThreadStructure *ThreadStartStruct);


extern GtkWindow 		*win;

extern struct _OptionsList *g_OptionsList;
extern struct _OptionsList *g_BookMarkList;


void UserCommand_CreateUserMenu(GtkWidget *rootmenu, struct UserDataList *UserData, struct MenuObjectList **MenuObjects){
    //in, in, out
    //!do not forget to free the MenuObjects LIST !!!!!!!!!!!!!!!!!!!!!!!!
    //mind, that USerData->UserCommandName is changed in here!

	TRACEIT(10,"UserCommand_CreateUserMenu		start");

	UserData = UserDataList_getFirstItem (UserData);

	if (rootmenu == NULL || UserData == NULL || MenuObjects == NULL){
        TRACEIT (2, "UserCommand_CreateUserMenu             end, parameter NULL");
        return;
	}

	const char sep[] = "/";
	char *buf;
	int found = 0;
	int werehere = 0;
	GString *gstrName = g_string_sized_new(10);
	GString *gstrPath = g_string_sized_new(10);
	GtkWidget *LastMenu = NULL;
	GtkWidget *MenuBuf = NULL;
	struct MenuObjectList *MenuObjectsBuf=NULL;

	//Create MenuStructure internal
	while (UserData && UserData->gstrMenuName){
	    //overwrite the menuname with the completeName (for the bookmarks)
        if (UserData->gstrMenuName && UserData->gstrCompleteName){
            g_string_assign (UserData->gstrMenuName, UserData->gstrCompleteName->str);
        }

	    g_string_assign(gstrName, UserData->gstrMenuName->str);
	    cutCharFromGString(gstrName, '/',1); //cut leading /
		if(strstr(gstrName->str,sep) != NULL){
			g_string_assign(gstrPath, UserData->gstrMenuName->str);
			GetPathFromPathWithName(gstrPath, 1);
			getLastStringInPath(gstrName);

			LastMenu = rootmenu;
			werehere = 0;
			for (buf = strtok(gstrPath->str,sep); buf; buf=strtok(NULL,sep)){
				werehere = 1;
				if (*MenuObjects != NULL)
                    MenuObjectsBuf = MenuObjectList_getFirstItem(*MenuObjects);
                else
                    MenuObjectsBuf = NULL;
				found = 0;
				while (MenuObjectsBuf){
					if (strcmp (MenuObjectsBuf->gstrName->str, buf) == 0 && MenuObjectsBuf->MenuToAddTo == LastMenu){
						found = 1;
						MenuBuf = MenuObjectsBuf->submenu;
						LastMenu = MenuBuf;
						break;
					}
					MenuObjectsBuf = MenuObjectsBuf->nextItem;
				}
				if (found != 1){
				    //printf("UserCommand_CreateUserMenu      new menu with name: %s\n",buf);
					MenuBuf = gtk_menu_new();
					MenuObjectList_AddItem(MenuObjects, buf, MenuBuf, LastMenu);
					LastMenu = MenuBuf;
				}
			}
			if (werehere != 0){
				g_string_assign(UserData->gstrMenuName, gstrName->str);
				UserData->MenuToAddTo = MenuBuf;
			}
		}
		if (UserData->MenuToAddTo == NULL){
			UserData->MenuToAddTo = rootmenu;
		}
		UserData = UserData->nextItem;
	}
	g_string_free(gstrName, TRUE);
	g_string_free(gstrPath, TRUE);
	TRACEIT(10,"UserCommand_CreateUserMenu		end");
}


void UserCommand_AttachUserMenu(struct MenuObjectList *MenuObjects){
	TRACEIT(10,"UserCommand_AttachUserMenu		start");
	GtkWidget *MenuBuf = NULL;

	//Now create Menustructure for viewing
	MenuObjects = MenuObjectList_getFirstItem(MenuObjects);
	if (MenuObjects == NULL){
        TRACEIT (2, "UserCommand_AttachUserMenu         end, parameter NULL");
        return;
	}

	while (MenuObjects){
		MenuBuf = gtk_menu_item_new_with_label(MenuObjects->gstrName->str);
		gtk_menu_item_set_submenu ((GtkMenuItem *)MenuBuf,MenuObjects->submenu);
		gtk_menu_shell_append(GTK_MENU_SHELL(MenuObjects->MenuToAddTo), MenuBuf);
		MenuObjects = MenuObjects->nextItem;
	}
	TRACEIT(10,"UserCommand_AttachUserMenu		end");
}

/////////////////////////////////////////////////////////////////////////////////////////////////
///////////////// USER DATA LIST //////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////

void UserDataList_AddItem(struct UserDataList **theList, char *cpCompleteName, void *ListElement){
	TRACEIT(10,"UserDataList_AddItem			start");
	if (theList == NULL || cpCompleteName == NULL){
	    TRACEIT(2,"UserDataList_AddItem			end, Parameter NULL");
		return;
	}
	struct UserDataList *NewItem=NULL;
	struct UserDataList *bufItem=NULL;

	NewItem = malloc (sizeof (struct UserDataList));

	if (NULL == NewItem){
		TRACEIT(1,"UserDataList_AddItem			end, Error allocating memory");
		return;
	}

    NewItem->iSourceType = -1;
    NewItem->Object = NULL;
    NewItem->gstrCompleteName = g_string_new (cpCompleteName);
    NewItem->gstrMenuName = g_string_new(cpCompleteName);
    NewItem->ListElement = ListElement;
    NewItem->MenuToAddTo = NULL;
    NewItem->prevItem = NULL;
    NewItem->nextItem = NULL;

    if (*theList == NULL){
        *theList = NewItem;
        //printf ("new List: %p\n", NewItem);
    }
    else{
        bufItem = UserDataList_getLastItem(*theList);
        //printf ("add to List: %p\n", *theList);
        bufItem->nextItem = NewItem;
        NewItem->prevItem = bufItem;
    }

	TRACEIT(10,"UserDataList_AddItem			end");
}


void UserDataList_DeleteList(struct UserDataList **theList){
	TRACEIT(10,"UserDataList_DeleteList			start");
	if (NULL == theList || *theList == NULL){
		TRACEIT(2,"UserDataList_DeleteList			end, parameter NULL");
		return;
	}

	struct UserDataList *bufItem = UserDataList_getFirstItem(*theList);
    struct UserDataList *bufItem2 = NULL;
    //printf ("delete List: %p\n", bufItem);

    while (bufItem){
        if (bufItem->gstrCompleteName){
            g_string_free(bufItem->gstrCompleteName,TRUE);
        }
        if (bufItem->gstrMenuName){
            g_string_free(bufItem->gstrMenuName,TRUE);
        }
        bufItem2 = bufItem;
        bufItem = bufItem->nextItem;
        free(bufItem2);
    }
    *theList = NULL;

	TRACEIT(10,"UserDataList_DeleteList			end");
}


struct UserDataList *UserDataList_getStandardItem(struct UserDataList *theList){
    TRACEIT(10,"UserDataList_getStandardItem			start");

    theList = UserDataList_getFirstItem (theList);
    if (theList == NULL){
        TRACEIT(2,"UserDataList_getStandardItem			end, parameter NULL");
        return NULL;
    }

    while (theList != NULL){
        if ( ((struct _UserCommandList *)(theList->ListElement))->iIsDefaultCommand == 1){
            return theList;
        }
        theList = theList->nextItem;
    }

    TRACEIT(10,"UserDataList_getStandardItem			end");
    return NULL;
}


struct UserDataList *UserDataList_getItemByListElement(struct UserDataList *theList, void *ListElement){
    TRACEIT(10,"UserDataList_getItemByListElement			start");

    theList = UserDataList_getFirstItem (theList);
    if (theList == NULL){
        TRACEIT(2,"UserDataList_getItemByListElement			end, parameter NULL");
        return NULL;
    }

    while (theList != NULL){
        if ( theList->ListElement == ListElement){
            return theList;
        }
        theList = theList->nextItem;
    }

    TRACEIT(10,"UserDataList_getItemByListElement			end");
    return NULL;
}


struct UserDataList *UserDataList_getFirstItem(struct UserDataList *theList){
    TRACEIT(10,"UserDataList_getFirstItem			start");
    if (theList == NULL){
        TRACEIT(2,"UserDataList_getFirstItem			end, parameter NULL");
        return NULL;
    }

    while (theList->prevItem != NULL){
        theList = theList->prevItem;
    }

    TRACEIT(10,"UserDataList_getFirstItem			end");
    return theList;
}


struct UserDataList *UserDataList_getLastItem(struct UserDataList *theList){
    TRACEIT(10,"UserDataList_getLastItem			start");
    if (theList == NULL){
        TRACEIT(2,"UserDataList_getLastItem			end, parameter NULL");
        return NULL;
    }

    while (theList->nextItem != NULL){
        theList = theList->nextItem;
    }

    TRACEIT(10,"UserDataList_getLastItem			end");
    return theList;
}


int UserDataList_CreateDummyCommand (struct UserDataList *theDummyItem, const char *cpCommand, void *Object, int iSource, int iCallType){
    //this function creates a UserCommand with the given information to start it with the common functions
    //theDummyItem has to be a valid pointer!
    TRACEIT(10, "UserDataList_CreateDummyCommand        start");
    if (theDummyItem==NULL || cpCommand==NULL || Object==NULL){
        TRACEIT(2, "UserDataList_CreateDummyCommand     parameter NULL");
        return -1;
    }
    GString *gstrWorkingPath = g_string_sized_new (10);

    theDummyItem->ListElement = NULL;
    theDummyItem->gstrCompleteName = g_string_new(cpCommand);
    theDummyItem->gstrMenuName = g_string_new(cpCommand);
    theDummyItem->prevItem = NULL;
    theDummyItem->nextItem = NULL;
    theDummyItem->MenuToAddTo = NULL;
    theDummyItem->iSourceType = iSource;
    theDummyItem->Object = Object;

    struct _UserCommandList *UserCommandList = NULL;
    UserCommandList_addItem(&UserCommandList, "UserDataList_CreateDummyCommand");
    if (UserCommandList == NULL){
        TRACEIT(4, "UserDataList_CreateDummyCommand  unable to create UserCommandList");
        g_string_free(gstrWorkingPath, TRUE);
        return -1;
    }

    if (iSource == SOURCE_SIDEOBJECT){
        g_string_assign (gstrWorkingPath, ((struct SideObjectList *)Object)->gstrPath->str);
    }
    else{
        getHomeDirectory (gstrWorkingPath);
    }

    theDummyItem->ListElement = UserCommandList;
    UserCommandList->iValid = 1;
    UserCommandList_editItem(UserCommandList, 0, NULL, NULL, cpCommand, iCallType, NULL, NULL, NULL, NULL, NULL, NULL, gstrWorkingPath->str);

    g_string_free(gstrWorkingPath, TRUE);

    TRACEIT(10, "UserDataList_CreateDummyCommand        end");
    return 0;
}


/////////////////////////////////////////////////////////////////////////////////////////////////
///////////////// MENU OBJECT LIST //////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////

void MenuObjectList_AddItem(struct MenuObjectList **theList, char *Name, GtkWidget *submenu, GtkWidget *MenuToAddTo){
	TRACEIT(10,"MenuObjectList_AddItem			start");
	if (Name == NULL || submenu == NULL || MenuToAddTo == NULL || theList == NULL){
		TRACEIT(2,"MenuObjectList_AddItem			end, parameter NULL ");
		return;
	}

	struct MenuObjectList *NewItem;
	struct MenuObjectList *bufItem;

	NewItem = malloc (sizeof (struct MenuObjectList));

	if (NULL == NewItem){
		TRACEIT(1,"MenuObjectList_AddItem			end, error allocating memory");
		return;
	}

	NewItem->gstrName = g_string_new(Name);
	NewItem->submenu = submenu;
	NewItem->MenuToAddTo = MenuToAddTo;
	NewItem->nextItem=NULL;
	NewItem->prevItem=NULL;

    if (*theList == NULL){
        *theList = NewItem;
    }
    else{
        bufItem = MenuObjectList_getLastItem (*theList);

        bufItem->nextItem = NewItem;
        NewItem->prevItem = bufItem;
    }

	TRACEIT(10,"MenuObjectList_AddItem			end");
}


void MenuObjectList_DeleteList(struct MenuObjectList **theList){
	TRACEIT(10,"MenuObjectList_DeleteList			start");

    if (NULL == theList){
		TRACEIT(2,"MenuObjectList_DeleteList			end, parameter NULL");
		return;
	}
	struct MenuObjectList *bufItem = MenuObjectList_getFirstItem(*theList);;
	struct MenuObjectList *bufItem2 = NULL;

    if (bufItem == NULL){
        TRACEIT(2, "MenuObjectList_DeleteList           WTF?");
    }

	while (bufItem){
        if (bufItem->gstrName){
            g_string_free(bufItem->gstrName, TRUE);
        }

	    bufItem2 = bufItem;
        bufItem = bufItem->nextItem;
        free (bufItem2);
	}
	*theList=NULL;

    TRACEIT(10,"MenuObjectList_DeleteList			end");
}


struct MenuObjectList *MenuObjectList_getFirstItem(struct MenuObjectList *theList){
    TRACEIT(10,"MenuObjectList_getFirstItem			start");
    if (theList == NULL){
        TRACEIT(2,"MenuObjectList_getFirstItem			end, parameter NULL");
        return NULL;
    }

    while (theList->prevItem != NULL){
        theList = theList->prevItem;
    }

    TRACEIT(10,"MenuObjectList_getFirstItem			end");
    return theList;
}


struct MenuObjectList *MenuObjectList_getLastItem(struct MenuObjectList *theList){
    TRACEIT(10,"MenuObjectList_getLastItem			start");
    if (theList == NULL){
        TRACEIT(2,"MenuObjectList_getLastItem			end, parameter NULL");
        return NULL;
    }

    while (theList->nextItem != NULL){
        theList = theList->nextItem;
    }

    TRACEIT(10,"MenuObjectList_getLastItem			end");
    return theList;
}


/////////////////////////////////////////////////////////////////////////////////////////////////
///////////////// calling usercommands  //////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////


int CallUserCommand_StartThread (struct UserDataList *OurUserDataListItem, GList *glistItemsWithPath, GList *glistItemsWithoutPath, GString *gstrThisSidePath, GString *gstrOtherSidePath, GString *gstrLeftSidePath, GString *gstrRightSidePath, GString *gstrHomeDir){
    //return: -1=error, 0=ok
    //this (or the following function must take care of freeing all the items given here
    //mind to free the ListItem member of OurUserDtaListItem, because this is a real copy of the data (but only one item)
    //gstrThisSidePath and gstrOtherSidePath can be NULL in the case we come from a fileshelf!

    //this function will collect all the data and write it to some structure.
    //after that it will start a thread which then will parse the command and takes control of all leading thinks
    //including freeing the memory given to the structure including the memory allocated for the structure!
    TRACEIT(10,"CallUserCommand_StartThread			start");
    if (OurUserDataListItem == NULL /*|| glistItemsWithPath==NULL || glistItemsWithoutPath==NULL */|| gstrLeftSidePath==NULL || gstrRightSidePath==NULL || gstrHomeDir==NULL){
        TRACEIT (2, "CallUserCommand_StartThread            parameter NULL");
        return -1;
    }

    struct _CallUserCommand_StartThreadStructure *ThreadStartStruct = malloc (sizeof (struct _CallUserCommand_StartThreadStructure));
    if (ThreadStartStruct == NULL){
        TRACEIT (2, "CallUserCommand_StartThread            unable to allocate memory");
        return -1;
    }

    int iRet=0;
    GError *Error=NULL;

    ThreadStartStruct->OurUserDataListItem = OurUserDataListItem;
    ThreadStartStruct->glistItemsWithPath = glistItemsWithPath;
    ThreadStartStruct->glistItemsWithoutPath = glistItemsWithoutPath;
    ThreadStartStruct->gstrThisSidePath = gstrThisSidePath;
    ThreadStartStruct->gstrOtherSidePath = gstrOtherSidePath;
    ThreadStartStruct->gstrLeftSidePath = gstrLeftSidePath;
    ThreadStartStruct->gstrRightSidePath = gstrRightSidePath;
    ThreadStartStruct->gstrHomeDir = gstrHomeDir;
    ThreadStartStruct->gstrWorkingDir = NULL;
    ThreadStartStruct->gstrParsedCommand = NULL;
    ThreadStartStruct->gstrQuitReplyText = NULL;
    ThreadStartStruct->gstrActionNoteText = NULL;
    ThreadStartStruct->uiCurrentPositionInList = 0;
    ThreadStartStruct->uiSizeOfLists = g_list_length (glistItemsWithPath);
    ThreadStartStruct->iCancelAll = 0;  //0=continue, 1=cancel (for each and list mode)
    ThreadStartStruct->commStru = NULL;
    ThreadStartStruct->wdgtUserDialog = NULL;
    ThreadStartStruct->wdgtUSerDialogMainVBox = NULL;
    ThreadStartStruct->gcontDialogCondition = NULL;
    ThreadStartStruct->gcontEachCondition = NULL;

    g_thread_create((GThreadFunc)CallUserCommand_MainEntryThread, (void *)ThreadStartStruct , FALSE, &Error);

    if (Error){
        TRACEIT(4,"CallUserCommand_StartThread			error starting thread:");
        TRACEIT(4, Error->message);

        CallUserCommand_StartThreadStructure_freeStruct ( &ThreadStartStruct);

        g_error_free(Error);
        Error = NULL;
        free(ThreadStartStruct);
        iRet = -1;
    }

    TRACEIT(10,"CallUserCommand_StartThread			end");
    return iRet;
}


int CallUserCommand_StartThreadStructure_freeStruct (struct _CallUserCommand_StartThreadStructure **ThreadStartStruct){
    if (ThreadStartStruct == NULL || *ThreadStartStruct==NULL){
        TRACEIT (2, "CallUserCommand_StartThreadStructure_freeStruct        parameter NULL");
        return -1;
    }
    UserCommandList_freeList ( (struct _UserCommandList **)&(((*ThreadStartStruct)->OurUserDataListItem)->ListElement));

    UserDataList_DeleteList( &((*ThreadStartStruct)->OurUserDataListItem));

    if ((*ThreadStartStruct)->glistItemsWithPath){
        g_list_foreach((*ThreadStartStruct)->glistItemsWithPath, (GFunc) GListFreeGStrings, NULL); g_list_free((*ThreadStartStruct)->glistItemsWithPath);
    }
    if ((*ThreadStartStruct)->glistItemsWithoutPath){
        g_list_foreach((*ThreadStartStruct)->glistItemsWithoutPath, (GFunc) GListFreeGStrings, NULL); g_list_free((*ThreadStartStruct)->glistItemsWithoutPath);
    }

    if ( (*ThreadStartStruct)->gstrThisSidePath ){
        g_string_free((*ThreadStartStruct)->gstrThisSidePath, TRUE);
    }
    if ( (*ThreadStartStruct)->gstrOtherSidePath ){
        g_string_free((*ThreadStartStruct)->gstrOtherSidePath, TRUE);
    }
    if ( (*ThreadStartStruct)->gstrLeftSidePath ){
        g_string_free((*ThreadStartStruct)->gstrLeftSidePath, TRUE);
    }
    if ( (*ThreadStartStruct)->gstrRightSidePath ){
        g_string_free((*ThreadStartStruct)->gstrRightSidePath, TRUE);
    }
    if ( (*ThreadStartStruct)->gstrHomeDir ){
        g_string_free((*ThreadStartStruct)->gstrHomeDir, TRUE);
    }
    if ( (*ThreadStartStruct)->gstrWorkingDir ){
        g_string_free((*ThreadStartStruct)->gstrWorkingDir, TRUE);
    }
    if ( (*ThreadStartStruct)->gstrParsedCommand ){
        g_string_free((*ThreadStartStruct)->gstrParsedCommand, TRUE);
    }
    if ( (*ThreadStartStruct)->gstrQuitReplyText ){
        g_string_free((*ThreadStartStruct)->gstrQuitReplyText, TRUE);
    }
    if ( (*ThreadStartStruct)->gstrActionNoteText ){
        g_string_free((*ThreadStartStruct)->gstrActionNoteText, TRUE);
    }

    free (*ThreadStartStruct);
    *ThreadStartStruct = NULL;

    return 0;
}


void CallUserCommand_MainEntryThread (void *data){
    //this is our main entry point which is supposed to be threaded and started from CallUserCommand_StartThread
    //data is from type struct _CallUserCommand_StartThreadStructur
    TRACEIT(10,"CallUserCommand_MainEntryThread			start");

    if (data == NULL){
        TRACEIT(2,"CallUserCommand_MainEntryThread			parameter NULL");
        g_thread_exit(0);
    }
    struct _CallUserCommand_StartThreadStructure *ThreadStartStruct = (struct _CallUserCommand_StartThreadStructure *)data;
    struct _UserCommandList *myUserCommandListItem = (struct _UserCommandList *)(ThreadStartStruct->OurUserDataListItem->ListElement);
    GList       *glistBuf = NULL;
    //GList       *glistEachThreads = NULL;
    //GThread     *gthreadEachThread = NULL;
    GError      *Error = NULL;
    int         iRet = 0;
    GMutex      *mutex=NULL;

    if (myUserCommandListItem->option_iCallType == USERCOMMAND_CALLTYPE_ALL){
        iRet = CallUserCommand_ParseCommand (ThreadStartStruct);
        if (iRet == 0 && ThreadStartStruct->iCancelAll == 0){
            iRet = CallUserCommand_CallToSystem ((void *)ThreadStartStruct);
        }
    }
    else if (myUserCommandListItem->option_iCallType == USERCOMMAND_CALLTYPE_EACH){
        //this is the only case in which we have to work a little bit more in here
        glistBuf = g_list_first(ThreadStartStruct->glistItemsWithPath);
        while (glistBuf && ThreadStartStruct->uiCurrentPositionInList < ThreadStartStruct->uiSizeOfLists){
            if (ThreadStartStruct->iCancelAll == 2){
                break;
            }
            iRet = CallUserCommand_ParseCommand (ThreadStartStruct);	//this is blocking call when asking the user for input
            if (iRet == 0 && ThreadStartStruct->iCancelAll == 0){
                if (ThreadStartStruct->gcontEachCondition != NULL){
                    TRACEIT (2, "CallUserCommand_MainEntryThread        internal error, CallUserCommand_MainEntryThread != NULL");
                    g_cond_wait (ThreadStartStruct->gcontEachCondition, mutex);
                }
                mutex = g_mutex_new();
                ThreadStartStruct->gcontEachCondition = g_cond_new ();
                g_mutex_lock (mutex);

                //gthreadEachThread = g_thread_create((GThreadFunc)CallUserCommand_CallToSystem, (void *)ThreadStartStruct , TRUE, &Error);
                g_thread_create((GThreadFunc)CallUserCommand_CallToSystem, (void *)ThreadStartStruct , FALSE, &Error);
				//we have to wait for this thread till condition signal because of our ThreadStartStruct, which is used for the next item too!
                g_cond_wait (ThreadStartStruct->gcontEachCondition, mutex);
                g_mutex_unlock (mutex);
                g_mutex_free (mutex);
                mutex = NULL;
                g_cond_free (ThreadStartStruct->gcontEachCondition);
                ThreadStartStruct->gcontEachCondition = NULL;
            }
            if (Error != NULL){
                TRACEIT (4, "CallUserCommand_MainEntryThread        error creating thread to call usercommand, error:");
                TRACEIT (4, Error->message);
                g_error_free(Error);
                Error = NULL;
            }
            //if (/*iRet == 0 && ThreadStartStruct->iCancelAll == 0 && */gthreadEachThread != NULL){
            //    glistEachThreads = g_list_prepend (glistEachThreads, (void *)gthreadEachThread);
            //    gthreadEachThread = NULL;
            //}

            glistBuf = glistBuf->next;
            ThreadStartStruct->uiCurrentPositionInList++;
        }
        //we have to wait for the threads to finish
        //do we really have to wait for all the threads to finish? Actually, CallUserCOmmandCallToSystem
        //will block us with a condition until it does not need the ThreadStartStruct any more, and therefore,
        //deleting this struct should be of no harm.
        /*
        glistBuf = glistEachThreads;
        while (glistBuf){
        	printf("Joining thread      at CallUserCommand_MainEntryThread\n");
            g_thread_join ( (GThread *)glistBuf->data);
            printf("Leaving thread join at CallUserCommand_MainEntryThread\n");
            glistBuf = glistBuf->next;
        }*/
    }
    else if (myUserCommandListItem->option_iCallType == USERCOMMAND_CALLTYPE_LIST){
        mutex = g_mutex_new();
        ThreadStartStruct->gcontDialogCondition = g_cond_new ();
        g_mutex_lock (mutex);

        CallUserCommand_ProcessViaList(ThreadStartStruct);

        g_cond_wait (ThreadStartStruct->gcontDialogCondition, mutex);
        g_mutex_unlock (mutex);
        g_mutex_free (mutex);
        mutex = NULL;
        g_cond_free (ThreadStartStruct->gcontDialogCondition);
        ThreadStartStruct->gcontEachCondition = NULL;
    }
	/*
    if (glistEachThreads){
        g_list_free (glistEachThreads);
    }*/
    if (Error){
        g_error_free(Error);
        Error = NULL;
    }
    CallUserCommand_StartThreadStructure_freeStruct ( &ThreadStartStruct);
    TRACEIT(10,"CallUserCommand_MainEntryThread			end");
    g_thread_exit(0);
}


int CallUserCommand_ParseCommand(struct _CallUserCommand_StartThreadStructure *ThreadStartStruct){
    TRACEIT (10, "CallUserCommand_ParseCommand      start");
    //this function is used to parse the command and fill the workingdir in ThreadStartStruct
    //it will return 0 even the user aports the operation (look into ThreadStartStruct)
    //returns -1 if an error occurs
    if (ThreadStartStruct == NULL || ThreadStartStruct->OurUserDataListItem==NULL || ThreadStartStruct->OurUserDataListItem->ListElement==NULL){
        TRACEIT (2, "CallUSerCommand_CallToSystem           Parameter NULL");
        return -1;
    }

    int         iRet = 0;
    GList       *glistBuf = NULL;
    int         iCallType;
    struct _UserCommandList *UserCommandStruct = ((struct _UserCommandList *)ThreadStartStruct->OurUserDataListItem->ListElement);


    if (ThreadStartStruct->gstrWorkingDir != NULL){
        g_string_free(ThreadStartStruct->gstrWorkingDir, TRUE);
        ThreadStartStruct->gstrWorkingDir = NULL;
    }
    if (ThreadStartStruct->gstrParsedCommand != NULL){
        g_string_free(ThreadStartStruct->gstrParsedCommand, TRUE);
        ThreadStartStruct->gstrParsedCommand = NULL;
    }
    if (ThreadStartStruct->gstrQuitReplyText != NULL){
        g_string_free(ThreadStartStruct->gstrQuitReplyText, TRUE);
        ThreadStartStruct->gstrQuitReplyText = NULL;
    }
    if (ThreadStartStruct->gstrActionNoteText != NULL){
        g_string_free(ThreadStartStruct->gstrActionNoteText, TRUE);
        ThreadStartStruct->gstrActionNoteText = NULL;
    }

    iCallType = UserCommandStruct->option_iCallType;

    //first, let us collect some Information:
    //the string which is copied to gstrWorking path depends on the calltype we are (all, each, list)
    //because we allways make use of the glist which is stored with pathes we are independent from the sourcetype (SideObject, Fileshelf)
    if (ThreadStartStruct->glistItemsWithPath!=NULL && ThreadStartStruct->glistItemsWithoutPath != NULL){
        if (iCallType == USERCOMMAND_CALLTYPE_EACH){
            //get the path stored with the item we currently used for the process
            glistBuf = g_list_nth (ThreadStartStruct->glistItemsWithPath, ThreadStartStruct->uiCurrentPositionInList);
            if (glistBuf == NULL){
                TRACEIT (2,"CallUserCommand_CallToSystem        internal error, item adressed not found");
                iRet = -1;
            }
            else {
                ThreadStartStruct->gstrWorkingDir = g_string_new( ((GString *)glistBuf->data)->str);
            }
        }
        else if (iCallType == USERCOMMAND_CALLTYPE_LIST){
            //get the path stored with the tem currently in the list
            glistBuf = g_list_nth (ThreadStartStruct->glistItemsWithPath, ThreadStartStruct->uiCurrentPositionInList);
            if (glistBuf == NULL){
                TRACEIT (2,"CallUserCommand_CallToSystem        internal error, item adressed not found");
                iRet = -1;
            }
            else{
                ThreadStartStruct->gstrWorkingDir = g_string_new( ((GString *)glistBuf->data)->str);
            }
        }
        else{ // (iCallType == USERCOMMAND_CALLTYPE_ALL){
            //get the path stored with the first item
            ThreadStartStruct->gstrWorkingDir = g_string_new( ((GString *)ThreadStartStruct->glistItemsWithPath->data)->str);
        }
        GetPathFromPathWithName(ThreadStartStruct->gstrWorkingDir, 0);
    }
    else if (UserCommandStruct->gstrWorkingDir){
        ThreadStartStruct->gstrWorkingDir = g_string_new (UserCommandStruct->gstrWorkingDir->str);
        //printf("CallUserCommand_CallToSystem: setting Listelement\n");
    }
    else{
        //printf("CallUserCommand_CallToSystem: setting homedir\n");
        ThreadStartStruct->gstrWorkingDir = g_string_sized_new (20);
        getHomeDirectory (ThreadStartStruct->gstrWorkingDir);
    }

    if (iRet == 0){
        //this function will fill the QuitReplyText and ActionNoteText too
        iRet = ParseXMLUserCommand (ThreadStartStruct, &(ThreadStartStruct->gstrParsedCommand), &(ThreadStartStruct->gstrQuitReplyText), &(ThreadStartStruct->gstrActionNoteText), ThreadStartStruct->gstrWorkingDir);
    }

    if (ThreadStartStruct->commStru){
        if (iRet == 1 ){//1 == user canceled this item, 2 == user canceled all items
            if (ThreadStartStruct->commStru->currentItem->ErrorMessage){
                g_string_assign (ThreadStartStruct->commStru->currentItem->ErrorMessage, "user canceled");
            }
            else{
                ThreadStartStruct->commStru->currentItem->ErrorMessage = g_string_new("user canceled");
            }
        }
        else if (iRet == 2){
            if (ThreadStartStruct->commStru->currentItem->ErrorMessage){
                g_string_assign (ThreadStartStruct->commStru->currentItem->ErrorMessage, "user canceled all");
            }
            else{
                ThreadStartStruct->commStru->currentItem->ErrorMessage = g_string_new("user canceled all");
            }
        }
    }

    TRACEIT (10, "CallUserCommand_ParseCommand      end");
    return iRet;
}


int CallUserCommand_CallToSystem (void *data){
    //this is our new function which calls the command to the system
    //it also will parse the command
    //commStru can be NULL, in this case we do not come from a list-call, but however, this info is also stored in ThreadStartStruct!
    //if we come from a list-call, list->Errors will set on errors
    struct _CallUserCommand_StartThreadStructure *ThreadStartStruct = (struct _CallUserCommand_StartThreadStructure *)data;
    //return: 0=ok
    //return: -1, error
    //1 = internal error, item adressed not found
    if (ThreadStartStruct == NULL){
        TRACEIT (2, "CallUSerCommand_CallToSystem           Parameter NULL");
        return -1;
    }

    //create a little widget, displaying the command currently processing
    int iRet=0;
    int iNotify=0;
    GtkWidget   *scrollwidget;
    GtkWidget   *ActionNoteWindow=NULL;
    GtkVBox     *VBox;
    GtkWidget   *label=NULL;
    struct ListWidgetCommunicationStruct *commStru = ThreadStartStruct->commStru;

    if (iRet == 0 && ThreadStartStruct->gstrActionNoteText){
        myWrap_gdk_threads_enter("callItemsToSystem 1");
        if (strlen (ThreadStartStruct->gstrActionNoteText->str) > 0){
            ActionNoteWindow = CreateMyWindow(ThreadStartStruct->gstrActionNoteText->str, NULL, 400,100, TRUE, FALSE, FALSE, &iNotify, FALSE, TRUE);
        }
        else{
            ActionNoteWindow = CreateMyWindow("running process", NULL, 400,100, TRUE, FALSE, FALSE, &iNotify, FALSE, TRUE);
        }
        AddMyIconsToWindow((GtkWindow *)ActionNoteWindow);
        label = gtk_label_new(NULL);
        gtk_label_set_selectable ( (GtkLabel *)label, TRUE);
        gtk_label_set_line_wrap ( (GtkLabel *)label, TRUE);

        VBox = g_object_new(GTK_TYPE_VBOX, "spacing", 5, "homogeneous", FALSE, NULL);

        scrollwidget = gtk_scrolled_window_new(NULL,NULL);
        gtk_scrolled_window_add_with_viewport ( (GtkScrolledWindow *)scrollwidget, label);
        gtk_container_add(GTK_CONTAINER(VBox),scrollwidget);

        //gtk_box_pack_start(GTK_BOX(VBox),label,FALSE,FALSE,1);
        gtk_container_add((GtkContainer *)ActionNoteWindow,(GtkWidget *)VBox);
        gtk_widget_show_all (ActionNoteWindow);
        myWrap_gdk_threads_leave("callItemsToSystem 1");
    }

    //COMMAND TO SYSTEM SHELL
    int         i_countCommand=0;
    int         i_exitstatus=0;
    gchar       *errors=NULL;
    int         int_error=0;
    GString     *gstrToken = g_string_sized_new(10);
    GString     *gstrErrorBuf = g_string_sized_new(10);
    GString     *gstrBuf = g_string_sized_new(10);

    GString *gstrParsedCommand = NULL;
    GString *gstrActionNoteText = NULL;
    GString *gstrWorkingDir = NULL;
    GString *gstrQuitReplyText = NULL;
    GString *gstrHomeDir = NULL;

    if (ThreadStartStruct->gstrParsedCommand){
        gstrParsedCommand = g_string_new (ThreadStartStruct->gstrParsedCommand->str);
    }
    if (ThreadStartStruct->gstrActionNoteText){
        gstrActionNoteText = g_string_new (ThreadStartStruct->gstrActionNoteText->str);
    }
    if (ThreadStartStruct->gstrQuitReplyText){
        gstrQuitReplyText = g_string_new (ThreadStartStruct->gstrQuitReplyText->str);
    }
    if (ThreadStartStruct->gstrWorkingDir){
        gstrWorkingDir = g_string_new (ThreadStartStruct->gstrWorkingDir->str);
    }
    if (ThreadStartStruct->gstrHomeDir){
        gstrHomeDir = g_string_new (ThreadStartStruct->gstrHomeDir->str);
    }


	//tell the waiting thread that we finished copying the information we need and we do not longer need the ThreadStartStruct
    if (ThreadStartStruct->gcontEachCondition)
        g_cond_signal (ThreadStartStruct->gcontEachCondition);

	while (iRet == 0 && myTokenFunction (gstrParsedCommand, i_countCommand, "&&", gstrToken) == 1){
        if (gstrActionNoteText){
            TRACEIT(10,"callItemsToSystem		ActionNote is defined");
            myWrap_gdk_threads_enter("callItemsToSystem 2");
            if (0 != iNotify)
                gtk_label_set_text ((GtkLabel *)label, gstrToken->str);
            myWrap_gdk_threads_leave("callItemsToSystem 2");
        }
        TRACEIT(10,"callItemsToSystem		going to call command to the system");
        chdir(gstrWorkingDir->str);

		if (!myWrapOf__g_spawn_command_line_sync (gstrToken->str, NULL, &errors, &i_exitstatus, NULL,TRUE)){
		    g_string_prepend(gstrToken, "\tCommand not found:\t\n\n");
		    g_string_append(gstrToken, "\n");
		    myWrap_gdk_threads_enter("callItemsToSystem 3");
            MyMessageWidget(win, "Invalid command", gstrToken->str);
            myWrap_gdk_threads_leave("callItemsToSystem 3");
            iRet = -1;
            break;
		}
		TRACEIT(10,"callItemsToSystem		Checking for errors....");
        if (i_exitstatus != 0 && errors!=NULL && strlen(errors)>1){
            g_string_assign (gstrErrorBuf, errors);
            if (strlen(gstrErrorBuf->str) > MAXCHARSOFERROROFUSERCOMMAND){
                cleverCutStringToNChars(gstrErrorBuf, "\n..._SOME_TEXT_WAS_DELETED_HERE_...\n",MAXCHARSOFERROROFUSERCOMMAND);
            }
            if (commStru){  //write error to lists GString
                TRACEIT(10,"callItemsToSystem		Write error to lists GString");
                if (commStru->currentItem->ErrorMessage == NULL)
                    commStru->currentItem->ErrorMessage = g_string_new(gstrErrorBuf->str);
                else {
                    g_string_assign (commStru->currentItem->ErrorMessage, gstrErrorBuf->str);
                }
                int_error=1;
            }
            else{   //open a message widget to show the error
                TRACEIT(10,"callItemsToSystem		creating error widget");
                g_string_sprintf(gstrBuf,"Command #%d returned with error.\nEvery following process is cancelled.\nError:\n%s",i_countCommand,gstrErrorBuf->str);
                myWrap_gdk_threads_enter("callItemsToSystem 4");
                MyMessageWidget(win, "Command quits with error",gstrBuf->str);
                myWrap_gdk_threads_leave("callItemsToSystem 4");
                int_error=1;
            }
            TRACEIT(10,"callItemsToSystem		break after error");
            iRet = -1;
            break;
        }

		if (errors){
            g_free(errors);
            errors = NULL;
		}
        i_countCommand++;
	}

	//destroy ActionNoteWindow
	if (ActionNoteWindow && 0 != iNotify){
	    TRACEIT(10,"callItemsToSystem		ActionNoteWindow is going to be destroyed");
	    myWrap_gdk_threads_enter("callItemsToSystem 5");
	    gtk_widget_destroy(ActionNoteWindow);
	    myWrap_gdk_threads_leave("callItemsToSystem 5");
    }

    TRACEIT(10,"callItemsToSystem		QuitReplayText?");
	if (gstrQuitReplyText && int_error == 0){
		myWrap_gdk_threads_enter("callItemsToSystem 6");
		MyMessageWidget(win, "Process end", gstrQuitReplyText->str);
		myWrap_gdk_threads_leave("callItemsToSystem 6");

	}
	//getHomeDirectory(&homedir);
	chdir(gstrHomeDir->str); //thsi causes an error ! replace it by the homedirctisy stored in ThreadList (function parameter 1)

    // list->Errors
    if (gstrBuf){
        g_string_free(gstrBuf, TRUE);
    }
    if (gstrErrorBuf){
        g_string_free(gstrErrorBuf, TRUE);
    }
    if (gstrToken){
        g_string_free (gstrToken, TRUE);
    }

    if (gstrParsedCommand){
        g_string_free (gstrParsedCommand, TRUE);
    }
    if (gstrActionNoteText){
        g_string_free (gstrActionNoteText, TRUE);
    }
    if (gstrQuitReplyText){
        g_string_free (gstrQuitReplyText, TRUE);
    }
    if (gstrWorkingDir){
        g_string_free (gstrWorkingDir, TRUE);
    }
    if (gstrHomeDir){
        g_string_free (gstrHomeDir, TRUE);
    }

    if (iRet  == 0){
        iRet = int_error;
    }
    return iRet;
}


int CallUserCommand_ProcessViaList(struct _CallUserCommand_StartThreadStructure *ThreadStartStruct){
    //data is from type struct _CallUserCommand_StartThreadStructur
    //this function processes userdata via list
    //we use our listwidget which we have developed over several versions
    //so we have to create the right list filled with the listitems
	TRACEIT(10,"CallUserCommand_ProcessViaList		start");
	if (ThreadStartStruct == NULL){
		TRACEIT(10,"CallUserCommand_ProcessViaList		ends");
		//g_thread_exit(0);
		return -1;
	}

	//we will need this list, but we have to create it in here!
	struct CopyReMoveList   *list = NULL;
	struct CopyReMoveList   *listbuf = NULL;
	int     i=0;
	//char    buf[320];
	GList   *glistBuf=NULL;
	//int iRet = 0;
	//int iItemsCount=0;
	struct _UserCommandList *UserCommandList = (struct _UserCommandList *) (ThreadStartStruct->OurUserDataListItem->ListElement);


	struct ListWidgetCommunicationStruct *commStru = malloc(sizeof(struct ListWidgetCommunicationStruct));
	if (commStru == NULL){
        TRACEIT (2, "CAllUserCommand_ProcessViaList     error allocating memory");
        return -1;
	}
	//First, lets create the list widget where we view the items
	GtkWidget               *Window;
	GtkWidget               *scrollWindow;
	GtkWidget               *listView;
	GtkListStore            *listStore;
	GtkTreeIter             iter;
	GtkTreeViewColumn   	*col_Items;
	GtkTreeViewColumn       *col_ItemNumbers;
	GtkTreeViewColumn       *col_Errors;
	GtkCellRenderer     	*renderer;
	GtkCellRenderer         *renderer2;
	GtkButton               *CancelButton;
	GtkButton               *CloseButton;
	GtkButton               *continueButton;
	GtkButton               *pauseButton;
	GtkWidget               *TheLabel;
	GtkVBox                 *VBox;
	GtkWidget               *ButtonBox;
	GString                 *WindowName = g_string_new(MYNAMESHORT);    //freed when the commStru gets freed

	myWrap_gdk_threads_enter("CallUserCommand_ProcessViaList 1");

	if (UserCommandList->option_gstrCaption){
        g_string_assign (WindowName, UserCommandList->option_gstrCaption->str);
	}

	Window = CreateMyWindow(WindowName->str, NULL, 400,200,TRUE, FALSE, TRUE, NULL, FALSE, FALSE);
	if (Window == NULL){
        g_string_free(WindowName, TRUE);
        free(commStru);
        TRACEIT(1,"CallUserCommand_ProcessViaList		end, window could not be created");
        return -1;
	}

    ThreadStartStruct->commStru = commStru;
    //Lets build the list (CopyReMoveList) needed to process the items with our widet
    //which list (with or without path) we use, we decide on the fact from which source we come
    if (ThreadStartStruct->OurUserDataListItem->iSourceType==SOURCE_SIDEOBJECT){
        glistBuf = g_list_first(ThreadStartStruct->glistItemsWithoutPath);

    }
    else if (ThreadStartStruct->OurUserDataListItem->iSourceType==SOURCE_FILESHELF){
        glistBuf = g_list_first(ThreadStartStruct->glistItemsWithPath);
    }
    while (glistBuf){
        CopyReMoveList_AddItem(&list, ((GString *)(glistBuf->data))->str, NULL, NULL, NULL, NULL);
        glistBuf = glistBuf->next;
    }

    //continue creating the list widget
	AddMyIconsToWindow((GtkWindow *)Window);

	scrollWindow = gtk_scrolled_window_new (NULL,NULL);
	listView = gtk_tree_view_new ();
	listStore = gtk_list_store_new( 3 , G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING  );

    col_Items=gtk_tree_view_column_new();
	col_ItemNumbers=gtk_tree_view_column_new();
	col_Errors=gtk_tree_view_column_new();
	gtk_tree_view_column_set_resizable(col_Items,TRUE);
	gtk_tree_view_column_set_resizable(col_ItemNumbers,TRUE);
	gtk_tree_view_column_set_resizable(col_Errors,TRUE);

	gtk_tree_view_column_set_title(col_ItemNumbers, "#");
	gtk_tree_view_append_column(GTK_TREE_VIEW(listView),col_ItemNumbers);
	renderer2 = gtk_cell_renderer_text_new();
	gtk_tree_view_column_pack_start(col_ItemNumbers, renderer2, TRUE);
	gtk_tree_view_column_add_attribute(col_ItemNumbers,renderer2,"text",0);

	gtk_tree_view_column_set_title(col_Items, "items to process");
	gtk_tree_view_append_column(GTK_TREE_VIEW(listView),col_Items);
	renderer = gtk_cell_renderer_text_new();
	gtk_tree_view_column_pack_start(col_Items, renderer, TRUE);
	gtk_tree_view_column_add_attribute(col_Items,renderer,"text",1);

	gtk_tree_view_column_set_title(col_Errors, "errors");
	gtk_tree_view_append_column(GTK_TREE_VIEW(listView),col_Errors);
	renderer = gtk_cell_renderer_text_new();
	gtk_tree_view_column_pack_start(col_Errors, renderer, TRUE);
	gtk_tree_view_column_add_attribute(col_Errors,renderer,"text",2);

	//g_signal_connect (listView,"button-press-event",G_CALLBACK(oneMouseButtonPressedInLittleWindow),NULL);
	g_signal_connect (Window,"delete-event",G_CALLBACK(ListWidget_DeleteEvent),commStru);
    //g_signal_connect (Window, "destroy", G_CALLBACK(ListWidget_End_FreeMemory),commStru); //do not use this signal in here!

    //was never used, its just a reminder!
    //    if (ThreadStartStruct->iCancelAll != 0){
    //    break;
    //    }

    listbuf = list;
	i=1;
	while(listbuf!= NULL){
		//snprintf(buf,319,"%d/%d",i,ThreadStartStruct->uiSizeOfLists);
		gtk_list_store_append(listStore,&iter);
		gtk_list_store_set (listStore, &iter, 1, listbuf->string1->str,-1);
        //listbuf->listStore=listStore;   //forget it, this is just for CalcSizeAfterCopyMove

        CopyReMoveList_initializeMemberTo (listbuf, listStore, iter, NULL, NULL, NULL, NULL, NULL, NULL, NULL, list->CopyMoveOrRemove, NULL );

		//listbuf->iter = iter;
		//listbuf->status = -1;
		//listbuf->CopyMoveOrRemove = list->CopyMoveOrRemove;

		i++;
		listbuf = listbuf->NextItem;
	}
	copyReMoveWidget_updateItemCounterColumn(list);

	CancelButton = (GtkButton *)gtk_button_new_with_label("cancel");
	g_signal_connect (CancelButton,"clicked",G_CALLBACK(ListWidget_CancelButton),(void *)commStru);

	CloseButton = (GtkButton *)gtk_button_new_with_label("close");
	g_signal_connect (CloseButton,"clicked",G_CALLBACK(ListWidget_CloseButton),commStru);

	continueButton = (GtkButton *)gtk_button_new_with_label("continue");
    g_signal_connect (continueButton, "clicked", G_CALLBACK(ListWidget_continueButton), commStru);

    pauseButton = (GtkButton *)gtk_button_new_with_label("pause");
    g_signal_connect (pauseButton, "clicked", G_CALLBACK(ListWidget_pauseButton), commStru);

	gtk_tree_view_set_model (GTK_TREE_VIEW(listView), GTK_TREE_MODEL(listStore));

    if (UserCommandList->option_gstrLabel){
        TheLabel = gtk_label_new(UserCommandList->option_gstrLabel->str);
    }
    else{
        TheLabel = gtk_label_new(UserCommandList->gstrCommand->str);
    }

	gtk_container_add((GtkContainer *)scrollWindow, listView);
	VBox = g_object_new(GTK_TYPE_VBOX, "spacing", 5, "homogeneous", FALSE, NULL);
	gtk_box_pack_start(GTK_BOX(VBox),TheLabel,FALSE,FALSE,1);
	gtk_box_pack_start(GTK_BOX(VBox),GTK_WIDGET(scrollWindow),TRUE,TRUE,1);

    ButtonBox = gtk_hbox_new ( TRUE, 3);

    gtk_box_pack_start(GTK_BOX(ButtonBox),GTK_WIDGET(pauseButton),TRUE,TRUE,1);
    gtk_box_pack_start(GTK_BOX(ButtonBox),GTK_WIDGET(continueButton),TRUE,TRUE,1);
	gtk_box_pack_start(GTK_BOX(ButtonBox),GTK_WIDGET(CancelButton),TRUE,TRUE,1);
    gtk_box_pack_start(GTK_BOX(ButtonBox),GTK_WIDGET(CloseButton),TRUE,TRUE,1);

    gtk_box_pack_start(GTK_BOX(VBox),GTK_WIDGET(ButtonBox),FALSE,FALSE,1);

    gtk_container_add((GtkContainer *)Window,(GtkWidget *)VBox);

    //The widget now is ready,

    //we now fill the communication struct with all the info neede
    commStru->mainListWidget=Window;
    commStru->listView=listView;
    commStru->cancelButton=CancelButton;
    commStru->closeButton=CloseButton;
    commStru->continueButton=continueButton;
    commStru->pauseButton=pauseButton;
    commStru->firstItem=list;
    commStru->currentItem=list;
    commStru->listStore=listStore;
    commStru->cancellable=NULL;//g_cancellable_new();
    commStru->gerror=NULL;
    commStru->iSomeError=0;
    commStru->SecondThreadRunning=0;
    commStru->TheLabel=TheLabel;
    commStru->WindowName=WindowName;
    commStru->iCancelled=0;  //becomes 1 if the process is cancelled
    commStru->iListPaused=0;
    commStru->iListPausedAutomatic=0;
    commStru->FreeMemoryAndDeleteWidgetFunction = FreeMemoryAndDeleteWidgetFunction;
    commStru->CancelListFunction = NULL;    // we do not need this in here
    commStru->iCancelDialogRuns=0;
    commStru->iDestroyFlag =0;

    commStru->bOverWriteAllExisting=-1;
    commStru->ui64TotalBytes=0;
    commStru->ui64TotalBytesProcessed=0;
    commStru->i64StartTotalBytesCopied=-1.0;
    commStru->dCopySpeed=0.0;
    commStru->iTotalBytesValid=0;
    commStru->progressBar=NULL;
    commStru->gthrdCopyReMoveThread_1 = NULL;
    commStru->gthrdCopyReMoveThread_2 = NULL;
    commStru->mtxContinue = g_mutex_new ();
    commStru->iNextThreadStarted = 1;
    commStru->ThreadStartStruct=NULL;

    //last thing to do before processing the list is to show the lsit widget
	gtk_widget_show_all(Window);
	gtk_widget_hide(GTK_WIDGET(CloseButton));
	gtk_widget_hide(GTK_WIDGET(continueButton));

	prepareWidgetForDrop ((GtkWidget *)commStru->listView, 1, DNDSignal_dragdatereceive_forCopyReMoveWidget, commStru);
	g_signal_connect ((GtkWidget *)commStru->listView,"drag-motion",G_CALLBACK(CopyReSizeWidget_dragmotion_signal),commStru);
	g_signal_connect ((GtkWidget *)commStru->listView,"drag-leave",G_CALLBACK(CopyReSizeWidget_dragleave_signal),commStru);
    g_signal_connect ((GtkWidget *)commStru->listView,"key-press-event",G_CALLBACK(CopyReSizeWidget_oneKeyPressedInView),commStru);
    g_signal_connect ((GtkWidget *)commStru->listView,"button-press-event",G_CALLBACK(ListWidget_MousePressEventCatcher),commStru);


	myWrap_gdk_threads_leave("CallUserCommand_ProcessViaList 1");

    //start processing the list
    //here are some variables we need
    //GtkTreePath         *pTreePath;
    //GtkTreeSelection    *selection;
    //int                 iSomeError=0;

    // at this moment I dont see that we need this in here
    //but I do clearly. When the lsit is processed and e.g. the user opens a UserDialog for each item,
    //its still possible to cancel the list and after that close the listwidget while the userdialog is still
    //open. Now when the userdialog gets closed, a segfault is the logical consequence.
    //therefore we need this here!
    //commStru->SecondThreadRunning = 1;  //I know I know this is the firstThread.

    ThreadStartStruct->uiCurrentPositionInList=0;
    commStru->ThreadStartStruct = ThreadStartStruct;

    commStru->gthrdCopyReMoveThread_1 = g_thread_create((GThreadFunc)CallUserCommand_ProcessItemThread, (void *)commStru , TRUE, NULL);

	TRACEIT(10,"CallUserCommand_ProcessViaList		ends");
    //g_thread_exit(0);
    return 0;
}


void CallUserCommand_ProcessItemThread (struct ListWidgetCommunicationStruct *commStru){
    TRACEIT(10,"CallUserCommand_ProcessItemThread		start");
    if (commStru == NULL){
        TRACEIT(2,"CallUserCommand_ProcessItemThread		PARAMETER NULL");
        g_thread_exit(0);
    }

    int                     iStillValid=1;
    int                     iBreak = 0;
    struct _CallUserCommand_StartThreadStructure *ThreadStartStruct = commStru->ThreadStartStruct;
    struct CopyReMoveList   *list = NULL;
    GtkTreeSelection        *selection;
    GtkTreePath             *pTreePath;
    int                     iRet = 0;
    //int                     iSomeError=0;

    //struct ListWidgetCommunicationStruct *commStru = ThreadStartStruct->commStru;

    if (ThreadStartStruct == NULL){
        TRACEIT(2,"CallUserCommand_ProcessItemThread		PARAMETER NULL 2");
        g_thread_exit(0);
    }

    myWrap_gdk_threads_enter("CallUserCommand_ProcessViaList 2");
    selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(commStru->listView));
    myWrap_gdk_threads_leave("CallUserCommand_ProcessViaList 2");

    commStru->SecondThreadRunning = 1;

    if (NULL != commStru->gthrdCopyReMoveThread_2){
        //this means that this is NOT the first CopyReMove Thread, as this thread will create the second thread to continue the list
        if (NULL != commStru->gthrdCopyReMoveThread_1){
            g_thread_join (commStru->gthrdCopyReMoveThread_1);  //make sure, that only one CopyReMove Thread is running at a time and the last threads memory gets freed
        }
        commStru->gthrdCopyReMoveThread_1 = commStru->gthrdCopyReMoveThread_2;  //forget the old thread and store this thread at the 1st position;
        commStru->gthrdCopyReMoveThread_2 = NULL;
    }

    list = commStru->currentItem;   //for faster writing. and shorter sentences
    if (NULL == list){
        //because this function will set the next currentItem for the following thread, the currentItem can be NULL, as the list ends.
        //So we reached the end of the list and we only have to clear everything as usual
        iBreak = 2;
    }
    if (iBreak==0 && list->string1==NULL  ){
        list->ibreak=1;
        iBreak = 1;
    }
    if (iBreak==0 && (list->status == -10 || commStru->iCancelled==1)){
        list->ibreak=1;
        iBreak = 1;
    }


    //while (commStru->currentItem && commStru->iCancelled==0){
    if (iBreak==0 && list != NULL && commStru->iCancelled==0){
        myWrap_gdk_threads_enter("CallUserCommand_ProcessViaList 3");

        gtk_tree_selection_select_iter(selection, &(list->iter));
        //make the scrolling follow the marked Item
        pTreePath = gtk_tree_model_get_path (gtk_tree_view_get_model ((GtkTreeView *)commStru->listView),&(list->iter));
        gtk_tree_view_scroll_to_cell ((GtkTreeView *)commStru->listView,pTreePath,NULL,TRUE,0.0,0.0);
        gtk_tree_path_free(pTreePath);

        myWrap_gdk_threads_leave("CallUserCommand_ProcessViaList 3");

        if (list->ErrorMessage){
            g_string_free(list->ErrorMessage,TRUE);
            list->ErrorMessage = NULL;
        }
        list->ErrorMessage = g_string_sized_new(10);    //make clear this pointer is valid

        if (commStru->gerror){
            g_error_free(commStru->gerror);
            commStru->gerror=NULL;
            TRACEIT(10,"CallUserCommand_ProcessViaList		error struct was not NULL, checkpoint 2");
        }

        //TODO check status before set/start ?
        list->status = 0;

        iRet = CallUserCommand_ParseCommand (ThreadStartStruct);
        if (iRet == 0 && ThreadStartStruct->iCancelAll == 0){
            iRet = CallUserCommand_CallToSystem (ThreadStartStruct);
        }

        list->status = 1;

        //if (commStru->iCancelled==1){
            //list->ibreak=1;
        //    break;
        //}
        if (ThreadStartStruct->iCancelAll == 1 || iRet == 2){
            commStru->iCancelled=1;
        }

        if (iRet != 0){
            myWrap_gdk_threads_enter("CallUserCommand_ProcessViaList 4");
            gtk_list_store_set (commStru->listStore, &(list->iter), 2, list->ErrorMessage->str,-1);
            myWrap_gdk_threads_leave("CallUserCommand_ProcessViaList 5");
            g_string_free (list->ErrorMessage, TRUE);
            list->ErrorMessage = NULL;
            commStru->iSomeError=1;
        }

        //ThreadStartStruct->uiCurrentPositionInList++;
        //commStru->currentItem = commStru->currentItem->NextItem;
    }



    g_mutex_lock (commStru->mtxContinue);
    iStillValid = 1;

    ThreadStartStruct->uiCurrentPositionInList++;

    if (iBreak == 2){   //iBreak=2 was set if the currentItem was NULL, in this case, this thread is the last one and we have finished the list
        if (commStru->iCancelled==1 || commStru->iSomeError!=0){ //error or cancel, its not our turn to close the widget, but to update some things

            myWrap_gdk_threads_enter("CallUserCommand_ProcessViaList 4");
            if (commStru->iSomeError == 1){
                MyMessageWidget((GtkWindow *)commStru->mainListWidget, "ERROR", "There was at least one Error \nwhile processing the list.\nLook at the column errors and scroll up\ntill you find the error message.");
                g_string_append(commStru->WindowName," ERROR");
            }
            else if (commStru->iCancelled == 1){
                g_string_append(commStru->WindowName," Cancelled");
            }
            gtk_window_set_title((GtkWindow *)commStru->mainListWidget,commStru->WindowName->str);

            gtk_widget_show(GTK_WIDGET(commStru->closeButton));
            gtk_widget_hide(GTK_WIDGET(commStru->cancelButton));

            gtk_widget_queue_draw ((GtkWidget *)commStru->mainListWidget);
            myWrap_gdk_threads_leave("CallUserCommand_ProcessViaList 4");

            commStru->SecondThreadRunning = 0;
        }
        else if (commStru->iCancelDialogRuns==0){
            g_mutex_unlock (commStru->mtxContinue);
            iStillValid = 0;
            myWrap_gdk_threads_enter("CallUserCommand_ProcessViaList 5");
            commStru->SecondThreadRunning = 0;
            (commStru->FreeMemoryAndDeleteWidgetFunction)( commStru);  //here Notify MUST BE 2 other wise this call will lead to a deadlock becuase of SecondThreadRunning
            myWrap_gdk_threads_leave("CallUserCommand_ProcessViaList 5");
        }
        else{ //the cancel dialog is open
            commStru->SecondThreadRunning = 0;
            commStru->iDestroyFlag = 1;
        }

        if (ThreadStartStruct->gcontDialogCondition)
            g_cond_signal (ThreadStartStruct->gcontDialogCondition);

    }
    else if (0 == commStru->iListPaused){
        //we will start the next thread, in the case that the next item is NULL, the thread is created without the option to join it:
        commStru->iNextThreadStarted = 1;

        commStru->currentItem = commStru->currentItem->NextItem;

        if (NULL != commStru->currentItem){
            commStru->gthrdCopyReMoveThread_2 = g_thread_create((GThreadFunc)CallUserCommand_ProcessItemThread, (void *)commStru , TRUE, NULL);
        }
        else {
            g_thread_create((GThreadFunc)CallUserCommand_ProcessItemThread, (void *)commStru , FALSE, NULL);
        }
    }
    else{
        commStru->iNextThreadStarted = 0;
    }


    if (1 == iStillValid){
        commStru->SecondThreadRunning = 0;
        g_mutex_unlock (commStru->mtxContinue);
    }


    TRACEIT(10,"CallUserCommand_ProcessItemThread		ends");
    g_thread_exit(0);
}



int UserOption_callTerminal(void *Object, int iSource){
    TRACEIT (10, "UserOption_callTerminal           start");

    if (Object == NULL){
        TRACEIT (2, "UserOption_callTerminal        parameter NULL");
        return -1;
	}
    int iRet = UserOption_callOption (Object, iSource, TERMINALKEY);

    TRACEIT (10, "UserOption_callTerminal           end");
    return iRet;
}


int UserOption_callStandardEditor(void *Object, int iSource){
    TRACEIT (10, "UserOption_callStandardEditor           start");

    if (Object == NULL){
        TRACEIT (2, "UserOption_callStandardEditor        parameter NULL");
        return -1;
	}
    int iRet = UserOption_callOption (Object, iSource, STANDARDEDITORKEY);

    TRACEIT (10, "UserOption_callStandardEditor           end");
    return iRet;
}


int UserOption_callOption (void *Object, int iSource, const char *ccpOptioniD){
    TRACEIT (10, "UserOption_callOption           start");
    if (Object == NULL || ccpOptioniD==NULL){
        TRACEIT (2, "UserOption_callOption        parameter NULL");
        return -1;
	}
    //first load the file if necessary
    configFileHandling_loadConfigFile ( FALSE, FALSE );

    const char *ccpCommand = OptionsList_getValueByID (g_OptionsList, ccpOptioniD);

    if (ccpCommand == NULL){
        TRACEIT (6, "UserOption_callOption            no terminal key defined");
        return -1;
    }

    struct UserDataList *theDummyItem = malloc (sizeof (struct UserDataList));
    if (theDummyItem == NULL){
        TRACEIT (3, "UserOption_callOption            unable to allocate memory");
        return -1;
    }

    UserDataList_CreateDummyCommand (theDummyItem, ccpCommand, Object, iSource, USERCOMMAND_CALLTYPE_ALL );
    startItemsWithUserCommand(NULL, theDummyItem);
    UserCommandList_freeList( (struct _UserCommandList **)&((theDummyItem->ListElement)) );
    UserDataList_DeleteList(&theDummyItem);
    TRACEIT (10, "UserOption_callOption           end");
    return 0;
}


int UserDataListFromBookmarkList(struct UserDataList **UserDataListToFill){
    //creates a UserDataList from g_BookMarkList
    //will delete USerDataListToFill first if it != NULL
    TRACEIT (10, "UserDataListFromBookmarkList          start");
    if (UserDataListToFill == NULL){
        TRACEIT(2, "UserDataListFromBookmarkList            parameter NULL");
        return -1;
    }
    if (*UserDataListToFill != NULL){
        UserDataList_DeleteList (UserDataListToFill);
    }

    configFileHandling_loadConfigFile ( FALSE, FALSE );

    struct _OptionsList *BookMarkListBuf = OptionsList_getFirstItem(g_BookMarkList);
    if (BookMarkListBuf == NULL){
        TRACEIT (2, "UserDataListFromBookmarkList       no bookmarks in file? Could not get g_BookMarkList");
        return -1;
    }

    while (BookMarkListBuf){
        UserDataList_AddItem(UserDataListToFill, BookMarkListBuf->gstrName->str, (void *)BookMarkListBuf);
        BookMarkListBuf = BookMarkListBuf->nextItem;
    }

    TRACEIT (10, "UserDataListFromBookmarkList          end");
    return 0;
}

