/*
    Copyright 2007, 2008, 2009, 2010 Geyer Klaus

    This file is part of Cat'sEyE.

    Cat'sEyE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Cat'sEyE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Cat'sEyE.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef USERDATA_H_
#define USERDATA_H_



/////////////////////////////////////////////////////////////////////////////////////////////////
///////////////// USER DATA LIST //////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////
struct UserDataList{
	//GString *UserCommand;
	//GString *UserCommandName;   //fill this with the name (including the path) of the usercommand as shown in the contextmenu
	GString *gstrCompleteName;  //means the name with path (more/item)
	GString *gstrMenuName;      //the name without the path (item)
	void *ListElement;      //will hold the element of the list from struct _UserCommandList, struct _OptionsList for bookmarks

    int iSourceType;        //from an Object or Fileshelf
	void *Object;  //SideObjectList or FileShelfObjectList for usercommands, Notebook for bookmarks

	GtkWidget *MenuToAddTo; //pointer to a menue to which this item should be added

	struct UserDataList *prevItem;
	struct UserDataList *nextItem;
};


void UserDataList_AddItem(struct UserDataList **theList, char *cpCompleteName, void *ListElement);
void UserDataList_DeleteList(struct UserDataList **theList);
struct UserDataList *UserDataList_getFirstItem(struct UserDataList *theList);
struct UserDataList *UserDataList_getLastItem(struct UserDataList *theList);
struct UserDataList *UserDataList_getStandardItem(struct UserDataList *theList);
struct UserDataList *UserDataList_getItemByListElement(struct UserDataList *theList, void *ListElement);
int UserDataList_CreateDummyCommand (struct UserDataList *theDummyItem, const char *cpCommand, void *Object, int iSource, int iCallType);
//struct UserDataList *UserDataList_CreateListFrom_UserCommandList (struct _UserCommandList *userCommandList);


/////////////////////////////////////////////////////////////////////////////////////////////////
///////////////// MENU OBJECT LIST //////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////
struct MenuObjectList{
	GtkWidget *submenu;
	GString *gstrName;
	GtkWidget *MenuToAddTo;

	struct MenuObjectList *prevItem;
	struct MenuObjectList *nextItem;
};

void MenuObjectList_DeleteList(struct MenuObjectList **theList);
void MenuObjectList_AddItem(struct MenuObjectList **theList, char *Name, GtkWidget *submenu, GtkWidget *MenuToAddTo);
struct MenuObjectList *MenuObjectList_getFirstItem(struct MenuObjectList *theList);
struct MenuObjectList *MenuObjectList_getLastItem(struct MenuObjectList *theList);


/////////////////////////////////////////////////////////////////////////////////////////////////
///////////////// _CallUserCommand_StartThreadStructure //////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////
struct _CallUserCommand_StartThreadStructure{
    //gets filled in CallUserCommand_StartThread

    //do not forget to free this thinks!!!!
    struct UserDataList *OurUserDataListItem;
    GList *glistItemsWithPath;
    GList *glistItemsWithoutPath;
    GString *gstrThisSidePath;
    GString *gstrOtherSidePath;
    GString *gstrLeftSidePath;
    GString *gstrRightSidePath;
    GString *gstrHomeDir;
    GString *gstrWorkingDir;
    GString *gstrParsedCommand;
    GString *gstrQuitReplyText;
    GString *gstrActionNoteText;

    struct ListWidgetCommunicationStruct *commStru; //if calling via list this comes in handy

    //creating the userdialog we had some trouble as we want Cat'sEyE not to be blocked as long the dialog runs.
    //for this we need some more variables:
    // following variables will not be freed in CallUserCommand_StartThreadStructure_freeStruct like the above items
    GtkWidget * wdgtUserDialog;         //init: NULL
    GtkWidget * wdgtUSerDialogMainVBox;     //init: NULL
    GCond     * gcontDialogCondition;   //init: NULL
    GCond     * gcontEachCondition;     //init: NULL
    int         iUserDialogReturnValue; //initialized by 0

    //No need to free this
    unsigned int uiCurrentPositionInList;   //saves the current position in the list glistItemsWithPath, glistItemsWithoutPath should have the same order
    unsigned int uiSizeOfLists;             //saves the size of the lists glistItemsWithPath and WithoutPath

    int iCancelAll;                 //0=continue, 1=cancel one, 2=cancel all to cancel processing the list (e.g. in each mode!, but also in list mode too)
};


void UserCommand_CreateUserMenu(GtkWidget *rootmenu, struct UserDataList *UserData, struct MenuObjectList **MenuObjects);
void UserCommand_AttachUserMenu(struct MenuObjectList *MenuObjects);



////////////////////////////////////////////////////////////////////////////////////////////
///////////////// Functions ////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////

int CallUserCommand_StartThread (struct UserDataList *OurUserDataListItem, GList *glistItemsWithPath, GList *glistItemsWithoutPath, GString *gstrThisSidePath, GString *gstrOtherSidePath, GString *gstrLeftSidePath, GString *gstrRightSidePath, GString *gstrHomeDir);
void CallUserCommand_ProcessItemThread (struct ListWidgetCommunicationStruct *commStru);
int UserOption_callTerminal(void *Object, int iSource);
int UserOption_callStandardEditor(void *Object, int iSource);
int UserOption_callOption (void *Object, int iSource, const char *ccpOptioniD);
int UserDataListFromBookmarkList(struct UserDataList **UserDataListToFill);


#endif //USERDATA_H_





